@@ -1,4 +1,4 @@
-/* $Id: e2p_config.c 539 2007-07-18 11:52:43Z tpgww $
+/* $Id: e2p_config.c 706 2007-11-06 09:13:06Z tpgww $
 
 Copyright (C) 2006-2007 tooar <tooar@gmx.net>
 
@@ -445,18 +445,26 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 	const gchar *filepath = gtk_entry_get_text (GTK_ENTRY (rt->open_entry));
 	gchar *realpath = (*filepath != '\0') ? (gchar *) filepath :
 		g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
-	gchar *contents;
-	//get file contents
+	gchar *localpath = F_FILENAME_TO_LOCALE (realpath);
+#ifdef E2_VFS
+	VPATH ddata = { localpath, NULL };	//only allow local config data
+#endif
+	gpointer contents;
+	//get file
 read_file:
-	if (e2_fs_get_file_contents (realpath, &contents E2_ERR_NONE()))
+#ifdef E2_VFS
+	if (e2_fs_get_file_contents (&ddata, &contents, NULL, TRUE E2_ERR_NONE()))
+#else
+	if (e2_fs_get_file_contents (localpath, &contents, NULL, TRUE E2_ERR_NONE()))
+#endif
 	{
 		//check for conforming version, hence format for tree options
 		gchar *sp, *sq, *st;
-		sp = strchr (contents, '\n');
+		sp = strchr ((gchar *)contents, '\n');
 		if (sp != NULL)
 		{
 			*sp = '\0';
-			sq = strstr (contents, "(v");
+			sq = strstr ((gchar *)contents, "(v");
 			if (sq != NULL)
 			{
 				st = g_strrstr (sq, ")");
@@ -487,7 +495,7 @@ read_file:
 								g_free (sq);
 								if (realpath != filepath)
 									g_free (realpath);
-								g_free (contents);
+								g_free (contents);	//need free() if file buffer allocated by malloc()
 								goto read_file;
 							}
 						}
@@ -498,7 +506,7 @@ read_file:
 							g_free (sq);
 							if (realpath != filepath)
 								g_free (realpath);
-							g_free (contents);
+							g_free (contents);	//need free() if file buffer allocated by malloc()
 							return;
 						}
 					}
@@ -512,8 +520,8 @@ read_file:
 
 		rebuild_needed = FALSE;
 		//process the file
-		_e2pc_filter_options (contents, import_flags);
-		g_free (contents);
+		_e2pc_filter_options ((gchar *)contents, import_flags);
+		g_free (contents);	//need free() if file buffer allocated by malloc()
 		if (rebuild_needed)
 		{
 			if (upgrade)
@@ -527,7 +535,13 @@ read_file:
 				gchar *local2 = F_FILENAME_TO_LOCALE (path3);
 				gchar *savepath = e2_utils_get_tempname (local2);
 				gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifdef E2_VFS
+				VPATH ddata = { local1, NULL };	//local config files only
+				VPATH tdata = { savepath, NULL };	//local config files only
+				e2_task_backend_rename (&ddata, &tdata);
+#else
 				e2_task_backend_rename (local1, savepath);
+#endif
 				gdk_threads_enter ();
 				g_free (path1);
 				g_free (path2);
@@ -640,6 +654,7 @@ read_file:
 	}
 	if (*filepath == '\0')
 		g_free (realpath);
+	F_FREE (localpath);
 	g_ptr_array_free (treeset_names, TRUE);
 }
 /**
@@ -691,24 +706,30 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
 {
 	const gchar *savepath = gtk_entry_get_text (GTK_ENTRY (rt->save_entry));
-	if (e2_option_bool_get ("confirm-overwrite"))
+	if (*savepath != '\0')
 	{
-#ifdef E2_FILES_UTF8ONLY
-		gchar *dlocal = F_FILENAME_TO_LOCALE ((gchar *)savepath);
-#else
-		gchar *dlocal = F_FILENAME_TO_LOCALE (savepath);
-#endif
-		if (e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
+		if (e2_option_bool_get ("confirm-overwrite"))
 		{
-			DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
-			if (choice != OK)
+			gchar *dlocal = D_FILENAME_TO_LOCALE (savepath);
+			g_strstrip (dlocal);
+#ifdef E2_VFS
+			VPATH ddata = { dlocal, NULL };
+			if (e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
+			if (e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
+#endif
 			{
-				F_FREE (dlocal);
-				return;
+				DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
+				if (choice != OK)
+				{
+					g_free (dlocal);
+					return;
+				}
 			}
+			g_free (dlocal);
 		}
+		e2_option_file_write (savepath);
 	}
-	e2_option_file_write ((gchar *) savepath);
 }
 /**
 @brief save config file with new name
@@ -743,8 +764,15 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		gchar *local = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 #ifndef USE_GTK2_8
 		//check for O/W if not done by the dialog itself
+#ifdef E2_VFS
+		VPATH ddata = { local, NULL };
+#endif
 		if (e2_option_bool_get ("confirm-overwrite")
-			&& e2_fs_access2 (local) E2_ERR_NONE() == 0)
+#ifdef E2_VFS
+		&& e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
+		&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
+#endif
 		{
 			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
 			if (choice != OK)
@@ -863,8 +891,15 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		gchar *local = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 #ifndef USE_GTK2_8
 		//check for O/W if not done by the dialog itself
+#ifdef E2_VFS
+		VPATH ddata = { local, NULL };
+#endif
 		if (e2_option_bool_get ("confirm-overwrite")
-			&& e2_fs_access2 (local) E2_ERR_NONE() == 0)
+#ifdef E2_VFS
+			&& e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
+			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
+#endif
 		{
 			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
 			if (choice != OK)
@@ -900,29 +935,39 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 
 	path = gtk_entry_get_text (GTK_ENTRY (rt->iconsavedir_entry));
 	if (!g_path_is_absolute (path))
-		dest =
-#ifdef E2_VFSTMP
-		e2_utils_get_absolute_dirpath (path, E2_DIRCUR_PRIVATE, NULL);
-#else
-		e2_utils_dircat (curr_view, path);
-#endif
+//E2_VFSTMPOK
+		dest = e2_utils_dircat (curr_view, path, FALSE);
 	else
 		dest = g_strdup (path);
 	if (g_str_has_suffix (dest, G_DIR_SEPARATOR_S))
-		*(dest + strlen (dest) - 1) = '\0';
+		*(dest + strlen (dest) - sizeof (gchar)) = '\0';
 	dlocal = F_FILENAME_TO_LOCALE (dest);
 
 	DialogButtons result;
+#ifdef E2_VFS
+	VPATH ddata = { dlocal, NULL };
+#endif
 	if (e2_option_bool_get ("confirm-overwrite")
+#ifdef E2_VFS
+		&& e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
 		&& e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
+#endif
 		result = e2_dialog_ow_check (slocal, dlocal, NONE);
 	else
 		result = OK;
 
 	if (result == OK)
 	{
+#ifdef E2_VFS
+		VPATH sdata = { slocal, NULL };
+#endif
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifdef E2_VFS
+		e2_task_backend_copy (&sdata, &ddata, E2_FTM_NORMAL);
+#else
 		e2_task_backend_copy (slocal, dlocal, E2_FTM_NORMAL);
+#endif
 		gdk_threads_enter ();
 	}
 
@@ -984,13 +1029,21 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
+#ifdef E2_ASSISTED
+	GtkWidget *label =
+#endif
 	e2_widget_add_label (vbox, _("Save configuration data in"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
 	const gchar *savepath;
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
 	//default export to config dir if it's usable
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };
+	if (e2_fs_is_dir3 (&ddata E2_ERR_NONE()) && !e2_fs_access (&ddata, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#else
 	if (e2_fs_is_dir3 (local E2_ERR_NONE()) && !e2_fs_access (local, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#endif
 		savepath = e2_cl_options.config_dir;
 	else
 		savepath = g_get_home_dir ();
@@ -1008,7 +1061,12 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		g_free (tempext);
 		F_FREE (local2);
 		E2_ERR_DECLARE
+#ifdef E2_VFS
+		ddata.localpath = temppath;
+		if (e2_fs_access2 (&ddata E2_ERR_PTR())	//checkme need for vfs ?
+#else
 		if (e2_fs_access2 (temppath E2_ERR_PTR())	//checkme need for vfs ?
+#endif
 				&& E2_ERR_IS (ENOENT))
 		{
 			E2_ERR_CLEAR
@@ -1025,6 +1083,9 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		g_free (temppath);
 
 	rt->save_entry = e2_widget_add_entry (vbox, (gchar *) savepath, TRUE, TRUE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (GTK_LABEL (label), rt->save_entry);
+#endif
 	gtk_widget_set_size_request (rt->save_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
@@ -1040,7 +1101,10 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		_("Save current config data in the specified file"), _e2pc_save_cb, rt);
 	gtk_container_add (GTK_CONTAINER (hbox), btn);
 
-	GtkWidget *label = gtk_label_new (_("export"));
+#ifndef E2_ASSISTED
+	GtkWidget *
+#endif
+	label = gtk_label_new (_("export"));
 	gtk_widget_show (label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
@@ -1056,13 +1120,21 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
+#ifdef E2_ASSISTED
+	GtkWidget *label =
+#endif
 	e2_widget_add_label (vbox, _("Get configuration data from"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
 	const gchar *openpath;
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
 	//default export to config dir if it's usable
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };
+	if (e2_fs_is_dir3 (&ddata E2_ERR_NONE()) && !e2_fs_access (&ddata, R_OK | X_OK E2_ERR_NONE()))
+#else
 	if (e2_fs_is_dir3 (local E2_ERR_NONE()) && !e2_fs_access (local, R_OK | X_OK E2_ERR_NONE()))
+#endif
 		openpath = e2_cl_options.config_dir;
 	else
 		openpath = g_get_home_dir ();
@@ -1071,6 +1143,9 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	gchar *openname = g_build_filename (openpath, default_config_file, NULL);
 
 	rt->open_entry = e2_widget_add_entry (vbox, (gchar *) openname, TRUE, TRUE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (GTK_LABEL (label), rt->open_entry);
+#endif
 	gtk_widget_set_size_request (rt->open_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
@@ -1110,53 +1185,56 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	gtk_widget_show (expvbox);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	gchar *label = _e2pc_get_setlabel (PANEBAR1_P);
-	_e2pc_create_check_button (hbox, PANEBAR1_P, FALSE, label);
-	g_free (label);
-	label = _e2pc_get_setlabel (PANEBAR2_P);
-	_e2pc_create_check_button (hbox, PANEBAR2_P, FALSE, label);
-	g_free (label);
+	gchar *labeltxt = _e2pc_get_setlabel (PANEBAR1_P);
+	_e2pc_create_check_button (hbox, PANEBAR1_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (PANEBAR2_P);
+	_e2pc_create_check_button (hbox, PANEBAR2_P, FALSE, labeltxt);
+	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	label = _e2pc_get_setlabel (TASKBAR_P);
-	_e2pc_create_check_button (hbox, TASKBAR_P, FALSE, label);
-	g_free (label);
-	label = _e2pc_get_setlabel (CMDBAR_P);
-	_e2pc_create_check_button (hbox, CMDBAR_P, FALSE, label);
-	g_free (label);
+	labeltxt = _e2pc_get_setlabel (TASKBAR_P);
+	_e2pc_create_check_button (hbox, TASKBAR_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (CMDBAR_P);
+	_e2pc_create_check_button (hbox, CMDBAR_P, FALSE, labeltxt);
+	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	label =  _e2pc_get_setlabel (MARKS_P);
-	_e2pc_create_check_button (hbox, MARKS_P, FALSE, label);
-	g_free (label);
-	label = _e2pc_get_setlabel (FILETYPES_P);
-	_e2pc_create_check_button (hbox, FILETYPES_P, FALSE, label);
-	g_free (label);
+	labeltxt =  _e2pc_get_setlabel (MARKS_P);
+	_e2pc_create_check_button (hbox, MARKS_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (FILETYPES_P);
+	_e2pc_create_check_button (hbox, FILETYPES_P, FALSE, labeltxt);
+	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	label = _e2pc_get_setlabel (BINDINGS_P);
-	_e2pc_create_check_button (hbox, BINDINGS_P, FALSE, label);
-	g_free (label);
-	label = _e2pc_get_setlabel (ALIASES_P);
-	_e2pc_create_check_button (hbox, ALIASES_P, FALSE, label);
-	g_free (label);
+	labeltxt = _e2pc_get_setlabel (BINDINGS_P);
+	_e2pc_create_check_button (hbox, BINDINGS_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (ALIASES_P);
+	_e2pc_create_check_button (hbox, ALIASES_P, FALSE, labeltxt);
+	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	label = _e2pc_get_setlabel (MENU_P);
-	_e2pc_create_check_button (hbox, MENU_P, FALSE, label);
-	g_free (label);
-	label = _e2pc_get_setlabel (PLUGINS_P);
-	_e2pc_create_check_button (hbox, PLUGINS_P, FALSE, label);
-	g_free (label);
+	labeltxt = _e2pc_get_setlabel (MENU_P);
+	_e2pc_create_check_button (hbox, MENU_P, FALSE, labeltxt);
+	g_free (labeltxt);
+	labeltxt = _e2pc_get_setlabel (PLUGINS_P);
+	_e2pc_create_check_button (hbox, PLUGINS_P, FALSE, labeltxt);
+	g_free (labeltxt);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
-	label = _e2pc_get_setlabel (CUSTMENU_P);
-	_e2pc_create_check_button (hbox, CUSTMENU_P, FALSE, label);
-	g_free (label);
+	labeltxt = _e2pc_get_setlabel (CUSTMENU_P);
+	_e2pc_create_check_button (hbox, CUSTMENU_P, FALSE, labeltxt);
+	g_free (labeltxt);
 
-	GtkWidget *pagelabel = gtk_label_new (_("import"));
-	gtk_widget_show (pagelabel);
-	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, pagelabel);
+#ifndef E2_ASSISTED
+	GtkWidget *
+#endif
+	label = gtk_label_new (_("import"));
+	gtk_widget_show (label);
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
 /**
 @brief  create and show notebook page with config import widgets
@@ -1170,21 +1248,32 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
+#ifdef E2_ASSISTED
+	GtkWidget *label =
+#endif
 	e2_widget_add_label (vbox, _("Use icons in"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
 	gchar *openpath;
 	gchar *localpath = e2_utils_get_icons_path (FALSE);
 	//default icons in config dir if that's usable
+#ifdef E2_VFS
+	VPATH ddata = { localpath, NULL };
+	if (!e2_fs_is_dir3 (&ddata E2_ERR_NONE()) || e2_fs_access (&ddata, R_OK | X_OK E2_ERR_NONE()))
+#else
 	if (!e2_fs_is_dir3 (localpath E2_ERR_NONE()) || e2_fs_access (localpath, R_OK | X_OK E2_ERR_NONE()))
+#endif
 		openpath = g_build_filename (e2_cl_options.config_dir, _("icons"), NULL);
 	else
 		openpath = D_FILENAME_FROM_LOCALE (localpath);
 	g_free (localpath);
 
 	rt->icondir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (GTK_LABEL (label), rt->icondir_entry);
+#endif
 	g_free (openpath);
-	gtk_widget_set_size_request (rt->open_entry, 400, -1);
+	gtk_widget_set_size_request (rt->icondir_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
@@ -1202,14 +1291,20 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 
 	e2_widget_add_separator (vbox, FALSE, E2_PADDING_SMALL);
 	//now the exporting
+#ifdef E2_ASSISTED
+	label =
+#endif
 	e2_widget_add_label (vbox, _("Copy current icons to"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
 	openpath = g_strconcat (e2_cl_options.config_dir,G_DIR_SEPARATOR_S BINNAME"-",
 		_("icons"), NULL);
 	rt->iconsavedir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (GTK_LABEL (label), rt->iconsavedir_entry);
+#endif
 	g_free (openpath);
-	gtk_widget_set_size_request (rt->open_entry, 400, -1);
+	gtk_widget_set_size_request (rt->iconsavedir_entry, 400, -1);
 
 	hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
