@@ -1,4 +1,4 @@
-/* $Id: e2p_cpbar.c 510 2007-07-14 09:28:38Z tpgww $
+/* $Id: e2p_cpbar.c 653 2007-09-05 11:22:04Z tpgww $
 **************************************
 
 Emelcopybar - a plugin for emelFM2
@@ -59,7 +59,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #ifdef IS_PAUSABLE
 typedef enum
 {
-	E2_BARTASK_STOPPED   = 1 << 0,
+	E2_BARTASK_STOPPED   = 1,
 	E2_BARTASK_PAUSEREQ  = 1 << 1,	//pause requested
 	E2_BARTASK_PAUSED    = 1 << 2,	//actually paused
 	E2_BARTASK_COMPLETED = 1 << 3,
@@ -77,7 +77,7 @@ typedef struct _E2_ProgressData
 {
 	pthread_mutex_t mutex;  //protects access to these data
 	pthread_cond_t cond;   //signals change
-	gchar *dlocal;	//localised destination (maybe temp) path of item copied
+	VPATH *dlocal;	//includes localised destination path (maybe temp) of item copied
 	guint64 done_size;
 	guint64 refresh_interval;
 } E2_ProgressData;
@@ -86,8 +86,8 @@ typedef struct _E2_ActionData
 {
 //	pthread_mutex_t mutex;  //protects access to these data
 	E2_FileTaskMode flags;	//flags for features of current copy
-	gchar *slocal;	//localised source path of item copied
-	gchar *dlocal;	//localised destination path of item copied
+	VPATH *slocal;	//includes localised source path of item copied
+	VPATH *dlocal;	//includes localised destination path of item copied
 	gboolean completed;	//set TRUE when the backend task is completed (valid or not)
 	gboolean result;	//value returned by backend task function
 } E2_ActionData;
@@ -120,23 +120,23 @@ Error message expects BGL to be open
 @return E2TW_CONTINUE always
 */
 static E2_TwResult
-_e2p_cpbar_twcb (const gchar *localpath, const struct stat *statptr,
+_e2p_cpbar_twcb (VPATH *localpath, const struct stat *statptr,
 	E2_TwStatus status, E2_BarData *twdata)
 {
 	switch (status)
 	{
-		case E2TW_F:	//not directory or link
+		case E2TW_F:	//not dir or link
 		case E2TW_SL:	//symbolic link
 		case E2TW_SLN:	//symbolic link naming non-existing file
 			twdata->totalsize += statptr->st_size;
-		case E2TW_DL:	//directory, not opened due to tree-depth limit
-		case E2TW_DM:	//directory, not opened due to different file system
-		case E2TW_D:	//directory (don't care about its reported size)
-		case E2TW_DNR:	//unreadable directory (for which, error is reported upstream)
-		case E2TW_NS:	//un-statable item (for which, error is reported upstream)
-		case E2TW_DRR:	//directory now readable
+		case E2TW_DL:	//dir, not opened due to tree-depth limit (reported upstream)
+		case E2TW_DM:	//dir, not opened due to different file system (reported upstream)
+		case E2TW_D:	//dir (don't care about its reported size)
+		case E2TW_DRR:	//dir now readable
+		case E2TW_DNR:	//unreadable dir (reported upstream)
+		case E2TW_NS:	//un-statable item (reported upstream)
 			twdata->count++;
-//		case E2TW_DP:	//directory, finished
+//		case E2TW_DP:	//dir, finished
 		default:
 			break;
 	}
@@ -186,12 +186,18 @@ BGL will be open
 static void *
 _e2p_cpbar_progress (E2_ProgressData *data)
 {
-	gchar *local;
+	gchar *localpath;
 	struct stat sb;
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
 //	printd (DEBUG, "monitor thread underway");
 
 	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
 
+#ifdef E2_VFS
+	ddata.spacedata = data->dlocal->spacedata;
+#endif
 	while (TRUE)
 	{
 		pthread_testcancel ();	//has a cancel-instruction been issued ?
@@ -202,26 +208,38 @@ _e2p_cpbar_progress (E2_ProgressData *data)
 		}
 		//get a local copy of the present item's destination path
 		//(that will be NULL if the action thread has finished already)
-		local = (data->dlocal == NULL) ? NULL : g_strdup (data->dlocal);
+		if (data->dlocal == NULL)
+			localpath = NULL;
+		else
+			localpath = g_strdup (VPSTR (data->dlocal));
 
 		if (pthread_mutex_unlock (&(data->mutex)))
 		{
 			printd (WARN,"Unlock error!");
 		}
 
-		if (local != NULL)
+		if (localpath != NULL)
 		{	//we're still underway
 			//determine how much of current item has been copied
 			E2_BarData pdata;
 			pdata.totalsize = 0;
 //			pdata.count = 0;
-			if (!e2_fs_lstat (local, &sb E2_ERR_NONE()))
+#ifdef E2_VFS
+			ddata.localpath = localpath;
+			if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE()))
+#else
+			if (!e2_fs_lstat (localpath, &sb E2_ERR_NONE()))
+#endif
 			{
 				if (S_ISDIR (sb.st_mode))
 				{
 					//if (!
-					e2_fs_tw (local, _e2p_cpbar_twcb, &pdata, -1,
-						E2TW_PHYS E2_ERR_NONE());
+#ifdef E2_VFS
+					e2_fs_tw (&ddata,
+#else
+					e2_fs_tw (localpath,
+#endif
+						_e2p_cpbar_twcb, &pdata, -1, E2TW_PHYS E2_ERR_NONE());
 					//)
 					//{
 						//FIXME handle error
@@ -233,7 +251,7 @@ _e2p_cpbar_progress (E2_ProgressData *data)
 //				pdata.count = 1;
 //			}
 			}
-			g_free (local);
+			g_free (localpath);
 
 			if (pthread_mutex_lock (&(data->mutex)))
 			{
@@ -282,9 +300,8 @@ _e2p_cpbar_action (E2_ActionData *data)
 /**
 @brief update progress window details, and copy item @a src to @a dest
 This is called with BGL open
-@param src path of item to be copied, utf-8 string
-@param dest new path of copied item, utf-8 string
-@param dest_dir the directory part of @a dest, utf-8 string
+@param slocal path of item to be copied, localised string
+@param dlocal new path of copied item, localised string
 @param breakflag pointer to store for abort-flag
 @param flags bitflags indicating task parameters
 @param bdata pointer to bar data struct holding progress data
@@ -293,7 +310,7 @@ This is called with BGL open
 
 @return
 */
-static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
+static void _e2p_cpbar_exec (VPATH *slocal, VPATH *dlocal,
 #ifndef IS_PAUSABLE
 	gboolean *breakflag,
 #endif
@@ -302,8 +319,9 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 {
 	gchar progresstext[64];	//utf-8 string, middle line of progress dialog
 	//localised strings
-	gchar *slocal = F_FILENAME_TO_LOCALE (src);
-	gchar *dlocal = F_FILENAME_TO_LOCALE (dest);
+	gchar *src = F_FILENAME_FROM_LOCALE (VPSTR (slocal));
+//	gchar *dest = F_FILENAME_FROM_LOCALE (VPSTR (dlocal));
+	gchar *dest_dir = g_path_get_dirname (VPSTR (dlocal));
 
 	//before copy starts, get size of item to be copied
 	E2_BarData pdata = { 0,0 };
@@ -317,8 +335,13 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
     //create action thread
 	//work with a temp name so that the backend doesn't create its own
 	//(which would prevent destination-monitoring)
-	gchar *templocal = e2_utils_get_tempname (dlocal);
+	gchar *templocal = e2_utils_get_tempname (VPSTR (dlocal));
+#ifdef E2_VFS
+	VPATH tempdata = { templocal, dlocal->spacedata };
+	E2_ActionData a_data = { flags, slocal , &tempdata, FALSE, FALSE };
+#else
 	E2_ActionData a_data = { flags, slocal , templocal, FALSE, FALSE };
+#endif
 	pthread_t action_thread_id;
 	gint status = pthread_create (&action_thread_id, NULL,
 		(void *) _e2p_cpbar_action, &a_data);
@@ -327,11 +350,10 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 		printd (WARN,"action-thread-create error!");
 //		if (errno == EAGAIN)
 //			printd (WARN, "not enough system resources");
-		F_FREE (slocal);
-		F_FREE (dlocal);
 		g_free (templocal);
 		return;
 	}
+
 	//wait for task to initiate, at least
 	g_usleep (50000);
 	//create progress monitor, if task not finished already
@@ -342,7 +364,11 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 		pthread_mutex_init (&(m_data.mutex), NULL);
 		pthread_cond_init (&(m_data.cond), NULL);
 		//get the temp name now in use for copying the item
+#ifdef E2_VFS
+		m_data.dlocal = &tempdata;
+#else
 		m_data.dlocal = templocal;
+#endif
 		m_data.done_size = 0;
 		//rough approach to setting the reporting interval (usec)
 //		m_data.refresh_interval = pdata.totalsize * 10;
@@ -353,12 +379,10 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 			m_data.refresh_interval = MAX_UPDATE_INTERVAL;	//with a cap
 		pthread_t mon_thread_id;
 		status = pthread_create (&mon_thread_id, NULL,
-			(void *) _e2p_cpbar_progress, &m_data);
+			(gpointer) _e2p_cpbar_progress, &m_data);
 		if (status != 0)
 		{
 			printd (WARN,"monitor-thread-create error!");
-			F_FREE (slocal);
-			F_FREE (dlocal);
 			g_free (templocal);
 			return;
 		}
@@ -372,14 +396,18 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 
 		gchar *shortsrc = e2_utils_str_shorten (src, MAX_CHAR, E2_DOTS_START);
 		gchar *shortdest = e2_utils_str_shorten (dest_dir, MAX_CHAR, E2_DOTS_START);
+		gchar *num1 = g_strdup_printf ("%"PRIu64, bdata->count);	//gettext workaround
+		gchar *num2 = g_strdup_printf ("%"PRIu64, tdata->count);
 		gchar *labeltext = g_strdup_printf (
-			_("copying %s\nto %s\nthis is item %llu of %llu"),
-			shortsrc, shortdest, bdata->count, tdata->count);
+			_("copying %s\nto %s\nthis is item %s of %s"),
+			shortsrc, shortdest, num1, num2);
 		gdk_threads_enter ();
 		gtk_label_set_text (GTK_LABEL (wdata->label), labeltext);
 		gdk_threads_leave ();
 		g_free (shortsrc);
 		g_free (shortdest);
+		g_free (num1);
+		g_free (num2);
 		g_free (labeltext);
 
 		gchar *progress_format = _("%.2f MB of %.2f MB  (%.0f\%%)");
@@ -426,12 +454,15 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 					printd (WARN,"Thread cancel error!\n");
 				}
 				//cleanup anything part-copied
+#ifdef E2_VFS
+				e2_task_backend_delete (&tempdata);
+#else
 				e2_task_backend_delete (templocal);
+#endif
 				g_free (templocal);
-				F_FREE (slocal);
-				F_FREE (dlocal);
 				return;
 			}
+
 			fraction = (gdouble) progress / tdata->totalsize;
 			//deal with rounding errors
 			if (fraction > 1.0)
@@ -500,13 +531,17 @@ static void _e2p_cpbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 		bdata->totalsize += pdata.totalsize;
 
 	if (a_data.result) //copy succeeded
+#ifdef E2_VFS
+		e2_task_backend_rename (&tempdata, dlocal);
+	else
+		e2_task_backend_delete (&tempdata); //cleanup anything part-copied
+#else
 		e2_task_backend_rename (templocal, dlocal);
 	else
 		e2_task_backend_delete (templocal); //cleanup anything part-copied
+#endif
 
 	g_free (templocal);
-	F_FREE (slocal);
-	F_FREE (dlocal);
 
 #ifdef IS_PAUSABLE
 	if (wdata->flags & E2_BARTASK_PAUSEREQ)
@@ -626,17 +661,29 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 		return FALSE;
 	}
 	E2_ERR_DECLARE
+	//FIXME change dir permission if possible ?
+#ifdef E2_VFS
+	VPATH sdata;
+	VPATH ddata = { qed->othrdir, qed->othrspace };
+	if (e2_fs_access (&ddata, W_OK E2_ERR_PTR()))
+#else
 	if (e2_fs_access (qed->othrdir, W_OK E2_ERR_PTR()))
+#endif
 	{
 		e2_fs_error_local (_("Cannot put anything in %s"),
+#ifdef E2_VFS
+			&ddata E2_ERR_MSGL());
+#else
 			qed->othrdir E2_ERR_MSGL());
+#endif
 		E2_ERR_CLEAR
 		return FALSE;
 	}
+
 	GPtrArray *names = qed->names;
 	GtkWidget *vbox;	//, *hbox;
-	gchar *src_dir, *dest_dir, *src, *dest;	//pointers to utf strings
-	gchar *slocal, *dlocal;	//pointers to localised strings
+	GString *src = g_string_sized_new (1024);
+	GString *dest = g_string_sized_new (1024);
 
 	//setup the information window
 	E2_BarWindowData windowdata;
@@ -689,28 +736,38 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 		_("Abort the copying"), _e2p_cpbar_break_cb, &breakflag);
 #endif
 
-	src_dir = F_FILENAME_FROM_LOCALE (qed->currdir);
-	dest_dir = F_FILENAME_FROM_LOCALE (qed->othrdir);
-
+#ifdef E2_VFS
+	sdata.spacedata = qed->currspace;
+#endif
 	//accumulate non-recursed count and total size of src item(s)
 	E2_BarData totaldata = { 0,0 };
 	guint count;
 	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
 	for (count=0; count < names->len; count++, iterator++)
 	{
-		slocal = e2_utils_strcat (qed->currdir, (*iterator)->filename);
+		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
+#ifdef E2_VFS
+		sdata.localpath = src->str;
+#endif
 		//if (!
-		e2_fs_tw (slocal, _e2p_cpbar_twcb, &totaldata, -1,
+#ifdef E2_VFS
+		e2_fs_tw (&sdata, _e2p_cpbar_twcb, &totaldata, -1,
+#else
+		e2_fs_tw (src->str, _e2p_cpbar_twcb, &totaldata, -1,
+#endif
 			E2TW_PHYS E2_ERR_NONE());
 		//)
 		//{
 			//FIXME handle error
 		//}
-		g_free (slocal);
 	}
 	totaldata.count = names->len;	//not interested in nested count
 
-	gboolean check = e2_option_bool_get ("confirm-overwrite");
+	gboolean check = e2_option_bool_get ("confirm-overwrite")
+#ifdef E2_VFS
+		&& qed->currspace == qed->othrspace
+#endif
+	;
 	E2_BarData progressdata = { 1,0 };
 	OW_ButtonFlags extras = (totaldata.count > 1) ? BOTHALL : NONE;
 	iterator = (E2_SelectedItemInfo **) names->pdata;
@@ -726,33 +783,39 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 #endif
 			break;	//user pressed stop btn or closed info window
 		//src_dir, dest_dir have trailing "/"
-		gchar *utf = F_FILENAME_FROM_LOCALE ((*iterator)->filename);
-		src = g_strconcat (src_dir, utf, NULL);
-		dest = g_strconcat (dest_dir, utf, NULL);
-		F_FREE (utf);
+		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
+		g_string_printf (dest, "%s%s", qed->othrdir, (*iterator)->filename);
 
-		dlocal = F_FILENAME_TO_LOCALE (dest);
-		if (check && e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
+#ifdef E2_VFS
+		sdata.localpath = src->str;
+		ddata.localpath = dest->str;
+		if (check && e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
+		if (check && e2_fs_access2 (dest->str E2_ERR_NONE()) == 0)
+#endif
 		{
 			e2_filelist_enable_refresh ();  //allow updates while we wait
-			slocal = F_FILENAME_TO_LOCALE (src);
 			gdk_threads_enter ();
 			*qed->status = E2_TASK_PAUSED;
-			DialogButtons result = e2_dialog_ow_check (slocal, dlocal, extras);
+			DialogButtons result = e2_dialog_ow_check (src->str, dest->str, extras);
 			*qed->status = E2_TASK_RUNNING;
 			gdk_threads_leave ();
-			F_FREE (slocal);
 			e2_filelist_disable_refresh ();
 			switch (result)
 			{
 				case YES_TO_ALL:
 					check = FALSE;
 				case OK:
-					_e2p_cpbar_exec (src, dest, dest_dir,
+#ifdef E2_VFS
+					_e2p_cpbar_exec (&sdata, &ddata,
+#else
+					_e2p_cpbar_exec (src->str, dest->str,
+#endif
 #ifndef IS_PAUSABLE
 						&breakflag,
 #endif
-				GPOINTER_TO_INT (qed->action->data), &progressdata, &totaldata, &windowdata);
+						GPOINTER_TO_INT (qed->action->data),
+						&progressdata, &totaldata, &windowdata);
 					break;
 				case CANCEL:
 					break;
@@ -762,30 +825,30 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 			}
 			if (result == NO_TO_ALL)
 			{
-				g_free (src);
-				g_free (dest);
-				F_FREE (dlocal);	//finished with dlocal for this item
 				break;
 			}
 		}
 		else  //no overwrite, or don't care
 		{
-			_e2p_cpbar_exec (src, dest, dest_dir,
+#ifdef E2_VFS
+			_e2p_cpbar_exec (&sdata, &ddata,
+#else
+			_e2p_cpbar_exec (src->str, dest->str,
+#endif
 #ifndef IS_PAUSABLE
 				&breakflag,
 #endif
-				GPOINTER_TO_INT (qed->action->data), &progressdata, &totaldata, &windowdata);
+				GPOINTER_TO_INT (qed->action->data),
+				&progressdata, &totaldata, &windowdata);
 		}
-		g_free (src);
-		g_free (dest);
-		F_FREE (dlocal);	//finished with dlocal for this item
 		progressdata.count++;
 	}
 	gdk_threads_enter ();
 	gtk_widget_destroy (windowdata.dialog);
 	gdk_threads_leave ();
-	F_FREE (src_dir);
-	F_FREE (dest_dir);
+
+	g_string_free (src, TRUE);
+	g_string_free (dest, TRUE);
 	e2_filelist_request_refresh (other_view->dir, TRUE); //src pane refreshed normally
 	e2_filelist_enable_refresh ();
 	return TRUE;
