@@ -1,4 +1,4 @@
-/* $Id: e2p_dircmp.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2p_dircmp.c 755 2007-11-26 22:02:18Z tpgww $
 
 Portions copyright (C) 2006-2007 tooar <tooar@gmx.net>
 
@@ -34,6 +34,7 @@ typedef struct _E2_CmpData
 {
 #ifdef E2_VFS
 	GError **operr;
+	PlaceInfo *otherspace;
 #endif
 	gint oldroot_len;
 	gchar *newroot;
@@ -378,12 +379,12 @@ Any error message here expects BGL to be open
 @param filepath localised string, absolute path of item to read
 @return pointer to newly-allocated string containing the computed hash, or NULL on error
 */
-static guchar *_e2p_diff_dohash (gchar *filepath)
+static guchar *_e2p_diff_dohash (VPATH *filepath)
 {
 	HASH_CONTEXT md;
 	guchar buffer [BUFSIZ];
 	guchar sum [HASH_LENGTH+1]; //computed hash + trailing \0
-	gint descriptor;
+	gint fdesc;
 	E2_ERR_DECLARE
 
 #if defined(__USE_GNU) && defined(O_NOATIME)
@@ -395,11 +396,11 @@ static guchar *_e2p_diff_dohash (gchar *filepath)
 		//try to proceed without atime preservation
 		E2_ERR_CLEAR
 	}
-	descriptor = e2_fs_safeopen (filepath, flags, 0);
+	fdesc = e2_fs_safeopen (VPSTR (filepath), flags, 0);
 #else	//be strict about open-flags
-	descriptor = e2_fs_safeopen (filepath, O_RDONLY, 0);
+	fdesc = e2_fs_safeopen (filepath, O_RDONLY, 0);
 #endif
-	if (descriptor < 0)
+	if (fdesc < 0)
 	{
 #ifdef E2_VFS
 		e2_fs_set_error_from_errno (&E2_ERR_NAME);
@@ -417,19 +418,16 @@ static guchar *_e2p_diff_dohash (gchar *filepath)
 	{
 		// clear buffer in case we need to pad the hash
 		memset (buffer, 0, BUFSIZ);
-		ssize_t n_read = e2_fs_read (descriptor, buffer, BUFSIZ E2_ERR_PTR());
+		ssize_t n_read = e2_fs_read (fdesc, buffer, BUFSIZ E2_ERR_PTR());
 		if (n_read == BUFSIZ)
 		{
 			HASH_Update (&md, buffer, BUFSIZ);
 			bytes_read += n_read;
 		}
 		else if (n_read == 0)
-			  break;
+			break;
 		else if (n_read < 0)
 		{
-#ifdef E2_VFSTMP
-		//FIXME respond to vfs error
-#endif
 //			if (1)	//FIXME fatal error handling
 			//these are the fatal errors used in md5deep
 			if (   E2_ERR_IS (EACCES)	//permission denied (changed since opened ?)
@@ -447,9 +445,9 @@ static guchar *_e2p_diff_dohash (gchar *filepath)
 			E2_ERR_CLEAR
 			HASH_Update(&md, buffer, BUFSIZ);
 			bytes_read += BUFSIZ;
-		 /* but the file pointer's now undefined. So
-			manually advance it to the next read position */
-			lseek (descriptor, bytes_read, SEEK_SET);
+		 /* but the file pointer's now undefined. So manually advance it to the
+			next read position */
+			lseek (fdesc, bytes_read, SEEK_SET);
 		}
 		else if (n_read < BUFSIZ)
 		{
@@ -491,7 +489,7 @@ else */
 		F_FREE (utf);
 	} */
 
-	TEMP_FAILURE_RETRY (close (descriptor));	//FIXME vfs
+	TEMP_FAILURE_RETRY (close (fdesc));	//FIXME vfs
 
 	return hash;
 }
@@ -504,8 +502,8 @@ BGL is expected to be open/off
 
 @return TRUE if the items are equal, (also FALSE after memory shortage)
 */
-static gboolean _e2p_diff1 (const gchar *localpath, const struct stat *statptr,
-	const gchar *localotherpath)
+static gboolean _e2p_diff1 (VPATH *localpath, const struct stat *statptr,
+	VPATH *localotherpath)
 {
 	gboolean retval;
 	struct stat othersb;
@@ -518,12 +516,12 @@ static gboolean _e2p_diff1 (const gchar *localpath, const struct stat *statptr,
 		retval = FALSE;	//unequal size = quick check
 	else if (S_ISREG (othersb.st_mode) && othersb.st_size > 0)
 	{	//determine equality of items' md5 sums
-		guchar *hash1 = _e2p_diff_dohash ((gchar *)localpath);
+		guchar *hash1 = _e2p_diff_dohash (localpath);
 		if (hash1 == NULL)
 			retval = FALSE;
 		else
 		{
-			guchar *hash2 = _e2p_diff_dohash ((gchar *)localotherpath);
+			guchar *hash2 = _e2p_diff_dohash (localotherpath);
 			if (hash2 == NULL)
 			{
 				g_free (hash1);
@@ -643,7 +641,7 @@ DP or DRR and DP reports).
 @param count pointer to store for items count
 @return E2TW_CONTINUE always
 */
-static E2_TwResult _e2p_diff_count_twcb (const gchar *local_name,
+static E2_TwResult _e2p_diff_count_twcb (VPATH *local_name,
 	const struct stat *statbuf, E2_TwStatus status, guint *count)
 {
 	(*count)++;
@@ -661,11 +659,14 @@ Error message expects BGL to be open/off
 
 @return E2TW_CONTINUE when items match, E2TW_STOP when they don't or can't decide
 */
-static E2_TwResult _e2p_diff_twcb (const gchar *localpath,
+static E2_TwResult _e2p_diff_twcb (VPATH *localpath,
 	const struct stat *statptr, E2_TwStatus status, E2_CmpData *twdata)
 {
 	gchar *localotherpath;
 	E2_TwResult retval = E2TW_STOP;
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
 
 	switch (status)
 	{
@@ -674,22 +675,45 @@ static E2_TwResult _e2p_diff_twcb (const gchar *localpath,
 		case E2TW_SLN:	//symbolic link naming non-existing file
 			localotherpath = e2_utils_strcat (twdata->newroot,
 				(gchar *)localpath + twdata->oldroot_len);
+#ifdef E2_VFS
+			ddata.localpath = localotherpath;
+			ddata.spacedata = twdata->otherspace;
+			if (_e2p_diff1 (localpath, statptr, &ddata))
+#else
 			if (_e2p_diff1 (localpath, statptr, localotherpath))
+#endif
 				retval = E2TW_CONTINUE;
 			g_free (localotherpath);
 			break;
 		case E2TW_D:	//directory
 		case E2TW_DRR:	//directory now readable
 		{
+			struct stat othrstatbuf;
+			localotherpath = e2_utils_strcat (twdata->newroot,
+				(gchar *)localpath + twdata->oldroot_len);
+#ifdef E2_VFS
+			ddata.localpath = localotherpath;
+			ddata.spacedata = twdata->otherspace;
+			if (e2_fs_stat (&ddata, &othrstatbuf E2_ERR_NONE())
+#else
+			if (e2_fs_stat (localotherpath, &othrstatbuf E2_ERR_NONE())
+#endif
+				|| !S_ISDIR (othrstatbuf.st_mode)
+				|| S_ISLNK (othrstatbuf.st_mode))
+				break;
 			//maybe there are more items in the other dir, or if this dir is empty
 			//there will be no other check that the other even exists
 			guint count = 0;
-			e2_fs_tw ((gchar *)localpath, _e2p_diff_count_twcb, &count, 1,
+			e2_fs_tw (localpath, _e2p_diff_count_twcb, &count, 1,
 				E2TW_XQT | E2TW_PHYS E2_ERR_NONE());
 			guint count2 = 0;
 			localotherpath = e2_utils_strcat (twdata->newroot,
 				(gchar *)localpath + twdata->oldroot_len);
+#ifdef E2_VFS
+			e2_fs_tw (&ddata, _e2p_diff_count_twcb, &count2, 1,
+#else
 			e2_fs_tw (localotherpath, _e2p_diff_count_twcb, &count2, 1,
+#endif
 				E2TW_XQT | E2TW_PHYS E2_ERR_NONE());
 			g_free (localotherpath);
 			if (count != count2)
@@ -720,26 +744,36 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 	GtkTreeModel *model = curr_view->model;
 	if (gtk_tree_model_get_iter_first (model, &iter));
 	{	//it's not empty
+		e2_filelist_disable_refresh ();
 		gdk_threads_enter (); //prevent thread-related hang!
 		e2_window_set_cursor (GDK_WATCH);
 		gdk_threads_leave ();
-		while (!curr_view->completed)
+		while (TRUE)
+		{
+			LISTS_LOCK
+			//CHECKME abort if cd detected ?
+			gboolean busy = curr_view->listcontrols.refresh_working
+						 || curr_view->listcontrols.cd_working;
+			LISTS_UNLOCK
+			if (!busy)
+				break;
 			usleep (100000);
-		e2_filelist_disable_refresh ();
+		}
 
+		gchar *curr_local, *other_local, *currpath;
 		FileInfo *info;
 		struct stat othersb;
-		gchar *curr_local, *other_local, *currpath;
 		E2_CmpData data;
 #ifdef E2_VFS
 		data.operr = NULL;	//or something ...
+		data.otherspace = other_view->spacedata;
+		VPATH sdata;
+		VPATH ddata;
+		sdata.spacedata = curr_view->spacedata;
+		ddata.spacedata = other_view->spacedata;
 #endif
-#ifdef E2_VFSTMP
-		//FIXME paths
-#else
 		curr_local = F_FILENAME_TO_LOCALE (curr_view->dir);
 		other_local = F_FILENAME_TO_LOCALE (other_view->dir);
-#endif
 		gboolean matched;
 		GtkTreeSelection *sel = curr_view->selection;
 		gdk_threads_enter ();
@@ -747,13 +781,17 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 		gdk_threads_leave ();
 		do
 		{
-#ifdef E2_VFS
+#ifdef E2_VFSTMP
 			//CLEAR any error data fom last loop
 #endif
 			gtk_tree_model_get (model, &iter, FINFO, &info, -1);
 			data.newroot = e2_utils_strcat (other_local, info->filename);
-
+#ifdef E2_VFS
+			ddata.localpath = data.newroot;
+			if (e2_fs_lstat (&ddata, &othersb E2_ERR_NONE()))
+#else
 			if (e2_fs_lstat (data.newroot, &othersb E2_ERR_NONE()))
+#endif
 				matched = FALSE;	//lstat() failed, crudely assume can't find anything to match
 			else
 			{
@@ -761,11 +799,23 @@ static gpointer _e2p_diff_all (gpointer thread_data)
 				if (S_ISDIR (info->statbuf.st_mode) && S_ISDIR (othersb.st_mode))
 				{
 					data.oldroot_len = strlen (currpath);
+#ifdef E2_VFS
+					sdata.localpath = currpath;
+					matched = e2_fs_tw (&sdata, _e2p_diff_twcb, &data, -1,
+#else
 					matched = e2_fs_tw (currpath, _e2p_diff_twcb, &data, -1,
+#endif
 						E2TW_PHYS E2_ERR_NONE());
 				}
 				else if (!S_ISDIR (info->statbuf.st_mode) && !S_ISDIR (othersb.st_mode))
+#ifdef E2_VFS
+				{
+					sdata.localpath = currpath;
+					matched = _e2p_diff1 (&sdata, &info->statbuf, &ddata);
+				}
+#else
 					matched = _e2p_diff1 (currpath, &info->statbuf, data.newroot);
+#endif
 				else
 					matched = FALSE;	//mixture of dir and non-dir
 
