@@ -1,6 +1,6 @@
-/* $Id: e2p_glob.c 544 2007-07-20 12:48:20Z tpgww $
+/* $Id: e2p_glob.c 799 2008-02-03 23:11:42Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -87,6 +87,7 @@ enum
 };
 
 static gint date_index;
+static gchar *previous_pattern;
 //these need to be in same order as the date options in the config dialog
 static gchar *date_format [5] =
 {
@@ -294,13 +295,16 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 			{
 				//filter pattern is utf
 				s = gtk_entry_get_text (GTK_ENTRY (data->pattern_entry));
-				if (strlen (s) == 0)
+				if (*s == 0)
 				{
 					choices.name_filter.use = FALSE;
 					e2_output_print_error (_("Invalid filename pattern"), FALSE);
 				}
 				else
 				{
+					if (previous_pattern != NULL)
+						g_free (previous_pattern);
+					previous_pattern = g_strdup (s);
 					choices.name_filter.pattern = s;
 					choices.name_filter.invert_mask = GTK_TOGGLE_BUTTON (data->invert_check)->active;
 					choices.name_filter.case_sensitive = GTK_TOGGLE_BUTTON (data->case_sensitive_check)->active;
@@ -341,10 +345,19 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 				if (gtk_tree_model_get_iter_first (model, &iter));
 				{	//it's not empty
 					FileInfo *info;
-					e2_window_set_cursor (GDK_WATCH);
-					while (!curr_view->completed)
-						usleep (100000);
 					e2_filelist_disable_refresh ();
+					e2_window_set_cursor (GDK_WATCH);
+					//wait until any current recreation is finished
+					while (TRUE)
+					{
+						LISTS_LOCK
+						gboolean busy = curr_view->listcontrols.refresh_working
+									 || curr_view->listcontrols.cd_working;
+						LISTS_UNLOCK
+						if (!busy)
+							break;
+						usleep (100000);
+					}
 
 					GtkTreeSelection *sel = curr_view->selection;
 					gtk_tree_selection_unselect_all (sel);	//start with clean slate
@@ -402,6 +415,9 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 		(hbox, TRUE, FALSE, NULL, NULL, FALSE, 0, NULL, &data);
 	e2_widget_add_mid_label (hbox, _("Named like"), 0.0, FALSE, 0);
 	data.pattern_entry = e2_widget_add_entry (hbox, "", TRUE, FALSE);
+//#ifdef E2_ASSISTED
+//	e2_widget_set_label_relations (GTK_LABEL (label), data.pattern_entry);
+//#endif
 	FileInfo *info = e2_fileview_get_selected_first_local (curr_view, FALSE);
 	if (info != NULL)
 	{
@@ -416,12 +432,15 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 			g_free (text);
 		F_FREE (utf);
 	}
+	else if (previous_pattern != NULL)
+		gtk_entry_set_text (GTK_ENTRY(data.pattern_entry), previous_pattern);
+
 	e2_widget_add_mid_label (hbox, _("example: *.c,*.h"), 0.0, FALSE, 0);
 
 	table = e2_widget_add_table (vbox, 1, 3, TRUE, TRUE, 0);
 	data.invert_check = e2_button_add_toggle_to_table (table, _("_Invert"),
 		FALSE, NULL, NULL, 1, 2, 0, 1);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -473,6 +492,9 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 
 	data.size_entry = e2_widget_add_entry (hbox, size_string, TRUE, FALSE);
 //	gtk_widget_set_size_request (data.size_entry, 100, 30);
+//#ifdef E2_ASSISTED
+//	e2_widget_set_label_relations (GTK_LABEL (label), data.size_entry);
+//#endif
 
 	data.units_combo = e2_combobox_add (hbox, FALSE, 0, NULL, NULL, NULL,
 	  E2_COMBOBOX_MENU_STYLE);
@@ -515,7 +537,9 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 	if (info != NULL)
 		g_free (utf);
 //	gtk_widget_set_size_request (data.date_entry, 120, 30);
-
+//#ifdef E2_ASSISTED
+//	e2_widget_set_label_relations (GTK_LABEL (label), data.date_entry);
+//#endif
 	//start at the name entry
 	gtk_editable_select_region (GTK_EDITABLE(data.pattern_entry), 0, -1);
 	gtk_widget_grab_focus (data.pattern_entry);
@@ -540,22 +564,22 @@ static gchar *aname;
 gboolean init_plugin (Plugin *p)
 {
 #define ANAME "glob"
-   aname = _("glob");
+	aname = _("glob");
 
-  p->signature = ANAME VERSION;
-  p->menu_name = _("_Glob..");
-  p->description = _("Select items matching a specified pattern");
-  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
+	p->signature = ANAME VERSION;
+	p->menu_name = _("_Glob..");
+	p->description = _("Select items matching a specified pattern");
+	p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate
 
-  if (p->action == NULL)
-  {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_glob, NULL, FALSE, 0, NULL);
-	return TRUE;
-  }
-  return FALSE;
+	if (p->action == NULL)
+	{
+		//no need to free this
+		gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
+		p->action = e2_plugins_action_register
+			(action_name, E2_ACTION_TYPE_ITEM, _e2p_glob, NULL, FALSE, 0, NULL);
+		return TRUE;
+	}
+	return FALSE;
 }
 /**
 @brief cleanup transient things for this plugin
@@ -566,8 +590,10 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
-  gboolean ret = e2_plugins_action_unregister (action_name);
-  g_free (action_name);
-  return ret;
+	gchar *action_name = g_strconcat (_A(10),".",aname,NULL);
+	gboolean ret = e2_plugins_action_unregister (action_name);
+	g_free (action_name);
+	if (previous_pattern != NULL)
+		g_free (previous_pattern);
+	return ret;
 }
