@@ -1,4 +1,4 @@
-/* $Id: e2p_mvbar.c 475 2007-07-09 11:42:44Z tpgww $
+/* $Id: e2p_mvbar.c 773 2007-11-30 21:19:14Z tpgww $
 **************************************
 
 Emelmovebar - a plugin for emelFM2
@@ -59,7 +59,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #ifdef IS_PAUSABLE
 typedef enum
 {
-	E2_BARTASK_STOPPED   = 1 << 0,
+	E2_BARTASK_STOPPED   = 1,
 	E2_BARTASK_PAUSEREQ  = 1 << 1,	//pause requested
 	E2_BARTASK_PAUSED    = 1 << 2,	//actually paused
 	E2_BARTASK_COMPLETED = 1 << 3,
@@ -77,7 +77,7 @@ typedef struct _E2_ProgressData
 {
 	pthread_mutex_t mutex;  //protects access to these data
 	pthread_cond_t cond;   //signals change
-	gchar *dlocal;	//localised destination (maybe temp) path of item copied
+	VPATH *dlocal;	//includes localised destination path (maybe temp) of item moved
 	guint64 done_size;
 	guint64 refresh_interval;
 } E2_ProgressData;
@@ -86,8 +86,8 @@ typedef struct _E2_ActionData
 {
 //	pthread_mutex_t mutex;  //protects access to these data
 	E2_FileTaskMode flags;	//flags for features of current copy
-	gchar *slocal;	//localised source path of item copied
-	gchar *dlocal;	//localised destination path of item copied
+	VPATH *slocal;	//includes localised source path of item copied
+	VPATH *dlocal;	//includes localised destination path of item copied
 	gboolean completed;	//set TRUE when the backend task is completed (valid or not)
 	gboolean result;	//value returned by backend task function
 } E2_ActionData;
@@ -111,8 +111,8 @@ static gboolean _e2p_mvbarQ (E2_ActionTaskData *qed);
 /**
 @brief update items' count and total size
 This is a callback for a treewalk function
-Error message expects BGL to be open/off
-@param localpath path of item reported by the walker, localised string
+Error message expects BGL to be open
+@param localpath absolute path of item reported by the walker, localised string
 @param statptr pointer to struct stat with data about @a localpath
 @param status code from the walker, indicating what type of report it is
 @param twdata pointer to tw data struct
@@ -120,7 +120,7 @@ Error message expects BGL to be open/off
 @return E2TW_CONTINUE always
 */
 static E2_TwResult
-_e2p_mvbar_twcb (const gchar *localpath, const struct stat *statptr,
+_e2p_mvbar_twcb (VPATH *localpath, const struct stat *statptr,
 	E2_TwStatus status, E2_BarData *twdata)
 {
 	switch (status)
@@ -131,10 +131,10 @@ _e2p_mvbar_twcb (const gchar *localpath, const struct stat *statptr,
 			twdata->totalsize += statptr->st_size;
 		case E2TW_DL:	//dir, not opened due to tree-depth limit (reported upstream)
 		case E2TW_DM:	//dir, not opened due to different file system (reported upstream)
-		case E2TW_DRR:	//dirnow readable
-		case E2TW_DNR:	//unreadable dir (for which, error is reported upstream)
 		case E2TW_D:	//dir (don't care about its reported size)
-		case E2TW_NS:	//un-statable item (for which, error is reported upstream)
+		case E2TW_DRR:	//dir now readable
+		case E2TW_DNR:	//unreadable dir (reported upstream)
+		case E2TW_NS:	//un-statable item (reported upstream)
 			twdata->count++;
 //		case E2TW_DP:	//dir, finished
 		default:
@@ -186,12 +186,18 @@ BGL will be open
 static void *
 _e2p_mvbar_progress (E2_ProgressData *data)
 {
-	gchar *local;
+	gchar *localpath;
 	struct stat sb;
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
 //	printd (DEBUG, "monitor thread underway");
 
 	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
 
+#ifdef E2_VFS
+	ddata.spacedata = data->dlocal->spacedata;
+#endif
 	while (TRUE)
 	{
 		pthread_testcancel ();	//has a cancel-instruction been issued ?
@@ -202,26 +208,38 @@ _e2p_mvbar_progress (E2_ProgressData *data)
 		}
 		//get a local copy of the present item's destination path
 		//(that will be NULL if the action thread has finished already)
-		local = (data->dlocal == NULL) ? NULL : g_strdup (data->dlocal);
+		if (data->dlocal == NULL)
+			localpath = NULL;
+		else
+			localpath = g_strdup (VPSTR (data->dlocal));
 
 		if (pthread_mutex_unlock (&(data->mutex)))
 		{
 			printd (WARN,"Unlock error!");
 		}
 
-		if (local != NULL)
+		if (localpath != NULL)
 		{	//we're still underway
 			//determine how much of current item has been moved
 			E2_BarData pdata;
 			pdata.totalsize = 0;
 //			pdata.count = 0;
-			if (!e2_fs_lstat (local, &sb E2_ERR_NONE()))
+#ifdef E2_VFS
+			ddata.localpath = localpath;
+			if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE()))
+#else
+			if (!e2_fs_lstat (localpath, &sb E2_ERR_NONE()))
+#endif
 			{
 				if (S_ISDIR (sb.st_mode))
 				{
 					//if (!
-					e2_fs_tw (local, _e2p_mvbar_twcb, &pdata, -1,
-						E2TW_PHYS E2_ERR_NONE());
+#ifdef E2_VFS
+					e2_fs_tw (&ddata,
+#else
+					e2_fs_tw (localpath,
+#endif
+					_e2p_mvbar_twcb, &pdata, -1, E2TW_PHYS E2_ERR_NONE());
 					//)
 					//{
 						//FIXME handle error
@@ -233,7 +251,7 @@ _e2p_mvbar_progress (E2_ProgressData *data)
 //				pdata.count = 1;
 //			}
 			}
-			g_free (local);
+			g_free (localpath);
 
 			if (pthread_mutex_lock (&(data->mutex)))
 			{
@@ -271,7 +289,7 @@ The thread simply exits when completed
 static void *
 _e2p_mvbar_action (E2_ActionData *data)
 {
-//	printd (DEBUG, "action thread underway");
+//	printd (DEBUG, "action thread underway for %s", data->slocal);
 	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
 
 	data->result = e2_task_backend_move	//no background-move option in this context
@@ -282,20 +300,18 @@ _e2p_mvbar_action (E2_ActionData *data)
 /**
 @brief update progress window details, and move item @a src to @a dest
 This is called with BGL open
-@param src path of item to be moved, utf-8 string
-@param dest new path of moved item, utf-8 string
-@param dest_dir the directory part of @a dest, utf-8 string
+@param slocal path of item to be moved, localised string
+@param dlocal new path of moved item, localised string
 @param realmove TRUE if this is a move between devices or partititions
 @param breakflag pointer to store for abort-flag
 @param flags bitflags indicating task parameters
-@param bdata pointer to bar data struct holding progress-to-date data
+@param bdata pointer to bar data struct holding progress data
 @param tdata pointer to bar data struct holding totals data
 @param wdata pointer to info-window data struct
 
 @return
 */
-static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
-	gboolean realmove,
+static void _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 #ifndef IS_PAUSABLE
 	gboolean *breakflag,
 #endif
@@ -304,8 +320,9 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 {
 	gchar progresstext[64];	//utf-8 string, middle line of progress dialog
 	//localised strings
-	gchar *slocal = F_FILENAME_TO_LOCALE (src);
-	gchar *dlocal = F_FILENAME_TO_LOCALE (dest);
+	gchar *src = F_FILENAME_FROM_LOCALE (VPSTR (slocal));
+//	gchar *dest = F_FILENAME_FROM_LOCALE (VPSTR (dlocal));
+	gchar *dest_dir = g_path_get_dirname (VPSTR (dlocal));
 
 	//before move starts, get size of item to be moved
 	E2_BarData pdata = { 0,0 };
@@ -317,18 +334,23 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 	//}
 
     //create action thread
-	//work with a temp name so that the backend doesn't create
-	//its own (which would prevent destination-monitoring)
-	gchar *templocal = e2_utils_get_tempname (dlocal);
-	E2_ActionData a_data = { flags, slocal , templocal, FALSE, FALSE };
+	//work with a temp name so that the backend doesn't create its own
+	//(which would prevent destination-monitoring)
+	gchar *templocal = e2_utils_get_tempname (VPSTR (dlocal));
+#ifdef E2_VFS
+	VPATH tempdata = { templocal, dlocal->spacedata };
+	E2_ActionData a_data = { flags, slocal , &tempdata, FALSE, FALSE };
+#else
+	E2_ActionData a_data = { flags, slocal, templocal, FALSE, FALSE };
+#endif
 	pthread_t action_thread_id;
 	gint status = pthread_create (&action_thread_id, NULL,
 		(void *) _e2p_mvbar_action, &a_data);
 	if (status != 0)
 	{
 		printd (WARN,"action-thread-create error!");
-		F_FREE (slocal);
-		F_FREE (dlocal);
+//		if (errno == EAGAIN)
+//			printd (WARN, "not enough system resources");
 		g_free (templocal);
 		return;
 	}
@@ -351,7 +373,11 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 		pthread_mutex_init (&(m_data.mutex), NULL);
 		pthread_cond_init (&(m_data.cond), NULL);
 		//get the temp name now in use for moving the item
+#ifdef E2_VFS
+		m_data.dlocal = &tempdata;
+#else
 		m_data.dlocal = templocal;
+#endif
 		m_data.done_size = 0;
 		//rough approach to setting the reporting interval (usec)
 //		m_data.refresh_interval = pdata.totalsize * 10;
@@ -362,12 +388,10 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 			m_data.refresh_interval = MAX_UPDATE_INTERVAL;	//with a cap
 		pthread_t mon_thread_id;
 		status = pthread_create (&mon_thread_id, NULL,
-			(void *) _e2p_mvbar_progress, &m_data);
+			(gpointer) _e2p_mvbar_progress, &m_data);
 		if (status != 0)
 		{
 			printd (WARN,"monitor-thread-create error!");
-			F_FREE (slocal);
-			F_FREE (dlocal);
 			g_free (templocal);
 			return;
 		}
@@ -381,14 +405,18 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 
 		gchar *shortsrc = e2_utils_str_shorten (src, MAX_CHAR, E2_DOTS_START);
 		gchar *shortdest = e2_utils_str_shorten (dest_dir, MAX_CHAR, E2_DOTS_START);
+		gchar *num1 = g_strdup_printf ("%"PRIu64, bdata->count);	//gettext workaround
+		gchar *num2 = g_strdup_printf ("%"PRIu64, tdata->count);
 		gchar *labeltext = g_strdup_printf (
-			_("moving %s\nto %s\nthis is item %llu of %llu"),
-			shortsrc, shortdest, bdata->count, tdata->count);
+			_("moving %s\nto %s\nthis is item %s of %s"),
+			shortsrc, shortdest, num1, num2);
 		gdk_threads_enter ();
 		gtk_label_set_text (GTK_LABEL (wdata->label), labeltext);
 		gdk_threads_leave ();
 		g_free (shortsrc);
 		g_free (shortdest);
+		g_free (num1);
+		g_free (num2);
 		g_free (labeltext);
 
 		gchar *progress_format = _("%.2f MB of %.2f MB  (%.0f\%%)");
@@ -435,11 +463,13 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 					printd (WARN,"Thread cancel error!\n");
 				}
 				//cleanup anything part-moved
+#ifdef E2_VFS
+				e2_task_backend_delete (&tempdata);
+#else
 				e2_task_backend_delete (templocal);
+#endif
 				g_free (templocal);
-				F_FREE (slocal);
-				F_FREE (dlocal);
-				return;	//FALSE;
+				return;
 			}
 
 			fraction = (gdouble) progress / tdata->totalsize;
@@ -509,7 +539,11 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 
 	if (a_data.result) //move succeeded
 	{
+#ifdef E2_VFS
+		e2_task_backend_rename (&tempdata, dlocal);
+#else
 		e2_task_backend_rename (templocal, dlocal);
+#endif
 		//update the progressive totals
 //		bdata->count += pdata.count;
 /* final display now done earlier
@@ -532,11 +566,13 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 */
 	}
 	else
+#ifdef E2_VFS
+		e2_task_backend_delete (&tempdata); //cleanup anything part-moved
+#else
 		e2_task_backend_delete (templocal); //cleanup anything part-moved
+#endif
 
 	g_free (templocal);
-	F_FREE (slocal);
-	F_FREE (dlocal);
 
 #ifdef IS_PAUSABLE
 	if (wdata->flags & E2_BARTASK_PAUSEREQ)
@@ -549,7 +585,7 @@ static void _e2p_mvbar_exec (gchar *src, gchar *dest, gchar *dest_dir,
 		gdk_threads_leave ();
 	}
 #endif
-	return;	//(a_data.result)
+	return;
 }
 #ifdef IS_PAUSABLE
 /**
@@ -592,13 +628,6 @@ _e2p_mvbar_pause_cb (GtkWidget *widget, E2_BarWindowData *wdata)
 	gtk_widget_grab_focus (wdata->resume_btn);
 	wdata->flags |= E2_BARTASK_PAUSEREQ;
 	//actual pause is started in the monitor loop
-//	pthread_cancel (mon_thread_id);
-// join act_thread_id
-//	pthread_cancel (act_thread_id);
-// ?? create a thread with a gtk_main ()
-//	gint status = pthread_create (&pausethreadID, NULL,
-//		(void *) _e2p_cpbar_paused, NULL);
-//	pthread_join (pausethreadID, NULL);
 	return TRUE;
 }
 /**
@@ -620,13 +649,9 @@ _e2p_mvbar_resume_cb (GtkWidget *widget, E2_BarWindowData *wdata)
 	wdata->flags &= ~E2_BARTASK_PAUSED;
 	e2_filelist_disable_refresh ();
 	gtk_main_quit ();
-	//FIXME resume moving
-//	pthread_create (mon_thread_id);
-//	pthread_create (act_thread_id);
-	//= terminate the gtk_main () thread
 	return TRUE;
 }
-#else
+#else //ndef IS_PAUSABLE
 /**
 @brief callback for stop-button press and window-close event
 
@@ -677,18 +702,62 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 		return FALSE;
 	}
 	E2_ERR_DECLARE
+	//FIXME change dir permission if possible ?
+#ifdef E2_VFS
+	VPATH sdata;
+	sdata.localpath = qed->currdir;
+	VPATH ddata = { qed->othrdir, qed->othrspace };
+	if (e2_fs_access (&ddata, W_OK E2_ERR_PTR()))
+#else
 	if (e2_fs_access (qed->othrdir, W_OK E2_ERR_PTR()))
+#endif
 	{
 		e2_fs_error_local (_("Cannot put anything in %s"),
+#ifdef E2_VFS
+			&ddata E2_ERR_MSGL());
+#else
 			qed->othrdir E2_ERR_MSGL());
+#endif
 		E2_ERR_CLEAR
 		return FALSE;
 	}
 
 	GPtrArray *names = qed->names;
 	GtkWidget *vbox;	//, *hbox;
-	gchar *src_dir, *dest_dir, *src, *dest;	//pointers to utf strings
-	gchar *slocal, *dlocal;	//pointers to localised strings
+	GString *src = g_string_sized_new (1024);
+	GString *dest = g_string_sized_new (1024);
+
+	//determine whether move will be physical or logical
+	gboolean realmove;
+#ifdef E2_VFS
+	if (qed->currspace != qed->othrspace)
+		realmove = TRUE;
+	else
+	{
+#endif
+		dev_t device;
+		struct stat statbuf;
+#ifdef E2_VFS
+		sdata.spacedata = qed->currspace;
+		if (!e2_fs_stat (&sdata, &statbuf E2_ERR_NONE()))
+#else
+		if (!e2_fs_stat (qed->currdir, &statbuf E2_ERR_NONE()))
+#endif
+			device = statbuf.st_dev;
+		else
+			device = -1;
+#ifdef E2_VFS
+		if (!e2_fs_stat (&ddata, &statbuf E2_ERR_NONE()))
+#else
+		if (!e2_fs_stat (qed->othrdir, &statbuf E2_ERR_NONE()))
+#endif
+			realmove = (statbuf.st_dev != device);
+		else
+			realmove = TRUE;
+#ifdef E2_VFS
+	}
+#endif
+	printd (DEBUG, (realmove) ? "physical move":"logical move");
 
 	//setup the information window
 	E2_BarWindowData windowdata;
@@ -741,47 +810,35 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 		_("Abort the moving"), _e2p_mvbar_break_cb, &breakflag);
 #endif
 
-	src_dir = F_FILENAME_FROM_LOCALE (qed->currdir);
-	dest_dir = F_FILENAME_FROM_LOCALE (qed->othrdir);
-
 	//accumulate non-recursed count and total size of src item(s)
 	E2_BarData totaldata = { 0,0 };
 	guint count;
 	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
 	for (count=0; count < names->len; count++, iterator++)
 	{
-		slocal = e2_utils_strcat (qed->currdir, (*iterator)->filename);
+		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
+#ifdef E2_VFS
+		sdata.localpath = src->str;
+#endif
 		//if (!
-		e2_fs_tw (slocal, _e2p_mvbar_twcb, &totaldata, -1,
+#ifdef E2_VFS
+		e2_fs_tw (&sdata, _e2p_mvbar_twcb, &totaldata, -1,
+#else
+		e2_fs_tw (src->str, _e2p_mvbar_twcb, &totaldata, -1,
+#endif
 			E2TW_PHYS E2_ERR_NONE());
 		//)
 		//{
 			//FIXME handle error
 		//}
-		g_free (slocal);
 	}
 	totaldata.count = names->len;	//not interested in nested count
 
-	//determine whether move will be physical or logical
-	//FIXME vfs
-	struct stat statbuf;
-	dev_t device;
-	gboolean realmove;
-	if (!e2_fs_stat (qed->currdir, &statbuf E2_ERR_NONE()))
-		device = statbuf.st_dev;
-	else
-		device = -1;
-	if (!e2_fs_stat (qed->othrdir, &statbuf E2_ERR_NONE()))
-		realmove = (statbuf.st_dev != device);
-	else
-		realmove = TRUE;
-
-	if (realmove)
-		printd (DEBUG, "physical move");
-	else
-		printd (DEBUG, "logical move");
-
-	gboolean check = e2_option_bool_get ("confirm-overwrite");
+	gboolean check = e2_option_bool_get ("confirm-overwrite")
+#ifdef E2_VFS
+		&& qed->currspace == qed->othrspace
+#endif
+	;
 	E2_BarData progressdata = { 1,0 };
 	OW_ButtonFlags extras = (totaldata.count > 1) ? BOTHALL : NONE;
 	iterator = (E2_SelectedItemInfo **) names->pdata;
@@ -797,34 +854,40 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 #endif
 			break;	//user pressed stop btn or closed info window
 		//src_dir, dest_dir have trailing "/"
-		gchar *utf = F_FILENAME_FROM_LOCALE ((*iterator)->filename);
-		src = g_strconcat (src_dir, utf, NULL);
-		dest = g_strconcat (dest_dir, utf, NULL);
-		F_FREE (utf);
+		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
+		g_string_printf (dest, "%s%s", qed->othrdir, (*iterator)->filename);
 
-		dlocal = F_FILENAME_TO_LOCALE (dest);
-		if (check && e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
+#ifdef E2_VFS
+		sdata.localpath = src->str;
+		ddata.localpath = dest->str;
+		if (check && e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
+		if (check && e2_fs_access2 (dest->str E2_ERR_NONE()) == 0)
+#endif
 		{
 			e2_filelist_enable_refresh ();  //allow updates while we wait
-			slocal = F_FILENAME_TO_LOCALE (src);
 			gdk_threads_enter ();
 			*qed->status = E2_TASK_PAUSED;
-			DialogButtons result = e2_dialog_ow_check (slocal, dlocal, extras);
+			DialogButtons result = e2_dialog_ow_check (src->str, dest->str, extras);
 			*qed->status = E2_TASK_RUNNING;
 			gdk_threads_leave ();
-			F_FREE (slocal);
 			e2_filelist_disable_refresh ();
 			switch (result)
 			{
 				case YES_TO_ALL:
 					check = FALSE;
 				case OK:
-					_e2p_mvbar_exec (src, dest, dest_dir, realmove,
-#ifndef IS_PAUSABLE
-							&breakflag,
+#ifdef E2_VFS
+					_e2p_mvbar_exec (&sdata, &ddata,
+#else
+					_e2p_mvbar_exec (src->str, dest->str,
 #endif
-
-							GPOINTER_TO_INT (qed->action->data), &progressdata, &totaldata, &windowdata);
+						realmove,
+#ifndef IS_PAUSABLE
+						&breakflag,
+#endif
+						GPOINTER_TO_INT (qed->action->data),
+						&progressdata, &totaldata, &windowdata);
 					break;
 				case CANCEL:
 					break;
@@ -834,32 +897,37 @@ _e2p_mvbarQ (E2_ActionTaskData *qed)
 			}
 			if (result == NO_TO_ALL)
 			{
-				g_free (src);
-				g_free (dest);
-				F_FREE (dlocal);	//finished with dlocal for this item
 				break;
 			}
 		}
 		else  //no overwrite, or don't care
 		{
-			_e2p_mvbar_exec (src, dest, dest_dir, realmove,
-#ifndef IS_PAUSABLE
-					&breakflag,
+#ifdef E2_VFS
+			_e2p_mvbar_exec (&sdata, &ddata,
+#else
+			_e2p_mvbar_exec (src->str, dest->str,
 #endif
-					GPOINTER_TO_INT (qed->action->data), &progressdata, &totaldata, &windowdata);
+				realmove,
+#ifndef IS_PAUSABLE
+				&breakflag,
+#endif
+				GPOINTER_TO_INT (qed->action->data),
+				&progressdata, &totaldata, &windowdata);
 		}
-		g_free (src);
-		g_free (dest);
-		F_FREE (dlocal);	//finished with dlocal for this item
 		progressdata.count++;
 	}
 	gdk_threads_enter ();
 	gtk_widget_destroy (windowdata.dialog);
 	gdk_threads_leave ();
-	F_FREE (src_dir);
-	F_FREE (dest_dir);
+
+	g_string_free (src, TRUE);
+	g_string_free (dest, TRUE);
+#ifdef E2_FAM
 	e2_filelist_request_refresh (other_view->dir, FALSE);
 	e2_filelist_request_refresh (curr_view->dir, TRUE);
+#else
+	e2_filelist_check_dirty (GINT_TO_POINTER (1));
+#endif
 	e2_filelist_enable_refresh ();
 	return TRUE;
 }
