@@ -1,4 +1,4 @@
-/* $Id: e2p_times.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2p_times.c 700 2007-11-05 08:55:39Z tpgww $
 
 Copyright (C) 2006-2007 tooar <tooar@gmx.net>
 
@@ -82,15 +82,15 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed);
 
 @return TRUE if the change was completed
 */
-static gboolean _e2pt_touch1 (gchar *path, struct stat *sb, E2_TouchData *data)
+static gboolean _e2pt_touch1 (VPATH *path, const struct stat *statptr, E2_TouchData *data)
 {
 	struct utimbuf tb;
 	struct timeval systime1, systime2, strt, nd;
 	struct timezone tz;
 	//old or new mod time
-	tb.modtime = (data->mtime == (time_t ) -1) ? sb->st_mtime : data->mtime;
+	tb.modtime = (data->mtime == (time_t ) -1) ? statptr->st_mtime : data->mtime;
 	//old or new access time
-	tb.actime = (data->atime == (time_t ) -1) ? sb->st_atime : data->atime;
+	tb.actime = (data->atime == (time_t ) -1) ? statptr->st_atime : data->atime;
 
 	gboolean retval = TRUE;
 	struct tm *tmptr;
@@ -112,7 +112,7 @@ static gboolean _e2pt_touch1 (gchar *path, struct stat *sb, E2_TouchData *data)
 
 		settimeofday (&strt, NULL);
 	}
-	if (fixc || tb.modtime != sb->st_mtime || tb.actime != sb->st_atime)
+	if (fixc || tb.modtime != statptr->st_mtime || tb.actime != statptr->st_atime)
 	{
 		E2_ERR_DECLARE
 		if (e2_fs_utime (path, &tb E2_ERR_PTR()))
@@ -170,7 +170,7 @@ Error messages expect BGL off
 
 @return E2TW_CONTINUE on success, others as appropriate
 */
-static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
+static E2_TwResult _e2_task_twcb_touch (VPATH *localpath,
 	const struct stat *statptr, E2_TwStatus status, E2_TouchData *user_data)
 {
 	E2_TwResult retval = E2TW_CONTINUE;	//default error code = none
@@ -181,22 +181,36 @@ static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
 		mode_t mode, newmode;
 		E2P_DirEnt *dirfix;
 		GList *member;
-
+#ifdef E2_VFS
+		VPATH ddata;
+#endif
 		case E2TW_DP:	//dir completion, touch, revert mode if needed, cleanup
-			for (member = g_list_last (user_data->dirdata); member != NULL; member = member->prev)
+#ifdef E2_VFS
+			ddata.spacedata = localpath->spacedata;
+#endif
+			for (member = user_data->dirdata; member != NULL; member = member->next)
 			{
 				dirfix = member->data;
 				if (dirfix != NULL)
 				{
 					if (g_str_equal (dirfix->path, localpath))
 					{
-						if (!_e2pt_touch1 (dirfix->path, (struct stat *)statptr, user_data))
+#ifdef E2_VFS
+						ddata.localpath = dirfix->path;
+						if (!_e2pt_touch1 (&ddata, statptr, user_data))
+#else
+						if (!_e2pt_touch1 (dirfix->path, statptr, user_data))
+#endif
 							retval = E2TW_FIXME;
+#ifdef E2_VFS
+						if (e2_fs_chmod (&ddata, dirfix->mode E2_ERR_PTR())
+#else
 						if (e2_fs_chmod (dirfix->path, dirfix->mode E2_ERR_PTR())
+#endif
 							&& E2_ERR_ISNOT (ENOENT))	//CHECKME ctime effect
 						{
-							e2_fs_error_local (_("Cannot change permissions of %s"),
-								(gchar *)localpath E2_ERR_MSGL());
+							e2_fs_error_local (_("Cannot change times of %s"),
+								localpath E2_ERR_MSGL());
 							retval = E2TW_FIXME;
 						}
 						g_free (dirfix->path);
@@ -216,7 +230,7 @@ static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
 			if (e2_fs_tw_adjust_dirmode (localpath, statptr, (W_OK | X_OK)) == 0)
 			{
 				//take a shot at doing the change, anyhow, probably fails
-				_e2pt_touch1 ((gchar *)localpath, (struct stat *) statptr, user_data);
+				_e2pt_touch1 (localpath, statptr, user_data);
 				//FIXME warn user about failure
 				retval |= E2TW_SKIPSUB;	//don't try to do any descendant
 			}
@@ -225,9 +239,9 @@ static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
 				//add this dir to list of items to touch afterwards
 				dirfix = ALLOCATE (E2P_DirEnt);
 				CHECKALLOCATEDWARNT (dirfix, retval=E2TW_STOP;break;)
-				dirfix->path = g_strdup (localpath);
+				dirfix->path = g_strdup (VPSTR (localpath));
 				dirfix->mode = statptr->st_mode & ALLPERMS;	//want to restore the original value
-				user_data->dirdata = g_list_append (user_data->dirdata, dirfix);
+				user_data->dirdata = g_list_prepend (user_data->dirdata, dirfix);
 			}
 			break;
 		case E2TW_DM:	//dir not opened (reported upstream)
@@ -240,13 +254,13 @@ static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
 			if (newmode == 0)
 			{
 				//take a shot at doing the change, anyhow, probably fails
-				_e2pt_touch1 ((gchar *)localpath, (struct stat *) statptr, user_data);
+				_e2pt_touch1 (localpath, statptr, user_data);
 				//FIXME warn user about failure
 				retval = E2TW_FIXME;
 			}
 			else	//dir can be processed
 			{
-				if (!_e2pt_touch1 ((gchar *)localpath, (struct stat *) statptr, user_data))
+				if (!_e2pt_touch1 (localpath, statptr, user_data))
 					retval = E2TW_FIXME;
 				if (newmode != mode)
 					e2_fs_chmod (localpath, mode E2_ERR_NONE());	//CHECKME ctime effect
@@ -255,7 +269,7 @@ static E2_TwResult _e2_task_twcb_touch (const gchar *localpath,
 		case E2TW_SL:
 		case E2TW_SLN:
 		case E2TW_F:
-			if (!_e2pt_touch1 ((gchar *)localpath, (struct stat *) statptr, user_data))
+			if (!_e2pt_touch1 (localpath, statptr, user_data))
 				retval = E2TW_FIXME;
 			break;
 //		case E2TW_NS:	//un-statable item (for which, error is reported upstream)
@@ -291,7 +305,7 @@ Error messages expect BGL open
 
 @return TRUE if operation succeeds
 */
-static gboolean _e2p_touch (gchar *localpath, E2_TouchData *data, gboolean recurse)
+static gboolean _e2p_touch (VPATH *localpath, E2_TouchData *data, gboolean recurse)
 {
 	struct stat statbuf;
 	E2_ERR_DECLARE
@@ -319,14 +333,28 @@ static gboolean _e2p_touch (gchar *localpath, E2_TouchData *data, gboolean recur
 
 			if (data->dirdata != NULL)
 			{	//touch and revert dir permissions, LIFO (=bottom-up) order
+#ifdef E2_VFS
+				VPATH ddata;
+				ddata.spacedata = localpath->spacedata;
+#endif
 				GList *member;
-				for (member = g_list_last (data->dirdata); member != NULL; member = member->prev)
+				for (member = data->dirdata; member != NULL; member = member->next)
 				{
 					E2P_DirEnt *dirfix = member->data;
+#ifdef E2_VFS
+					ddata.localpath = dirfix->path;
+					if (e2_fs_lstat (&ddata, &statbuf E2_ERR_NONE())  //no link pass-thru
+						|| !_e2pt_touch1 (&ddata, &statbuf, data))
+#else
 					if (e2_fs_lstat (dirfix->path, &statbuf E2_ERR_NONE())  //no link pass-thru
 						|| !_e2pt_touch1 (dirfix->path, &statbuf, data))
+#endif
 						retval = FALSE;
+#ifdef E2_VFS
+					if (e2_fs_chmod (&ddata, dirfix->mode E2_ERR_PTR())
+#else
 					if (e2_fs_chmod (dirfix->path, dirfix->mode E2_ERR_PTR())
+#endif
 						&& E2_ERR_ISNOT (ENOENT))
 						e2_fs_error_local (_("Cannot change permissions of %s"),
 							localpath E2_ERR_MSGL());
@@ -603,6 +631,7 @@ static GtkWidget *_e2p_times_create_combo (GList *history)
 @brief create and run time-change dialog
 
 @param local_dir path of dir containing item to process, localised string with trailer
+@param spacedata pointer to space data for active pane
 @param info pointer to selection data struct for the item being processed
 @param recurse_ret pointer to store for user's choice for a recursive change
 @param statbuf pointer to filesystem data struct for the item being processed
@@ -610,8 +639,11 @@ static GtkWidget *_e2p_times_create_combo (GList *history)
 
 @return enumerator corresponing to the clicked dialog button
 */
-static DialogButtons _e2p_times_dialog_run (gchar *local_dir, E2_SelectedItemInfo *info,
-	gboolean *recurse_ret, E2_TouchData *data, gboolean multi)
+static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
+#ifdef E2_VFS
+	PlaceInfo *spacedata,
+#endif
+	E2_SelectedItemInfo *info, gboolean *recurse_ret, E2_TouchData *data, gboolean multi)
 {
 	DialogButtons choice;
 	GtkWidget *times_dialog;
@@ -624,15 +656,30 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir, E2_SelectedItemInf
 	E2_TimesDlgRuntime rt;
 
 	gchar *local = e2_utils_strcat (local_dir, info->filename);
+#ifdef E2_VFS
+	VPATH tdata = { local, spacedata };
+	if (e2_fs_lstat (&tdata, &statbuf E2_ERR_NONE()))
+#else
 	if (e2_fs_lstat (local, &statbuf E2_ERR_NONE()))
+#endif
 	{
 		g_free (local);
 		return CANCEL;
 	}
 	E2_ERR_DECLARE
-	rt.permission = e2_fs_check_write_permission (local E2_ERR_PTR());
+	rt.permission =
+#ifdef E2_VFS
+	e2_fs_check_write_permission (&tdata E2_ERR_PTR());
+#else
+	e2_fs_check_write_permission (local E2_ERR_PTR());
+#endif
 //FIXME handle error
-	gboolean thisis_dir = e2_fs_is_dir3 (local E2_ERR_PTR());
+	gboolean thisis_dir =
+#ifdef E2_VFS
+	e2_fs_is_dir3 (&tdata E2_ERR_PTR());
+#else
+	e2_fs_is_dir3 (local E2_ERR_PTR());
+#endif
 //FIXME handle error
 	g_free (local);
 	//copy pointers
@@ -798,10 +845,6 @@ static gboolean _e2p_task_times (gpointer from, E2_ActionRuntime *art)
 static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 {
 	//printd (DEBUG, "task: times");
-#ifdef E2_VFSTMP
-	if (0)	//FIXME (<fs flag for qed->currdir> != FS_LOCAL)
-		return FALSE;
-#endif
 	GPtrArray *names = qed->names;
 	gchar *curr_local = qed->currdir;
 	GString *path = g_string_sized_new (PATH_MAX+NAME_MAX);
@@ -810,6 +853,10 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 	gboolean multisrc = names->len > 1;
 	gboolean all = FALSE;
 	E2_TouchData data;
+#ifdef E2_VFS
+	VPATH sdata;
+	sdata.spacedata = qed->currspace;
+#endif
 
 /* out-of-loop setup = FIXME
 	GtkWidget *dialog;
@@ -830,17 +877,30 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 		gboolean recurse;
 		//".." entries filtered when names compiled
 		g_string_printf (path, "%s%s", curr_local, (*iterator)->filename); //separator comes with dir
-
+#ifdef E2_VFS
+		sdata.localpath = path->str;
+#endif
 		if (all)
 		{
 			//check if we have permission to change this item
 			//NB this tests _all_ w permissions, touch not so sophisticated ...
+#ifdef E2_VFS
+			permission = e2_fs_check_write_permission (&sdata E2_ERR_NONE());
+#else
 			permission = e2_fs_check_write_permission (path->str E2_ERR_NONE());
+#endif
 			if (!permission)
 			{
+#ifdef E2_VFS
+				sdata.localpath = (*iterator)->filename;
+#endif
 				e2_fs_error_simple (
 					_("You do not have authority to change time(s) for %s"),
-					(*iterator)->filename);
+#ifdef E2_VFS
+				&sdata);
+#else
+				(*iterator)->filename);
+#endif
 			}
 			choice = (permission) ? OK : CANCEL;
 		}
@@ -858,8 +918,11 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			e2_filelist_enable_refresh ();  //allow updates while we wait
 
 			*qed->status = E2_TASK_PAUSED;
-			choice = _e2p_times_dialog_run (curr_local, *iterator, &recurse,
-				&data, multisrc);
+			choice = _e2p_times_dialog_run (curr_local,
+#ifdef E2_VFS
+			qed->currspace,
+#endif
+			*iterator, &recurse, &data, multisrc);
 			*qed->status = E2_TASK_RUNNING;
 
 #ifdef E2_REFRESH_DEBUG
@@ -875,21 +938,34 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			choice = OK;
 		  case OK:
 #ifdef E2_INCLIST
+# ifdef E2_VFS
+			if (_e2p_touch (&sdata, &data, recurse))
+# else
 			if (_e2p_touch (path->str, &data, recurse))
+# endif
 			{
 				//FIXME update line in treeview
 			}
 #else
 # ifdef E2_FAM
+#  ifdef E2_VFS
+			_e2p_touch (&sdata, &data, recurse);
+#  else
 			_e2p_touch (path->str, &data, recurse);
+#  endif
 # else
+#  ifdef E2_VFS
+			if (_e2p_touch (&sdata, &data, recurse))
+			{
+				sdata.localpath = curr_local;
+				//make the file-list refresher notice successful change
+				e2_fs_utime (&sdata, NULL E2_ERR_NONE());
+			}
+#  else
 			if (_e2p_touch (path->str, &data, recurse))
 			//make the file-list refresher notice successful change
-//#  ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-//#  else
 				e2_fs_utime (curr_local, NULL E2_ERR_NONE());
-//#  endif
+#  endif
 # endif
 #endif
 		  case CANCEL:
@@ -924,7 +1000,7 @@ static gchar *aname;
 */
 gboolean init_plugin (Plugin *p)
 {
-#define ANAME "timeset"
+#define ANAME "times"
 	aname = _("timeset");
 
 	p->signature = ANAME VERSION;
