@@ -1,6 +1,6 @@
-/* $Id: e2p_upgrade.c 475 2007-07-09 11:42:44Z tpgww $
+/* $Id: e2p_upgrade.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2005-2007 tooar <tooar@gmx.net>
+Copyright (C) 2005-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -78,11 +78,21 @@ static void _e2p_upgrade_backup (gchar *file)
 	gchar *config_file = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s",
 		e2_cl_options.config_dir, file);
 	gchar *local = F_FILENAME_TO_LOCALE (config_file);
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };
+	if (e2_fs_access (&ddata, F_OK E2_ERR_NONE()) == 0)	//traverse link, if any
+#else
 	if (e2_fs_access (local, F_OK E2_ERR_NONE()) == 0)	//traverse link, if any
+#endif
 	{
 		gchar *saved_file = g_strconcat (local, ".save", NULL);
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifdef E2_VFS
+		VPATH sdata = { saved_file, NULL };
+		e2_task_backend_rename (&ddata, &sdata);
+#else
 		e2_task_backend_rename (local, saved_file);
+#endif
 		gdk_threads_enter ();
 		g_free (saved_file);
 	}
@@ -195,10 +205,10 @@ static void _e2p_upgrade_0_1 (void)
 		gchar *newstr1 = g_strconcat (_A(5),".",_A(20),NULL);
 		//keybinding to add
 		gchar *oldstr2 = g_strconcat ("\t\t|",_("Delete"),"|false|",_A(5),".",_A(38),"|",NULL); //"delete"
-		gchar *newstr2 = g_strconcat ("\t\t|<Shift>",_("Delete"),"|false|",_A(1),".",_A(89),"|",NULL); //"trashempty"
+		gchar *newstr2 = g_strconcat ("\t\t|<Shift>",_("Delete"),"|false|",_A(1),".",_A(98),"|",NULL); //"trashempty"
 		//filetype string to revise
-		gchar *oldstr3 = g_strconcat ("<", _A(13), ".", _A(81),";<", _A(13), ".", _A(71), ";>bzip2 -d -c %F [\\]| tar xf -;<", _A(13), ".", _A(70), ";<", _A(13), ".", _A(81),NULL);
-		gchar *newstr3 = g_strconcat (_A(13), ".", _A(71), ";>bzip2 -d -c %f ","\\\\","| tar -C %D -xf -;<", _A(13), ".", _A(70), NULL);
+		gchar *oldstr3 = g_strconcat ("<", _A(13), ".", _A(90),";<", _A(13), ".", _A(72), ";>bzip2 -d -c %F [\\]| tar xf -;<", _A(13), ".", _A(71), ";<", _A(13), ".", _A(90),NULL);
+		gchar *newstr3 = g_strconcat (_A(13), ".", _A(72), ";>bzip2 -d -c %f ","\\\\","| tar -C %D -xf -;<", _A(13), ".", _A(71), NULL);
 
 			gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 				default_config_file, NULL);
@@ -242,7 +252,7 @@ static void _e2p_upgrade_0_1_1 (void)
 
 	//keybindings to add
 			//general
-		gchar *oldstr2 = g_strconcat("\t|<Ctrl>r","|false|",_A(13),".",_A(69),"|",NULL); //"refresh"
+		gchar *oldstr2 = g_strconcat("\t|<Ctrl>r","|false|",_A(13),".",_A(70),"|",NULL); //"refresh"
 		gchar *newstr2 = g_strconcat
 			("\t|<Control>1","|false|",_A(4),".",_A(43),"|1\\\n""\t|<CControl>2","|false|",_A(4),".",_A(43),"|2",NULL);
 
@@ -261,7 +271,7 @@ static void _e2p_upgrade_0_1_1 (void)
 		gchar *oldstr6 = g_strconcat
 			(_("su.."),"|.*.png|",_("Run something as root"),"|",_A(17),"|xterm -e 'su -c \"%{.*read'",NULL);
 		gchar *newstr6 = g_strconcat
-			(_("mts.."),"|mounts_"E2IP".png|",_("Mount or unmount a device"),"|",_A(56),".",_A(24),"|",_C(4), NULL);
+			(_("mts.."),"|mounts_"E2IP".png|",_("Mount or unmount a device"),"|",_A(57),".",_A(24),"|",_C(4), NULL);
 
 			gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 				default_config_file, NULL);
@@ -309,8 +319,8 @@ static void _e2p_upgrade_0_1_2 (void)
 	if (sed != NULL)
 	{	//get rid of icon file paths
 		//keybinding to add
-		gchar *oldstr2 = g_strconcat ("\t\t|<Ctrl>a","|false|",_A(10),".",_A(86),"|",NULL);
-		gchar *newstr2 = g_strconcat("\t\t|<CControl>i","||",_A(10),".",_A(53),"|",NULL);
+		gchar *oldstr2 = g_strconcat ("\t\t|<Ctrl>a","|false|",_A(10),".",_A(95),"|",NULL);
+		gchar *newstr2 = g_strconcat("\t\t|<CControl>i","||",_A(10),".",_A(54),"|",NULL);
 		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 			default_config_file, NULL);
 		gchar *command = g_strconcat ("cp -f ", cfg_file, " ", cfg_file, ".save;",
@@ -346,22 +356,22 @@ static void _e2p_upgrade_0_1_3 (void)
 		{
 			//keybinding to add
 			gchar *oldstr1 = g_strconcat ("\t",_C(32),"||false|",_A(17),"|",NULL);
-			gchar *newstr1 = g_strconcat ("\t\t|<Shift><Control>space"),"|false|",_A(10),".",_A(87),"|",NULL);
+			gchar *newstr1 = g_strconcat ("\t\t|<Shift><Control>space"),"|false|",_A(10),".",_A(96),"|",NULL);
 			//actions to rename
-			gchar *oldstr2 = g_strconcat (_A(13),".",_A(44), NULL);
-			gchar *newstr2 = g_strconcat (_A(10),".",_A(44), NULL);
-			gchar *oldstr3 = g_strconcat (_A(13),".",_A(45),NULL);
-			gchar *newstr3 = g_strconcat (_A(10),".",_A(45),NULL);
-			gchar *oldstr4 = g_strconcat (_A(13),".",_A(53),NULL);
-			gchar *newstr4 = g_strconcat (_A(10),".",_A(53),NULL);
-			gchar *oldstr5 = g_strconcat (_A(13),".",_A(80),NULL);
-			gchar *newstr5 = g_strconcat (_A(10),".",_A(80),NULL);
-			gchar *oldstr6 = g_strconcat (_A(13),".",_A(81),NULL);
-			gchar *newstr6 = g_strconcat (_A(10),".",_A(81),NULL);
-			gchar *oldstr7 = g_strconcat (_A(13),".",_A(79),NULL);
-			gchar *newstr7 = g_strconcat (_A(10),".",_A(79),NULL);
-			gchar *oldstr8 = g_strconcat (_A(13),".",_A(86),NULL);
-			gchar *newstr8 = g_strconcat (_A(10),".",_A(86),NULL);
+			gchar *oldstr2 = g_strconcat (_A(13),".",_A(45), NULL);
+			gchar *newstr2 = g_strconcat (_A(10),".",_A(45), NULL);
+			gchar *oldstr3 = g_strconcat (_A(13),".",_A(46),NULL);
+			gchar *newstr3 = g_strconcat (_A(10),".",_A(46),NULL);
+			gchar *oldstr4 = g_strconcat (_A(13),".",_A(54),NULL);
+			gchar *newstr4 = g_strconcat (_A(10),".",_A(54),NULL);
+			gchar *oldstr5 = g_strconcat (_A(13),".",_A(81),NULL);
+			gchar *newstr5 = g_strconcat (_A(10),".",_A(81),NULL);
+			gchar *oldstr6 = g_strconcat (_A(13),".",_A(90),NULL);
+			gchar *newstr6 = g_strconcat (_A(10),".",_A(90),NULL);
+			gchar *oldstr7 = g_strconcat (_A(13),".",_A(80),NULL);
+			gchar *newstr7 = g_strconcat (_A(10),".",_A(80),NULL);
+			gchar *oldstr8 = g_strconcat (_A(13),".",_A(95),NULL);
+			gchar *newstr8 = g_strconcat (_A(10),".",_A(95),NULL);
 
 			gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 				default_config_file, NULL);
@@ -429,21 +439,21 @@ static void _e2p_upgrade_0_1_6 (void)
 		if (sed != NULL)
 		{
 			//keybindings to add
-			gchar *oldstr1 = g_strconcat("\t|F3|false|",_A(5),".",_A(92),"|",NULL);
-			gchar *newstr1 = g_strconcat("\t|<Shift>F3||",_A(5),".",_A(93),"|",NULL);
+			gchar *oldstr1 = g_strconcat("\t|F3|false|",_A(5),".",_A(101),"|",NULL);
+			gchar *newstr1 = g_strconcat("\t|<Shift>F3||",_A(5),".",_A(102),"|",NULL);
 			gchar *oldstr2 = g_strconcat("\t|F4|false|",_A(5),".",_A(39),"|",NULL);
 			gchar *newstr2 = g_strconcat("\t|<Shift>F4||",_A(5),".",_A(40),"|",NULL);
 			gchar *oldstr3 = g_strconcat("\t|F5|false|",_A(5),".",_A(33),"|",NULL);
 			gchar *newstr3 = g_strconcat("\t|<Shift>F5||",_A(5),".",_A(34),"|",NULL);
-			gchar *oldstr4 = g_strconcat("\t|F6|false|",_A(5),".",_A(57),"|",NULL);
-			gchar *newstr4 = g_strconcat("\t|<Shift>F6||",_A(5),".",_A(58),"|",NULL);
+			gchar *oldstr4 = g_strconcat("\t|F6|false|",_A(5),".",_A(58),"|",NULL);
+			gchar *newstr4 = g_strconcat("\t|<Shift>F6||",_A(5),".",_A(59),"|",NULL);
 			//keybindings to change
 			gchar *oldstr5 = g_strconcat("\t\t|<Shft>Up|false|",_A(17),NULL); //FIXME //_A(17)
 			gchar *newstr5 = g_strconcat("\t\t|<Alt>Up||",_A(17),NULL);
-			gchar *oldstr6 = g_strconcat("\t|<Ctrl>space||",_A(9),".",_A(67),NULL);
-			gchar *newstr6 = g_strconcat("\t|<CControl>g","||",_A(9),".",_A(67),NULL);
-			gchar *oldstr7 = g_strconcat("\t\t|<Shft><Ctrl>space|false|",_A(10),".",_A(87),NULL);
-			gchar *newstr7 = g_strconcat("\t\t|space||",_A(10),".",_A(87),NULL);
+			gchar *oldstr6 = g_strconcat("\t|<Ctrl>space||",_A(9),".",_A(68),NULL);
+			gchar *newstr6 = g_strconcat("\t|<CControl>g","||",_A(9),".",_A(68),NULL);
+			gchar *oldstr7 = g_strconcat("\t\t|<Shft><Ctrl>space|false|",_A(10),".",_A(96),NULL);
+			gchar *newstr7 = g_strconcat("\t\t|space||",_A(10),".",_A(96),NULL);
 			//action parameter to kill
 			gchar *oldstr8 = _("expand");
 
@@ -495,10 +505,10 @@ static void _e2p_upgrade_0_1_6_2 (void)
 		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 				default_config_file, NULL);
 		//keybindings to add
-		gchar *oldstr1 = g_strconcat("\t\t|<Ctrl>h","|false|",_A(10),".",_A(79),"|",NULL);
+		gchar *oldstr1 = g_strconcat("\t\t|<Ctrl>h","|false|",_A(10),".",_A(80),"|",NULL);
 		gchar *newstr1 = g_strconcat("\t\t|<Ctrl>m","|false|",_A(10),".",_A(21),"|",NULL);
 		gchar *oldstr2 = g_strconcat("\t",_C(5),"||false||",NULL);
-		gchar *newstr2 = g_strconcat("\t\t|<Shft>Return||",_A(1),".",_A(76),"|",NULL);
+		gchar *newstr2 = g_strconcat("\t\t|<Shft>Return||",_A(1),".",_A(77),"|",NULL);
 
 		gchar *command = g_strconcat ("cp -f ", cfg_file, " ", cfg_file, ".save;",
 			sed,
@@ -541,20 +551,20 @@ static void _e2p_upgrade_0_1_6_3 (void)
 		gchar *newstr10 = g_strconcat("\t||false|true|",_A(18),"|",NULL);
 		gchar *newstr11 = g_strconcat("\t",_("_Edit user commands.."),"|gtk-preferences|false|true|",_A(2),".",_A(32),"|",_C(8),NULL);
 		//toggle-action reformatting
-		gchar *oldstr2 = g_strconcat("\\([^|]*",_A(99),"\\)|true",NULL);
-		gchar *newstr2 = g_strconcat(_A(15),".",_A(102),"|\\1",NULL);
-		gchar *oldstr3 = g_strconcat("\\([^|]*",_A(79),"\\)|true",NULL);
-		gchar *newstr3 = g_strconcat(_A(15),".",_A(102),"|\\1",NULL);
+		gchar *oldstr2 = g_strconcat("\\([^|]*",_A(108),"\\)|true",NULL);
+		gchar *newstr2 = g_strconcat(_A(15),".",_A(111),"|\\1",NULL);
+		gchar *oldstr3 = g_strconcat("\\([^|]*",_A(80),"\\)|true",NULL);
+		gchar *newstr3 = g_strconcat(_A(15),".",_A(111),"|\\1",NULL);
 		gchar *oldstr4 = g_strconcat("\\([^|]*",_A(22),"\\)|true",NULL);
-		gchar *newstr4 = g_strconcat(_A(15),".",_A(102),"|\\1",NULL);
-		gchar *oldstr5 = g_strconcat("\\([^|]*",_A(99),"\\)|false",NULL);
-		gchar *newstr5 = g_strconcat(_A(15),".",_A(101),"|\\1",NULL);
-		gchar *oldstr6 = g_strconcat("\\([^|]*",_A(79),"\\)|false",NULL);
-		gchar *newstr6 = g_strconcat(_A(15),".",_A(101),"|\\1",NULL);
+		gchar *newstr4 = g_strconcat(_A(15),".",_A(111),"|\\1",NULL);
+		gchar *oldstr5 = g_strconcat("\\([^|]*",_A(108),"\\)|false",NULL);
+		gchar *newstr5 = g_strconcat(_A(15),".",_A(110),"|\\1",NULL);
+		gchar *oldstr6 = g_strconcat("\\([^|]*",_A(80),"\\)|false",NULL);
+		gchar *newstr6 = g_strconcat(_A(15),".",_A(110),"|\\1",NULL);
 		gchar *oldstr7 = g_strconcat("\\([^|]*",_A(22),"\\)|false",NULL);
-		gchar *newstr7 = g_strconcat(_A(15),".",_A(101),"|\\1",NULL);
-		gchar *oldstr8 = g_strconcat (_A(9),".",_A(79),NULL);
-		gchar *newstr8 = g_strconcat (_A(9),".",_A(78),NULL);
+		gchar *newstr7 = g_strconcat(_A(15),".",_A(110),"|\\1",NULL);
+		gchar *oldstr8 = g_strconcat (_A(9),".",_A(80),NULL);
+		gchar *newstr8 = g_strconcat (_A(9),".",_A(79),NULL);
 		gchar *command = g_strconcat (
 				"cp -f ", local, " ", local, ".save;",
 				sed,
@@ -567,8 +577,8 @@ static void _e2p_upgrade_0_1_6_3 (void)
  				 " -e '/",oldstr10,".*$/a\\\n",newstr10,"\\\n",newstr11,"'",
 				//renamed actions FIXME later - remove specific _() so is not translated
 				 " -e 's/",_("<filter>"),"/",_A(22),"/'",
-				 " -e 's/",_("toggle_full"),"/",_A(99),"/'",
-				 " -e 's/",_("toggle_hidden"),"/",_A(79),"/'",
+				 " -e 's/",_("toggle_full"),"/",_A(108),"/'",
+				 " -e 's/",_("toggle_hidden"),"/",_A(80),"/'",
 				 " -e 's/",oldstr2,"/",newstr2,"/'",
 				 " -e 's/",oldstr3,"/",newstr3,"/'",
 				 " -e 's/",oldstr4,"/",newstr4,"/'",
@@ -618,18 +628,18 @@ static void _e2p_upgrade_0_1_7_1 (void)
 	if (sed != NULL)
 	{
 		//actions to rename
-		gchar *oldstr1 = g_strconcat (_A(3),"\\.",_A(55), NULL);
-		gchar *newstr1 = g_strconcat (_A(1),".",_A(55), NULL);
+		gchar *oldstr1 = g_strconcat (_A(3),"\\.",_A(56), NULL);
+		gchar *newstr1 = g_strconcat (_A(1),".",_A(56), NULL);
 		//commandbar item re-assigned
 		gchar *oldstr2 = g_strconcat(_("ps"),"|ps_"E2IP".png|",_("List child processes"),"|",_A(1),".",_("list_children"),"|",NULL);
 		gchar *newstr2 = g_strconcat(_("ps"),"|ps_"E2IP".png|",_("Child processes"),"|",_A(28),".",_A(24),"|",NULL);
 		gchar *oldstr7 = g_strconcat(_A(1),"\\.",_("list_children"),NULL);
 		gchar *newstr7 = g_strconcat(_A(7),".",_A(28),NULL);
 		//keybindings to add
-		gchar *oldstr3 = g_strconcat("\t|F1|false|",_A(13),".",_A(69),"|",NULL);
+		gchar *oldstr3 = g_strconcat("\t|F1|false|",_A(13),".",_A(70),"|",NULL);
 		gchar *newstr3 = g_strconcat("\t|<Shift>F1||",_A(7),".",_A(28),"|\\n"
-									"\t|<Control>F1||",_A(7),".",_A(65),"|\\n"
-									"\t|<Alt>F1||",_A(7),".",_A(50),"|",NULL);
+									"\t|<Control>F1||",_A(7),".",_A(66),"|\\n"
+									"\t|<Alt>F1||",_A(7),".",_A(51),"|",NULL);
 		gchar *oldstr4 = g_strconcat("\t\t|Tab|false|",_A(1),".",_A(31),"|",NULL);
 		gchar *newstr4 = g_strconcat("\t\t|<Alt>Insert||",_A(7),".",_A(28),"|",NULL);
 		gchar *oldstr8 = g_strconcat("\t|<Control>F5|false|",_A(5),".",_A(36),"|",NULL);
@@ -794,18 +804,18 @@ static void _e2p_upgrade_0_3_1 (void)
 	{
 		//renamed actions
 		gchar *oldlist = _("<list>");
-		gchar *oldstr1 = g_strconcat (_A(65),".",_A(38),NULL);
-		gchar *newstr1 = g_strconcat (_A(29),".",_A(65),NULL);
+		gchar *oldstr1 = g_strconcat (_A(66),".",_A(38),NULL);
+		gchar *newstr1 = g_strconcat (_A(29),".",_A(66),NULL);
 		gchar *oldstr2 = g_strconcat (_A(0),".",oldlist,NULL);
 		gchar *newstr2 = g_strconcat (_A(0),".",_A(24),NULL);
 		gchar *oldstr3 = g_strconcat (_A(28),".",oldlist,NULL);
 		gchar *newstr3 = g_strconcat (_A(7),".",_A(28),NULL);
-		gchar *oldstr4 = g_strconcat (_A(50),".",oldlist,NULL);
-		gchar *newstr4 = g_strconcat (_A(7),".",_A(50),NULL);
-		gchar *oldstr5 = g_strconcat (_A(56),".",oldlist,NULL);
-		gchar *newstr5 = g_strconcat (_A(56),".",_A(24),NULL);
-		gchar *oldstr6 = g_strconcat (_A(65),".",oldlist,NULL);
-		gchar *newstr6 = g_strconcat (_A(7),".",_A(65),NULL);
+		gchar *oldstr4 = g_strconcat (_A(51),".",oldlist,NULL);
+		gchar *newstr4 = g_strconcat (_A(7),".",_A(51),NULL);
+		gchar *oldstr5 = g_strconcat (_A(57),".",oldlist,NULL);
+		gchar *newstr5 = g_strconcat (_A(57),".",_A(24),NULL);
+		gchar *oldstr6 = g_strconcat (_A(66),".",oldlist,NULL);
+		gchar *newstr6 = g_strconcat (_A(7),".",_A(66),NULL);
 		gchar *oldstr7 = g_strconcat (_A(14),".",oldlist,NULL);
 		gchar *newstr7 = g_strconcat (_A(14),".",_A(24),NULL);
 
@@ -857,8 +867,8 @@ static void _e2p_upgrade_0_3_3 (void)
 	if (sed != NULL)
 	{
 		//keybinding to add
-		gchar *oldstr1 = g_strconcat("\t|F9|false|",_A(5),".",_A(51),"|",NULL);
-		gchar *newstr1 = g_strconcat("\t|<Shift>F9||",_A(10),".",_A(90),"|",NULL);
+		gchar *oldstr1 = g_strconcat("\t|F9|false|",_A(5),".",_A(52),"|",NULL);
+		gchar *newstr1 = g_strconcat("\t|<Shift>F9||",_A(10),".",_A(99),"|",NULL);
 
 		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
 			default_config_file, NULL);
@@ -884,8 +894,17 @@ static void _e2p_upgrade_0_3_3 (void)
 }
 #endif
 
+static void _e2p_upgrade_0_3_6 (void)
+{
+	//get rid of outdated cache flags for find plugin, which hasn't been loaded yet
+	e2_cache_clean1 ("find-plugin-flags");
+}
+
 gboolean init_plugin (Plugin *p)
 {
+#define ANAME "uprade"
+	p->signature = ANAME VERSION;
+
 	if (strcmp (app.cfgfile_version,"0.1") < 0)
 		//this one's major - dump the lot and start afresh
 		_e2p_upgrade_pre_0_1();
@@ -936,6 +955,8 @@ gboolean init_plugin (Plugin *p)
 				if (strcmp (app.cfgfile_version,"0.3.3") < 0)
 					_e2p_upgrade_0_3_3 ();
 #endif
+				if (strcmp (app.cfgfile_version,"0.3.5.1") < 0)
+					_e2p_upgrade_0_3_6 ();
 				else
 					cancelled = TRUE;
 			}
