@@ -1,6 +1,6 @@
-/* $Id: e2p_thumbs.c 544 2007-07-20 12:48:20Z tpgww $
+/* $Id: e2p_thumbs.c 854 2008-04-19 11:45:33Z tpgww $
 
-Copyright (C) 2007 tooar <tooar@gmx.net>
+Copyright (C) 2007-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -67,6 +67,7 @@ typedef struct _E2_ThumbDialogRuntime
 //	GtkWidget *hiddenbtn;	//action-area hide-button
 //	gboolean show_hidden;	//TRUE to show hidden items in the iconview
 	ViewInfo *view;			//data for displayed file pane
+	gboolean clamp;			//TRUE to clamp image display into range 32..128 px
 	gboolean replicate;		//TRUE to cause iconview selections to be replicated in related treeview
 	gint sort_type;			//to determine the correct sort func, corresponds to column in original store
 	GtkSortType sort_order;
@@ -207,7 +208,8 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 
 	//transfer data from filelist store to iconview store
 	GtkTreeIter itert;
-	GtkTreeModel *mdlt = GTK_TREE_MODEL (rt->view->store);	//the child model
+//	GtkTreeModel *mdlt = GTK_TREE_MODEL (rt->view->store);	//the child model (everything)
+	GtkTreeModel *mdlt = rt->view->model;	//the filter-model
 
 	if (gtk_tree_model_get_iter_first (mdlt, &itert))
 	{
@@ -216,10 +218,7 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 		GimpThumbnail* thumbnail;
 		GError *error;
 		gboolean makepxb;
-//		gint len;
-//		gchar target[PATH_MAX+NAME_MAX];
 		gchar *filename, *namekey, *dlocal, *localpath;
-//		struct stat statbuf2;
 		GtkTreeIter iteri;
 
 		gdk_threads_enter ();
@@ -238,8 +237,7 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 				NAMEKEY, &namekey,
 				FINFO, &info,	//FIXME handle refreshing which changes the address of info's
 				-1);
-			makepxb = FALSE;
-			//we do a bit of filtering to reduce irrelevant pixbuf attempts ...
+			//do a bit of filtering to reduce irrelevant pixbuf attempts ...
 			switch (info->statbuf.st_mode & S_IFMT)
 			{
 				case S_IFLNK:
@@ -283,6 +281,7 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 					makepxb = TRUE;
 					break;
 				default:
+					makepxb = FALSE;
 					break;
 			}
 
@@ -292,9 +291,11 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 				error = NULL;
 				if (!gimp_thumbnail_set_filename (thumbnail, localpath, &error))
 				{
-					//warn the user
 					if (error != NULL)
+					{
+						//FIXME warn the user
 						g_error_free (error);
+					}
 					makepxb = FALSE;
 				}
 				if (makepxb)
@@ -305,24 +306,33 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 						gboolean scaled;
 						gint scaleto;
 recreate:
-						if (width < E2_SCALEUP_SIZE && height < E2_SCALEUP_SIZE)
+						if (rt->clamp)
 						{
-							scaled = TRUE;
-							scaleto = E2_SCALEUP_SIZE;
-						}
-						else if (width > GIMP_THUMB_SIZE_NORMAL || height > GIMP_THUMB_SIZE_NORMAL)
-						{
-							scaled = TRUE;
-							scaleto = GIMP_THUMB_SIZE_NORMAL;
+							if (width < E2_SCALEUP_SIZE && height < E2_SCALEUP_SIZE)
+							{
+								scaled = TRUE;
+								scaleto = E2_SCALEUP_SIZE;
+							}
+							else if (width > GIMP_THUMB_SIZE_NORMAL || height > GIMP_THUMB_SIZE_NORMAL)
+							{
+								scaled = TRUE;
+								scaleto = GIMP_THUMB_SIZE_NORMAL;
+							}
+							else
+							{
+								scaled = FALSE;
+								scaleto = MAX (width, height);
+							}
 						}
 						else
 						{
 							scaled = FALSE;
 							scaleto = MAX (width, height);
 						}
-						if (gimp_thumbnail_peek_thumb (thumbnail, scaleto) != GIMP_THUMB_STATE_EXISTS)
-//						if (gimp_thumbnail_check_thumb (thumbnail, scaleto) != GIMP_THUMB_STATE_OK)
+//						if (gimp_thumbnail_peek_thumb (thumbnail, scaleto) != GIMP_THUMB_STATE_EXISTS)
+						if (gimp_thumbnail_check_thumb (thumbnail, scaleto) != GIMP_THUMB_STATE_OK)
 						{
+rescale:
 							if (scaled)
 								pxb = gdk_pixbuf_new_from_file_at_scale (localpath,
 									scaleto, scaleto, TRUE, &error);
@@ -358,10 +368,16 @@ recreate:
 							if (gimp_thumbnail_peek_image (thumbnail) == GIMP_THUMB_STATE_EXISTS)
 							{
 								width = MAX (thumbnail->image_width, thumbnail->image_height);
-								if (width != 0	//0 is reprorted for non-standard-size ?
-									&& (width < E2_SCALEUP_SIZE
-									||  width > GIMP_THUMB_SIZE_NORMAL
-									||  width != scaleto))
+								if (width == 0)	//0 is reprorted for non-standard-size ?
+								{
+									height = width = scaleto;
+									goto rescale;
+								}
+								//if (width != 0	//0 is reprorted for non-standard-size ?
+								//	&&
+								else if (width < E2_SCALEUP_SIZE
+									  || width > GIMP_THUMB_SIZE_NORMAL
+									  || width != scaleto)
 								{
 									height = width;
 									goto recreate;
@@ -379,6 +395,7 @@ recreate:
 					}
 					else //no file info
 					{
+						//FIXME warn the user
 						makepxb = FALSE;
 					}
 				}
@@ -437,9 +454,11 @@ recreate:
 # ifdef E2_VFSTMP
 	//FIXME
 # else
+#  ifdef E2_FAM_KERNEL
 	//this is a bad hack to prevent extra refresh
 	if (!ref)
 		e2_fs_FAM_clean_reports (rt->view->dir);
+#  endif
 # endif
 #endif */
 	e2_filelist_enable_one_refresh (pane);
@@ -526,13 +545,21 @@ static gboolean _e2p_thumbs_check_dirty (E2_ThumbDialogRuntime *rt)
 	printd (DEBUG, "_e2p_thumbs_check_dirty");
 	if (!GTK_IS_WIDGET (rt->dialog))
 		return FALSE;
+	static gboolean busy = FALSE;
+	if (!busy)
+	{
 #ifdef E2_VFSTMP
 	//FIXME when dir is not mounted local
 #endif
-	if (rt->dir_mtime < rt->view->dir_mtime
-		&& !rt->blocked
-		&& g_str_equal (rt->path, rt->view->dir))
-		_e2p_thumbs_refresh_store (rt);
+		if (rt->dir_mtime < rt->view->dir_mtime
+			&& !rt->blocked
+			&& g_str_equal (rt->path, rt->view->dir))
+		{
+			busy = TRUE;
+			_e2p_thumbs_refresh_store (rt);
+			busy = FALSE;
+		}
+	}
 	return TRUE;
 }
 /**
@@ -547,7 +574,7 @@ static gboolean _e2p_thumbs_check_dirty (E2_ThumbDialogRuntime *rt)
 	return FALSE;
 } */
 /**
-@brief timer callback to do a store replacemnent after refreshing is unblocked
+@brief timer callback to do a store replacement after refreshing is unblocked
 @param rt pointer to dialog's data struct
 @return TRUE if the associated-filelist refresh is still in progress
 */
@@ -555,21 +582,37 @@ static gboolean _e2p_thumbs_wait_to_replace (E2_ThumbDialogRuntime *rt)
 {
 	if (GTK_IS_WIDGET (rt->dialog) && !rt->blocked)
 	{
-		if (rt->view->blocked)
+		LISTS_LOCK
+		gboolean busy = rt->view->listcontrols.refresh_working
+					 || rt->view->listcontrols.cd_working;
+		LISTS_UNLOCK
+		if (busy)
 			return TRUE;
+		static gboolean working = FALSE;	//simple blocker instead of killing timer
 		//to prevent automatic refreshing of 2 views from feeding off each other,
 		//suspend for a while the automatic refreshing here
 		//(too bad if we miss something in the meantime)
-		printd (DEBUG, "deferred replication of CHANGE-DIR for icons liststore");
+//		printd (DEBUG, "doing deferred replication of CHANGE-DIR for icons liststore");
 //		e2_hook_unregister (&rt->view->hook_refresh, _e2p_thumbs_refresh_hook, rt, TRUE);
-		_e2p_thumbs_replace_store (rt);
-//		g_timeout_add (10000, (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+		if (!working)
+		{	//previous timer callback hasn't yet finished
+			working = TRUE;
+			_e2p_thumbs_replace_store (rt);
+			working = FALSE;
+		}
+/*#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (10,
+#else
+		g_timeout_add (10000,
+#endif
+		 (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+*/
 	}
 	return FALSE;
 }
 /**
 @brief hook function for app.paneX.hook_change_dir
-This is initiated with BGL on/closed
+This is initiated with BGL off/open
 @param newpath UNUSED path of opened directory, utf-8 string
 @param rt pointer to dialog's data struct
 @return TRUE always
@@ -579,16 +622,24 @@ static gboolean _e2p_thumbs_change_dir_hook (gchar *newpath, E2_ThumbDialogRunti
 	printd (DEBUG, "replicating CHANGE-DIR for icons liststore");
 	g_free (rt->path);
 	rt->path = g_strdup (newpath);
-	if (!rt->view->blocked)
+	LISTS_LOCK
+	gboolean busy = rt->view->listcontrols.refresh_working
+				 || rt->view->listcontrols.cd_working;
+	LISTS_UNLOCK
+	if (!busy)
 	{
 		//to prevent automatic refreshing of 2 views from feeding off each other,
 		//suspend for a while the automatic refreshing here
 		//(too bad if we miss something in the meantime)
 //		e2_hook_unregister (&rt->view->hook_refresh, _e2p_thumbs_refresh_hook, rt, TRUE);
-		gdk_threads_leave ();	//open BGL before update
 		_e2p_thumbs_replace_store (rt);
-		gdk_threads_enter ();
-//		g_timeout_add (10000, (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+/*#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (10,
+#else
+		g_timeout_add (10000,
+#endif
+		 (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+*/
 	}
 	else
 	{
@@ -607,15 +658,31 @@ static gboolean _e2p_thumbs_wait_to_refresh (E2_ThumbDialogRuntime *rt)
 {
 	if (GTK_IS_WIDGET (rt->dialog) && !rt->blocked)
 	{
-		if (rt->view->blocked)
+		LISTS_LOCK
+		gboolean disabled = rt->view->listcontrols.norefresh;	//CHECKME ->listing ?
+		LISTS_UNLOCK
+		if (disabled)
 			return TRUE;
 		printd (DEBUG, "deferred replication of REFRESH of icons liststore");
 		//to prevent automatic refreshing of 2 views from feeding off each other,
 		//suspend for a while the automatic refreshing here
 		//(too bad if we miss something in the meantime)
 //		e2_hook_unregister (&rt->view->hook_refresh, _e2p_thumbs_refresh_hook, rt, TRUE);
-		_e2p_thumbs_refresh_store (rt);
-//		g_timeout_add (10000, (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+
+		static gboolean working = FALSE;	//simple blocker instead of killing timer
+		if (!working)
+		{	//previous timer callback hasn't yet finished
+			working = TRUE;
+			_e2p_thumbs_refresh_store (rt);
+			working = FALSE;
+		}
+/*#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (10,
+#else
+		g_timeout_add (10000,
+#endif
+		 (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+*/
 	}
 	return FALSE;
 }
@@ -629,14 +696,19 @@ This is initiated from an idle-callback, so BGL is off
 /*static gboolean _e2p_thumbs_refresh_hook (ViewInfo *view, E2_ThumbDialogRuntime *rt)
 {
 	printd (DEBUG, "replicating REFRESH of icons liststore");
-	if (!rt->view->blocked)
+	if (!rt->view->norefresh)	//CHECKME ->listing ?
 	{
 		//to prevent automatic refreshing of 2 views from feeding off each other,
 		//suspend for a while the automatic refreshing here
 		//(too bad if we miss something in the meantime)
 		e2_hook_unregister (&rt->view->hook_refresh, _e2p_thumbs_refresh_hook, rt, TRUE);
 		_e2p_thumbs_refresh_store (rt);
-		g_timeout_add (10000, (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
+#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (10,
+#else
+		g_timeout_add (10000,
+#endif
+		 (GSourceFunc)_e2p_thumbs_enable_refresh, rt);
 	}
 	else
 	{
@@ -792,7 +864,53 @@ static void _e2p_thumbs_set_menu_position (GtkMenu *menu,
 	GtkAllocation alloc = rt->iconview->allocation;
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
-	*push_in = TRUE;
+	*push_in = FALSE;
+}
+/**
+@brief execute action corresponding to item selected from filetype tasks menu
+This is the callback for handling a selection of a filetype action from
+the context menu
+@param widget the selected menu item widget
+@param rt data struct for the dialog where the menu key was pressed
+
+@return
+*/
+static void _e2p_thumbs_menu_choose_filetype_action_cb (GtkWidget *widget,
+	E2_ThumbDialogRuntime *rt)
+{
+	printd (DEBUG, "context menu choose filetype action cb");
+	GList *sel = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (rt->iconview));
+	if (sel != NULL && g_list_length (sel) > 0)
+	{
+		GString *command;
+		gchar *name;
+		GList *member;
+		GtkTreePath *tp;
+		GtkTreeModel *mdl = GTK_TREE_MODEL (rt->store);
+		GtkTreeIter iter;
+
+		command = g_string_sized_new (512);
+		command = g_string_assign (command, gtk_widget_get_name (widget));
+		for (member = sel; member != NULL; member = member->next)
+		{
+			tp = (GtkTreePath *)member->data;
+			gtk_tree_model_get_iter (mdl, &iter, tp);
+			gtk_tree_model_get (mdl, &iter, FILENAME, &name, -1);
+			//FIXME there's a small chance that the dir may have changed
+			//without syncing to the dialog content
+			g_string_append_printf (command, " \"%s%s\"", rt->view->dir, name);
+			g_free (name);
+			gtk_tree_path_free (tp);
+		}
+		g_list_free (sel);
+
+#ifdef E2_COMMANDQ
+		e2_command_run (command->str, E2_COMMAND_RANGE_DEFAULT, FALSE);
+#else
+		e2_command_run (command->str, E2_COMMAND_RANGE_DEFAULT);
+#endif
+		g_string_free (command, TRUE);
+	}
 }
 /**
 @brief iconview un-select-all callback
@@ -816,7 +934,10 @@ static void _e2p_thumbs_unselect_all_cb (GtkWidget *widget, E2_ThumbDialogRuntim
 */
 static void _e2p_thumbs_refresh_cb (GtkWidget *widget, E2_ThumbDialogRuntime *rt)
 {
-	if (!rt->view->blocked)
+	LISTS_LOCK
+	gboolean busy = rt->view->listcontrols.norefresh;
+	LISTS_UNLOCK
+	if (!busy)
 	{
 		gdk_threads_leave ();
 		_e2p_thumbs_refresh_store (rt);
@@ -882,6 +1003,55 @@ static void _e2p_thumbs_toggle_replication_cb (GtkWidget *widget, E2_ThumbDialog
 //	}
 }
 /**
+@brief iconview clamp-image-size toggle callback
+
+@param widget the menu item widget which activated the callback
+@param rt pointer to dialog data struct
+
+@return
+*/
+static void _e2p_thumbs_toggle_clamp_cb (GtkWidget *widget, E2_ThumbDialogRuntime *rt)
+{
+	rt->clamp = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
+	_e2p_thumbs_refresh_cb (NULL, rt);
+}
+/**
+@brief populate @a menu with items for the actions for a filetype
+Can't use the standard function for this, it initiates commands using %f, which
+may be wrong, and has no data for the callback
+Each member of @a actions is like "command" or "label@command"
+@param menu the menu widget to which the action menu-items are to be added
+@param actions NULL-terminated array of utf8 strings, each a command for a filetype
+@param rt data struct for the dialog where the menu key was pressed
+
+@return
+*/
+static void _e2p_thumbs_menu_filetype_actions (GtkWidget *menu,
+	const gchar **actions, E2_ThumbDialogRuntime *rt)
+{
+	gchar *sep;
+	GtkWidget *menu_item;
+
+	while (*actions != NULL)
+	{
+		if ((sep = strchr (*actions, '@')) != NULL)  //if always ascii @, don't need g_utf8_strchr()
+		{
+			*sep = '\0';
+			menu_item = e2_menu_add (menu, (gchar *)*actions, NULL, NULL,
+				_e2p_thumbs_menu_choose_filetype_action_cb, rt);
+			*sep = '@';	//revert to original form (this is the 'source' data)
+			gtk_widget_set_name (menu_item, sep+1);
+		}
+		else
+		{
+			menu_item = e2_menu_add (menu, (gchar *)*actions, NULL, NULL,
+				_e2p_thumbs_menu_choose_filetype_action_cb, rt);
+			gtk_widget_set_name (menu_item, *actions);
+		}
+		actions++;
+	}
+}
+/**
 @brief construct and show dialog context menu
 This provides a subset of the filelist context menu, plus a couple of things
 @param iconview the widget where the click happened
@@ -922,13 +1092,13 @@ static void _e2p_thumbs_show_context_menu (GtkWidget *iconview,
 				actions = e2_filetype_get_actions (ext);
 				if (actions != NULL)
 				{
-					e2_context_menu_create_filetype_actions_menu (menu, actions);
+					_e2p_thumbs_menu_filetype_actions (menu, actions, rt);
 					break;
 				}
 			}
 			g_free (filename);
 		}
-		gchar *aname = g_strconcat (_A(5),".",_A(61), NULL);
+		gchar *aname = g_strconcat (_A(5),".",_A(62), NULL);
 		e2_menu_add_action (menu, _("Open _with.."),"open_with_"E2IP".png", NULL,
 			aname, NULL, NULL);
 		e2_menu_add_separator (menu);
@@ -958,18 +1128,29 @@ static void _e2p_thumbs_show_context_menu (GtkWidget *iconview,
 	item =
 	e2_menu_add_check (menu, _("Replicate _selection"),
 		rt->replicate, _e2p_thumbs_toggle_replication_cb, rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
 #endif
 		item,
 		_("If activated, items selected in this window will also be selected in the associated filelist"));
+
+	item =
+	e2_menu_add_check (menu, _("_Clamp size"),
+		rt->clamp, _e2p_thumbs_toggle_clamp_cb, rt);
+#ifdef USE_GTK2_12TIPS
+	gtk_widget_set_tooltip_text (
+#else
+	e2_widget_set_tooltip (NULL,
+#endif
+		item,
+		_("If activated, thumbnails will be scaled up or down if needed, into the range 32 to 128 pixels"));
 /*	item =
 	e2_menu_add_check (menu, _("_Hidden items"), rt->show_hidden,
 		_e2_treedlg_toggle_strict_cb, rt);
 	if (!rt->show_hidden)
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 		gtk_widget_set_tooltip_text (
 #else
 		e2_widget_set_tooltip (NULL,
@@ -978,6 +1159,7 @@ static void _e2p_thumbs_show_context_menu (GtkWidget *iconview,
 		_("If activated, hidden image files will be displayed"));
 	gtk_widget_set_sensitive (item, !rt->show_hidden);
 */
+
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 		G_CALLBACK (e2_menu_destroy_cb), NULL);
 
@@ -1068,18 +1250,17 @@ static void _e2p_thumbs_item_activated_cb (GtkIconView *iconview,
     if (gtk_tree_model_get_iter (model, &iter, tpath))
 	{
 		//get the activated item
-		gchar *fullpath, *dlocal;
+		gchar *localpath;
 		FileInfo *info;
-#ifdef E2_VFSTMP
-		dlocal = ;//FIXME
-#else
-		dlocal = F_FILENAME_TO_LOCALE (view->dir);
-#endif
 		gtk_tree_model_get (model, &iter, FINFO, &info, -1);
-		fullpath = e2_utils_strcat (dlocal, info->filename);
-		e2_task_backend_open (fullpath, TRUE);
-		F_FREE (dlocal);
-		g_free (fullpath);
+		localpath = e2_utils_dircat (view, info->filename, TRUE);
+#ifdef E2_VFS
+		VPATH ddata = { localpath, view->spacedata };
+		e2_task_backend_open (&ddata, TRUE);
+#else
+		e2_task_backend_open (localpath, TRUE);
+#endif
+		g_free (localpath);
 	}
 }
 /**
@@ -1155,7 +1336,7 @@ static GtkWidget *_e2p_thumbs_create_sorting_menu (E2_ThumbDialogRuntime *rt)
 	item =
 	e2_menu_add_check (menu, _("Ascending"), (rt->sort_order == GTK_SORT_ASCENDING),
 		_e2p_thumbs_toggle_sortorder_cb, rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1197,7 +1378,7 @@ static void _e2p_thumbs_set_sortmenu_position (GtkMenu *menu, gint *x, gint *y,
 		*y = button_y + button->allocation.height + 2;
 	else
 		*y = button_y - menu_size.height - 2;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 /**
 @brief handle button click, window-close etc for directory-tree dialog
@@ -1278,6 +1459,7 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 
 //	rt->show_hidden = show_hidden;	//before dialog is filled CHECKME use view->show_hidden ?
 	rt->replicate = TRUE;	//cause iconview selections to be replicated in related treeview
+	rt->clamp = TRUE;	//cause thumbs to be scaled up or down if outside 32..128 px
 	rt->sort_type = view->sort_column;
 	rt->sort_order = view->sort_order;
 	rt->blocked = FALSE;
@@ -1431,7 +1613,11 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 		&E2_BUTTON_CLOSE, NULL);
 
 	//get the actual data for the iconview
-	if (!rt->view->blocked)
+	LISTS_LOCK
+	gboolean busy = rt->view->listcontrols.refresh_working
+				 || rt->view->listcontrols.cd_working;
+	LISTS_UNLOCK
+	if (!busy)
 		_e2p_thumbs_replace_store (rt);
 	else
 		g_timeout_add_full (G_PRIORITY_HIGH, 100,
@@ -1444,7 +1630,12 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 		app.pane1.hook_change_dir : app.pane2.hook_change_dir;
 	e2_hook_register (&thisview_cdhook, _e2p_thumbs_change_dir_hook, rt);
 //	e2_hook_register (&rt->view->hook_refresh, _e2p_thumbs_refresh_hook, rt);
-	rt->refreshtimer_id = g_timeout_add (E2_FILESCHECK_INTERVAL,
+	rt->refreshtimer_id =
+#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
+#else
+		g_timeout_add (E2_FILESCHECK_INTERVAL,
+#endif
 		(GSourceFunc) _e2p_thumbs_check_dirty, rt);
 
 	return NULL;
@@ -1481,8 +1672,8 @@ static void _e2p_thumbs_keybindings (E2_OptionSet *set)
 	//the category string(s) here need to match the binding name
 	g_strconcat(_C(11),"||||",NULL),  //_("dialogs"
 	g_strconcat("\t",aname,"||||",NULL),  //_("thumbs"
-	g_strconcat("\t\t|<Control>j","||",_A(108),".",_A(109),"|<Control>a",NULL),
-	g_strconcat("\t\t|<Control>k","||",_A(108),".",_A(109),"|<Control>c",NULL),
+	g_strconcat("\t\t|<Control>j","||",_A(119),".",_A(120),"|<Control>a",NULL),
+	g_strconcat("\t\t|<Control>k","||",_A(119),".",_A(120),"|<Control>c",NULL),
 	g_strdup(">"),
 	NULL);
 }
