@@ -1,6 +1,6 @@
-/* $Id: e2_action.c 535 2007-07-16 23:51:52Z tpgww $
+/* $Id: e2_action.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -39,9 +39,6 @@ ToDo
 #include "e2_action.h"
 #include "e2_action_option.h"
 #include "e2_task.h"
-#ifdef E2_VFS
-# include "e2_vfs.h"
-#endif
 #include "e2_plugins.h"
 #include "e2_filetype.h"
 #include "e2_dialog.h"
@@ -83,8 +80,8 @@ static gboolean _e2_action_do_toggle (gpointer from, E2_ActionRuntime *art)
 		newstate = !ex->current_state;
 	else
 		newstate = e2_toolbar_button_toggle_custom (hash_name);
-
-	gchar *cc = g_strdup ((newstate) ? ex->true_action : ex->false_action);
+	//get the operation corresponding to the un-toggled state
+	gchar *cc = g_strdup ((newstate) ? ex->false_action : ex->true_action);
 	if (e2_action_check (cc) != NULL)
 	{
 		gchar *gap = e2_utils_find_whitespace (cc);
@@ -126,6 +123,20 @@ static gboolean _e2_action_custom_command (gpointer from, E2_ActionRuntime *art)
 	g_free (command);
 	return (res == 0);
 }
+#ifdef E2_VFS
+/**
+@brief dummy action for plugins which do not really belong on a plugins menu
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return FALSE always
+*/
+gboolean e2_action_inaction (gpointer from, E2_ActionRuntime *art)
+{
+	return FALSE;
+}
+#endif
 
   /*********************/
  /***** callbacks *****/
@@ -677,17 +688,17 @@ void e2_actions_init (void)
 
 	toggles_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 		(GDestroyNotify) _e2_action_free_toggle);
-	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(99),NULL);
-	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(99),NULL);
-	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(79),NULL);
-	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(79),NULL);
+	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(108),NULL);
+	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(108),NULL);
+	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(80),NULL);
+	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(80),NULL);
 	toggles_array [E2_TOGGLE_PANE1FILTERS] = g_strconcat (_A(11),".",_A(22),NULL);
 	toggles_array [E2_TOGGLE_PANE2FILTERS] = g_strconcat (_A(12),".",_A(22),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(9),".",_A(99),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(9),".",_A(78),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(9),".",_A(108),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(9),".",_A(79),NULL);
 #ifdef E2_VFS
-	toggles_array [E2_TOGGLE_PANE1VIRTUAL] = g_strconcat (_A(11),".",_A(106),NULL);
-	toggles_array [E2_TOGGLE_PANE2VIRTUAL] = g_strconcat (_A(12),".",_A(106),NULL);
+	toggles_array [E2_TOGGLE_PANE1SPACE] = g_strconcat (_A(11),".",_A(115),NULL);
+	toggles_array [E2_TOGGLE_PANE2SPACE] = g_strconcat (_A(12),".",_A(115),NULL);
 #endif
 	//register most actions
 	//note translated action labels needed here, i.e. before config loaded
@@ -719,14 +730,14 @@ void e2_actions_init (void)
 	homeless_actions[2].name = g_strdup(_A(18));
 	homeless_actions[3].name = g_strdup(_A(19));
 
-	homeless_actions[4].name = g_strconcat(_A(1),".",_A(68),NULL);
+	homeless_actions[4].name = g_strconcat(_A(1),".",_A(69),NULL);
 	//configuration
 	homeless_actions[5].name = g_strconcat(_A(2),".",_A(32),NULL);
 	homeless_actions[6].name = g_strconcat(_A(2),".",_A(37),NULL);
 	homeless_actions[7].name = g_strconcat(_A(2),".",_C(33),NULL);
 	homeless_actions[8].name = g_strdup(_A(17));
-	homeless_actions[9].name = g_strconcat(_A(15),".",_A(101),NULL);
-	homeless_actions[10].name = g_strconcat(_A(15),".",_A(102),NULL);
+	homeless_actions[9].name = g_strconcat(_A(15),".",_A(110),NULL);
+	homeless_actions[10].name = g_strconcat(_A(15),".",_A(111),NULL);
 
 	gint count = sizeof (homeless_actions) / sizeof (E2_Action);
 	gint i;
@@ -744,9 +755,6 @@ void e2_actions_init (void)
 	//register most other actions
 	// (some others are in pane create fn)
 	e2_task_actions_register ();
-#ifdef E2_VFS
-	e2_vfs_actions_register ();
-#endif
 	e2_output_actions_register();
 //	e2_context_menu_actions_register();
 	e2_bookmark_actions_register ();
@@ -834,81 +842,92 @@ void e2_action_setup_labels (void)
 	action_labels[41] = _("filetype");
 	action_labels[42] = _("focus");
 	action_labels[43] = _("focus_toggle");
-	action_labels[44] = _("go_back");
-	action_labels[45] = _("go_forward");
-	action_labels[46] = _("go_up");
-	action_labels[47] = _("goto_bottom");
-	action_labels[48] = _("goto_top");
-	action_labels[49] = _("help");
-	action_labels[50] = _("history");
-	action_labels[51] = _("info");
-	action_labels[52] = _("insert_selection");
-	action_labels[53] = _("invert_selection");
-	action_labels[54] = _("mirror");
-	action_labels[55] = _("mkdir");
-	action_labels[56] = _("mountpoints");
-	action_labels[57] = _("move");
-	action_labels[58] = _("move_as");
-	action_labels[59] = _("open");
-	action_labels[60] = _("open_in_other");
-	action_labels[61] = _("open_with");
-	action_labels[62] = _("owners");
-	action_labels[63] = _("page_down");
-	action_labels[64] = _("page_up");
-	action_labels[65] = _("pending");
-	action_labels[66] = _("permissions");
-	action_labels[67] = _("print");
-	action_labels[68] = _("quit");
-	action_labels[69] = _("refresh");
-	action_labels[70] = _("refreshresume");
-	action_labels[71] = _("refreshsuspend");
-	action_labels[72] = _("rename");
-	action_labels[73] = _("scroll_down");
-	action_labels[74] = _("scroll_up");
-	action_labels[75] = _("search");  //= _C(35)
-	action_labels[76] = _("send");
-	action_labels[77] = _("set");
-	action_labels[78] = _("show");
-	action_labels[79] = _("show_hidden");
-	action_labels[80] = _("show_menu");
-	action_labels[81] = _("switch");
-	action_labels[82] = _("symlink");
-	action_labels[83] = _("symlink_as");
-	action_labels[84] = _("sync");
-	action_labels[85] = _("toggle_direction");
-//	action_labels[86] = _("toggle_full");	//changed to _A(99)"expand" with new toggles
-	action_labels[86] = _("toggle_select_all");
-	action_labels[87] = _("toggle_selected");
-	action_labels[88] = _("trash");
-	action_labels[89] = _("trashempty");
+	action_labels[44] = _("fullscreen");
+	action_labels[45] = _("go_back");
+	action_labels[46] = _("go_forward");
+	action_labels[47] = _("go_up");
+	action_labels[48] = _("goto_bottom");
+	action_labels[49] = _("goto_top");
+	action_labels[50] = _("help");
+	action_labels[51] = _("history");
+	action_labels[52] = _("info");
+	action_labels[53] = _("insert_selection");
+	action_labels[54] = _("invert_selection");
+	action_labels[55] = _("mirror");
+	action_labels[56] = _("mkdir");
+	action_labels[57] = _("mountpoints");
+	action_labels[58] = _("move");
+	action_labels[59] = _("move_as");
+	action_labels[60] = _("open");
+	action_labels[61] = _("open_in_other");
+	action_labels[62] = _("open_with");
+	action_labels[63] = _("owners");
+	action_labels[64] = _("page_down");
+	action_labels[65] = _("page_up");
+	action_labels[66] = _("pending");
+	action_labels[67] = _("permissions");
+	action_labels[68] = _("print");
+	action_labels[69] = _("quit");
+	action_labels[70] = _("refresh");
+	action_labels[71] = _("refreshresume");
+	action_labels[72] = _("refreshsuspend");
+	action_labels[73] = _("rename");
+	action_labels[74] = _("scroll_down");
+	action_labels[75] = _("scroll_up");
+	action_labels[76] = _("search");  //= _C(35)
+	action_labels[77] = _("send");
+	action_labels[78] = _("set");
+	action_labels[79] = _("show");
+	action_labels[80] = _("show_hidden");
+	action_labels[81] = _("show_menu");
+	action_labels[82] = _("sortaccesssed");
+	action_labels[83] = _("sortchanged");
+	action_labels[84] = _("sortgroup");
+	action_labels[85] = _("sortmodified");
+	action_labels[86] = _("sortname");
+	action_labels[87] = _("sortpermission");
+	action_labels[88] = _("sortsize");
+	action_labels[89] = _("sortuser");
+	action_labels[90] = _("switch");
+	action_labels[91] = _("symlink");
+	action_labels[92] = _("symlink_as");
+	action_labels[93] = _("sync");
+	action_labels[94] = _("toggle_direction");
+//	action_labels[95] = _("toggle_full");	//changed to _A(108)"expand" with new toggles
+	action_labels[95] = _("toggle_select_all");
+	action_labels[96] = _("toggle_selected");
+	action_labels[97] = _("trash");
+	action_labels[98] = _("trashempty");
 #ifdef E2_TREEDIALOG
-	action_labels[90] = _("tree");
+	action_labels[99] = _("tree");
 #endif
-	action_labels[91] = _("unpack");
-	action_labels[92] = _("view");  //= _C(40)
-	action_labels[93] = _("view_again");
-	action_labels[94] = _("view_at");
+	action_labels[100] = _("unpack");
+	action_labels[101] = _("view");  //= _C(40)
+	action_labels[102] = _("view_again");
+	action_labels[103] = _("view_at");
 	//these are action _parameter_ strings
-	action_labels[95] = _("child");
-	action_labels[96] = _("ctrl");
-	action_labels[97] = _("dirs");
-	action_labels[98] = _("escape");
-	action_labels[99] = _("expand");
-	action_labels[100] = _("files");
-	action_labels[101] = _("off");
-	action_labels[102] = _("on");
-	action_labels[103] = _("quote");
-	action_labels[104] = _("shift");
-	action_labels[105] = _("top");
+	action_labels[104] = _("child");
+	action_labels[105] = _("ctrl");
+	action_labels[106] = _("dirs");
+	action_labels[107] = _("escape");
+	action_labels[108] = _("expand");
+	action_labels[109] = _("files");
+	action_labels[110] = _("off");
+	action_labels[111] = _("on");
+	action_labels[112] = _("quote");
+	action_labels[113] = _("shift");
+	action_labels[114] = _("top");
 	//late entries, out of order
 	//maybe more space here, see array definition in header file
 #ifdef E2_VFS
-	action_labels[106] = _("<virtual>");
-	action_labels[107] = _("unpack_in_other");
+	action_labels[115] = _("<spacemenu>");
+	action_labels[116] = _("dummy");
+	action_labels[117] = _("namespace");
+	action_labels[118] = _("unpack_in_other");
 #endif
 #ifdef E2_KEYALIAS
-	action_labels[108] = _("key");
-	action_labels[109] = _("alias");
+	action_labels[119] = _("key");
+	action_labels[120] = _("alias");
 #endif
 }
 
