@@ -1,6 +1,6 @@
-/* $Id: e2_alias.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_alias.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_ALIAS_H
-#define E2_ALIAS_H
+#ifndef __E2_ALIAS_H__
+#define __E2_ALIAS_H__
 
 #include "emelfm2.h"
 #include <regex.h>
@@ -47,4 +47,4 @@ void e2_alias_sync (E2_AliasRuntime *rt);
 void e2_alias_init (void);
 void e2_alias_clean (void);
 
-#endif /* ifndef E2_ALIAS_H */
+#endif //ndef __E2_ALIAS_H__
