@@ -1,6 +1,6 @@
-/* $Id: e2_command.c 567 2007-07-25 10:35:45Z tpgww $
+/* $Id: e2_command.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -159,6 +159,7 @@ rewait:
 		if (!(rt == NULL || rt->action))
 		{
 			printd (DEBUG, "ending after waitpid returned %d", endpid);
+			pthread_mutex_lock (&task_mutex);
 			if (WIFEXITED (exitstatus))
 			{
 				rt->pid = -2L;	//prevent further matches for this rt
@@ -194,7 +195,7 @@ rewait:
 					rt->pid = -2L;	//prevent further matches for this rt
 					rt->status = E2_TASK_INCOMPLETE;
 					rt->ex.command.exit = signal;
-					printd (DEBUG, "_e2_command_watch() detected command '%s' terminated by signal %d (%s)",
+					printd (DEBUG, "child-signal handler detected command '%s' terminated by signal %d (%s)",
 						rt->ex.command.command, signal, g_strsignal (signal));
 					printd (DEBUG, "child status (%d) stored", signal);
 				}
@@ -206,6 +207,7 @@ rewait:
 				printd (DEBUG, "process %d restarted", endpid);
 			}
 # endif
+			pthread_mutex_unlock (&task_mutex);
 		}
 		else	//no matching running-command data (could be a sync command (old version))
 //			if (!rt->action)
@@ -397,6 +399,7 @@ static void _e2_command_sigpoll_handler (gint num, siginfo_t *info, void *contex
 * /
 }
 */
+#define E2CMDTYPEB
 /**
 @brief watch for and forward to output pane any stdout or stderr text from running command
 This is a thread function.
@@ -407,8 +410,10 @@ Assumes BGL is off
 */
 static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 {
-	struct timeval wait;
-	struct timeval *waitptr;
+//	struct timeval wait;
+//	struct timeval *waitptr;
+	struct timespec wait;
+	struct timespec *waitptr;
 	gchar *buffer;
 #ifdef USE_GLIB2_10
 	buffer = g_slice_alloc (READBUFSIZE);
@@ -434,6 +439,10 @@ static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 	//block until select succeeds
 	//and note, ouput pipe (if used) is always selectable
 	waitptr = NULL;
+#ifdef E2CMDTYPEB
+	gboolean outread, errread;
+	outread = errread = TRUE;	//we want to pselect() the fd's at least once
+#endif
 
 	while (TRUE)
 	{
@@ -445,33 +454,58 @@ static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 #ifdef OUTPUT_POLL
 		fd_set out_fds;
 #endif
-
 reselect:
+/*#ifdef E2CMDTYPEB
+		//FIXME handle case where outread and errread are both FALSE;
+		if (!(outread || errread))
+		{
+			selcount = 0;
+			goto ?
+		}
+#endif */
 		FD_ZERO (&in_fds);
-		FD_SET (cmddata->child_stdout_fd, &in_fds);
-		FD_SET (cmddata->child_stderr_fd, &in_fds);
+//FIXME find a way to may these tests work without stuffing up the completion reporting
+#ifdef E2CMDTYPEB
+		if (outread)	//don't select fd's that are closed
+#endif
+			FD_SET (cmddata->child_stdout_fd, &in_fds);
+#ifdef E2CMDTYPEB
+		if (errread)
+#endif
+			FD_SET (cmddata->child_stderr_fd, &in_fds);
 #ifdef OUTPUT_POLL
 		FD_ZERO (&out_fds);
-		FD_SET (cmddata->child_stdin_fd, &out_fds);
+# ifdef E2CMDTYPEB
+		if (?)
+# endif
+			FD_SET (cmddata->child_stdin_fd, &out_fds);
 #endif
 		//timer data undefined after last use, re-init if used now
 		if (waitptr != NULL)
 		{
 			wait.tv_sec = maxwait; //0 for immediate return
-			wait.tv_usec = 0;
+//			wait.tv_usec = 0;
+			wait.tv_nsec = 0;
 		}
+
+#ifndef E2CMDTYPEB
 		outread = errread = FALSE;	//ensure var's are initialised
-//CHECKME pselect instead of select ?
+#endif
+
 #ifdef OUTPUT_POLL
-		selcount = select (maxfd, &in_fds, &out_fds, NULL, waitptr);
+		selcount = pselect (maxfd, &in_fds, &out_fds, NULL, waitptr, NULL);
 #else
-		selcount = select (maxfd, &in_fds, NULL, NULL, waitptr);
+		selcount = pselect (maxfd, &in_fds, NULL, NULL, waitptr, NULL);
 #endif
 		if (selcount == -1 && errno == EINTR)
+		{
+			usleep (1000);	//wait awhile for any interrrupt processing
 			goto reselect;
+		}
 		else if (selcount < 0)
 		{
 			printd (DEBUG, "pipe selection error %d (%s)", errno, g_strerror (errno));
+//			outread = errread = FALSE;	//ensure no more selects
 			selcount = 0;	//fake value to trigger loop exit
 		}
 		else if (selcount > 0)
@@ -482,6 +516,9 @@ reselect:
 			{
 //				printd (DEBUG, "child stdout ready");
 				//read input, starting with normal pipe
+#ifdef E2CMDTYPEB
+				outread = FALSE;	//in case this one fails
+#endif
 				while ((bytecount =
 					_e2_command_piperead (cmddata->child_stdout_fd, buffer, READMAX)) > 0)
 				{
@@ -492,22 +529,36 @@ reselect:
 					if (bytecount != READMAX)
 						break;
 				}
+				if (!outread)
+				{
+					//pipe closed or closing
+					printd (DEBUG, "FALSE positive child stdout ready");
+					usleep (10000);	//allow for some cleanup
+				}
 			}
+//#ifdef E2CMDTYPEB
+//			else
+//				outread = TRUE;	//ensure reselection happens
+//#endif
+
 			//to minimise race-risk, check again for stderr if stdout was processed
 			if (!FD_ISSET (cmddata->child_stderr_fd, &in_fds) && outread)
 			{
 //				printd (DEBUG, "re-check child stderr ready");
 				wait.tv_sec = 0;
-				wait.tv_usec = 0;
+//				wait.tv_usec = 0;
+				wait.tv_nsec = 0;
 				FD_ZERO (&in_fds);
 				FD_SET (cmddata->child_stderr_fd, &in_fds);
-				TEMP_FAILURE_RETRY (select (cmddata->child_stderr_fd+1, &in_fds, NULL, NULL, &wait));
+				TEMP_FAILURE_RETRY (pselect (cmddata->child_stderr_fd+1, &in_fds, NULL, NULL, &wait, NULL));
 			}
-
 			if (FD_ISSET (cmddata->child_stderr_fd, &in_fds))
 			{
 //				printd (DEBUG, "child stderr ready");
 				//read input, starting with error pipe
+#ifdef E2CMDTYPEB
+				errread = FALSE;	//in case this one fails
+#endif
 				while ((bytecount =
 					_e2_command_piperead (cmddata->child_stderr_fd, buffer, READMAX)) > 0)
 				{
@@ -518,8 +569,17 @@ reselect:
 					if (bytecount != READMAX)
 						break;
 				}
+				if (!errread)
+				{
+					//pipe closed or closing
+					printd (DEBUG, "FALSE positive child stderr ready");
+					usleep (10000);	//allow for some cleanup
+				}
 			}
-
+//#ifdef E2CMDTYPEB
+//			else
+//				errread = TRUE;	//ensure reselection happens
+//#endif
 #ifdef OUTPUT_POLL
 //FIXME grab any commandline input and pass direct to running sync command
 //MAYBE only if child is stopped ?
@@ -547,40 +607,76 @@ reselect:
 #endif
 		}
 
-		if (rt->status >= E2_TASK_COMPLETED)	//completed of one of the failures
+		pthread_mutex_lock (&task_mutex);
+		gboolean done = (rt->status >= E2_TASK_COMPLETED);
+		pthread_mutex_unlock (&task_mutex);
+#ifdef E2CMDTYPEB
+		if (done || selcount == 0 || !(outread || errread))
+#else
+		if (done)
 		{
 			if (selcount == 0 || !(outread || errread))
+#endif
 			{
-				printd (DEBUG, "watch function cleanup after end of process %d", rt->pid);
+				printd (DEBUG, "watch function cleanup after end of process %s", rt->pidstr);
 				close (cmddata->child_stdin_fd);
 				close (cmddata->child_stdout_fd);
 				close (cmddata->child_stderr_fd);
-				E2_CommandTaskData *cmddata = &rt->ex.command;
 				if (cmddata->show && e2_option_bool_get ("fileop-show"))
 				{
-					gchar *message = g_strdup_printf ("%s>%s (%s) %s '%d'\n",
-						(cmddata->extshell) ? "sh" : "", cmddata->command,
-					rt->pidstr, _("returned"), cmddata->exit);
-					gdk_threads_enter ();
-					e2_output_print (rt->current_tab, message, rt->pidstr,
-						TRUE, "small", (cmddata->exit == 0) ? "green" : "red", NULL);
-					gdk_threads_leave ();
-					g_free (message);
+#ifdef E2CMDTYPEB
+					//wait at most 5 secs for completion code to be logged
+					guint i = 0;
+					while (!done && i < 250)
+					{
+						usleep (20000);
+						pthread_mutex_lock (&task_mutex);
+						done = (rt->status >= E2_TASK_COMPLETED);
+						pthread_mutex_unlock (&task_mutex);
+						i++;
+					}
+					if (i < 250)
+					{
+#endif
+						gchar *message = g_strdup_printf ("%s>%s (%s) %s '%d'\n",
+							(cmddata->extshell) ? "sh" : "", cmddata->command,
+						rt->pidstr, _("returned"), cmddata->exit);
+						gdk_threads_enter ();
+						e2_output_print (rt->current_tab, message, rt->pidstr,
+							TRUE, "small", (cmddata->exit == 0) ? "green" : "red", NULL);
+						gdk_threads_leave ();
+						g_free (message);
+#ifdef E2CMDTYPEB
+					}
+#endif
 				}
+		//CHECKME sometimes (how?) we can end this func without breaking here
+				printd (DEBUG, "Exiting command select loop");
 				break;
+#ifndef E2CMDTYPEB
 			}
 			else	//CHECKME maybe not really needed ?
 			{
-				printd (DEBUG, "scanning child pipes again after watch function detected process %d completion flag", rt->pid);
+				printd (DEBUG, "scanning child pipes again after watch function detected process %s completion flag", rt->pidstr);
 				//do one last quick scan in case of race between SIGCHILD and pipes interrogation
 				maxwait = 0;
 				waitptr = &wait;
 				goto reselect;
 			}
+#endif
 		}
 #ifdef DEBUG_MESSAGES
-		else if (selcount == 0 || !(outread || errread))
-			printd (DEBUG, "no pipe from child was actually read");
+		else
+		{
+	//		if (selcount == 0 || !(outread || errread))
+	//			printd (DEBUG, "no pipe from child was actually read");
+			if (selcount == 0)
+				printd (DEBUG, "selected 0 fd's");
+			if (!outread)
+				printd (DEBUG, "flag outread FALSE");
+			if (!errread)
+				printd (DEBUG, "flag errread FALSE");
+		}
 #endif
 	//CHECKME can we do this without so many loops ? SIGPOLL ?
 	//CHECKME for sync commands, omit WNOHANG ?
@@ -656,27 +752,32 @@ THIS RELIES ON select() returning regularly ...
 #else
 	g_free (buffer);
 #endif
-
+//	printd (DEBUG, "End command watch");
 	return (GINT_TO_POINTER (rt->ex.command.exit));
 }
 /**
 @brief run command @a command
-This sets the command's CWD to curr_view->dir
 Assumes BGL is closed upon entry
 @param command the command string, used in messages, utf8
 @param args pointer to NULL-terminated command arguments array, each localised
+@param cwd path of directory to use as CWD for the command, utf8
 @param sync TRUE to run @a command in foreground, FALSE for background
 @param show TRUE to show @a command and its exit status in the output pane
 @param extshell TRUE if @a command is to be run in external shell
 
 @return the exit code of the command, > 0 if error
 */
-static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
-	gboolean show, gboolean extshell)
+static gint _e2_command_fork (gchar *command, gchar **args, const gchar *cwd,
+	gboolean sync, gboolean show, gboolean extshell)
 {
 	//there are no embedded tests for command existence, so we need to do that
 	if (!g_path_is_absolute (args[0]))
 	{
+#ifdef E2_VFSTMP
+	if supporting get-and-exec, then local $PATH may be irrelevant
+	need relevant PlaceInfo to sort this out, might not be curr_view->spacedata->workplace
+	cwd needs to be a VPATH*
+#endif
 		gchar *path = g_find_program_in_path (args[0]);
 		if (path != NULL)
 		{
@@ -692,6 +793,9 @@ static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
 		}
 	}
 
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
 	gchar *message;
 	pid_t pid;
 /*
@@ -766,6 +870,7 @@ static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
 	}
 
 	_e2_command_block_childsignal ();
+	gdk_threads_leave ();
 
 	if ((pid = fork ()) == 0)
 	{	//child
@@ -806,24 +911,35 @@ static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
 
 		_e2_command_unblock_childsignal ();
 
-# ifdef E2_VFSTMP
+#ifdef E2_VFS
 		if (curr_view->spacedata != NULL)
 		{
-			//FIXME set the correct CWD
-			if (!e2_fs_chdir_local (curr_view->spacedata->workplace))
+#ifdef E2_VFSTMP
+	if supporting get-and-exec, and getting is slow, some cd-race might occur
+	need relevant PlaceInfo which might not be curr_view->spacedata->workplace
+#endif
+			//there's no native analog for the vdir, so go here...
+			//all command args need to have been modified accordingly, by the command interpreter
+			ddata.localpath = curr_view->spacedata->workplace;
+			ddata.spacedata = NULL;
+			if (!e2_fs_chdir_local (&ddata E2_ERR_NONE()))
 				execvp (args[0], args);	//should not return
 		}
-		else
-# else
-		if (e2_fs_chdir (curr_view->dir E2_ERR_NONE()))	//native execution only, no vfs error check
-# endif
+		else	//native execution
+#endif
 		{
-//			printd (DEBUG, "Child will run '%s'", args[0]);
-			execvp (args[0], args);	//should not return
+			if (cwd == NULL)
+				cwd = curr_view->dir;
+			if (e2_fs_chdir ((gchar *)cwd E2_ERR_NONE()))
+			{
+	//			printd (DEBUG, "Child will run '%s'", args[0]);
+				execvp (args[0], args);	//should not return
+			}
 		}
 		_exit (1);
 		//end of child process
 	}
+	gdk_threads_enter ();
 	if (pid < 0)
 	{	//fork error
 		close (stdin_pipe[INWARDS]);
@@ -838,9 +954,9 @@ static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
 		goto launch_error;
 	}
 	//parent process
+	rt->status = E2_TASK_RUNNING;
 	rt->pid = (glong) pid;
 	rt->pidstr = g_strdup_printf ("%ld", rt->pid);
-	rt->status = E2_TASK_RUNNING;
 	E2_CommandTaskData *cmddata = &rt->ex.command;
 	cmddata->child_stdin_fd = stdin_pipe[OUTWARDS];	//setup for later sending to child
 	cmddata->child_stdout_fd = stdout_pipe[INWARDS];	//setup for later receiving from child
@@ -938,8 +1054,21 @@ static gint _e2_command_fork (gchar *command, gchar **args, gboolean sync,
 	}
 
 launch_error:
+#ifdef E2_VFS
+# ifdef E2_VFSTMP
+	if supporting get-and-exec, and getting is slow, some cd-race might occur
+	need relevant PlaceInfo which might not be curr_view->spacedata->workplace
+# endif
+	ddata.localpath = args[0];
+	ddata.spacedata = curr_view->spacedata;
+#endif
 	gdk_threads_leave ();	//downstream does mutex management
-	e2_fs_error_simple (_("Error while launching '%s'"), args[0]);
+	e2_fs_error_simple (_("Error while launching '%s'"),
+#ifdef E2_VFS
+		&ddata);
+#else
+		args[0]);
+#endif
 	gdk_threads_enter ();
 	return 1;
 }
@@ -1243,6 +1372,7 @@ This sets the executed command's CWD to curr_view->dir
 Expects BGL closed
 @param command the command string, used in messages, utf8
 @param args pointer to command arguments array, each localised
+@param cwd path of directory to use as CWD for the command, utf8
 @param show TRUE to show @a command and its pid in the output pane
 @param extshell TRUE when command is to be run in external shell
 
@@ -1253,19 +1383,22 @@ Expects BGL closed
 GIOChannel *stdoutchannel;
 GIOChannel *stderrchannel;
 #endif
-static gint _e2_command_run_async (gchar *command, gchar **args, gboolean show,
-	gboolean extshell)
+static gint _e2_command_run_async (gchar *command, gchar **args, const gchar *cwd,
+	gboolean show, gboolean extshell)
 {
 	printd (DEBUG, "asynchronously run command %s", command);
 	_e2_command_block_childsignal ();	//prevent feedback from child until ready to get it
 
+	if (cwd == NULL)
+		cwd = curr_view->dir;
 	gchar *local;
-#ifdef E2_VFSTMP
-	//FIXME set the correct CWD
+#ifdef E2_VFS
+	//for a non-native place, there is no analog for curr_view->dir, so go elsewhere
+	//all args need to have been conformed, by the command interpreter
 	local = (curr_view->spacedata == NULL) ?
-		F_FILENAME_TO_LOCALE (curr_view->dir) : curr_view->spacedata->workplace;
+		F_FILENAME_TO_LOCALE (cwd) : curr_view->spacedata->workplace;
 #else
-	local = F_FILENAME_TO_LOCALE (curr_view->dir);
+	local = F_FILENAME_TO_LOCALE (cwd);
 #endif
 	GPid pid = 0;
 	gint stdinfd = 0, stdoutfd = 0, stderrfd = 0;	//file descriptors to write to and read from child
@@ -1273,10 +1406,12 @@ static gint _e2_command_run_async (gchar *command, gchar **args, gboolean show,
 	g_spawn_async_with_pipes (local, args, NULL,
 		G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
 		NULL, NULL, &pid, &stdinfd, &stdoutfd, &stderrfd, &error);
-#ifdef E2_VFSTMP
+#ifdef E2_VFS
 	if (curr_view->spacedata == NULL)
 #endif
-	F_FREE (local);
+		F_FREE (local);
+
+	//CHECKME revert CWD to curr_view->dir if that still exists ?
 
 	if (error != NULL)
 	{
@@ -1353,13 +1488,14 @@ This sets the executed command's CWD to curr_view->dir
 Expects BGL to be closed
 @param command the command string, used in messages, utf8
 @param args pointer to command arguments array, each localised
+@param cwd path of directory to use as CWD for the command, utf8
 @param show TRUE to show @a command and its exit status in the output pane
 @param extshell TRUE when command is to be run in external shell
 
 @return the exit code of the command, > 0 if error
 */
-static gint _e2_command_run_sync (gchar *command, gchar **args, gboolean show,
-	gboolean extshell)
+static gint _e2_command_run_sync (gchar *command, gchar **args, const gchar *cwd,
+	gboolean show, gboolean extshell)
 {
 	printd (DEBUG, "synchronously run command %s", command);
 	//we dont get real pid's from static commands, so we fake
@@ -1398,16 +1534,19 @@ static gint _e2_command_run_sync (gchar *command, gchar **args, gboolean show,
 		WAIT_FOR_EVENTS;
 	}
 
+	if (cwd == NULL)
+		cwd = curr_view->dir;
 //	GError *error;
 	gint exit;
 	gchar *sout = NULL, *serr = NULL;
 	gchar *local;
-#ifdef E2_VFSTMP
-	//FIXME set the correct CWD
+#ifdef E2_VFS
+	//for a non-native place, there is no analog for curr_view->dir, so go elsewhere
+	//all args need to have been conformed, by the command interpreter
 	local = (curr_view->spacedata == NULL) ?
-		F_FILENAME_TO_LOCALE (curr_view->dir) : curr_view->spacedata->workplace;
+		F_FILENAME_TO_LOCALE (cwd) : curr_view->spacedata->workplace;
 #else
-	local = F_FILENAME_TO_LOCALE (curr_view->dir);
+	local = F_FILENAME_TO_LOCALE (cwd);
 #endif
 	_e2_command_block_childsignal ();
 
@@ -1419,6 +1558,8 @@ static gint _e2_command_run_sync (gchar *command, gchar **args, gboolean show,
 	//no rt cleanup after error
 	rt->status = (success) ? E2_TASK_COMPLETED : E2_TASK_FAILED;
 
+	//CHECKME revert CWD to curr_view->dir if that still exists ?
+
 	_e2_command_unblock_childsignal ();
 
 	rt->ex.command.exit = exit;
@@ -1435,10 +1576,10 @@ static gint _e2_command_run_sync (gchar *command, gchar **args, gboolean show,
 		g_free (sout);
 	if (serr != NULL)
 		g_free (serr);
-#ifdef E2_VFSTMP
+#ifdef E2_VFS
 	if (curr_view->spacedata == NULL)
 #endif
-	F_FREE (local);
+		F_FREE (local);
 
 	if (success && show && e2_option_bool_get ("fileop-show"))
 	{
@@ -2105,14 +2246,18 @@ interpreted as "cd $HOME".
 Command arguments for must be separated by one or more ' ' chars (ie no tabs)
 NOTE used to return pid of process running the command (if any)
 @param raw utf-8 command string
+@param cwd utf8 path string, maybe with trailer, to use as (native) CWD when running @a command NULL for curr_view->dir
 @param range flags for the scope of the command
 
 @return 0 for success, >0 for error
 */
 //FIXME more consistent returned value
-static gint _e2_command_run_single (gchar *raw, E2_CommandRange range)
+static gint _e2_command_run_single (gchar *raw, const gchar *cwd, E2_CommandRange range)
 {
 	//CHECKME should variables be volatilized for threaded usage ?
+#ifdef E2_VFSTMP
+//BIG FIXME handling command arguments etc when active pane is vdir
+#endif
 	static gchar *prior_cmd = NULL;	//for preventing recursive alias substitution
 	gchar *command1, *command2, *freeme;
 	gint result;
@@ -2202,6 +2347,8 @@ static gint _e2_command_run_single (gchar *raw, E2_CommandRange range)
 			gchar *s1 = e2_utils_translate_relative_path (
 #ifdef E2_VFSTMP
 			//FIXME which CWD ?
+			OR should we "get" and run an executable in an active-pane vdir
+			or some dir realtive to that?
 #else
 			curr_view->dir,
 #endif
@@ -2268,17 +2415,19 @@ static gint _e2_command_run_single (gchar *raw, E2_CommandRange range)
 
 	//if command is: cd [%]%f, when updir '../' line was clicked, that
 	//item would not be available as a selected item, so ...
-	if (g_str_has_prefix (command1, "cd ")
-		//FIXME support macros other than with english letters
-		&& strstr (command1, "%f") != NULL)
+	if (g_str_has_prefix (command1, "cd "))
 	{
-		FileInfo *info = e2_fileview_get_selected_first_local (curr_view, TRUE);
-		if (info != NULL //something selected
-			&& g_str_equal (info->filename, ".."))
+		//FIXME support macros other than with english letters
+		if (strstr (command1, "%f") != NULL || strstr (command1, "%p") != NULL)
 		{
-			g_free (command1);
-			command1 = g_strdup ("cd ..");	//no translation
-			expand_macros = FALSE;	//avoid error message
+			FileInfo *info = e2_fileview_get_selected_first_local (curr_view, TRUE);
+			if (info != NULL //something selected
+				&& g_str_equal (info->filename, ".."))
+			{
+				g_free (command1);
+				command1 = g_strdup ("cd ..");	//no translation
+				expand_macros = FALSE;	//avoid error message
+			}
 		}
 	}
 	//expand macros, if wanted
@@ -2359,13 +2508,12 @@ static gint _e2_command_run_single (gchar *raw, E2_CommandRange range)
 			*command2 = ':';
 		}
 	}
-
 	//"cd" commands that are not run in a separate shell
 	//just change the dir in the active pane
 	if (!run_in_shell && g_str_has_prefix (command1, "cd"))	//no translation
 	{	//note: tests here rely on ascii single-byte chars in command string
 		gint len = strlen (command1);
-		if ((len > 3) && (command1[2] == ' '))	//FIXME or \t ?
+		if (len > 3 && command1[2] == ' ')	//FIXME or \t ?
 		{
 			command2 = e2_utf8_unescape (command1+3, ' ');
 			e2_pane_change_dir (NULL, command2);	//command1 is utf-8 or ascii
@@ -2462,11 +2610,11 @@ static gint _e2_command_run_single (gchar *raw, E2_CommandRange range)
 	}
 #endif
 #ifdef E2_NEW_COMMAND
-	result = _e2_command_fork (command1, args, wait, show, run_in_shell);
+	result = _e2_command_fork (command1, args, cwd, wait, show, run_in_shell);
 #else
 	result = (wait) ?
-		_e2_command_run_sync (command1, args, show, run_in_shell):
-		_e2_command_run_async (command1, args, show, run_in_shell);
+		_e2_command_run_sync (command1, args, cwd, show, run_in_shell):
+		_e2_command_run_async (command1, args, cwd, show, run_in_shell);
 #endif
 //	if (free)
 		g_free (command1);
@@ -2515,7 +2663,7 @@ static gboolean _e2_command_send_to_lastchild (gpointer from, E2_ActionRuntime *
 	}
 //	tochild = TRUE;
 	//this is an action, can only return T/F, so we lose the result of the command
-	_e2_command_run_single (command, E2_COMMAND_RANGE_DEFAULT | E2_COMMAND_RANGE_TOCHILD);
+	_e2_command_run_single (command, NULL, E2_COMMAND_RANGE_DEFAULT | E2_COMMAND_RANGE_TOCHILD);
 //	tochild = FALSE;
 	//we don't want any history, just clear the text
 	gtk_entry_set_text (GTK_ENTRY (from), "");
@@ -2543,72 +2691,28 @@ that string temporarily
 The change of CWD will probably not persist for the duration of command execution
 if the command is asynchronous
 @param command utf8 string containing the command(s)
-@param workdir utf8 path string, maybe with trailer, to use as CWD when running @a command
+@param workdir utf8 path string, maybe with trailer, to use as (native) CWD when running @a command, NULL for curr_view->dir
 @param range flags for the scope of the command (may be over-ridden, for actions)
 @return integer return code from last command executed, 0 = success, > 0 on error
 */
-gint e2_command_run_at (gchar *command, gchar *workdir, E2_CommandRange range)
-{
-	e2_filelist_disable_refresh ();	//mimimise CWD changes
-	//FIXME also block cd's if not already blocked
-	//FIXME handle case where CWD is already in use eg refresh in progress
-	//cd_blocked = TRUE;
-#ifdef E2_VFSTMP
-	ensure workdir is not virtual
-#endif
-	if (!e2_fs_chdir (workdir E2_ERR_NONE()))	//run commands only in native dirs, no vfs error check
-	{
-		e2_filelist_enable_refresh ();
-		return 1;
-	}
-#ifdef E2_VFSTMP
-	//FIXME do not do this, dir may be virtual, command may be queued
-#endif
-	g_strlcpy (curr_view->dir, workdir, sizeof (curr_view->dir));
-#ifdef E2_COMMANDQ
-	gint res = e2_command_run (command, range, TRUE);
-#else
-	gint res = e2_command_run (command, range);
-#endif
-	gchar *local = F_FILENAME_TO_LOCALE (workdir);
-	if (e2_fs_access (local, X_OK E2_ERR_NONE()))
-		e2_fs_chdir (curr_pane->path E2_ERR_NONE()); //with an unmount command, this may not exist
-	F_FREE (local);
-#ifdef E2_VFSTMP
-	//FIXME do not do this, dir may be virtual, command may be queued
-#endif
-	g_strlcpy (curr_view->dir, curr_pane->path, sizeof (curr_view->dir));
-	e2_filelist_enable_refresh ();
-	//cd_blocked = FALSE; ??
-	return res;
-}
-#ifndef E2_COMMANDQ
-/**
-@brief run command @a command, after interpreting its contents
-
-@a command may be a single command or a series joined by ';'
-(any ';' inside single- or double-parentheses is ignored)
-NOTE used to return command's pid
-@param command utf string containing the command(s)
-@param range flags for the scope of the command (may be over-ridden, for actions)
-
-@return integer return code from last command executed, 0 = success, > 0 on error
-*/
-gint e2_command_run (gchar *command, E2_CommandRange range)
+gint e2_command_run_at (gchar *command, const gchar *workdir, E2_CommandRange range)
 {
+//E2_VFSTMP workdir probably needs to become a VPATH*
 	//CHECKME should these variables be volatilized for threaded usage ?
 	if ((command == NULL) || (*command == '\0'))
 		return 1;
 	g_strchug (command);	//command arg may be a path with intended trailing whitespace
 	if (*command == '\0')
 		return 1;
+	printd (DEBUG, "e2_command_run_at (command:%s,range:%d)", command, range);
+
+//FIXME umount commands sometimes should get eject appended - but what about aliases ?
 
-	printd (DEBUG, "e2_command_run (command:%s,range:%d)", command, range);
 	//parse & handle joined commands here
 	//in particular, ignore separators inside parentheses
 	if (strchr (command, ';') == NULL)	//if always ascii ;, don't need g_utf8_strchr()
 		//no joined commands
-		return _e2_command_run_single (command, range);
+		return _e2_command_run_single (command, workdir, range);
 	//don't split joined commands if ">>" modifier-prefix is provided
 	gchar *s = command;
 	gboolean run_in_shell = FALSE, done = FALSE;
@@ -2632,7 +2736,7 @@ gint e2_command_run (gchar *command, E2_CommandRange range)
 			break;
 	}
 	if (run_in_shell)
-		return _e2_command_run_single (command, range);
+		return _e2_command_run_single (command, workdir, range);
 
 	gint result = 0;
 	gchar *p;
@@ -2641,15 +2745,30 @@ gint e2_command_run (gchar *command, E2_CommandRange range)
 	{
 		//process this command
 		*p = '\0';
-		result = _e2_command_run_single (s, range);
+		result = _e2_command_run_single (s, workdir, range);
 		s = p; s++;	//resume from the next spot
 		if (*s == '\0') //the ';' was at end of line
 			return result;
 	}
 	//process the last (or only) command
-	return (_e2_command_run_single (s, range));
+	return (_e2_command_run_single (s, workdir, range));
 }
-#endif	//ndef E2_COMMANDQ
+/* *
+NOW a #define
+@brief run command @a command, after interpreting its contents
+
+@a command may be a single command or a series joined by ';'
+(any ';' inside single- or double-parentheses is ignored)
+NOTE used to return command's pid
+@param command utf string containing the command(s)
+@param range flags for the scope of the command (may be over-ridden, for actions)
+
+@return integer return code from last command executed, 0 = success, > 0 on error
+*/
+/*gint e2_command_run (gchar *command, E2_CommandRange range)
+{
+	return (e2_command_run_at (command, NULL, range));
+} */
 /**
 @brief display commands help document
 The document name is hard-coded with the help-file name
@@ -2665,7 +2784,7 @@ void e2_command_output_help (void)
 	{	//we can't find what the user has set for a help command
 		//so use the default instead
 		g_free (command);
-		command = g_strconcat(_A(5),".",_A(94)," ",
+		command = g_strconcat(_A(5),".",_A(103)," ",
 			e2_option_str_get ("usage-help-doc"), " [commands]",NULL);	//file.view_at _I( helpfile titles not (yet?) translated
 	}
 #ifdef E2_COMMANDQ
@@ -2726,16 +2845,16 @@ void e2_command_actions_register (void)
 	gchar *action_name = g_strconcat(_A(7),".",_A(28),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_list_children, NULL, FALSE);
-	action_name = g_strconcat(_A(7),".",_A(50),NULL);
+	action_name = g_strconcat(_A(7),".",_A(51),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_list_history, NULL, FALSE);
-	action_name = g_strconcat(_A(7),".",_A(65),NULL);
+	action_name = g_strconcat(_A(7),".",_A(66),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_list_pending, NULL, FALSE);
-	action_name = g_strconcat(_A(29),".",_A(65),NULL);
+	action_name = g_strconcat(_A(29),".",_A(66),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		e2_command_clear_pending, NULL, FALSE);
-	action_name = g_strconcat(_A(1),".",_A(76),NULL);
+	action_name = g_strconcat(_A(1),".",_A(77),NULL);
 	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_send_to_lastchild, NULL, FALSE,
 		E2_ACTION_EXCLUDE_TOOLBAR | E2_ACTION_EXCLUDE_MENU, NULL);
