@@ -1,6 +1,6 @@
-/* $Id: e2_command_line.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_command_line.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -181,8 +181,7 @@ This function tries to find the corresponding entry widget if
 @a rt != NULL
 Or if @a rt is NULL, tries to find the @a rt corresponding to @a widget.
 But then, if @a entry is not actually an entry widget, @a rt is just set
-to the one for the first-registered command line (as opposed
-to dir line) if any
+to the one for the first-registered command line (as opposed to dir line) if any
 
 @param widget pointer to widget to check/set (may be NULL)
 @param rt pointer to the command line runtime object to check/set, or to NULL
@@ -241,7 +240,8 @@ This is initiated with BGL closed/on
 
 @return TRUE always
 */
-static gboolean _e2_command_line_change_dir_hook (gchar *path,
+//static gboolean
+static void _e2_command_line_change_dir_hook (gchar *path,
 	E2_CommandLineRuntime *rt)
 {
 	GtkWidget *entry = GTK_BIN (rt->cl)->child;
@@ -251,13 +251,13 @@ static gboolean _e2_command_line_change_dir_hook (gchar *path,
 	if (!e2_option_bool_get_direct (rt->opt_history_last))
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
 	if (e2_option_bool_get ("dir-line-pathname-hint"))
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 		gtk_widget_set_tooltip_text (
 #else
 		e2_widget_set_tooltip (NULL,
 #endif
 		entry, path);
-	return TRUE;
+//	return TRUE;
 }
 /**
 @brief mouse button-press callback
@@ -345,10 +345,7 @@ void e2_command_line_cd_activated_cb (GtkWidget *entry, E2_CommandLineRuntime *r
 
 	gchar *dir = e2_utf8_unescape (command_raw, ' ');
 
-#ifdef E2_VFSTMP
-	//FIXME extract any URI and process it
-#endif
-
+//E2_VFSTMPOK no need to handle any URI here
 	E2_PaneRuntime *pane_rt = rt->pane;
 	e2_pane_change_dir (pane_rt, dir);
 
@@ -474,8 +471,8 @@ static gboolean _e2_command_line_insert_action
 	//find out from the action argument if the user wants quotes/escaping
 	g_strstrip (arg);
 	gchar *str_down = g_utf8_strdown (arg, -1);
-	gboolean escape = g_str_equal (arg, _A(98));	//_("escape"
-	gboolean quote = (escape) ? FALSE : g_str_equal (arg, _A(103));	//_("quote"
+	gboolean escape = g_str_equal (arg, _A(107));	//_("escape"
+	gboolean quote = (escape) ? FALSE : g_str_equal (arg, _A(112));	//_("quote"
 	g_free (str_down);
 
 #ifdef E2_REFRESH_DEBUG
@@ -573,9 +570,9 @@ static gboolean _e2_command_line_complete_action
 						flags &= ~E2_COMPLETE_FLAG_PATH;
 						g_strstrip (arg);
 				//		gchar *str_down = g_utf8_strdown (arg, -1);
-						if (strstr (arg, _A(97)) != NULL)
+						if (strstr (arg, _A(106)) != NULL)
 							flags |= E2_COMPLETE_FLAG_DIRS;
-						if (strstr (arg, _A(100)) != NULL)
+						if (strstr (arg, _A(109)) != NULL)
 							flags |= E2_COMPLETE_FLAG_FILES;
 						if (strstr (arg, _("mounts")) != NULL)	//_I("mounts")
 							flags |= E2_COMPLETE_FLAG_MOUNT;
@@ -837,8 +834,9 @@ E2_CommandLineRuntime *e2_command_line_create (gchar *name_default,
 			"button-press-event", G_CALLBACK (_e2_command_line_button_press_cb), rt->pane);
 		//register to change_dir hook for dir-lines so we always know the
 		//latest directory to add it to the history and show it if wanted
-		e2_hook_register (&rt->pane->hook_change_dir,
-			_e2_command_line_change_dir_hook, rt);
+//		gboolean DEBUGhookhere;
+//		e2_hook_register (&rt->pane->hook_change_dir,
+//			_e2_command_line_change_dir_hook, rt);
 		bindingname = g_strconcat (_C(17),".",_C(12),NULL);  //_(general.dir-line"
 	}
 
@@ -847,7 +845,7 @@ E2_CommandLineRuntime *e2_command_line_create (gchar *name_default,
 
 	return rt;
 }
-/**
+/* *
 @brief
 
 @param rt pointer to command line runtime data struct
@@ -887,12 +885,13 @@ void e2_command_line_destroy (E2_CommandLineRuntime *rt)
 	}
 	else */
 	app.command_lines = g_list_remove (app.command_lines, rt);
+/*	gboolean DEBUGhookhere;
 	if (!rt->original)
 	{	//line is a dir line
 		E2_PaneRuntime *pane_rt = rt->pane;
 		e2_hook_unregister (&pane_rt->hook_change_dir,
 			_e2_command_line_change_dir_hook, rt, TRUE);
-	}
+	} */
 
 	_e2_command_line_sync_history (rt);	//CHECKME a cache-unregister sync function?
 
@@ -916,6 +915,27 @@ void e2_command_line_clean (void)
 		DEALLOCATE (E2_CommandLineRuntime, (E2_CommandLineRuntime *)member->data);
 }
 /**
+@brief change-dirline contents function
+This is initiated with BGL closed/on
+@param path utf-8 string, path of the dir that is being opened, with trailer
+@param rt pointer to data struct for the pane being changed
+
+@return
+*/
+void e2_command_line_change_dir (gchar *utfpath, E2_PaneRuntime *rt)
+{
+	GList *member;
+	for (member = app.command_lines; member != NULL; member = member->next)
+	{
+		E2_CommandLineRuntime *cl = (E2_CommandLineRuntime *)member->data;
+		if (cl->pane == rt)
+		{
+			_e2_command_line_change_dir_hook (utfpath, cl);
+			break;
+		}
+	}
+}
+/**
 @brief initialize command line actions
 
 This function initializes the command line actions.
@@ -944,7 +964,7 @@ void e2_command_line_actions_register (void)
 	action_name = g_strconcat(_A(1),".",_A(31),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_line_complete_action, NULL, FALSE);
-	action_name = g_strconcat(_A(1),".",_A(52),NULL);
+	action_name = g_strconcat(_A(1),".",_A(53),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_command_line_insert_action, NULL, FALSE);
 	action_name = g_strconcat(_A(28),".",_A(24),NULL);
