@@ -1,7 +1,7 @@
-/* $Id: e2_cache.c 557 2007-07-24 11:59:11Z tpgww $
+/* $Id: e2_cache.c 851 2008-04-07 10:34:28Z tpgww $
 
 Copyright (C) 2004 Florian Zaehringer (flo.zaehringer@web.de)
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -113,7 +113,7 @@ the cache
 @param size icon size enumerator
 @return pointer to E2_Image struct for the image, or NULL if problem occurred
 */
-E2_Image *e2_cache_image_get (gchar *name, GtkIconSize size)
+E2_Image *e2_cache_image_get (const gchar *name, GtkIconSize size)
 {
 	if (name == NULL)
 		name = GTK_STOCK_MISSING_IMAGE;	//revert to default icon image
@@ -140,7 +140,7 @@ E2_Image *e2_cache_image_get (gchar *name, GtkIconSize size)
 		{	//custom icon file
 			gchar *fullname;
 			if (g_path_is_absolute (name))
-				fullname = name;
+				fullname = (gchar *)name;
 			else
 			{
 				gchar *freeme = e2_utils_get_icons_path (TRUE);
@@ -400,29 +400,34 @@ and convert a list of rowstrings (possibly NULL), to data in that store.
 @a syncfunc must be void syncfunc (gpointer*) and convert store data from
 store *pointer to a GList of rowstrings, then set *pointer to address of that
 list
-
+As for lists, the value of the store pointer may be altered after cacheing, as
+the address of that pointer is remembered and used
 @param name identifier string used in cache file
-@param store addess to save liststore or treestore to hold cached data if any
-@param fillfunc function which creates store and adds cached data if any
-@param syncfunc function which converts store data to a list ready for cacheing
+@param store address to save pointer of liststore (or treestore?) holding any ached data
+@param fillfunc function which creates store and adds any cached data
+@param syncfunc function which converts store data to a list ready for cacheing, or NULL
+@param syncdata data to be provided as second argument to @a syncfunc
 
 @return
 */
-void e2_cache_store_register (gchar *name, gpointer *store, gpointer fillfunc,
-	gpointer syncfunc)
+void e2_cache_store_register (gchar *name, gpointer *store,
+	void (*fillfunc) (gpointer*, GList*), GList*(*syncfunc) (gpointer, gpointer),
+	gpointer syncdata)
 {
-	//put any cache data into a temporary list for conversion
+	//any cached data will be a list. Try to get that
 	GList *rowdata;
 	E2_Cache *cache = e2_cache_list_register (name, &rowdata);
 
-	void (*func) (gpointer*, GList*) = fillfunc;
-	(*func) (store, rowdata);
+//	void (*func) (gpointer*, GList*) = fillfunc;
+//	(*func) (store, rowdata);
+	fillfunc (store, rowdata);
 
-	e2_list_free_with_data (&rowdata);
-	//before writing cache, this will convert liststore data
+	e2_list_free_with_data (&rowdata);	//data stored, this now redundant
+	//adjust parameters to suit subsequent cacheing
+	cache->type = E2_CACHE_TYPE_STORE;
+	cache->data = store;
 	cache->sync_func = syncfunc;
-	cache->sync_data = &(cache->data);	//for syncfunc to use and change
-	cache->data = *store;	//hack re-assignment, for syncfunc to use
+	cache->sync_data = syncdata;
 }
 /**
 @brief get data-list element corresponding to @a name from the cache list
@@ -521,17 +526,26 @@ void e2_cache_array_register (gchar *name, guint size, gint *values, gint *defs)
 		gchar *end = NULL;
 		gchar *str = (gchar *) unknown->data;
 		gchar **split = g_strsplit (str, ",", -1);
-		for (i = 0 ; split[i] != NULL ; i++)
+		if (g_strv_length (split) == size)
 		{
-			*stores = g_ascii_strtoull (split[i], &end, 10);
-			if (end == split[i])
+			for (i = 0 ; split[i] != NULL ; i++)
 			{
-				printd (WARN, "array cache '%s' data conversion failed from '%s", name, split[i]);
-				*stores = defs[i];
+				*stores = g_ascii_strtoull (split[i], &end, 10);
+				if (end == split[i])
+				{
+					printd (WARN, "array cache '%s' data conversion failed from '%s", name, split[i]);
+					*stores = defs[i];
+				}
+				stores++;
 			}
-			stores++;
 		}
-		if (i < size)
+		else
+		{
+			printd (WARN, "array cache '%s' wrong no. of value(s)", name);
+			for (i = 0 ; i < size ; i++)
+				*stores++ = *defs++;
+		}
+/*		if (i < size)
 		{
 			printd (WARN, "array cache '%s' missing value(s)", name);
 			//set all values to default, in case earlier ones are missing
@@ -541,7 +555,7 @@ void e2_cache_array_register (gchar *name, guint size, gint *values, gint *defs)
 			//append missing values
 //			for (; i < size ; i++)
 //				*stores++ = defs[i];
-		}
+		} */
 		g_strfreev (split);
 		g_free (str);
 		unknown_list = g_list_remove_link (unknown_list, found);
@@ -553,6 +567,29 @@ void e2_cache_array_register (gchar *name, guint size, gint *values, gint *defs)
 	cache_list = g_list_append (cache_list, cache);
 	return;
 }
+#ifdef E2_VFS
+/**
+@brief helper func to recursively walk a list/tree store and convert all its
+contents to strings
+CHECKME is there another func that does this already e.g. for config backup
+*/
+static void _e2_cache_store_walk (GtkTreeModel *model, GtkTreeIter *iter, gint level, GList **stringlist)
+{
+	gchar *rowstring;
+	GtkTreeIter child;
+	do
+	{
+#ifdef E2_VFSTMP
+//e2_tree_row_to_string() is currently parked in the vfs plugin
+//		rowstring = e2_tree_row_to_string (model, &iter, -1, level);
+#endif
+		rowstring = g_strdup ("FIXME");
+		*stringlist = g_list_append (*stringlist, rowstring);
+		if (gtk_tree_model_iter_children (model, &child, iter))
+			_e2_cache_store_walk (model, &child, level+1, stringlist);
+	} while (gtk_tree_model_iter_next (model, iter));
+}
+#endif
 /**
 @brief remove an item from the cache list, and park in the unknown list
 All data is copied. Any cleanup should be done by the caller, using a saved
@@ -592,13 +629,38 @@ void e2_cache_unregister (gchar *name)
 			//copy this, in case original cleared
 			cache->data = (gpointer) g_strdup ((gchar *) cache->data);
 			break;
+#ifdef E2_VFS
+//sofar, only cached store is for vfs data
+		case E2_CACHE_TYPE_STORE:
+			if (*(gpointer *)cache->data == NULL)	//make sure store still exists
+				break;
+			GList *stringlist;
+			if (cache->sync_func != NULL)
+			{	//custom conversion to list for backup
+				GList* (*fun) (gpointer, gpointer) = cache->sync_func;
+				stringlist = fun (*(gpointer *)cache->data, cache->sync_data);
+			}
+			else
+			{
+				stringlist = NULL;
+				GtkTreeIter iter;
+				GtkTreeModel *model = GTK_TREE_MODEL ((GtkListStore *)*cache->data);
+				if (gtk_tree_model_get_iter_first (model, &iter))
+					_e2_cache_store_walk (model, &iter, 0, &stringlist);
+			}
+			cache->data = (gpointer) stringlist; //backup records list start, not a pointer address
+			break;
+#endif
 		case E2_CACHE_TYPE_LIST:
 			if (cache->sync_func != NULL)
 			{
 				void (*fun) (gpointer) = cache->sync_func;
 				fun (cache->sync_data);
 			}
-			GList *stringlist = NULL;
+#ifndef E2_VFS
+			GList *
+#endif
+			stringlist = NULL;
 			if ((cache->data != NULL) && (*cache->data != NULL))
 			{
 				GList *node;
@@ -633,19 +695,22 @@ void e2_cache_unregister (gchar *name)
 }
 /**
 @brief write cache file to disk
-File contents are saved in UTF-8 encoding, not localised.
-This expects BGL to be closed for printing any error message.
+File contents are saved in utf-8 encoding, not localised.
+Expects BGL to be on/closed
 @return
 */
 void e2_cache_file_write (void)
 {
+#ifdef E2_VFS
+	VPATH ddata;
+	VPATH tdata;
+#endif
 	gchar *utfpath = g_build_filename
 		(e2_cl_options.config_dir, default_cache_file, NULL);
 	gchar *local = F_FILENAME_TO_LOCALE (utfpath);
 	gchar *tempname = e2_utils_get_tempname (local);
-	gchar *utf = F_FILENAME_FROM_LOCALE (tempname);
 
-	E2_FILE *f = e2_fs_open_file_to_write (utf E2_ERR_NONE());
+	E2_FILE *f = e2_fs_open_writestream (tempname E2_ERR_NONE());
 
 	if (f != NULL)
 	{
@@ -664,8 +729,28 @@ void e2_cache_file_write (void)
 				goto error_handler;
 		if (cache_list != NULL)
 		{
+extern gint real_width;
+extern gint real_height;
 			//these data need special 'syncing' FIXME use array sync process for this
 			e2_fileview_update_col_cachedata ();
+			//do not cache current window size in fullscreen mode
+			if (app.mainwindow_state & GDK_WINDOW_STATE_FULLSCREEN)
+			{
+				member = g_list_find_custom (cache_list, "window-width",
+					(GCompareFunc) _e2_cache_str_compare);
+				if (member != NULL)
+				{
+					cache = (E2_Cache *)member->data;
+					*((gint *)cache->data) = real_width;
+				}
+				member = g_list_find_custom (cache_list, "window-height",
+					(GCompareFunc) _e2_cache_str_compare);
+				if (member != NULL)
+				{
+					cache = (E2_Cache *)member->data;
+					*((gint *)cache->data) = real_height;
+				}
+			}
 
 			for (member = cache_list; member != NULL; member = member->next)
 			{
@@ -698,6 +783,47 @@ void e2_cache_file_write (void)
 						if (e2_fs_file_write (f, "%s=%s\n", cache->name, (gchar *) *cache->data) == 0)
 							goto error_handler;
 						break;
+#ifdef E2_VFS
+//sofar, only cached store is for vfs data
+					case E2_CACHE_TYPE_STORE:
+						if (*(gpointer *)cache->data != NULL)	//make sure store still exists
+						{
+							GList *stringlist;
+							if (cache->sync_func != NULL)
+							{	//custom conversion to list for backup
+								GList* (*fun) (gpointer, gpointer) = cache->sync_func;
+								stringlist = fun (*(gpointer *)cache->data, cache->sync_data);
+							}
+							else
+							{
+								stringlist = NULL;
+								GtkTreeIter iter;
+								GtkTreeModel *model = GTK_TREE_MODEL ((GtkListStore *)*cache->data);
+								if (gtk_tree_model_get_iter_first (model, &iter))
+									_e2_cache_store_walk (model, &iter, 0, &stringlist);
+							}
+							if (stringlist != NULL)
+							{
+								if (e2_fs_file_write (f, "%s=<\n", cache->name) == 0)
+									goto error_handler;
+								GList *node;
+								for (node = stringlist; node != NULL; node = node->next)
+								{
+									//escape anything that would seem like a tree-option start or end
+									if (//*((gchar *)tmp->data) == '<'//ascii check is ok here
+										//||
+										*((gchar *)node->data) == '>')
+											e2_fs_file_write (f, "\\");
+									if (e2_fs_file_write (f, "%s\n", (gchar *) node->data) == 0)
+										goto error_handler;
+								}
+								if (e2_fs_file_write (f, ">\n") == 0)
+									goto error_handler;
+									//CHECKME = leaks when this is done before session-end ??
+							}
+						}
+						break;
+#endif
 					case E2_CACHE_TYPE_LIST:
 						if (cache->sync_func != NULL)
 						{
@@ -770,6 +896,7 @@ void e2_cache_file_write (void)
 							goto error_handler;
 					break;
 				case E2_CACHE_TYPE_LIST:
+				case E2_CACHE_TYPE_STORE:
 					if (e2_fs_file_write (f, "%s=<\n", cache->name) == 0)
 						goto error_handler;
 					GList *node;
@@ -796,9 +923,20 @@ void e2_cache_file_write (void)
 			}
 		}
 		e2_fs_close_stream (f);
+#ifdef E2_VFS
+		tdata.localpath = tempname;
+		tdata.spacedata = NULL;
+		ddata.localpath = local;
+		ddata.spacedata = NULL;
+#endif
 		gdk_threads_leave ();	//downstream error messages invoke local mutex management
+#ifdef E2_VFS
+		e2_task_backend_rename (&tdata, &ddata);
+		e2_fs_chmod (&ddata, 0600 E2_ERR_NONE());
+#else
 		e2_task_backend_rename (tempname, local);
 		e2_fs_chmod (local, 0600 E2_ERR_NONE());
+#endif
 		gdk_threads_enter ();
 		goto cleanup;
 	}
@@ -807,17 +945,20 @@ error_handler:
 	{
 		e2_fs_close_stream (f);
 		gdk_threads_leave ();
+#ifdef E2_VFS
+		e2_task_backend_delete (&tdata);
+#else
 		e2_task_backend_delete (tempname);
+#endif
 		gdk_threads_enter ();
 	}
 	gchar *msg = g_strdup_printf (_("Cannot write cache file %s - %s"),
 		utfpath, g_strerror (errno));	//ok for native-only config file
 	e2_output_print_error (msg, TRUE);
 cleanup:
-	F_FREE (local);
-	F_FREE (utf);
-	g_free (tempname);
 	g_free (utfpath);
+	F_FREE (local);
+	g_free (tempname);
 }
 /**
 @brief interpret a line of the cache file
@@ -900,7 +1041,7 @@ static void _e2_cache_read_from_string (gchar *f[])
 	}
 }
 /**
-@brief read cache file named @a fn from disk
+@brief read cache file named @a fn from disk, and process its contents
 @param fn name of cache file
 @return
 */
@@ -908,35 +1049,51 @@ static void _e2_cache_file_read (gchar *fn)
 {
 	//find absolute path to config file
 	gchar *filepath = g_build_filename (e2_cl_options.config_dir, fn, NULL);
-	gchar *cont;
-	if (! e2_fs_get_file_contents (filepath, &cont E2_ERR_NONE()))
+	gchar *localpath = F_FILENAME_TO_LOCALE (filepath);
+#ifdef E2_VFS
+	VPATH ddata = { localpath, NULL };	//only local cache file
+#endif
+	gpointer contents;
+	//NOTE any error during this read must not print error message, as
+	//config and output data are not yet known
+	if (e2_fs_get_file_contents (
+#ifdef E2_VFS
+		&ddata,
+#else
+		localpath,
+#endif
+		&contents, NULL, TRUE E2_ERR_NONE()))
 	{
-		printd (WARN, "could not open cache file '%s'", filepath);
+		printd (DEBUG, "cache read from file '%s'", filepath);
+		gchar **split = g_strsplit ((gchar *)contents, "\n", -1);
+		_e2_cache_read_from_string (split);
+		g_strfreev (split);
+		g_free (contents);	//need free() if file buffer allocated by malloc()
 	}
 	else
 	{
-		printd (DEBUG, "cache read from file '%s'", filepath);
-		gchar **split = g_strsplit (cont, "\n", -1);
-		_e2_cache_read_from_string (split);
-		g_strfreev (split);
-		g_free (cont);
+		printd (WARN, "could not open cache file '%s'", filepath);
 	}
+
 	g_free (filepath);
+	F_FREE (localpath);
 }
 /**
 @brief initialize the cache
 
 This function will initilize the cache system of e2. It will automatically
 be called at startup.
+@param config_dir_ready TRUE when config dir found or created
 
 @return
 */
-void e2_cache_init (void)
+void e2_cache_init (gboolean config_dir_ready)
 {
 	cache_list = NULL;
 	unknown_list = NULL;
 	default_cache_file = "cache"; //DO NOT TRANSLATE THIS
-	_e2_cache_file_read (default_cache_file);
+	if (config_dir_ready)
+		_e2_cache_file_read (default_cache_file);
 }
 /**
 @brief clean cache memory-allocations
@@ -946,14 +1103,54 @@ void e2_cache_init (void)
 void e2_cache_clean (void)
 {
 	GList *member;
-	for (member = cache_list; member != NULL; member = member->next)
+	if (cache_list != NULL)
 	{
-		g_free (((E2_Cache*) member->data)->name);
-		DEALLOCATE (E2_Cache, (E2_Cache*) member->data);
+		for (member = cache_list; member != NULL; member = member->next)
+		{
+			g_free (((E2_Cache*) member->data)->name);
+			DEALLOCATE (E2_Cache, (E2_Cache*) member->data);
+		}
+		g_list_free (cache_list);
+		cache_list = NULL;
 	}
-	for (member = unknown_list; member != NULL; member = member->next)
+
+	if (unknown_list != NULL)
 	{
-		g_free (((E2_Cache*) member->data)->name);
-		DEALLOCATE (E2_Cache, (E2_Cache*) member->data);
+		for (member = unknown_list; member != NULL; member = member->next)
+		{
+			g_free (((E2_Cache*) member->data)->name);
+			DEALLOCATE (E2_Cache, (E2_Cache*) member->data);
+		}
+		g_list_free (unknown_list);
+		unknown_list = NULL;
+	}
+}
+/**
+@brief clean one item from cache runtime data
+This is used only to cleanup things during a version-upgrade
+@param name the key of the item to clear
+
+@return
+*/
+void e2_cache_clean1 (const gchar *name)
+{
+	GList *found = g_list_find_custom (cache_list, name,
+		(GCompareFunc) _e2_cache_str_compare);
+	if (found != NULL)
+	{
+		g_free (((E2_Cache*) found->data)->name);
+		if (((E2_Cache*) found->data)->type == E2_CACHE_TYPE_STR)
+			g_free (((E2_Cache*) found->data)->data);	//other things will leak
+		DEALLOCATE (E2_Cache, (E2_Cache*) found->data);
+		cache_list = g_list_delete_link (cache_list, found);
+	}
+	found = g_list_find_custom (unknown_list, name,
+			(GCompareFunc) _e2_cache_str_compare);
+	if (found != NULL)
+	{
+		g_free (((E2_Cache*) found->data)->name);
+		g_free (((E2_Cache*) found->data)->data);
+		DEALLOCATE (E2_Cache, (E2_Cache*) found->data);
+		unknown_list = g_list_delete_link (unknown_list, found);
 	}
 }
