@@ -1,4 +1,4 @@
-/* $Id: e2_option.c 557 2007-07-24 11:59:11Z tpgww $
+/* $Id: e2_option.c 641 2007-09-05 03:14:54Z tpgww $
 
 Copyright (C) 2003-2007 tooar <tooar@gmx.net>
 
@@ -63,10 +63,19 @@ gboolean e2_option_set_config_dir (void)
 {
 	//check if the config dir is really there
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };	//only local config data
+	if (!e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 	if (!e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 	{	//if not, try to create it
 		printd (INFO, "creating config directory '%s'", e2_cl_options.config_dir);
+#ifdef E2_VFS
+		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
+#else
 		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
+#endif
 		{
 			printd (WARN, "could not create config directory '%s'",
 				e2_cl_options.config_dir);
@@ -75,7 +84,11 @@ gboolean e2_option_set_config_dir (void)
 		}
 	}
 	//it's there, but can we use it ?
+#ifdef E2_VFS
+	else if (e2_fs_access (&ddata, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#else
 	else if (e2_fs_access (local, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#endif
 	{	//nope
 		printd (WARN, "cannot access config directory '%s'",
 			e2_cl_options.config_dir);
@@ -95,7 +108,7 @@ Creates dir if doesn't exist, or print error message.
 Sets e2_cl_options.trash_dir to the path (a trailing '/'),
 or it stays NULL if dir not available.
 Needs to be called after the config dir is checked/created
-
+Expects BGL off/open
 @return
 */
 void e2_option_set_trash_dir (void)
@@ -123,7 +136,11 @@ void e2_option_set_trash_dir (void)
 				dhome = e2_cl_options.config_dir;
 			else
 			{
-				e2_output_print_error(_("Cannot identify where to put trash. Please set the XDG_DATA_HOME or HOME environment variable."), FALSE);
+				gdk_threads_enter ();
+				e2_output_print_error (
+				_("Cannot identify where to put trash. Please set the XDG_DATA_HOME or HOME environment variable."),
+				FALSE);
+				gdk_threads_leave ();
 				return;
 			}
 		}
@@ -147,19 +164,35 @@ void e2_option_set_trash_dir (void)
 */
 	//make the trash dir if it doesn't exist
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.trash_dir);
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };	//only local config data
+	if (!e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 	if (!e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 	{
 		printd (INFO, "creating trash directory '%s'", e2_cl_options.trash_dir);
+#ifdef E2_VFS
+		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
+#else
 		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
+#endif
 		{
-			gchar *message = g_strdup_printf (_("Cannot create trash directory %s"), e2_cl_options.trash_dir);
-			e2_output_print_error(message, TRUE);
+			gchar *message = g_strdup_printf (_("Cannot create trash directory %s"),
+				e2_cl_options.trash_dir);
+			gdk_threads_enter ();
+			e2_output_print_error (message, TRUE);
+			gdk_threads_leave ();
 			g_free (e2_cl_options.trash_dir);
 			e2_cl_options.trash_dir = NULL;
 		}
 	}
 	//it's there, but can we use it ?
+#ifdef E2_VFS
+	else if (e2_fs_access (&ddata, R_OK | X_OK E2_ERR_NONE()))
+#else
 	else if (e2_fs_access (local, R_OK | X_OK E2_ERR_NONE()))
+#endif
 	{	//nope
 		printd (WARN, "cannot access trash directory '%s'",
 			e2_cl_options.trash_dir);
@@ -189,7 +222,7 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 	gboolean advanced = e2_option_bool_get ("advanced-config");
 	e2_keybinding_clean ();
 	g_hash_table_destroy (app.filetypes);
-	e2_plugins_unload_all ();	//do this before data are cleared
+	e2_plugins_unload_all (FALSE);	//do this before data are cleared
 	//clear relevant current information
 	e2_option_clear_data ();
 	//now re-create things, in the same order as at session-start
@@ -255,7 +288,12 @@ void e2_option_enable_config_checks (void)
 		if (app.monitor_type != E2_MONITOR_DEFAULT)
 			e2_fs_FAM_monitor_config ();
 #endif
-		app.timers[CONFIG_T] = g_timeout_add (E2_CONFIGCHECK_INTERVAL,
+		app.timers[CONFIG_T] =
+#ifdef USE_GLIB2_14
+			g_timeout_add_seconds (E2_CONFIGCHECK_INTERVAL_S,
+#else
+			g_timeout_add (E2_CONFIGCHECK_INTERVAL,
+#endif
 			(GSourceFunc) e2_option_check_config_files, NULL);
 	}
 }
@@ -272,10 +310,10 @@ same stream
 Otherwise, here we check config file mtim.tvsec against a stored
 value.
 If update is indicated, recreate window after re-reading config data
-
+@param user_data UNUSED pointer specified when the timer was initiated
 @return TRUE, always, so timer continues
 */
-gboolean e2_option_check_config_files (void)
+gboolean e2_option_check_config_files (gpointer User_data)
 {
 #ifdef E2_FAM
 	if (app.monitor_type == E2_MONITOR_DEFAULT)
@@ -284,7 +322,12 @@ gboolean e2_option_check_config_files (void)
 		struct stat stat_buf;
 		gchar *filename = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
 		gchar *local = F_FILENAME_TO_LOCALE (filename);
+#ifdef E2_VFS
+		VPATH ddata = { local, NULL }; //config data always local
+		cfgdirty = (!e2_fs_stat (&ddata, &stat_buf E2_ERR_NONE())
+#else
 		cfgdirty = (!e2_fs_stat (local, &stat_buf E2_ERR_NONE())
+#endif
 				&& stat_buf.st_mtime != app.config_mtime);
 		if (cfgdirty && app.rebuild_enabled)
 			app.config_mtime = stat_buf.st_mtime;
@@ -324,12 +367,12 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 	E2_OptionSet *set = e2_option_get_simple (name);
 	if (set == NULL)
 	{
-	#ifdef USE_GLIB2_10
+#ifdef USE_GLIB2_10
 		set = (E2_OptionSet *) g_slice_alloc (sizeof (E2_OptionSet));
 //		set = ALLOCATE (E2_OptionSet);
-	#else
+#else
 		set = ALLOCATE (E2_OptionSet);
-	#endif
+#endif
 		CHECKALLOCATEDWARN (set, )
 		if (set != NULL)
 		{
@@ -537,25 +580,28 @@ static void _e2_option_file_write_unknowns (gpointer key, gpointer value, E2_FIL
 /**
 @brief write current configuration to native file @a utfpath
 The current configuration will be written to the default config file 'config'.
-This expects BGL to be closed for printing any error message.
+Expects BGL to be on/closed
 @param utfpath absolute path of file to write (utf-8 string), or NULL to use default
 
 @return
 */
-void e2_option_file_write (gchar *utfpath)
+void e2_option_file_write (const gchar *utfpath)
 {
-	gboolean freeme = (utfpath == NULL);
-	if (freeme)
-		utfpath = g_build_filename (e2_cl_options.config_dir,
-			default_config_file, NULL);
-	gchar *local = F_FILENAME_TO_LOCALE (utfpath);
+#ifdef E2_VFS
+	VPATH ddata;
+	VPATH tdata;
+#endif
+	gboolean freepath = (utfpath == NULL);
+	gchar *usepath = (freepath) ?
+		g_build_filename (e2_cl_options.config_dir, default_config_file, NULL):
+		(gchar *)utfpath;
+	gchar *local = F_FILENAME_TO_LOCALE (usepath);
 	gchar *tempname = e2_utils_get_tempname (local);
-	gchar *utf = F_FILENAME_FROM_LOCALE (tempname);
 
-	E2_FILE *f = e2_fs_open_file_to_write (utf E2_ERR_NONE());
+	E2_FILE *f = e2_fs_open_writestream (tempname E2_ERR_NONE());
 	if (f != NULL)
 	{
-		printd (DEBUG, "write config file: %s", utfpath);
+		printd (DEBUG, "write config file: %s", usepath);
 		if (e2_fs_file_write (f,
 			//the 1st line is language-independent, for version verification
 			"# "PROGNAME" (v "VERSION RELEASE")\n\n" ) == 0)
@@ -626,9 +672,20 @@ void e2_option_file_write (gchar *utfpath)
 			goto error_handler;
 
 		e2_fs_close_stream (f);
+#ifdef E2_VFS
+		tdata.localpath = tempname;
+		tdata.spacedata = NULL;
+		ddata.localpath = local;
+		ddata.spacedata = NULL;
+#endif
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifdef E2_VFS
+		e2_task_backend_rename (&tdata, &ddata);
+		e2_fs_chmod (&ddata, 0600 E2_ERR_NONE());
+#else
 		e2_task_backend_rename (tempname, local);
 		e2_fs_chmod (local, 0600 E2_ERR_NONE());
+#endif
 		gdk_threads_enter ();
 		goto cleanup;
 	}
@@ -637,19 +694,22 @@ error_handler:
 	{
 		e2_fs_close_stream (f);
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifdef E2_VFS
+		e2_task_backend_delete (&tdata);
+#else
 		e2_task_backend_delete (tempname);
+#endif
 		gdk_threads_enter ();
 	}
 	gchar *msg = g_strdup_printf (_("Cannot write config file %s - %s"),
-		utfpath, g_strerror (errno));	//ok for native-only config file
+		usepath, g_strerror (errno));	//ok for native-only config file
 	e2_output_print_error (msg, TRUE);
 	sleep (1);
 cleanup:
+	if (freepath)
+		g_free (usepath);
 	F_FREE (local);
-	F_FREE (utf);
 	g_free (tempname);
-	if (freeme)
-		g_free (utfpath);
 }
 /**
 @brief Set value of "single-valued" option named @a option
@@ -790,18 +850,28 @@ If the file is read successfully, log the post-read config dir timestamp
 
 @return TRUE if the file was read
 */
-static gboolean _e2_option_config_file_read (gchar **contents)
+static gboolean _e2_option_config_file_read (gpointer *contents)
 {
 	//find absolute path to config file
 	gchar *filepath = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
 	gchar *local = F_FILENAME_TO_LOCALE (filepath);
+#ifdef E2_VFS
+	VPATH ddata = { local, NULL };	//only local config data
+	if (e2_fs_access (&ddata, R_OK E2_ERR_NONE()))
+#else
 	if (e2_fs_access (local, R_OK E2_ERR_NONE()))
+#endif
 	{	//try for the old-style name
 		g_free (filepath);
 		F_FREE (local);
 		filepath = g_build_filename (e2_cl_options.config_dir, "config", NULL);
 		local = F_FILENAME_TO_LOCALE (filepath);
+#ifdef E2_VFS
+		ddata.localpath = local;
+		if (e2_fs_access (&ddata, R_OK E2_ERR_NONE()))
+#else
 		if (e2_fs_access (local, R_OK E2_ERR_NONE()))
+#endif
 		{
 			g_free (filepath);
 			F_FREE (local);
@@ -810,13 +880,21 @@ static gboolean _e2_option_config_file_read (gchar **contents)
 	}
 
 	gboolean retval;
-	//get file contents
-	if (e2_fs_get_file_contents (filepath, contents E2_ERR_NONE()))
+	//get file
+#ifdef E2_VFS
+	if (e2_fs_get_file_contents (&ddata, contents, NULL, TRUE E2_ERR_NONE()))
+#else
+	if (e2_fs_get_file_contents (local, contents, NULL, TRUE E2_ERR_NONE()))
+#endif
 	{
 //#ifndef E2_FAM do this anyway ...
 		//log the current timestamp
 		struct stat stat_buf;
+#ifdef E2_VFS
+		e2_fs_stat (&ddata, &stat_buf E2_ERR_NONE());
+#else
 		e2_fs_stat (local, &stat_buf E2_ERR_NONE());
+#endif
 		app.config_mtime = stat_buf.st_mtime;
 //#endif
 		printd (DEBUG, "config file '%s' read", filepath);
@@ -842,12 +920,12 @@ they are initialized with a default one.
 */
 gboolean e2_option_file_read (void)
 {
-	gchar *contents;
+	gpointer contents;
 	if (_e2_option_config_file_read (&contents))
 	{
 		gboolean retval;
 		//everything goes into an array
-		gchar **split = g_strsplit (contents, "\n", -1);
+		gchar **split = g_strsplit ((gchar *)contents, "\n", -1);
 		if (split[0] != NULL)
 		{	//there is something to work with, so parse it
 			//get the config file version (always ascii) from 1st line of file
@@ -867,7 +945,7 @@ gboolean e2_option_file_read (void)
 			retval = FALSE;
 
 		g_strfreev (split);
-		g_free (contents);
+		g_free (contents);	//need free() if file buffer allocated by malloc()
 		return retval;
 	}
 	else
