@@ -1,4 +1,4 @@
-/* $Id: e2_option__default.c 489 2007-07-12 01:40:25Z tpgww $
+/* $Id: e2_option__default.c 706 2007-11-06 09:13:06Z tpgww $
 
 Copyright (C) 2003-2007 tooar <tooar@gmx.net>
 
@@ -111,13 +111,11 @@ void e2_option_default_register (void)
 
 	group_name = g_strconcat(_C(20) ,":",_C(25),NULL);  //_("interface:miscellaneous"
 #ifdef E2_COMPOSIT
-	UNCOMMENT THIS
-/*	//window opacity set to (1 - this-value/100), so this-value = 0 means fully opaque
-	e2_option_int_register ("window-bleed", group_name, _("translucence"),
-		_("Window translucence, 0 (opaque) to 50 (faint)"),
-		NULL, 0, 0, 50,
+//	UNCOMMENT THIS
+	e2_option_int_register ("window-opacity", group_name, _("opacity"),
+		_("Window translucence, 30 (faint) to 100 (opaque)"),
+		NULL, 100, 30, 100,
 		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDALL);
-*/
 #endif
 	e2_option_bool_register ("button2-updir", group_name, _("'go up' on middle-button click"),
 		_("This is a faster alternative to double clicking '..' or clicking the 'go up' button"), NULL, TRUE,
@@ -284,6 +282,16 @@ void e2_option_default_register (void)
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDLISTS);
 
 	group_name = g_strconcat(_C(32),".",_C(2),":",_C(25),NULL); //_("panes.colors:miscellaneous"
+#ifdef E2_ASSISTED
+	e2_option_bool_register ("color-background-set", group_name, _("custom background color"),
+		_("If enabled, the color specified below will be used for background"),
+		NULL, FALSE,
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
+	e2_option_color_register ("color-background", group_name, _("background color"),
+		_("Background color used instead of theme color"),
+		"color-background-set", "#2E3436",
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
+#endif
 	e2_option_color_register ("color-active-pane", group_name, _("active pane header color"),
 		_("This color is used for the background of column headers in the active pane"), NULL, "dark grey",
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_BUILDPANES);
