@@ -1,6 +1,6 @@
-/* $Id: e2_option_color.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_option_color.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_OPTION_COLOR_H
-#define E2_OPTION_COLOR_H
+#ifndef __E2_OPTION_COLOR_H__
+#define __E2_OPTION_COLOR_H__
 
 #include "e2_option.h"
 #include "e2_option_color.h"
@@ -33,4 +33,4 @@ GdkColor *e2_option_color_get (gchar *name);
 gboolean e2_option_color_set_str (gchar *name, gchar *value);
 gboolean e2_option_color_set_str_direct (E2_OptionSet *set, gchar *value);
 
-#endif //ndef E2_OPTION_COLOR_H
+#endif //ndef __E2_OPTION_COLOR_H__
