@@ -1,6 +1,6 @@
-/* $Id: e2_option_str.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_option_str.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_OPTION_STR_H
-#define E2_OPTION_STR_H
+#ifndef __E2_OPTION_STR_H__
+#define __E2_OPTION_STR_H__
 
 #include "emelfm2.h"
 #include "e2_option.h"
@@ -32,4 +32,4 @@ gchar *e2_option_str_get (gchar *option);
 gchar *e2_option_str_get_direct (E2_OptionSet *set);
 void e2_option_str_set_direct (E2_OptionSet *set, gchar *value);
 
-#endif //ndef E2_OPTION_STR_H
+#endif //ndef __E2_OPTION_STR_H__
