@@ -1,6 +1,6 @@
-/* $Id: e2_option_unknown.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_option_unknown.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,12 +18,12 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_OPTION_UNKNOWN_H
-#define E2_OPTION_UNKNOWN_H
+#ifndef __E2_OPTION_UNKNOWN_H__
+#define __E2_OPTION_UNKNOWN_H__
 
 #include "emelfm2.h"
 
 void e2_option_unknown_record (gchar *option, gchar *value);
 void e2_option_transient_value_get (E2_OptionSet *set);
 
-#endif // ndef E2_OPTION_UNKNOWN_H
+#endif // ndef __E2_OPTION_UNKNOWN_H__
