@@ -1,6 +1,6 @@
-/* $Id: e2_about_dialog.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_about_dialog.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,9 +18,9 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_ABOUT_DIALOG_H
-#define E2_ABOUT_DIALOG_H
+#ifndef __E2_ABOUT_DIALOG_H__
+#define __E2_ABOUT_DIALOG_H__
 
 void e2_about_dialog_actions_register (void);
 
-#endif //ndef E2_ABOUT_DIALOG_H
+#endif //ndef __E2_ABOUT_DIALOG_H__
