@@ -1,6 +1,6 @@
-/* $Id: e2_config_dialog.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_config_dialog.c 876 2008-05-08 12:55:21Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -29,6 +29,10 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_plugins.h"
 #include "e2_filetype.h"
 
+//define to apply scrolledwindow to each page of the config notebeok, allowing
+//omission of bars for whole book, which cause nested bars for tree-set data
+#define NEWSCROLL
+
 typedef struct _E2_TreeCfgData
 {
 	E2_OptionSet *set;
@@ -212,6 +216,22 @@ E2_SpecificConfDialogRuntime *e2_config_dialog_single (gchar *set_name,
  /***** callbacks *****/
 /*********************/
 
+
+/**
+@brief callback for options hash cleanup
+Size-group opjects in the hash need to be unreffed so that they will be cleared
+when all widgets in the group are destroyed
+@param data pointer to data item to be cleaned
+
+@return
+*/
+static void _e2_hashfree (gpointer data)
+{
+	if (GTK_IS_SIZE_GROUP (data))
+		g_object_unref (data);
+}
+
+#ifndef NEWSCROLL
 /**
 @brief callback for show dialog
 
@@ -225,6 +245,7 @@ static void _e2_confdlg_show_cb (GtkWidget *dialog, GtkWidget *sw)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 }
+#endif
 /**
 @brief callback for show dialog
 
@@ -549,29 +570,45 @@ static gboolean _e2_confdlg_ok_cb (gboolean apply)
 		e2_fs_touch_config_dir ();
 		e2_option_enable_config_checks ();
 		//may need to show the revised config file data
-		//FIXME this may cause crash
-//		if (!(buildflags & (E2_OPTION_FLAG_BUILDALL | E2_OPTION_FLAG_BUILDPANES)))
-//			e2_filelist_check_dirty (GINT_TO_POINTER(1));
+/*	FIXME this may cause crash
+		if (!(buildflags & (E2_OPTION_FLAG_BUILDALL | E2_OPTION_FLAG_BUILDPANES)))
+		{
+#ifdef E2_FAM
+			e2_filelist_request_refresh (curr_view->dir, FALSE);
+			e2_filelist_request_refresh (other_view->dir, TRUE);
+#else
+			e2_filelist_check_dirty (GINT_TO_POINTER(1));
+#endif
+		}
+*/
 	}
 
 	//set filelist monitoring according to current option
 	if (e2_option_bool_get ("auto-refresh"))
 	{
 		if (app.timers[DIRTYCHECK_T] == 0)
-			app.timers[DIRTYCHECK_T] = g_timeout_add (E2_FILESCHECK_INTERVAL,
-				(GSourceFunc) e2_filelist_check_dirty, NULL);
+		{
+			e2_filelist_start_refresh_polling ();
+#ifdef E2_FAM
+			//e2_fs_FAM_connect ();
+			//CHECKME what things to start monitoring ?
+			//e2_fs_FAM_change (gchar *olddir, E2_PaneRuntime *rt);
+#endif
+		}
 	}
 	else if (app.timers[DIRTYCHECK_T] != 0)
 	{
-#ifdef E2_FAM
-	//FIXME don't want FAM reports to pile up due to lack of polling
-#endif
 		g_source_remove (app.timers[DIRTYCHECK_T]);
 		app.timers[DIRTYCHECK_T] = 0;
+#ifdef E2_FAM
+		//FIXME don't want FAM reports to pile up due to lack of polling
+		//e2_fs_FAM_cancel_monitor_dir (gchar *path)
+		//e2_fs_FAM_clean_reports (gchar *path)
+#endif
 	}
-	e2_filelist_enable_refresh ();	//clear settings changed at start of func
 	//after config file update, set config monitoring according to current option
 	e2_option_enable_config_checks ();
+	e2_filelist_enable_refresh ();	//clear settings changed at start of func
 
 	return TRUE;
 }
@@ -862,7 +899,7 @@ void e2_confdlg_choose_plugin_cb (GtkWidget *button, E2_OptionSet *set)
 		if (*ppath == '\0')
 		{
 			g_free (ppath);
-			ppath = g_strconcat (PLUGINS_DIR G_DIR_SEPARATOR_S, pname, NULL);	//mixed local & utf
+			ppath = g_strconcat (PLUGINS_DIR G_DIR_SEPARATOR_S, pname, NULL);	//mixed ascii & utf
 		}
 		else
 		{
@@ -974,7 +1011,7 @@ static void _e2_confdlg_font_select_cb (GtkWidget *button, E2_OptionSet *set)
 		gtk_font_selection_dialog_set_font_name (dialog,
 			gtk_entry_get_text (GTK_ENTRY (set->widget)));
 #ifdef E2_COMPOSIT
-		e2_window_set_translucency (GTK_WIDGET (dialog), DIALOG_TRANS_LEVEL);	//constant opacity for dialogs
+		e2_window_set_opacity (GTK_WIDGET (dialog), DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
 		g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
 			G_CALLBACK (_e2_confdlg_font_select_ok_cb), set->widget);
@@ -1125,7 +1162,7 @@ void e2_confdlg_extcolorpick_cb (GtkWidget *button, E2_OptionSet *set)
 	gtk_color_selection_set_current_color (GTK_COLOR_SELECTION
 		(GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel), &color);
 #ifdef E2_COMPOSIT
-	e2_window_set_translucency (dialog, DIALOG_TRANS_LEVEL);	//constant opacity for dialogs
+	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
 	e2_dialog_add_defined_button (dialog, &E2_BUTTON_APPLY);
 
@@ -1258,7 +1295,7 @@ static gboolean _e2_confdlg_category_selected_cb
 	return FALSE;
 }
 /**
-@brief setup an entry widget for a dialog page
+@brief setup an entry widget for a set on a dialog page
 
 @param box box widget to hold the entry
 @param set pointer to set data
@@ -1268,7 +1305,7 @@ static gboolean _e2_confdlg_category_selected_cb
 static void _e2_confdlg_set_entry (GtkWidget *box, E2_OptionSet *set)
 {
 	set->widget = e2_widget_add_entry (box, set->sval, TRUE, FALSE);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1286,12 +1323,17 @@ Expects page_last_name to be non-NULL
 (even if wrong, when changing to basic config)
 @param parent_iter pointer to iter in categories treeview, can be NULL for a new category
 @param label_text name of the category
+@param with_sw TRUE to put the data into a scrolled window
 @param rt pointer to data struct for the dialog
 
 @return a vbox widget
 */
 static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
-	gchar *label_text, E2_ConfigDialogRuntime *rt)
+	gchar *label_text,
+#ifdef NEWSCROLL
+	gboolean with_sw,
+#endif
+	E2_ConfigDialogRuntime *rt)
 {
 	//add page to the treeview model
 	GtkTreeIter child_iter;
@@ -1311,8 +1353,28 @@ static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
 		rt->openpage = page_num;
 	}
 
-	//page vbox containing the headline and config widgets
-	GtkWidget *vbox = gtk_vbox_new (FALSE, E2_PADDING);
+	GtkWidget *vbox;
+#ifdef NEWSCROLL
+	if (with_sw)
+	{
+		GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC,
+			GTK_POLICY_AUTOMATIC, GTK_SHADOW_NONE);
+		vbox = gtk_vbox_new (FALSE, E2_PADDING);
+		e2_widget_sw_add_with_viewport (sw, vbox);
+		gtk_widget_show_all (sw);
+		gtk_notebook_append_page (rt->notebook, sw, NULL);
+	}
+	else
+	{
+#endif
+		//page vbox containing the headline and config widgets
+		vbox = gtk_vbox_new (FALSE, E2_PADDING);
+		gtk_widget_show (vbox);
+		gtk_notebook_append_page (rt->notebook, vbox, NULL);
+#ifdef NEWSCROLL
+	}
+#endif
+
 	g_object_set_data_full (G_OBJECT (vbox), "iter", gtk_tree_iter_copy (&child_iter),
 		(void *) gtk_tree_iter_free);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), E2_PADDING);
@@ -1324,33 +1386,31 @@ static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
 	g_free (label_text2);
 	gtk_misc_set_padding (GTK_MISC (label), E2_PADDING_XSMALL, E2_PADDING_XSMALL);
 	gtk_container_add (GTK_CONTAINER (frame), label);
-	gtk_notebook_append_page (rt->notebook, vbox, NULL);
 
 	return vbox;
 }
 /**
 @brief get notebook page for option group @a group, after creating it if necessary
-Recursive if @a group is a child (name includes '.')
-or is a grandchild (name includes ':')
+Recursive if @a group is a child (name includes '.') or is a grandchild (name includes ':')
+(and in those cases, it alters the set->group temporarily)
 This also sets page_last_name, if it was NULL
-@param group option group-name string
+@param set pointer to set data struct
 @param rt pointer to data struct for the dialog
 
-@return a box widget into which option data can be packed
+@return a vbox widget into which option data can be packed
 */
-static GtkWidget *_e2_confdlg_get_page (gchar *group, E2_ConfigDialogRuntime *rt)
+static GtkWidget *_e2_confdlg_get_page (E2_OptionSet *set, E2_ConfigDialogRuntime *rt)
 {
 	GtkWidget *box;
-	if ((box = (GtkWidget *) g_hash_table_lookup (rt->opthash, group)) == NULL)
+	if ((box = (GtkWidget *) g_hash_table_lookup (rt->opthash, set->group)) == NULL)
 	{
-		group = g_strdup (group);
-		gchar *frame = strrchr (group, ':');	//always ascii :, don't need g_utf8_strrchr()
+		gchar *frame = strrchr (set->group, ':');	//always ascii :, don't need g_utf8_strrchr()
 		if (frame != NULL)
 		{
 			*frame = '\0';
 			GtkWidget *frame_box;
-			if ((frame_box = (GtkWidget *) g_hash_table_lookup (rt->opthash, group)) == NULL)
-				frame_box = _e2_confdlg_get_page (group, rt);
+			if ((frame_box = (GtkWidget *) g_hash_table_lookup (rt->opthash, set->group)) == NULL)
+				frame_box = _e2_confdlg_get_page (set, rt); //recurse with altered set->group
 			gchar *frame_title = e2_utils_str_stretch (frame + 1);
 			GtkWidget *frame_widget = gtk_frame_new (frame_title);
 			g_free (frame_title);
@@ -1363,29 +1423,37 @@ static GtkWidget *_e2_confdlg_get_page (gchar *group, E2_ConfigDialogRuntime *rt
 		}
 		else
 		{
-			gchar *child = strrchr (group, '.');	//always ascii '.', don't need g_utf8_strrchr()
+			gchar *child = strrchr (set->group, '.');	//always ascii '.', don't need g_utf8_strrchr()
 			if (child != NULL)
 			{
 				*child = '\0';
-				box = _e2_confdlg_get_page (group, rt);
+				box = _e2_confdlg_get_page (set, rt); //recurse with altered set->group
 				if (page_last_name == NULL)
 				//we didn't have a page-name from last time or earlier this time
-					page_last_name = g_strdup (group);
+					page_last_name = g_strdup (set->group);
 				GtkTreeIter *iter2;
 				iter2 = g_object_get_data (G_OBJECT (box), "iter");
 				if (iter2)
-					box = _e2_confdlg_add_page (iter2, child + 1, rt);
+					box = _e2_confdlg_add_page (iter2, child + 1,
+#ifdef NEWSCROLL
+						set->type != E2_OPTION_TYPE_TREE,
+#endif
+						rt);
 				*child = '.';
 			}
-			else	//this is a group without a child
+			else	//this is a group without a descendant
 			{
 				if (page_last_name == NULL)
 				//we didn't have a page-name from last time or earlier this time
-					page_last_name = g_strdup (group);
-				box = _e2_confdlg_add_page (NULL, group, rt);
+					page_last_name = g_strdup (set->group);
+				box = _e2_confdlg_add_page (NULL, set->group,
+#ifdef NEWSCROLL
+					set->type != E2_OPTION_TYPE_TREE,
+#endif
+					rt);
 			}
 		}
-		g_hash_table_insert (rt->opthash, group, box);
+		g_hash_table_insert (rt->opthash, g_strdup(set->group), box);
 	}
 	return box;
 }
@@ -1456,13 +1524,15 @@ void e2_config_dialog_create (gchar *page)
 	rt->notebook = GTK_NOTEBOOK (gtk_notebook_new ());
 	gtk_notebook_set_show_tabs (rt->notebook, FALSE);
 	gtk_notebook_set_show_border (rt->notebook, FALSE);
+#ifdef NEWSCROLL
+	//in this case, each page has its own sw
+	gtk_paned_pack2 (GTK_PANED (hpane), GTK_WIDGET(rt->notebook), TRUE, TRUE);
+	gtk_widget_set_size_request (GTK_WIDGET(rt->notebook), 10, 10);
+#else
 	sw = e2_widget_get_sw_plain (GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw),
-		GTK_WIDGET (rt->notebook));
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (sw)->child),
-		GTK_SHADOW_NONE);
+	e2_widget_sw_add_with_viewport (sw, GTK_WIDGET (rt->notebook));
 	gtk_paned_pack2 (GTK_PANED (hpane), sw, TRUE, TRUE);
-
+#endif
 	//note which page we want to open at
 	if (page != NULL && *page != '\0')
 	{
@@ -1478,36 +1548,42 @@ void e2_config_dialog_create (gchar *page)
 	E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_ADVANCED :
 	E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BASICONLY ;
 
-	rt->opthash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	rt->opthash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
+		(GDestroyNotify)_e2_hashfree);
 	guint i;
 	gpointer *walker;
 	E2_OptionSet *set;
 	for (i = 0, walker = options_array->pdata; i < options_array->len; i++, walker++)
 	{
+		GtkSizeGroup *size_group;
 		set = *walker;
 		if (!(set->flags & includemask))
 		{
 			set->widget = NULL;	//make sure that this is ignored when updating after 'ok' or 'apply'
 			continue;
 		}
-		//make sure there's a size group for the page
-		gchar *size_group_key = g_strconcat (set->group, "_sizegroup", NULL);	//no translation
-		GtkSizeGroup *size_group = (GtkSizeGroup *) g_hash_table_lookup (rt->opthash, size_group_key);
-		if (GTK_IS_SIZE_GROUP (size_group))
-			g_free (size_group_key);
-		else
+		if (set->type != E2_OPTION_TYPE_TREE)
 		{
-			size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-			g_hash_table_insert (rt->opthash, size_group_key, size_group);
+			//make sure there's a size group for the page
+			gchar *size_group_key = g_strconcat (set->group, "_sizegroup", NULL);	//no translation
+			size_group = (GtkSizeGroup *) g_hash_table_lookup (rt->opthash, size_group_key);
+			if (GTK_IS_SIZE_GROUP (size_group))
+				g_free (size_group_key);
+			else
+			{
+				size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+				g_hash_table_insert (rt->opthash, size_group_key, size_group);
+			}
 		}
+		else
+			size_group = NULL;	//warning prevention only
+
 		//get page for this set
 		//and if necessary, create categories view item(s) and
 		//notebook page(s) for parent group and any child(ren)
-		GtkWidget *box = _e2_confdlg_get_page (set->group, rt);
-
+		GtkWidget *box = _e2_confdlg_get_page (set, rt);
 		//FIXME create only the the startup page now, rest after display
-
-		GtkWidget *hbox, *label;
+		GtkWidget *hbox, *label;	//, *button;
 		gchar *label_text;
 		switch (set->type)
 		{
@@ -1515,17 +1591,20 @@ void e2_config_dialog_create (gchar *page)
 				e2_option_tree_add_widget (config_dialog, box, set);
 				break;
 			case E2_OPTION_TYPE_BOOL:
-				e2_option_bool_add_dialog_widget (config_dialog, box, size_group, set);
+				//button =
+				e2_widget_add_tied_check_button (box, set, config_dialog);
+				//CHECKME size_group needed ?
+				//gtk_size_group_add_widget (size_group, button);
 				break;
 			case E2_OPTION_TYPE_STR:
 				hbox = gtk_hbox_new (FALSE, E2_PADDING);
 				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, E2_PADDING);
 				label = e2_widget_add_mid_label (hbox, set->desc, 0.0, FALSE, 0);
 				gtk_size_group_add_widget (size_group, label);
-//				set->widget = e2_widget_add_entry (hbox, set->sval, TRUE, FALSE);
-//				gtk_widget_set_tooltip_text (set->widget, set->tip);
-		//FIXME enable <Esc> press during editing to abort the change, not the whole dialog
 				_e2_confdlg_set_entry (hbox, set);
+#ifdef E2_ASSISTED
+				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+#endif
 				break;
 			case E2_OPTION_TYPE_INT:
 				e2_option_int_add_widget (config_dialog, box, size_group, set);
@@ -1543,10 +1622,10 @@ void e2_config_dialog_create (gchar *page)
 				g_free (label_text);
 				gtk_size_group_add_widget (size_group, label);
 				//font entry for font name
-//				set->widget = e2_widget_add_entry (hbox, NULL, TRUE, FALSE);
-//				gtk_widget_set_tooltip_text (set->widget, set->tip);
-		//FIXME enable <Esc> press during editing to abort the change, not the whole dialog
 				_e2_confdlg_set_entry (hbox, set);
+#ifdef E2_ASSISTED
+				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+#endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for font select dialog
 				e2_button_add (hbox, FALSE, 0, _("change"), GTK_STOCK_SELECT_FONT,
@@ -1570,10 +1649,10 @@ void e2_config_dialog_create (gchar *page)
 				g_free (label_text);
 				gtk_size_group_add_widget (size_group, label);
 				//color entry for color names/color hex values
-//				set->widget = e2_widget_add_entry (hbox, NULL, TRUE, FALSE);
-//				gtk_widget_set_tooltip_text (set->widget, set->tip);
-		//FIXME enable <Esc> press during editing to abort the change, not the whole dialog
 				_e2_confdlg_set_entry (hbox, set);
+#ifdef E2_ASSISTED
+				e2_widget_set_label_relations (GTK_LABEL (label), set->widget);
+#endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for color select dialog
 				e2_button_add (hbox, FALSE, 0, _("change"), GTK_STOCK_SELECT_COLOR,
@@ -1593,8 +1672,8 @@ void e2_config_dialog_create (gchar *page)
 		}
 	}
 	/*a set's dependency (if any) may be 'forward' in the array, in which
-	case the relevant widget not known when the set is processed, so
-	need to setup dependencies after all options are setup i.e. iterate again ...
+	case the relevant widget not known when the set was processed in the previous
+	walk, so need to setup dependencies after all options are setup i.e. iterate again ...
 	FIXME in pass 1, log the items with un-resolved dependencies, then
 	just do those here */
 	for (i = 0, walker = options_array->pdata; i < options_array->len; i++, walker++)
@@ -1652,9 +1731,13 @@ void e2_config_dialog_create (gchar *page)
 		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (catsview), path, NULL, FALSE, 0, 0);
 		gtk_tree_path_free (path);
 	}
-//	gtk_window_set_default_size -- does not work ...
+	//if no cache data from last usage, we don't want an unpleasant size
+	if (app.cfgdlg_width < 10)
+		app.cfgdlg_width = 400;
+	if (app.cfgdlg_height < 10)
+		app.cfgdlg_height = 300;
 	gtk_window_resize (GTK_WINDOW(config_dialog), app.cfgdlg_width,
-		app.cfgdlg_height);	//revert to last size if not too small (initially -1,-1 = default)
+		app.cfgdlg_height);	//revert to last size if not too small
 
 	//receptacle for cut/copied row(s) data generated from context menu
 	//the keys are constant strings, not to be replaced or freed
@@ -1676,9 +1759,14 @@ void e2_config_dialog_create (gchar *page)
 	//(can't get it to work, if set sooner than in the 'show' cb ...)
 	g_signal_connect (G_OBJECT (config_dialog), "show",
 		G_CALLBACK (_e2_confdlg_show_cb2), rt);
+//#ifdef NEWSCROLL
+//	g_signal_connect (G_OBJECT (config_dialog), "show",
+//		G_CALLBACK (_e2_confdlg_show_cb), rt->notebook);
+//#else
+#ifndef NEWSCROLL
 	g_signal_connect (G_OBJECT (config_dialog), "show",
 		G_CALLBACK (_e2_confdlg_show_cb), sw);
-
+#endif
 	e2_dialog_add_undefined_button_custom (config_dialog, FALSE,
 		E2_RESPONSE_USER1, _("_Default"), GTK_STOCK_CLEAR,
 		_("Revert all options to their default settings"),
