@@ -1,6 +1,6 @@
-/* $Id: e2_dialog.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_dialog.c 838 2008-03-23 22:25:52Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -61,57 +61,46 @@ static void _e2_dialog_resize_destroy_cb (GtkWidget *dialog, gpointer data)
 	}
 }
 /**
-@brief show scrollbars in @a dialog if appropriate
-@param dialog the widget to be updated
-@return FALSE always, to cancel the timer
-*/
-/*static gboolean _e2_dialog_resize_idle2_cb (GtkWidget *dialog)
-{
-	printd (DEBUG, "dialog idle-time scrollbars cb");
-	GtkScrolledWindow *scrolled = g_object_get_data (G_OBJECT (dialog),
-		"e2-dialog-resize-scrolled");
-	gdk_threads_enter ();	//protect idle fn against thread glitches
-	WAIT_FOR_EVENTS;
-	gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC,
-		GTK_POLICY_AUTOMATIC);
-	gdk_threads_leave ();
-	//record 0 idle id, we're done now
-	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id", NULL);
-	return FALSE;
-} */
-/**
-@brief idle callback to (larger only) resize @a dialog
+@brief idle callback to resize @a dialog
 @param dialog the widget to be resized
-@return FALSE always, to remove the source
+@return FALSE always, to remove the idle source
 */
 static gboolean _e2_dialog_resize_idle_cb (GtkWidget *dialog)
 {
-	if ((dialog->allocation.width < dialog->requisition.width) ||
-		(dialog->allocation.height < dialog->requisition.height))
+	if ((dialog->allocation.width != dialog->requisition.width) ||
+		(dialog->allocation.height != dialog->requisition.height))
 	{
+//		GtkPolicyType barpolicy = GTK_POLICY_NEVER;
 		GtkScrolledWindow *scrolled = g_object_get_data (G_OBJECT (dialog),
 			"e2-dialog-resize-scrolled");
-//		gdk_threads_enter ();	//protect idle fn against thread glitches
+		gdk_threads_enter ();
+		if (scrolled != NULL)
+		{
+			//turn any scrollbar(s) off, so the resize works as intended
+			//assume both bars have same policy
+//			barpolicy = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (scrolled), "hscrollbar-policy"));
+			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+				GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+			WAIT_FOR_EVENTS
+		}
 		gtk_window_resize (GTK_WINDOW (dialog),
 			dialog->requisition.width, dialog->requisition.height);
-		//turn bars back on (was in separate function)
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-//		gdk_threads_leave ();
-		//record idle id for 2nd part of the resize process
-/*		guint app.timers[IDLE_T] = g_timeout_add (750,
-			(GSourceFunc) _e2_dialog_resize_idle2_cb, dialog);
-		g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id",
-			GUINT_TO_POINTER (app.timers[IDLE_T]));
-*/
-	} //else
-//		_e2_dialog_resize_idle2_cb (dialog);
+		WAIT_FOR_EVENTS
+		//turn bars back on
+		if (scrolled != NULL)	//&& barpolicy != GTK_POLICY_NEVER)
+		{
+			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);	//barpolicy, barpolicy);
+		}
+		gdk_threads_leave ();
+	}
 	//record 0 idle id, we're done now
 	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id", NULL);
 	return FALSE;
 }
 /**
-@brief turn on automatic scrollbars for @a scrolled
+@brief "show" callback for a dialog with scrolled window
+Turns on automatic scrollbars for @a scrolled
 @param dialog UNUSED the dialog widget containing @a scrolled
 @param scrolled scrolled window to be updated
 @return
@@ -122,6 +111,27 @@ void e2_dialog_show_cb (GtkWidget *dialog, GtkScrolledWindow *scrolled)
 		GTK_POLICY_AUTOMATIC);
 }
 /**
+@brief "show" callback for a dialog with notebook
+Turns on automatic scrollbars for each page of @a book
+@param dialog UNUSED the dialog widget containing @a book
+@param book notebook to be updated
+@return
+*/
+void e2_dialog_show_notebook_cb (GtkWidget *dialog, GtkNotebook *book)
+{
+	gint i, pagecount = gtk_notebook_get_n_pages (book);
+	for (i = 0; i < pagecount; i++)
+	{
+		GtkWidget *page;
+		GtkScrolledWindow *sw;
+		page = gtk_notebook_get_nth_page (book, i);
+		sw = (GtkScrolledWindow *)g_object_get_data (G_OBJECT (page),
+			"e2-tab-scrolled-window");
+		if (sw != NULL)
+			gtk_scrolled_window_set_policy (sw, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	}
+}
+/**
 @brief dialog button-press callback, to pop up a context menu
 @param dialog the widget which received the mouse button press
 @param event pointer to event data struct
@@ -279,16 +289,19 @@ gboolean e2_dialog_key_pos_cb (GtkWidget *widget, GdkEventKey *event, gpointer d
 */
 void e2_dialog_setup (GtkWidget *dialog, GtkWidget *parent)
 {
+	//some funcs here go down to xlib etc CHECKME problem if re-entrant ?
+//	CLOSEBGL_IF_OPEN
 	GtkWindow *thiswindow = GTK_WINDOW (dialog);
-	gtk_window_set_transient_for (thiswindow, GTK_WINDOW (parent));
+	if (parent != NULL)
+		gtk_window_set_transient_for (thiswindow, GTK_WINDOW (parent));
 	gtk_window_set_destroy_with_parent (thiswindow, TRUE);
 	gtk_window_set_wmclass (thiswindow,
 		gtk_window_get_title (thiswindow), PROGNAME);
 	gtk_window_set_role (thiswindow, "dialog");
-	gtk_window_set_position (thiswindow,
-		e2_option_int_get ("dialog-position"));
+	gtk_window_set_position (thiswindow, e2_option_int_get ("dialog-position"));
 	gtk_window_set_resizable (thiswindow, TRUE);  //moved to here from ... show
 //	gtk_widget_set_size_request (dialog, 0, 0);	//KILLS LATESHOW  etc in this spot (at least) !!
+//	OPENBGL_IF_CLOSED
 }
 /**
 @brief setup and run dialog
@@ -373,12 +386,19 @@ label must have been escaped before calling here
 @param data pointer to data to be provided to @a callback
 @return the dialog widget
 */
-GtkWidget *e2_dialog_create (gchar *stock, gchar *label_text, gchar *title,
-	void *callback, void *data)
+GtkWidget *e2_dialog_create (const gchar *stock, const gchar *label_text,
+	const gchar *title, gpointer callback, gpointer data)
 {
 	GtkWidget *dialog = gtk_dialog_new ();
-	if (title != NULL)
-	    E2_DIALOG_TITLE (GTK_WINDOW (dialog), title);
+
+	//BGL may be open or closed here
+	CLOSEBGL_IF_OPEN
+
+	e2_window_set_title (dialog, title);
+
+	gtk_widget_add_events (dialog, GDK_ALL_EVENTS_MASK);	//uses xlib?
+
+	OPENBGL_IF_CLOSED
 
 	GtkWidget *img = (stock != NULL ) ?
 		e2_widget_get_icon (stock, GTK_ICON_SIZE_DIALOG) : NULL;
@@ -411,10 +431,10 @@ GtkWidget *e2_dialog_create (gchar *stock, gchar *label_text, gchar *title,
 	}
 
 #ifdef E2_COMPOSIT
-	e2_window_set_translucency (dialog, DIALOG_TRANS_LEVEL);	//constant opacity for dialogs
+	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
 
-	gtk_widget_add_events (dialog, GDK_ALL_EVENTS_MASK);
+//see above	gtk_widget_add_events (dialog, GDK_ALL_EVENTS_MASK);
 
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (dialog),
@@ -439,18 +459,24 @@ GtkWidget *e2_dialog_create (gchar *stock, gchar *label_text, gchar *title,
 */
 GtkWidget *e2_dialog_add_sw (GtkWidget *dialog)
 {
+	//start without scrollbars, so that the dialog is first displayed with the
+	//size of the things packed into the sw
 	GtkWidget *scrolled = e2_widget_add_sw (GTK_DIALOG (dialog)->vbox,
 		GTK_POLICY_NEVER, GTK_POLICY_NEVER, TRUE, 0);
+	//arrange for automatic scrollbars when the dialog is shown, after sizing
+	g_signal_connect (G_OBJECT (GTK_DIALOG (dialog)), "show",
+		G_CALLBACK (e2_dialog_show_cb), scrolled);
 	GtkWidget *vbox = e2_widget_get_box (TRUE, FALSE, 0);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), vbox);
 	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (scrolled)->child),
 		GTK_SHADOW_NONE);
-	g_signal_connect (G_OBJECT (GTK_DIALOG (dialog)), "show",
-		G_CALLBACK (e2_dialog_show_cb), scrolled);
 	GtkWidget *hbox = g_object_get_data (G_OBJECT (dialog), "e2-dialog-hbox");
-	gtk_widget_reparent (hbox, vbox);
-	gtk_box_set_child_packing (GTK_BOX (vbox), hbox, FALSE, FALSE, 0,
-		GTK_PACK_START);
+	if (hbox != NULL)
+	{
+		gtk_widget_reparent (hbox, vbox);
+		gtk_box_set_child_packing (GTK_BOX (vbox), hbox, FALSE, FALSE, 0,
+			GTK_PACK_START);
+	}
 	gtk_widget_show (vbox);
 	return vbox;
 }
@@ -643,10 +669,9 @@ void e2_dialog_setup_chooser (GtkWidget *dialog, const gchar *title,
 	gtk_box_pack_start (GTK_BOX(vbox), wchooser, TRUE, TRUE, 0);
 	e2_dialog_setup (dialog, parent);
 */
-	if (title != NULL)
-		E2_DIALOG_TITLE (GTK_WINDOW (dialog), title);
+	e2_window_set_title (dialog, title);
 #ifdef E2_COMPOSIT
-	e2_window_set_translucency (dialog, DIALOG_TRANS_LEVEL);	//constant opacity for dialogs
+	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
 
 /*	//hide the dialog's standard 'open' button
@@ -801,7 +826,7 @@ static DialogButtons _e2_dialog_line_input (gchar* window_title, gchar *prompt,
 	prompt);
 #endif
 	gchar *title = (window_title != NULL) ? window_title : _("user input");
-	E2_DIALOG_TITLE (GTK_WINDOW (dialog), title);
+	e2_dialog_set_title (dialog, title);
 */
 	gchar *title = (window_title != NULL) ? window_title : _("user input");
 	DialogButtons retval;
@@ -834,6 +859,12 @@ static DialogButtons _e2_dialog_line_input (gchar* window_title, gchar *prompt,
 			suggestion, FALSE, select_text);
 #endif
 
+#ifdef E2_ASSISTED
+	GtkWidget *label = (GtkWidget *) g_object_get_data (G_OBJECT (dialog),
+		"e2-dialog-label");
+	e2_widget_set_label_relations (GTK_LABEL (label), dialogentry);
+#endif
+
 	//handle Return-key presses when the entry is focused
 	g_signal_connect (G_OBJECT (dialogentry), "activate",
 		G_CALLBACK (_e2_dialog_activated_cb), &retval);
@@ -875,6 +906,10 @@ static DialogButtons _e2_dialog_line_input (gchar* window_title, gchar *prompt,
 	GtkLabel *label = GTK_LABEL (GTK_DIALOG (dialog)->label);
 	gtk_label_set_text (label, prompt);
 	gtk_label_set_use_markup (label, TRUE);
+#ifdef E2_ASSISTED
+	e2_widget_set_label_relations (label, dialogentry);
+#endif
+
 //  & add default/suggestion now
 	if (history)
 	{
@@ -920,6 +955,11 @@ Assumes that @a dlocal (at least) exists
 DialogButtons e2_dialog_ow_check (gchar *slocal, gchar *dlocal, OW_ButtonFlags extras)
 {
 //	printd (DEBUG, "overwrite check");
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
+	struct stat sb;
+
 	GString *dialog_prompt = g_string_sized_new (NAME_MAX + 64);	//pick a reasonable initial size
 	gchar *utf = F_DISPLAYNAME_FROM_LOCALE (dlocal);
 	//before applying bold markup, escape any pango-annoying component of the item name
@@ -927,14 +967,30 @@ DialogButtons e2_dialog_ow_check (gchar *slocal, gchar *dlocal, OW_ButtonFlags e
 	gchar *type = NULL;
 	if (slocal != NULL)
 	{
-		struct stat sb;
 //		E2_ERR_DECLARE
 
-		if (!e2_fs_stat (slocal, &sb E2_ERR_NONE()))
+#ifdef E2_VFS
+		ddata.localpath = slocal;
+#ifdef E2_VFSTMP
+		//FIXME spacedata
+#endif
+		ddata.spacedata = NULL;
+		if (!e2_fs_stat (&ddata, &sb E2_ERR_NONE()))
+#else
+		if (!e2_fs_stat (slocal, &sb E2_ERR_NONE()))	//should never fail
+#endif
 		{
 			gboolean sdir = S_ISDIR (sb.st_mode);
 			time_t stime = sb.st_mtime;
+#ifdef E2_VFS
+			ddata.localpath = dlocal;
+#ifdef E2_VFSTMP
+		//FIXME spacedata
+#endif
+			if (e2_fs_stat (&ddata, &sb E2_ERR_NONE()))
+#else
 			if (e2_fs_stat (dlocal, &sb E2_ERR_NONE()))
+#endif
 			{
 				//FIXME handle error properly - maybe nothing to overwrite now
 				//E2_ERR_CLEAR
@@ -955,9 +1011,20 @@ DialogButtons e2_dialog_ow_check (gchar *slocal, gchar *dlocal, OW_ButtonFlags e
 	if (type == NULL)
 		type = _("existing");
 
-	if (e2_fs_is_dir3 (dlocal E2_ERR_NONE()))
-		g_string_printf (dialog_prompt, _("Remove all contents of %s\n<b>%s</b> ?"),
-			type, public);
+#ifdef E2_VFS
+	ddata.localpath = dlocal;
+# ifdef E2_VFSTMP
+		//FIXME spacedata
+# endif
+	ddata.spacedata = NULL;
+	if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE())
+#else
+	if (!e2_fs_lstat (dlocal, &sb E2_ERR_NONE())	//should never fail
+#endif
+		&& S_ISDIR (sb.st_mode)
+		&& !S_ISLNK (sb.st_mode))
+			g_string_printf (dialog_prompt,
+				_("Remove all contents of %s\n<b>%s</b> ?"), type, public);
 	else
 	{
 		gchar *dbase = g_path_get_basename (public);
@@ -1116,16 +1183,11 @@ void e2_dialog_resize (GtkWidget *dialog, gfloat factor)
 /**
 @brief setup to resize @a dialog at an idle time
 @param dialog the dialog widget to be resized
-@param scrolled a scrolled window widget in @a dialog
+@param scrolled a scrolled window widget in @a dialog, or NULL
 @return
 */
 void e2_dialog_resize_with_sw (GtkWidget *dialog, GtkWidget *scrolled)
 {
-	//must turn bars off now, for the resize to work
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-		GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-scrolled",
-		scrolled);
 	guint idle_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (dialog),
 			"e2-dialog-resize-idle-id"));
 	if (idle_id != 0)
@@ -1133,6 +1195,8 @@ void e2_dialog_resize_with_sw (GtkWidget *dialog, GtkWidget *scrolled)
 	idle_id = g_idle_add ((GSourceFunc) _e2_dialog_resize_idle_cb, dialog);
 	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id",
 		GUINT_TO_POINTER (idle_id));
+	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-scrolled",
+		scrolled);
 	g_signal_connect (G_OBJECT (dialog), "destroy",
 		G_CALLBACK (_e2_dialog_resize_destroy_cb), NULL);
 }
