@@ -1,6 +1,6 @@
-/* $Id: e2_edit_dialog.c 557 2007-07-24 11:59:11Z tpgww $
+/* $Id: e2_edit_dialog.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2006-2007 tooar <tooar@gmx.net>
+Copyright (C) 2006-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -89,12 +89,22 @@ static void _e2_edit_dialog_cancel_spell (E2_ViewDialogRuntime *rt)
 @brief save text selected in @a buffer
 This func is also used to save output pane selection
 @param buffer text buffer from which the text will be saved
-@param parent parent widget
+#ifdef E2_VFS
+@param spacedata pointer to namspace data
+#endif
+@param parent parent widget, dialog or main window
 
 @return
 */
-void e2_edit_dialog_save_selected (GtkTextBuffer *buffer, GtkWidget *parent)
+void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
+#ifdef E2_VFS
+	PlaceInfo *spacedata,
+#endif
+	GtkWidget *parent)
 {
+#ifdef E2_VFSTMP
+	check assumption that selector only works for current namespace
+#endif
 	GtkTextIter start, end;
 	//get this first, as the dialog unselects the text
 	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
@@ -121,10 +131,17 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer, GtkWidget *parent)
 		if (response == GTK_RESPONSE_OK)
 		{
 			gchar *local = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+#ifdef E2_VFS
+			VPATH ddata = { local, spacedata };
+#endif
 #ifndef USE_GTK2_8
 			//check for o/w
 			if (e2_option_bool_get ("confirm-overwrite")
+#ifdef E2_VFS
+				&& e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
 				&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
+#endif
 			{
 				DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
 				if (choice != OK)
@@ -136,20 +153,15 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer, GtkWidget *parent)
 				}
 			}
 #endif
-			gchar *utf = F_FILENAME_FROM_LOCALE (local);
-			E2_FILE *file = e2_fs_open_file_to_write (utf E2_ERR_NONE());
-			if (file != NULL)
-			{
-				gchar *text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-				e2_fs_file_put (utf, text, file E2_ERR_NONE());
-#ifdef E2_VFSTMP
-	make this work with non-local files - no file streaming ?
+			gchar *text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+			e2_fs_set_file_contents (
+#ifdef E2_VFS
+			&ddata, text, strlen (text), 0644 E2_ERR_NONE());
+#else
+			local, text, strlen (text), 0644 E2_ERR_NONE());
 #endif
-				e2_fs_close_stream (file);
-				g_free (text);
-			}
+			g_free (text);
 			g_free (local);
-			F_FREE (utf);
 		}
 		gtk_widget_destroy (dialog);
 		gtk_text_buffer_select_range (buffer, &start, &end);
@@ -166,7 +178,11 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer, GtkWidget *parent)
 static void _e2_edit_dialog_savesel_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
-	e2_edit_dialog_save_selected (rt->textbuffer, rt->dialog);
+	e2_edit_dialog_save_selected (rt->textbuffer,
+#ifdef E2_VFS
+	rt->spacedata,
+#endif
+	rt->dialog);
 }
 /**
 @brief save file
@@ -179,7 +195,6 @@ static void _e2_edit_dialog_savesel_cb (GtkWidget *menuitem,
 static void _e2_edit_dialog_save_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
-	rt->saved_ok = FALSE;
 	GtkTextIter start, end;
 	gtk_text_buffer_get_bounds (rt->textbuffer, &start, &end);
 	gchar *text = gtk_text_buffer_get_text (rt->textbuffer, &start, &end, FALSE);
@@ -199,100 +214,110 @@ static void _e2_edit_dialog_save_cb (GtkWidget *menuitem,
 	{
 		gsize bytes_old, bytes_new;
 		GError *error = NULL;
-		text = g_convert (text, -1, rt->charset, "UTF-8", &bytes_old, &bytes_new, &error);
-		if (error != NULL)
+		gchar *newtext;
+		newtext = g_convert (text, -1, rt->charset, "UTF-8", &bytes_old, &bytes_new, &error);
+		if (error == NULL)
+		{
+			g_free (text);
+			text = newtext;
+		}
+		else
 		{
 			gchar *msg;
+			if (newtext != NULL)
+				g_free (newtext);
 			switch (error->code)
 			{
 				case G_CONVERT_ERROR_ILLEGAL_SEQUENCE:
 					msg = g_strdup_printf (
 					_("Cannot save %s in its original encoding %s. Reverting to UTF-8"),
-					rt->filepath, rt->charset);
+					rt->localpath, rt->charset);
 					e2_output_print_error (msg, TRUE);
 					g_error_free (error);
 					break;
 				default:
 					msg = g_strdup_printf (
-					_("Encoding conversion failed for %s"), rt->filepath);
+					_("Encoding conversion failed for %s"), rt->localpath);
 					e2_output_print_error (msg, TRUE);
 					e2_output_print_error (error->message, FALSE);
 					g_error_free (error);
+					rt->saved_ok = FALSE;
 					return;
-					break;
 			}
 		}
 	}
 
-	gchar *local = F_FILENAME_TO_LOCALE (rt->filepath);
-	struct stat sb;
-	gboolean renamed = FALSE;
-#ifdef E2_VFSTMP
-	E2_ERR_DECLARE
+	const gchar *usepath;
+	//rt->localpath may be a replacement name, when "saving as"
+	usepath = (rt->saveas) ? rt->newlocalpath : rt->localpath;
+
+#ifdef E2_VFS
+	VPATH ddata = { usepath, rt->spacedata };
 #endif
-	if (e2_option_bool_get ("edit-save-backup") &&
-#ifdef E2_VFSTMP
-	//FIXME handle vfs error
-		!e2_fs_access (local, W_OK E2_ERR_PTR()))
+	if (e2_option_bool_get ("edit-save-backup")
+		&& !rt->saveas
+		&& !e2_fs_access (
+#ifdef E2_VFS
+		&ddata, W_OK E2_ERR_NONE()))
 #else
-		!e2_fs_access (local, W_OK E2_ERR_NONE()))
+		usepath, W_OK E2_ERR_NONE()))
 #endif
 	{
-		renamed = !e2_fs_stat (local, &sb E2_ERR_NONE());
-		gchar *saved = e2_utils_strcat (local, "~");
+		gchar *saved = e2_utils_strcat (usepath, "~");
+#ifdef E2_VFS
+		VPATH sdata = { saved, rt->spacedata };
+#endif
 		gdk_threads_leave ();	//downstream errors invoke local mutex locking
-		e2_task_backend_rename (local, saved);	//ignore any warning !!
+#ifdef E2_VFS
+		e2_task_backend_rename (&ddata, &sdata);	//ignore any warning !!
+#else
+		e2_task_backend_rename (usepath, saved);	//ignore any warning !!
+#endif
 		gdk_threads_enter ();
 		g_free (saved);
 	}
-#ifdef E2_VFSTMP
-	E2_ERR_CLEAR
+
+	//allow new file permissions to be the same as an old one, if any
+	struct stat sb;
+	gboolean replaced = !rt->saveas &&
+#ifdef E2_VFS
+		!e2_fs_stat (&ddata, &sb E2_ERR_NONE());	//through a link
+#else
+		!e2_fs_stat (usepath, &sb);
 #endif
 
-	E2_FILE *file = e2_fs_open_file_to_write (rt->filepath E2_ERR_NONE());
-	if (file == NULL)
-	{
-		g_free (text);
-		F_FREE (local);
-		return;
-		//FIXME undo backup-rename
-	}
-	if (e2_fs_file_put (rt->filepath, text, file E2_ERR_NONE()) == EOF)
-	{
-		//FIXME warn user
-		g_free (text);
-		F_FREE (local);
-#ifdef E2_VFSTMP
-		make this work with remote file - no streaming ?
+	rt->saved_ok = e2_fs_set_file_contents (
+#ifdef E2_VFS
+		&ddata, text, strlen (text), 0600 E2_ERR_NONE());
+#else
+		usepath, text, strlen (text), 0600);
 #endif
-		e2_fs_close_stream (file);
-		return;
-		//FIXME undo backup-rename
-	}
-	e2_fs_close_stream (file);	//E2_ERR_NONE());
-
-	if (renamed)
-	{
-		//make new file the same as the old one
-//	if (
-		e2_fs_chmod (local, sb.st_mode E2_ERR_NONE());
-//	) handle error
-//		uid_t owner_id = ?;
-//		gid_t group_id = ?;
-//		e2_fs_chown (local, owner_id, group_id);
-	}
 
 	g_free (text);
-	F_FREE (local);
-
-	if (g_str_equal (rt->charset, "UNKNOWN"))
+	if (rt->saved_ok)
 	{
-		gchar *msg = g_strdup_printf
-		(_("Original encoding of '%s' is unknown, now saved as UTF-8"), rt->filepath);
-		e2_output_print_error (msg, TRUE);
+		mode_t perms = (replaced) ? sb.st_mode & ALLPERMS : 0644;
+#ifdef E2_VFS
+		e2_fs_chmod (&ddata, perms E2_ERR_NONE());
+#else
+		e2_fs_chmod (usepath, perms);
+#endif
+	//	uid_t owner_id = ?;
+	//	gid_t group_id = ?;
+	//	e2_fs_chown (local, owner_id, group_id);
+
+		if (g_str_equal (rt->charset, "UNKNOWN"))
+		{
+			e2_fs_error_simple (
+			_("Original encoding of '%s' is unknown, now saved as UTF-8"),
+#ifdef E2_VFS
+			&ddata);
+#else
+			usepath);
+#endif
+		}
+		gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (rt->textbuffer), FALSE);
 	}
-	rt->saved_ok = TRUE;
-	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (rt->textbuffer), FALSE);
 }
 /**
 @brief save file with new name
@@ -305,19 +330,24 @@ static void _e2_edit_dialog_save_cb (GtkWidget *menuitem,
 static void _e2_edit_dialog_saveas_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+#ifdef E2_VFSTMP
+	check assumption that selector only works for current namespace
+#endif
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 		NULL);
 
+	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
 	e2_dialog_setup_chooser (dialog,
 		_("file save as"),
-		rt->filepath,
+		utf,
 		GTK_FILE_CHOOSER_ACTION_SAVE,
 		FALSE, 	//hide-hidden
 		FALSE,	//single-selection
 		GTK_RESPONSE_OK);	//default response
+	F_FREE (utf);
 
 	gint response;
 	while ((response = gtk_dialog_run (GTK_DIALOG (dialog))) == E2_RESPONSE_USER1)
@@ -328,8 +358,15 @@ static void _e2_edit_dialog_saveas_cb (GtkWidget *menuitem,
 		gchar *local = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 #ifndef USE_GTK2_8
 		//check for O/W if not done by the dialog
+#ifdef E2_VFS
+		VPATH ddata = { local, rt->spacedata };
+#endif
 		if (e2_option_bool_get ("confirm-overwrite")
+#ifdef E2_VFS
+			&& e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+#else
 			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
+#endif
 		{
 			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
 			if (choice != OK)
@@ -340,16 +377,20 @@ static void _e2_edit_dialog_saveas_cb (GtkWidget *menuitem,
 			}
 		}
 #endif
-		g_free (rt->filepath);
-		rt->filepath = F_FILENAME_FROM_LOCALE (local);
-		if (rt->filepath != local)
-			g_free (local);
+		rt->saveas = TRUE;
+		rt->newlocalpath = local;
 		_e2_edit_dialog_save_cb (NULL, rt);
+		rt->saveas = FALSE;
 
 		if (rt->saved_ok)
-		{	//update the dialog label
+		{
+			g_free (rt->localpath);
+			rt->localpath = local;
+			//update the dialog label
 			GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog), "e2-dialog-label");
-			gtk_label_set_text (GTK_LABEL (label), rt->filepath);
+			gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
+			gtk_label_set_text (GTK_LABEL (label), utf);
+			F_FREE (utf);
 		}
 	}
 	gtk_widget_destroy (dialog);
@@ -371,12 +412,16 @@ static void _e2_edit_dialog_refresh_cb (GtkWidget *menuitem,
 	_("Reverting to saved version cannot be undone"));
 	if (choice == OK)
 	{
-		//current rt->filepath is replaced by allocated string, in read text func ???
-		gchar *freeme = rt->filepath;
+#ifdef E2_VFS
+		VPATH ddata = { rt->localpath, rt->spacedata };
+#endif
 		gdk_threads_leave ();
-		e2_view_dialog_read_text (rt);	//probably won't fail when refreshing
+#ifdef E2_VFS
+		e2_view_dialog_read_text (&ddata, rt);	//no failure test when refreshing
+#else
+		e2_view_dialog_read_text (rt->localpath, rt);	//no failure test when refreshing
+#endif
 		gdk_threads_enter ();
-		g_free (freeme);
 		gtk_text_buffer_set_modified (rt->textbuffer, FALSE);
 	}
 	gtk_widget_grab_focus (rt->textview);
@@ -711,7 +756,7 @@ static void _e2_edit_dialog_replace_cb (GtkWidget *menuitem,
 	}
 }
 /**
-@brief construct and show output-pane context menu
+@brief construct and show edit-dialog context menu
 
 @param textview the textview widget where the click happened
 @param event_button which mouse button was clicked (0 for a menu key)
@@ -826,7 +871,7 @@ static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
 	item =
 	e2_menu_add_check (menu, _("_Wrap"), rt->textwrap,
 		 _e2_edit_dialog_wrap_cb, rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1277,8 +1322,8 @@ static void _e2_edit_dialog_keybindings (E2_OptionSet *set)
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 	g_strconcat(_C(11),"||||",NULL),  //_("dialogs"
 	g_strconcat("\t",_A(39),"||||",NULL),  //_("edit"
-	g_strconcat("\t\t|<Control>u","||",_A(108),".",_A(109),"|<Control>Home",NULL),
-	g_strconcat("\t\t|<Control>n","||",_A(108),".",_A(109),"|<Control>End",NULL),
+	g_strconcat("\t\t|<Control>u","||",_A(119),".",_A(120),"|<Control>Home",NULL),
+	g_strconcat("\t\t|<Control>n","||",_A(119),".",_A(120),"|<Control>End",NULL),
 	g_strdup(">"),
 	NULL);
 }
@@ -1289,17 +1334,17 @@ static void _e2_edit_dialog_keybindings (E2_OptionSet *set)
 /**********************/
 /**
 @brief create and show edit dialog
-@a filename may be full path, or at least the name, of item to be edited.
-If @a filename is not an absolute path, curr_view->dir is prepended
+@a localpath may be full path, or at least the name, of item to be edited.
+If @a localpath is not an absolute path, curr_view->dir is prepended
 If @a srt is non-NULL, and there's a problem during setup, @a srt is set to NULL
 Expects BGL closed when we arrive here
-@param filename utf8 string, or NULL if editing a buffer
+@param localpath localised string, or NULL if editing a buffer
 @param buf pointer to output pane tab textbuffer, or NULL id editing a file
 @param srt pointer to store for returned dialog data, or NULL
 
 @return
 */
-static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_ViewDialogRuntime **srt)
+static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_ViewDialogRuntime **srt)
 {
 	//init view runtime object (0's to ensure view-dialog things are NULL)
 	E2_ViewDialogRuntime *rt = ALLOCATE0 (E2_ViewDialogRuntime);
@@ -1312,14 +1357,13 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 	}
 
 	gint char_width, char_height;
-	gboolean fileedit = (filename != NULL);
+	gboolean fileedit = (localpath != NULL);
 	if (fileedit)
 	{	//edit file
 		rt->textview = gtk_text_view_new ();
-		rt->filepath = filename;	//interim store, for the file-read func
 		// add the item's content
 		gdk_threads_leave ();	//any downstream error message does its own mutex management
-		if (!e2_view_dialog_read_text (rt))
+		if (!e2_view_dialog_read_text (localpath, rt))
 		{
 			gdk_threads_enter ();
 			gtk_widget_destroy (rt->textview);
@@ -1335,11 +1379,9 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 	{	//edit output pane buffer
 		//CHECKME any benefit from a flag to distinguish output edtis ?
 		const gchar *home = g_get_home_dir ();
-		gchar *utf = F_FILENAME_FROM_LOCALE ((gchar *)home);
 		gchar *name = g_strconcat (_C(27), "-", _A(39), NULL);	//_("output-edit")
-		rt->filepath = g_build_filename (utf, name, NULL);
+		rt->localpath = g_build_filename (home, name, NULL);
 		rt->charset = "UTF-8";	//the text is from our buffer
-		F_FREE (utf);
 		g_free (name);
 		//textview defaults to editable and cursor visible
 		rt->textview = gtk_text_view_new_with_buffer (buf);
@@ -1370,8 +1412,10 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 //	rt->ui_tmp = NULL;
 //	rt->seq_reserve = FALSE;	//default value for ui->seq ??
 
-	rt->dialog = e2_dialog_create (NULL, rt->filepath, _("editing file"),
+	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
+	rt->dialog = e2_dialog_create (NULL, utf, _("editing file"),
 		_e2_edit_dialog_response_cb, rt);
+	F_FREE (utf);
 	gtk_window_set_type_hint (GTK_WINDOW (rt->dialog),
 		    GDK_WINDOW_TYPE_HINT_NORMAL);
 	//override some default label properties
@@ -1398,7 +1442,7 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (rt->textview),
 			mark, 0.2, TRUE, 0.0, 0.5);
 	}
-	rt->idle_id = 0;
+//	rt->idle_id = 0;
 	//init search runtime data
 	rt->history = e2_list_copy_with_data (find_history);	//CHECKME why copy these ?
 	rt->rephistory = e2_list_copy_with_data (replace_history);
@@ -1439,7 +1483,7 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 	GtkWidget *entry = GTK_BIN (rt->combo2)->child;
 	g_signal_connect_after (G_OBJECT (entry), "key-release-event",
 		G_CALLBACK (_e2_edit_dialog_combokey_cb), rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1472,7 +1516,7 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 	labeltext = _("_Replace");
 	rt->replacebtn =
 	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_CONVERT, labeltext, E2_RESPONSE_USER1);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1483,7 +1527,7 @@ static void _e2_edit_dialog_create (gchar *filename, GtkTextBuffer *buf, E2_View
 	labeltext = _("_Find");
 	rt->findbtn =
 	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1575,21 +1619,31 @@ static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
 	//".." entries filtered when names compiled
 	//for name comparisons, we need the full path
 	gchar *local = e2_utils_strcat (qed->currdir, (*iterator)->filename);
-	gchar *edit_this = D_FILENAME_FROM_LOCALE (local);
-	g_free (local);
+#ifdef E2_VFS
+	VPATH ddata = { local, qed->currspace };
+#endif
 	E2_ViewDialogRuntime *vrt = NULL;
 	gdk_threads_enter ();
-	_e2_edit_dialog_create (edit_this, NULL, &vrt);
+#ifdef E2_VFS
+	_e2_edit_dialog_create (&ddata, NULL, &vrt);
+#else
+	_e2_edit_dialog_create (local, NULL, &vrt);
+#endif
 	gdk_threads_leave ();
 	if (vrt != NULL)
 	{
 		gdk_threads_enter ();
-		e2_view_dialog_show_atlast (edit_this, vrt);
+		e2_view_dialog_show_atlast
+#ifdef E2_VFS
+			(&ddata, vrt);
+#else
+			(local, vrt);
+#endif
 		gdk_threads_leave ();
-		g_free (edit_this);
+		g_free (local);
 		return TRUE;
 	}
-	g_free (edit_this);
+	g_free (local);
 	return FALSE;
 }
 
@@ -1603,20 +1657,21 @@ static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
 In those cases, curr_view->dir will be prepended
 This returns immediately, so if run in a Q-thread, that will end
 Expects BGL closed
-@param file utf8 string which has at least the name of item to be processed, or NULL
+@param localpath localised string which has at least the name of item to be processed, or NULL
 @param buf pointer to a GtkTextBuffer to edit an output pane tab, or NULL
 @return TRUE if the dialog was created
 */
-gboolean e2_edit_dialog_create (gchar *file, GtkTextBuffer *buf)
+gboolean e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf)
 {
 	//there is no async thread protection here, as this func can be called from
 	//different contexts - any protection is done by the caller
 	E2_ViewDialogRuntime *vrt;
-	if (file != NULL || buf != NULL)
+	if (localpath != NULL || buf != NULL)
 	{
-		_e2_edit_dialog_create (file, buf, &vrt);
+		_e2_edit_dialog_create (localpath, buf, &vrt);
 		if (vrt != NULL)
 		{
+			WAIT_FOR_EVENTS_SLOWLY
 			e2_dialog_show (vrt->dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
 			gtk_window_present (GTK_WINDOW (vrt->dialog));
 			return TRUE;
