@@ -1,6 +1,6 @@
-/* $Id: e2_file_info_dialog.c 568 2007-07-25 23:08:53Z tpgww $
+/* $Id: e2_file_info_dialog.c 863 2008-05-05 22:55:54Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -22,6 +22,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "emelfm2.h"
 #include <string.h>
 #include <unistd.h>
+#include <langinfo.h>
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
@@ -50,14 +51,18 @@ typedef struct _E2_FileInfoRuntime
 @a readable_type and/or @a mime_type are set to NULL if nothing valid is available
 @param filepath localised path string of item to be processed
 @param readable_type store for pointer to determined description string
-@param mime_type store for pointer to determined mime string
+@param mime_type store for pointer to determined mime (and for text files, charset too) string
 
 @return TRUE if both description and mume commands are executed successfully (which happens if file options are bad)
 */
-static gboolean _e2_file_info_dialog_get_file_type (gchar *filepath,
+static gboolean _e2_file_info_dialog_get_file_type (VPATH *filepath,
 	gchar **readable_type, gchar **mime_type)
 {
-	size_t bsize = strlen (filepath) + 512;
+#ifdef E2_VFS
+	if (e2_fs_item_is_mounted (filepath))
+	{
+#endif
+	size_t bsize = 512;
 #ifdef USE_GLIB2_10
 	gchar *buf = (gchar *) g_slice_alloc ((gulong) bsize);
 #else
@@ -70,11 +75,8 @@ static gboolean _e2_file_info_dialog_get_file_type (gchar *filepath,
 		*mime_type = NULL;
 		return FALSE;
 	}
-#ifdef E2_VFSTMP
-	FIXME get information about non-local items
-#else
 	//CHECKME -b option ok ? -i option to get mimetype
-	gchar *s, *command = g_strdup_printf ("file -pr \"%s\"", filepath);
+	gchar *command = g_strdup_printf ("file -bhnprs \"%s\"", VPSTR (filepath));
 	E2_FILE *pipe = e2_fs_open_pipe (command);
 	g_free (command);
 	if (pipe == NULL)
@@ -95,18 +97,17 @@ static gboolean _e2_file_info_dialog_get_file_type (gchar *filepath,
 	{
 		*(buf+strlen(buf)-1) = '\0';
 # endif
-		if ((s = strchr (buf, ':')) != NULL)	//scan for ascii ':', don't need g_utf8_strchr()
-			*readable_type = g_strdup (s+2);
-		else
-			*readable_type = g_strdup (buf);
+		*readable_type = g_strdup (buf);
 	}
 	else
 	{
 		*readable_type = NULL;	//NOTE this is really an error, but returns TRUE
 	}
 	e2_fs_pipe_close (pipe);
-	//get mimetype string
-	command = g_strdup_printf ("file -bpri \"%s\"", filepath);
+	//get mimetype (and for text files, encoding type) string
+	//we use -i option to get encoding for text files, instead of just --mime-type,
+	//because separate usage of  --mime-encoding does not work properly
+	command = g_strdup_printf ("file -bhnpri \"%s\"", VPSTR (filepath));
 	pipe = e2_fs_open_pipe (command);
 	g_free (command);
 	if (pipe == NULL)
@@ -143,8 +144,19 @@ static gboolean _e2_file_info_dialog_get_file_type (gchar *filepath,
 # else
 	g_free (buf);
 # endif
-#endif	//def E2_VFSTMP
 	return TRUE;
+#ifdef E2_VFS
+	}
+	else	//not a local item
+	{
+# ifdef E2_VFSTMP
+		FIXME
+# endif
+		*readable_type = NULL;
+		*mime_type = NULL;
+		return FALSE;
+	}
+#endif
 }
 /**
 @brief set popup menu position
@@ -170,7 +182,7 @@ static void _e2_file_info_dialog_set_menu_position (GtkWidget *menu,
 	GtkAllocation alloc = dialog->allocation;
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y + alloc.height/2;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 /**
 @brief copy dialog data to clipboard
@@ -307,45 +319,46 @@ static void _e2_file_info_dialog_response_cb (GtkDialog *dialog, gint response,
 
 @return enumerator for pressed dialog button
 */
-DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
+DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 {
 	GtkWidget *dialog;
 	GtkWidget *dialog_vbox;
 	GtkWidget *table;
 	GString *label_text = g_string_sized_new (NAME_MAX + 32);  //this is plenty big for all uses here
 	gchar *s1, *s2;
-	gchar perm_string[32];
+	gchar *mime;
 	gchar date_string[32];
     struct stat statbuf;
 	struct tm *tm_ptr;
 	struct passwd *pw_buf;
 	struct group *grp_buf;
 	E2_FileInfoRuntime rt;
+	DialogButtons choice;
 
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
 //explain to user ??
 		return CANCEL;
 
 	//must prevent the default dialog response, it stuffs up Q cleanups
-	DialogButtons choice;
 	rt.dialog =
 	dialog = e2_dialog_create (NULL, NULL, _("file info"),
 		_e2_file_info_dialog_response_cb, &choice);
 	dialog_vbox = GTK_DIALOG (dialog)->vbox;
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 
-	rt.localpath = localpath;	//store data for copying
-	s1 = g_filename_display_basename (localpath);
+	rt.localpath = VPSTR (localpath);	//store data for copying
+	s1 = g_filename_display_basename (VPSTR (localpath));
 	s2 = g_markup_escape_text (s1, -1);
 	g_string_printf (label_text, "<b>%s</b>", s2);
 	e2_widget_add_mid_label (dialog_vbox, label_text->str, 0.5, TRUE, 0);
 	g_free (s1);
 	g_free (s2);
 	e2_widget_add_separator (dialog_vbox, TRUE, E2_PADDING);
+	mime = NULL;
 
 	if (S_ISREG (statbuf.st_mode))
 	{
-		gchar *mime = NULL, *readable = NULL;
+		gchar *readable = NULL;
 		//CHECKME
 		if (statbuf.st_dev == 0 || statbuf.st_size > statbuf.st_blocks * statbuf.st_blksize)
 		{
@@ -372,25 +385,26 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 		GdkPixbuf *pixbuf = NULL;
 		if (mime != NULL && *mime != '\0')
 		{
-			GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen
-				(gtk_widget_get_screen (app.main_window));	//app.main_window->screen;
+			gchar *m2 = g_strdup (mime);
 			//mime string returned by 'find' is of the form "text/plain; <stuff>"
 			//from freedesktop.org, icon name is of the form media-subtype (e.g. text-plain).
 			//If no specific icon is found, fallback to media-x-generic (e.g. text-x-generic).
-			s1 = strchr (mime, ';');
+			s1 = strchr (m2, ';');
 			if (s1 != NULL)
 				*s1 = '\0';
-			s1 = strchr (mime, '/');
+			s1 = strchr (m2, '/');
 			if (s1 != NULL)
 				*s1 = '-';
-			pixbuf = gtk_icon_theme_load_icon (icon_theme, mime,
+			GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen
+				(gtk_widget_get_screen (app.main_window));	//app.main_window->screen;
+			pixbuf = gtk_icon_theme_load_icon (icon_theme, m2,
 				GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
 			if (pixbuf == NULL)
 			{
 				if (s1 != NULL)
 				{
 					*s1 = '\0';
-					gchar *inm = g_strconcat (mime, "-x-generic", NULL);
+					gchar *inm = g_strconcat (m2, "-x-generic", NULL);
 					pixbuf = gtk_icon_theme_load_icon (icon_theme, inm,
 						GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
 					g_free (inm);
@@ -414,6 +428,7 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 					}
 				}
 			}
+			g_free (m2);
 		}
 		if (pixbuf != NULL)
 		{
@@ -438,8 +453,6 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 			g_free (readable);
 			g_free (s2);
 		}
-		if (mime != NULL)
-			g_free (mime);
 	}
 	//links to dirs reported as dirs by (e2_fs_is_dir3 (localpath))
 	else if (S_ISDIR (statbuf.st_mode))
@@ -491,12 +504,16 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 	if (S_ISLNK (statbuf.st_mode))
 	{
 		gchar target[PATH_MAX];
-		gchar *filename = g_strdup (localpath);
+		gchar *filename = g_strdup (VPSTR (localpath));
 		gint len = strlen (filename);
 		if (filename[len-1] == G_DIR_SEPARATOR)
 			filename[len-1] = '\0';
-
+#ifdef E2_VFS
+		VPATH ddata = { filename, localpath->spacedata };
+		len = e2_fs_readlink (&ddata, target, sizeof (target) E2_ERR_NONE());
+#else
 		len = e2_fs_readlink (filename, target, sizeof (target) E2_ERR_NONE());
+#endif
 		if (len > 0)
 		{
 			target[len] = '\0';
@@ -545,7 +562,12 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 			E2_ERR_DECLARE
 			//check the real target, without looking thru chained links
 			struct stat statbuf2;	//don't disturb the size etc data for the link itelf
+#ifdef E2_VFS
+			ddata.localpath = target;
+			if (e2_fs_lstat (&ddata, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
+#else
 			if (e2_fs_lstat (target, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
+#endif
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>missing</b>)"));
 			else if (g_str_equal (localpath, target))
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>itself</b>)"));
@@ -584,74 +606,142 @@ DialogButtons e2_file_info_dialog_run (gchar *localpath, gboolean multi)
 	else
 		rt.target = NULL;	//don't use this label when copying
 
-	table = e2_widget_add_table (dialog_vbox, 9, 2, FALSE, TRUE, E2_PADDING);
+	table = e2_widget_add_table (dialog_vbox, 10, 2, FALSE, TRUE, E2_PADDING);
 	gtk_table_set_col_spacings (GTK_TABLE (table), E2_PADDING);
 
-	e2_widget_add_mid_label_to_table (table, _("Size:"), 0, 0,1,0,1);
+	if (mime != NULL)
+	{
+		if (g_str_has_prefix (mime, "text/"))
+		{
+			s2 = strstr (mime, " charset");
+			if (s2 != NULL)
+			{
+				s1 = g_utf8_strup (s2 + 9, -1);	//uppercase encoding
+				*s2 = '\0';
+				s2 = g_strconcat (mime, "\n", s1, NULL);
+				g_free (s1);
+				s1 = g_strconcat (_("Mime:"), "\n", _("Encoding:"), NULL);
+			}
+			else
+			{
+				s1 = _("Mime:");
+				s2 = mime;
+			}
+		}
+		else
+		{
+			s1 = _("Mime:");
+			s2 = mime;
+		}
+		e2_widget_add_mid_label_to_table (table, s1, 0, 0,1,0,1);
+		e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,0,1);
+		if (s2 != mime)
+		{
+			g_free (s1);
+			g_free (s2);
+		}
+	}
 
-	if (statbuf.st_size < 10240) // less than 10K
-		g_string_printf (label_text, "%d %s", (gint) statbuf.st_size, _("bytes"));
-	else if (statbuf.st_size < 1048576) // less than a meg
-		g_string_printf (label_text, "%.2f %s", (gfloat) statbuf.st_size / 1024.0, _("kbytes"));
-	else // more than a meg
-		g_string_printf (label_text, "%.2f %s", (gfloat) statbuf.st_size / 1048576.0, _("Mbytes"));
+	e2_widget_add_mid_label_to_table (table, _("Size:"), 0, 0,1,1,2);
+
+	g_string_printf (label_text, "%"PRIu64, statbuf.st_size);
+	//expand number to add separator char(s)
+	const gchar *comma = nl_langinfo (THOUSEP);
+	if (comma == NULL || *comma == '\0')
+		comma = ",";
+	guint len = label_text->len;
+	guint ths = len-1;  //0-based index
+	guint i;
+	while (ths > 2 && len < label_text->allocated_len)
+	{
+		for (i = len-1; i > ths-3; i--)
+			label_text->str[i+1] = label_text->str[i];
+		label_text->str[i+1] = *comma;
+		label_text->str[++len] = '\0';
+		ths = i;
+	}
+	label_text->len = len;
+
+	label_text = g_string_append_c (label_text, ' ');
+	label_text = g_string_append (label_text, _("bytes"));
 	rt.size =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,0,1);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,1,2);
 
 	GtkWidget *sep = gtk_hseparator_new ();
-	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,1,2);
-	e2_widget_add_mid_label_to_table (table, _("User:"), 0, 0,1,2,3);
+	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,2,3);
+	e2_widget_add_mid_label_to_table (table, _("User:"), 0, 0,1,3,4);
 	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
 		g_string_printf (label_text, "%s", pw_buf->pw_name);
 	else
 		g_string_printf (label_text, "%d", (guint) statbuf.st_uid);
 	rt.user =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,2,3);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,3,4);
 
-	e2_widget_add_mid_label_to_table (table, _("Group:"), 0, 0,1,3,4);
+	e2_widget_add_mid_label_to_table (table, _("Group:"), 0, 0,1,4,5);
 	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
 		g_string_printf (label_text, "%s", grp_buf->gr_name);
 	else
 		g_string_printf (label_text, "%d", (guint) statbuf.st_gid);
 	rt.group =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,3,4);
-	e2_fs_get_perm_string (perm_string, sizeof (perm_string), statbuf.st_mode);
-	g_string_assign (label_text, perm_string + 1);	//skip the "type code" in 1st byte of string
-	g_string_insert_c (label_text, 6, ' ');
-	g_string_insert_c (label_text, 3, ' ');
-	e2_widget_add_mid_label_to_table (table, _("Permissions:"), 0, 0,1,4,5);
-	rt.permissions =
 	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,4,5);
+	gchar *perm_string = e2_fs_get_perm_string (statbuf.st_mode);
+	g_string_assign (label_text, g_utf8_next_char (perm_string));	//skip the "type code" in 1st byte of string
+	g_free (perm_string);
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 6), ' ');
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 3), ' ');
+	e2_widget_add_mid_label_to_table (table, _("Permissions:"), 0, 0,1,5,6);
+	rt.permissions =
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,5,6);
 
 	sep = gtk_hseparator_new ();
-	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,5,6);
+	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,6,7);
 
 	s1 = "%a %Od %b %EY %OI:%OM %p";
 	tm_ptr = localtime (&statbuf.st_atime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0, 0,1,6,7);
+	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0, 0,1,7,8);
 	rt.accessdate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,6,7);
+	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,7,8);
+#ifdef USE_GTK2_12TIPS
+	gtk_widget_set_tooltip_text (
+#else
+	e2_widget_set_tooltip (NULL,
+#endif
+		rt.accessdate, _("Item was last opened, run, read etc"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_mtime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Modified:"), 0, 0,1,7,8);
+	e2_widget_add_mid_label_to_table (table, _("Modified:"), 0, 0,1,8,9);
 	rt.moddate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,7,8);
+	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,8,9);
+#ifdef USE_GTK2_12TIPS
+	gtk_widget_set_tooltip_text (
+#else
+	e2_widget_set_tooltip (NULL,
+#endif
+		rt.moddate, _("Content of the item was last altered"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_ctime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Changed:"), 0, 0,1,8,9);
+	e2_widget_add_mid_label_to_table (table, _("Changed:"), 0, 0,1,9,10);
 	rt.changedate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,8,9);
+	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,9,10);
+#ifdef USE_GTK2_12TIPS
+	gtk_widget_set_tooltip_text (
+#else
+	e2_widget_set_tooltip (NULL,
+#endif
+		rt.changedate, _("Property of the item (name, permission, owner etc) was last altered"));
 	g_free (s2);
 
 	g_string_free (label_text,TRUE);
+	if (mime != NULL)
+		g_free (mime);
 
 	if (multi)
 	{
