@@ -1,6 +1,6 @@
-/* $Id: e2_filetype_dialog.c 539 2007-07-18 11:52:43Z tpgww $
+/* $Id: e2_filetype_dialog.c 854 2008-04-19 11:45:33Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -670,7 +670,7 @@ static void _e2_edftdlg_set_menu_position (GtkMenu *menu,
 	GtkAllocation alloc = treeview->allocation;
 	*x = left+ alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 /**
 @brief create treeview context menu
@@ -1034,7 +1034,7 @@ typedef struct _E2_FileTypeDlg2Runtime
 {
 	GtkWidget *view_btn;
 	GtkWidget *open_btn;
-	gchar *filepath;
+	gchar *utfpath;	//utf8 absolute path of item
 	gint processed;	//store for return-code
 } E2_FileTypeDlg2Runtime;
 
@@ -1054,7 +1054,7 @@ static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	gchar *action;
 	//remember this when rt is cleared at end of dialog func, and handle any whitespace
-	gchar *item = g_strconcat ("\"", rt->filepath, "\"", NULL);
+	gchar *item = g_strconcat ("\"", rt->utfpath, "\"", NULL);
 	//send back "handled" type flag to caller
 	switch (response)
 	{
@@ -1091,14 +1091,14 @@ static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 			g_free (action);
 			break;
 		case E2_RESPONSE_USER2:	//view
-			action = g_strconcat (_A(5),".",_A(92), NULL);
+			action = g_strconcat (_A(5),".",_A(101), NULL);
 			gdk_threads_leave ();	//open BGL
 			e2_action_run_simple_from (action, item, NULL);
 			gdk_threads_enter ();
 			g_free (action);
 			break;
 		case E2_RESPONSE_USER3:	//open with
-			action = g_strconcat (_A(5),".",_A(61), NULL);
+			action = g_strconcat (_A(5),".",_A(62), NULL);
 			gdk_threads_leave ();	//open BGL
 			e2_action_run_simple_from (action, item, NULL);
 			gdk_threads_enter ();
@@ -1121,40 +1121,50 @@ static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 @brief create and show dialog for handling unknown or ambiguous filetype
 @a newtype should be TRUE when @a filename does have an extension
 @a newtype and @a ambig should not both be TRUE
-@param path absolute or relative path of item to be processed, utf8 string
+@param localpath absolute or relative path of item to be processed, localised string
 @param text TRUE to include view or edit option in the dialog
 @param ambig TRUE to enable choice to execute or open item
 @param newtype TRUE to include a button to add a new filetype
 
 @return enumerator signalling how @a filename was "actioned" via the dialog
 */
-gint e2_filetype_dialog_create (gchar *path, gboolean text, gboolean ambig,
+gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	gboolean newtype)
 {
-//	printd (DEBUG, "e2_filetype_dialog_create: %s", filename);
+	gint retval;
+	gboolean addpath;
+	gchar *usepath, *dir, *base, *title, *public, *message;
+	struct stat sb;
+
 	E2_FileTypeDlg2Runtime *rt = ALLOCATE (E2_FileTypeDlg2Runtime);
 	CHECKALLOCATEDWARN (rt, return);
-	struct stat sb;
-	gboolean addpath = !g_path_is_absolute (path);
+
+	usepath = VPSTR (localpath);
+//	printd (DEBUG, "e2_filetype_dialog_create: %s", usepath);
+	addpath = !g_path_is_absolute (usepath);
 	if (addpath)
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
-		path = e2_utils_dircat (curr_view, path);
+	{
+	//FIXME race (unlikely) if dir goes and cd happens, worse if past virtual root
+#ifdef E2_VFS
+		if (localpath->spacedata == other_view->spacedata)
+			usepath = e2_utils_dircat (other_view, usepath, TRUE);
+		else
 #endif
-	rt->filepath = path;
-	gchar *local = F_FILENAME_TO_LOCALE (path);
-	gchar *title = (ambig) ? _("ambiguous filetype") : _("unrecognised filetype");
-	gchar *public = g_markup_escape_text (path, -1);
-	gchar *dir = g_path_get_dirname (public);
-	gchar *base = g_path_get_basename (public);
-	gchar *message = g_strdup_printf (
+			usepath = e2_utils_dircat (curr_view, usepath, TRUE);
+	}
+
+	rt->utfpath = D_FILENAME_FROM_LOCALE (usepath);
+	title = (ambig) ? _("ambiguous filetype") : _("unrecognised filetype");
+	public = g_markup_escape_text (rt->utfpath, -1);
+	dir = g_path_get_dirname (public);
+	base = g_path_get_basename (public);
+	message = g_strdup_printf (
 		_("What would you like to do with\n<b>%s</b>\nin %s ?"), base, dir);
 	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
 		message, title, _e2_filetype_dialog_response_cb, rt);
+	g_free (public);
 	g_free (dir);
 	g_free (base);
-	g_free (public);
 	g_free (message);
 
 	//can always add to category, even if item does not presently exist
@@ -1164,7 +1174,12 @@ gint e2_filetype_dialog_create (gchar *path, gboolean text, gboolean ambig,
 		_("Create a new filetype for this extension, or add it to an existing filetype"),
 		NULL, NULL);
 
-	if (!e2_fs_stat (local, &sb E2_ERR_NONE()))
+#ifdef E2_VFS
+	VPATH ddata = { usepath, localpath->spacedata };
+	if (!e2_fs_stat (&ddata, &sb E2_ERR_NONE()))
+#else
+	if (!e2_fs_stat (usepath, &sb E2_ERR_NONE()))
+#endif
 	{
 		if (S_ISREG (sb.st_mode))
 		{
@@ -1196,9 +1211,8 @@ gint e2_filetype_dialog_create (gchar *path, gboolean text, gboolean ambig,
 	e2_dialog_show (dialog, NULL, 0, &E2_BUTTON_CANCEL, NULL);
 	gtk_main ();	//block until user selects something
 	if (addpath)
-		g_free (path);
-	F_FREE (local);
-	gint retval = rt->processed;	//get code for what was selected
+		g_free (usepath);
+	retval = rt->processed;	//get code for what was selected
 	DEALLOCATE (E2_FileTypeDlg2Runtime, rt);
 	return retval;
 }
