@@ -1,6 +1,6 @@
-/* $Id: e2_mkdir_dialog.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_mkdir_dialog.c 892 2008-05-20 09:16:37Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -87,7 +87,12 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 	gchar *path = g_build_filename (part1, part2, part3, NULL);
 
 	gchar *local = F_FILENAME_TO_LOCALE (path);
+#ifdef E2_VFS
+	VPATH ddata = { local, curr_view->spacedata };
+	if (!e2_fs_access (&ddata, F_OK E2_ERR_NONE()))	//through links
+#else
 	if (!e2_fs_access (local, F_OK E2_ERR_NONE()))	//through links
+#endif
 	{
 		//the directory in the combo entry already exists,
 		//check (backwards through the name) whether the name includes a number
@@ -151,7 +156,11 @@ static gchar *_e2_mkdirdlg_find_dir (const gchar *dir, const gchar *parent_dir)
 			g_free (freeme);
 		}
 		//now check for a non-existent item with an incremented number
+#ifdef E2_VFS
+		while (!e2_fs_access (&ddata, F_OK E2_ERR_NONE()))
+#else
 		while (!e2_fs_access (local, F_OK E2_ERR_NONE()))
+#endif
 		{
 			i++;
 			g_free (path);
@@ -205,7 +214,12 @@ static gchar *_e2_mkdirdlg_real_parent (gchar *parent, gboolean *exists, gint *i
 //FIXME handle vfs parent
 #endif
 	gchar *local = F_FILENAME_TO_LOCALE (parent);
+#ifdef E2_VFS
+	VPATH ddata = { local, curr_view->spacedata };
+	while (e2_fs_access (&ddata, F_OK E2_ERR_NONE()) || ! e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 	while (e2_fs_access (local, F_OK E2_ERR_NONE()) || ! e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 	{
 		(*i)++;
 		*exists = FALSE;
@@ -270,11 +284,7 @@ This is called from several placces, all inside BGL
 */
 static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
 {
-//	if ((!GTK_IS_WIDGET (rt->dialog)) || (!GTK_WIDGET_VISIBLE (rt->dialog)))
-//		return;
-	if (!gtk_expander_get_expanded (GTK_EXPANDER (rt->info_expander)))
-		return;
-	printd (DEBUG, "mkdir dialog resize pending");
+//	printd (DEBUG, "mkdir dialog resize pending");
 	e2_dialog_resize_with_sw (rt->dialog, rt->scrolled);	//this works at idle-time
 }
 /**
@@ -347,6 +357,7 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 	printd (DEBUG, "mkdir dialog update status");
 	gboolean absolute;
 	gchar *path, *parent;
+	E2_ERR_DECLARE
 
 	rt->creation_possible = FALSE;
 
@@ -389,8 +400,12 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 		return;
 	}
 	gchar *local = F_FILENAME_TO_LOCALE (parent);
-	E2_ERR_DECLARE
+#ifdef E2_VFS
+	VPATH ddata = { local, curr_view->spacedata };
+	if ((e2_fs_access (&ddata, W_OK | X_OK E2_ERR_PTR())) != 0)
+#else
 	if ((e2_fs_access (local, W_OK | X_OK E2_ERR_PTR())) != 0)
+#endif
 	{
 		if (absolute)
 			reason = g_strdup_printf (_("cannot write to '%s' - %s"), parent,
@@ -426,9 +441,18 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 	}
 	F_FREE (local);
 	local = F_FILENAME_TO_LOCALE (path);
+#ifdef E2_VFS
+	ddata.localpath = local;
+	if (!e2_fs_access (&ddata, F_OK E2_ERR_NONE()))
+#else
 	if (!e2_fs_access (local, F_OK E2_ERR_NONE()))
+#endif
 	{
+#ifdef E2_VFS
+		if  (e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 		if  (e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 			reason = _("the directory already exists.");
 		else
 			reason = _("something is in the way.");
@@ -518,7 +542,7 @@ static void _e2_mkdirdlg_update_follow_dir (E2_MkdirDialogRuntime *rt, gboolean
 }
 /**
 @brief hook function for app.paneX.hook_change_dir
-This is initated with BGL closed/on
+This is called with BGL off/open
 @param path path of an opened directory, utf-8 string
 @param rt pointer to dialog's data struct
 @return TRUE
@@ -527,13 +551,15 @@ static gboolean _e2_mkdirdlg_change_dir_hook (gchar *path, E2_MkdirDialogRuntime
 {
 	g_free (rt->path);
 	rt->path = g_strdup (path);
+	gdk_threads_enter ();
 	_e2_mkdirdlg_update_status (rt);
 	_e2_mkdirdlg_update_dialog_size (rt);
+	gdk_threads_leave ();
 	return TRUE;
 }
 /**
 @brief hook function for app.hook_pane_focus_changed
-This is called from inside (CHECKME or out of?) BGL
+This is called with BGL off/open
 @param pane_rt data struct for the currenly-focused pane
 @param rt pointer to dialog's data struct
 @return TRUE
@@ -544,14 +570,18 @@ static gboolean _e2_mkdirdlg_change_focus_hook (E2_PaneRuntime *pane_rt, E2_Mkdi
 #ifdef E2_VFSTMP
 	//FIXME path for non-mounted dirs
 #else
+	LISTS_LOCK
 	rt->path = g_strdup (pane_rt->path);
+	LISTS_UNLOCK
 #endif
-	_e2_mkdirdlg_update_status (rt); 	//CHECKME must have BGL before going there
+	gdk_threads_enter ();
+	_e2_mkdirdlg_update_status (rt);
 	_e2_mkdirdlg_update_dialog_size (rt);
+	gdk_threads_leave ();
 	return TRUE;
 }
 /**
-@brief create a new directory, along with any missing "ancestor(s)"
+@brief create a new directory, along with any missing "ancestor(s)", in the active-pane dir
 The string provided may be an absolute or relative path, the latter
 including just a name. If relative, the last-used dir?? will be prepended
 This is called only from _e2_mkdirdlg_response_cb () with BGL closed
@@ -561,8 +591,8 @@ This is called only from _e2_mkdirdlg_response_cb () with BGL closed
 
 @return
 */
-static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
-	E2_MkdirDialogRuntime *rt, gboolean close)
+static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt,
+	gboolean close)
 {
 	printd (DEBUG, "mkdir dialog create dir");
 	if (!rt->creation_possible)
@@ -573,14 +603,10 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
 		return;
 
 	gchar *path, *parent, *local, *freeme = NULL;
-#ifdef E2_VFSTMP
-	//FIXME path for virtual dirs
-#else
 	if (g_path_is_absolute (dir))
 		path = g_strdup (dir);
 	else
 		path = g_strconcat (rt->path, dir, NULL);
-#endif
 
 	//find closest existing "ancestor" of the dir to be created, and any
 	//intervening ancestor dir(s) that need to be created
@@ -593,7 +619,12 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
 		g_free (freeme);
 	}
 	freeme = local = F_FILENAME_TO_LOCALE (parent);
+#ifdef E2_VFS
+	VPATH ddata = { local, curr_view->spacedata };
+	while (e2_fs_access (&ddata, F_OK E2_ERR_NONE()) || !e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 	while (e2_fs_access (local, F_OK E2_ERR_NONE()) || !e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 	{
 #ifdef E2_VFSTMP
 	//FIXME parent path for virtual dirs
@@ -614,10 +645,20 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
 	{
 		local = F_FILENAME_TO_LOCALE ((gchar *)tmp->data);
 //		E2_ERR_NAME = NULL;
+#ifdef E2_VFS
+		ddata.localpath = local;
+		if (e2_fs_mkdir (&ddata, 0777 E2_ERR_PTR()))	//FIXME vfs
+#else
 		if (e2_fs_mkdir (local, 0777 E2_ERR_PTR()))	//FIXME vfs
+#endif
 		{
 			gdk_threads_leave (); //downstream does local mutex management
-			e2_fs_error_local (_("Cannot create directory %s"), local E2_ERR_MSGL());
+			e2_fs_error_local (_("Cannot create directory %s"),
+#ifdef E2_VFS
+				&ddata E2_ERR_MSGL());
+#else
+				local E2_ERR_MSGL());
+#endif
 			gdk_threads_enter ();
 			E2_ERR_CLEAR
 
@@ -638,7 +679,12 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
 
 	//now create the chosen dir itself
 	local = F_FILENAME_TO_LOCALE (path);
+#ifdef E2_VFS
+	ddata.localpath = local;
+	if (!e2_fs_mkdir (&ddata, 0777 E2_ERR_PTR()))
+#else
 	if (!e2_fs_mkdir (local, 0777 E2_ERR_PTR()))
+#endif
 	{	//succeeded
 		if (!close)
 			_e2_mkdirdlg_update_name_and_status (rt);
@@ -652,7 +698,12 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry,
 	else	//mkdir failed
 	{
 		gdk_threads_leave (); //downstream does local mutex management
-		e2_fs_error_local (_("Cannot create directory %s"), local E2_ERR_MSGL());
+		e2_fs_error_local (_("Cannot create directory %s"),
+#ifdef E2_VFS
+			&ddata E2_ERR_MSGL());
+#else
+			local E2_ERR_MSGL());
+#endif
 		gdk_threads_enter ();
 		E2_ERR_CLEAR
 
@@ -792,13 +843,13 @@ static void _e2_mkdirdlg_toggled3_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 		e2_option_bool_set_direct (suggest_dir, rt->opt_suggest_dir);
 }
 /**
-@brief callback for expander opened
+@brief callback for info expander opened or closed
 @param expander the activated widget
 @param spec
 @param rt pointer to dialog's data struct
 @return
 */
-static void _e2_mkdirdlg_toggled_expander_cb (GtkWidget *expander,
+static void _e2_mkdirdlg_expander_toggled_cb (GtkWidget *expander,
 	GParamSpec *spec, E2_MkdirDialogRuntime *rt)
 {
 //	printd (DEBUG, "mkdir dialog expander toggle");
@@ -852,7 +903,7 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 		_e2_mkdirdlg_response_cb, rt);
 
 	GtkWidget *vbox = e2_dialog_add_sw (rt->dialog);
-	rt->scrolled = vbox->parent->parent;
+	rt->scrolled = vbox->parent->parent;	//bad !!
 
 	//CHECKME this is needed to workaround gtk weirdness,
 	//= bad delay when any key is pressed during a cursor-visible interval
@@ -862,26 +913,27 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 		_e2_mkdirdlg_activate_cb, rt, &mkdir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
 	gdk_threads_leave ();
-	//add info box with expander
-	rt->info_box = e2_widget_get_box (TRUE, TRUE, TRUE);
-	rt->info_expander = e2_option_bool_add_expander_widget (rt->dialog,
-		vbox, _("info"), e2_option_get ("dialog-mkdir-show-info"), rt->info_box);
-	gtk_widget_show (rt->info_expander);
+	//add expander with info box as its child
+	GtkWidget *info_box = e2_widget_get_box (TRUE, FALSE, 1);
+	rt->info_expander = e2_widget_add_expander (vbox, _("info"),
+		e2_option_get ("dialog-mkdir-show-info"), rt->dialog, info_box);
+
 	g_signal_connect (G_OBJECT (rt->info_expander), "notify::expanded",
-		G_CALLBACK (_e2_mkdirdlg_toggled_expander_cb), rt);
+		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb), rt);
 
 	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	GtkWidget *hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
+	GtkWidget *hbox = e2_widget_add_box (info_box, FALSE, 0, FALSE, FALSE, 0);
 	GtkWidget *label = e2_widget_add_label (hbox,
 		_("<small>parent directory:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
 	gtk_size_group_add_widget (group, label);
 
 	rt->info_label = e2_widget_add_mid_label (hbox, "", 0.0, TRUE, E2_PADDING_SMALL);
-	hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
+	hbox = e2_widget_add_box (info_box, FALSE, 0, FALSE, FALSE, 0);
 	label = e2_widget_add_label (hbox,
 		_("<small>creation possible:</small>"), 0.0, 0.0, FALSE, E2_PADDING_SMALL);
 	gtk_size_group_add_widget (group, label);
 	rt->info_label2 = e2_widget_add_label (hbox, "", 0.0, 0.0, TRUE, E2_PADDING_SMALL);
+	g_object_unref (G_OBJECT (group));
 
 	if (follow_pane == NULL)
 		follow_pane = e2_option_get ("dialog-mkdir-follow-pane");
@@ -918,10 +970,8 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 
 //	_e2_mkdirdlg_update_follow_dir (rt, e2_option_bool_get_direct (follow_pane));
 
-	gdk_threads_enter ();
-	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL,
+    e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_THREADED | E2_DIALOG_DONT_SHOW_ALL,
 		&E2_BUTTON_CANCEL, &E2_BUTTON_CREATE, &E2_BUTTON_OK, NULL);
-	gdk_threads_leave ();
 	//do this after showing, to reduce repitition
 	g_signal_connect (G_OBJECT (rt->combo), "changed",
 		G_CALLBACK (_e2_mkdirdlg_changed_cb), rt);
@@ -941,7 +991,7 @@ void e2_mkdir_dialog_actions_register ()
 {
 	e2_cache_list_register ("mkdir-history", &mkdir_history);
 
-	gchar *action_name = g_strconcat(_A(1),".",_A(55),NULL);
+	gchar *action_name = g_strconcat(_A(1),".",_A(56),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_mkdir_dialog_create, NULL, FALSE);
 }
