@@ -1,6 +1,6 @@
-/* $Id: e2_permissions_dialog.c 544 2007-07-20 12:48:20Z tpgww $
+/* $Id: e2_permissions_dialog.c 868 2008-05-06 04:42:09Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -287,11 +287,11 @@ static void _e2_permissions_dialog_response_cb (GtkDialog *dialog,
 @return enumerator corresponing to the clicked dialog button
 */
 DialogButtons e2_permissions_dialog_run (
-					gchar *localpath,
-					gchar **mode_ret,
-					E2_RecurseType *recurse_ret,
-					gboolean *permission_ret,
-					gboolean multi )
+				VPATH *localpath,
+				gchar **mode_ret,
+				E2_RecurseType *recurse_ret,
+				gboolean *permission_ret,
+				gboolean multi )
 {
 	GtkWidget *permissions_dialog;
 	GtkWidget *dialog_vbox, *sub_vbox;
@@ -300,7 +300,7 @@ DialogButtons e2_permissions_dialog_run (
 	GtkWidget *action_area;
 	GtkWidget *table;
 
-	gchar perm_string[32];
+//	gchar perm_string[32];
 	struct stat statbuf;
 	struct passwd *pw_buf;
 	struct group *grp_buf;
@@ -325,7 +325,7 @@ DialogButtons e2_permissions_dialog_run (
 	//  table = e2_widget_add_table (dialog_vbox, 3, 3, TRUE, TRUE, 0);  //3 rows, 3 cols, homogen, fill, no pad
 	//  gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
 	gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
-	gchar *name = g_filename_display_basename (localpath);
+	gchar *name = g_filename_display_basename (VPSTR (localpath));
 	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
 	g_free (name);
 
@@ -370,10 +370,11 @@ DialogButtons e2_permissions_dialog_run (
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
 	gtk_widget_show (hbox);
 
-	e2_fs_get_perm_string (perm_string, sizeof (perm_string), statbuf.st_mode);
-	g_string_assign (label_text, perm_string + 1);	//skip the "type code" in 1st byte of string
-	g_string_insert_c (label_text, 6, ' ');
-	g_string_insert_c (label_text, 3, ' ');
+	gchar *s = e2_fs_get_perm_string (statbuf.st_mode);
+	g_string_assign (label_text, g_utf8_next_char (s));	//skip the "type code" in 1st byte of string
+	g_free (s);
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 6), ' ');
+	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 3), ' ');
 	g_string_insert_c (label_text, 0, ' ');
 	label_text = g_string_prepend (label_text, _("currently"));
 
@@ -465,7 +466,7 @@ DialogButtons e2_permissions_dialog_run (
 			rt.recurse_button = e2_button_add_toggle_to_table (table, _("R_ecurse:"),
 				FALSE, _e2_permissions_dialog_toggle_recurse_button_cb, &rt,
 				1,2,0,1 );
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 			gtk_widget_set_tooltip_text (
 #else
 			e2_widget_set_tooltip (NULL,
