@@ -1,6 +1,6 @@
-/* $Id: e2_permissions_dialog.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_permissions_dialog.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_PERMISSIONS_DIALOG_H
-#define E2_PERMISSIONS_DIALOG_H
+#ifndef __E2_PERMISSIONS_DIALOG_H__
+#define __E2_PERMISSIONS_DIALOG_H__
 
 #include "emelfm2.h"
 #include "e2_dialog.h"
@@ -32,7 +32,7 @@ typedef enum
 	E2_RECURSE_ALL = 1 << 3
 } E2_RecurseType;
 
-DialogButtons e2_permissions_dialog_run (gchar *localpath, gchar **mode_ret,
+DialogButtons e2_permissions_dialog_run (VPATH *localpath, gchar **mode_ret,
 	E2_RecurseType *recurse_ret, gboolean *permission_ret, gboolean multi);
 
-#endif //ndef E2_PERMISSIONS_DIALOG_H
+#endif //ndef __E2_PERMISSIONS_DIALOG_H__
