@@ -1,6 +1,6 @@
-/* $Id: e2_select_image_dialog.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_select_image_dialog.h 885 2008-05-19 11:13:10Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_SELECT_IMAGE_DIALOG_H
-#define E2_SELECT_IMAGE_DIALOG_H
+#ifndef __E2_SELECT_IMAGE_DIALOG_H__
+#define __E2_SELECT_IMAGE_DIALOG_H__
 
 #include "emelfm2.h"
 
@@ -31,12 +31,14 @@ typedef struct _E2_SID_Runtime
 	GtkWidget *parent;
 	GtkNotebook *notebook;
 	gint page;
-	GtkWidget *file_chooser;
-	GtkTreeModel *model;
-	GtkTreeView *treeview;
+	GtkIconView *stockview;
+	GtkTreeModel *stockmodel;
+	GtkIconView *customview;
+	GtkTreeModel *custommodel;
+	GtkWidget *dir_chooser; //button
 } E2_SID_Runtime;
 
 GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name,
 	gchar *icon, gpointer response_func, E2_OptionSet *set);
 
-#endif /* ifndef E2_SELECT_IMAGE_DIALOG_H */
+#endif //ndef __E2_SELECT_IMAGE_DIALOG_H__
