@@ -1,6 +1,6 @@
-/* $Id: e2_tree_dialog.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_tree_dialog.c 868 2008-05-06 04:42:09Z tpgww $
 
-Copyright (C) 2007 tooar <tooar@gmx.net>
+Copyright (C) 2007-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1996-2007 Steve Baker <ice@mama.indstate.edu>
 
 This file is part of emelFM2.
@@ -386,6 +386,7 @@ static void _e2_treedlg_show_path (const gchar *pathstr, gboolean expanded, E2_T
 			gtk_tree_path_free (parent);
 		}
 		//maybe gtk bug here ?? any vertical scroll < about 0.5 fails
+		//?WAIT_FOR_EVENTS
 		gtk_tree_view_scroll_to_cell (tvw, tp, NULL, TRUE, 0.5, 0.1);
 
 		GtkTreeSelection *sel = gtk_tree_view_get_selection (tvw);
@@ -596,7 +597,7 @@ The treewalk has been setup for breadth-first, dirs only operation
 
 @return E2TW_CONTINUE on success, others as appropriate
 */
-static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
+static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 	const struct stat *statptr, E2_TwStatus status, E2_TreeDialogRuntime *rt)
 {
 	E2_TwResult retval;
@@ -693,7 +694,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 		if (g_str_equal (dirpath, G_DIR_SEPARATOR_S))
 			return retval;
 
-		gchar *lastseg = strrchr (dirpath, G_DIR_SEPARATOR);
+		gchar *lastseg = strrchr (VPSTR (dirpath), G_DIR_SEPARATOR);
 		if (lastseg != NULL)	//should always succeed
 		{
 #ifdef STRICT_HIDE
@@ -729,7 +730,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 				}
 #endif
 				const gchar *parentpath;
-				if (lastseg == dirpath)
+				if (lastseg == VPSTR (dirpath))
 				{
 #ifdef E2_VFSTMP
 //FIXME alternate tree roots ?
@@ -739,7 +740,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 				else
 				{
 					*lastseg = '\0';
-					parentpath = dirpath;
+					parentpath = VPCSTR (dirpath);
 				}
 				//confirm that parent path is already in treestore
 				if ((ref = g_hash_table_lookup (rt->dirhash, parentpath)) != NULL)
@@ -786,7 +787,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 					//remember this new addition
 					tp = gtk_tree_model_get_path (model, &iter);
 					ref = gtk_tree_row_reference_new (model, tp);
-					g_hash_table_insert (rt->dirhash, g_strdup (dirpath), ref);
+					g_hash_table_insert (rt->dirhash, g_strdup (VPSTR(dirpath)), ref);
 					gtk_tree_path_free (tp);
 				}
 				else
@@ -799,7 +800,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 				//get parent iter
 				GtkTreeIter parent;
 				const gchar *parentpath;
-				if (lastseg == dirpath)
+				if (lastseg == VPSTR (dirpath))
 				{
 #ifdef E2_VFSTMP
 //FIXME alternate tree roots ?
@@ -809,7 +810,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (const gchar *dirpath,
 				else
 				{
 					*lastseg = '\0';
-					parentpath = dirpath;
+					parentpath = VPCSTR (dirpath);
 				}
 				//confirm that parent path is already in treestore
 				GtkTreePath	*tp;
@@ -997,7 +998,7 @@ static void _e2_treedlg_set_menu_position (GtkMenu *menu,
 	GtkAllocation alloc = rt->treeview->allocation;
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 /**
 @brief tree path copy callback
@@ -1117,7 +1118,7 @@ static void _e2_treedlg_show_context_menu (GtkWidget *treeview,
 	item = e2_menu_add_check (menu, _("_Strict hiding"), rt->strict_hide,
 		_e2_treedlg_toggle_strict_cb, rt);
 	if (!rt->show_hidden)
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 		gtk_widget_set_tooltip_text (
 #else
 		e2_widget_set_tooltip (NULL,
@@ -1271,16 +1272,23 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 		gboolean wasnot_hidden = (rt->strict_hide) ? FALSE : !rt->show_hidden;
 		if (wasnot_hidden)
 			rt->show_hidden = TRUE;
+#endif
+#ifdef E2_VFS
+		VPATH ddata = { thisdir, rt->view->spacedata };
 #endif
 		gdk_threads_leave ();
 		//if (!
+#ifdef E2_VFS
+			e2_fs_tw (&ddata, _e2_treedlg_twcb_scan, rt, 2,
+#else
 			e2_fs_tw (thisdir, _e2_treedlg_twcb_scan, rt, 2,
+#endif
 #ifdef WARNBADS
 			E2TW_QT |
 #else
 			E2TW_XQT |
 #endif
-		E2TW_ONLYDIR E2_ERR_NONE());
+			E2TW_ONLYDIR E2_ERR_NONE());
 		//)
 		//{
 			//FIXME handle error
