@@ -1,6 +1,6 @@
-/* $Id: e2_view_dialog.c 544 2007-07-20 12:48:20Z tpgww $
+/* $Id: e2_view_dialog.c 866 2008-05-06 01:05:28Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -373,81 +373,102 @@ gboolean e2_view_dialog_search (gboolean first, gboolean incremental,
 /**
 @brief read text file to be viewed
 This is used by both view and edit dialogs
+Assumes the item being read is in curr_view i.e. no cd race from queueing
 Character encoding is performed if necessary and possible.
-@a rt ->filepath has the name of the file to be read. It will be
+@a rt ->filepath has the localised name of the file to be read. It will be
 replaced by a newly-allocated string, to be freed elsewhere
-Error messages expect BGL open
+Error messages and any macro-instigated dialog expect BGL open
+@param localfile name of item to read, localised text, not neccessarily absolute
 @param rt dialog runtime data struct
 
 @return TRUE if the read and conversion was successful
 */
-gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
+gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 {
-	gchar *file;
-	if (!g_path_is_absolute (rt->filepath))
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
-		file = g_build_filename (curr_view->dir, rt->filepath, NULL);
-#endif
-	else
-		file = g_strdup (rt->filepath);
-	gchar *local = F_FILENAME_TO_LOCALE (file);
 	E2_ERR_DECLARE
-	if (e2_fs_access (local, R_OK E2_ERR_PTR()))	//traverse a link
+	gchar *localpath;
+	if (!g_path_is_absolute (VPSTR (localfile)))
+		localpath = g_build_filename (curr_view->dir, VPSTR (localfile), NULL);
+	else
+		localpath = g_strdup (VPSTR (localfile));	//copy in case we want to keep it
+#ifdef E2_VFS
+	VPATH ddata = { localpath, localfile->spacedata };
+	if (e2_fs_access (&ddata, R_OK E2_ERR_PTR()))	//traverse a link
+#else
+	if (e2_fs_access (localpath, R_OK E2_ERR_PTR()))	//traverse a link
+#endif
 	{
-		e2_fs_error_local (_("Cannot read '%s'"), local E2_ERR_MSGL());
+		e2_fs_error_local (_("Cannot read '%s'"),
+#ifdef E2_VFS
+		&ddata E2_ERR_MSGL());
+#else
+		localpath E2_ERR_MSGL());
+#endif
 		E2_ERR_CLEAR
-		g_free (file);
-		F_FREE (local);
+		g_free (localpath);
 		return FALSE;
 	}
-	if (e2_fs_is_dir3 (local E2_ERR_NONE()))
+#ifdef E2_VFS
+	if (e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
+	if (e2_fs_is_dir3 (localpath E2_ERR_NONE()))
+#endif
 	{
-		gchar *msg = g_strdup_printf (_("'%s' is a directory"), file);
-		e2_output_print_error (msg, TRUE);
-		g_free (file);
-		F_FREE (local);
+		e2_fs_error_simple (_("'%s' is a directory"),
+#ifdef E2_VFS
+		&ddata);
+#else
+		localpath);
+#endif
+		g_free (localpath);
 		return FALSE;
 	}
 
 	gboolean usable;
-	gchar *cont, *converter = NULL;
+	gpointer contents;
+	gchar *converter = NULL;
 	if (e2_option_bool_get ("use-external-encoder"))
 	{
 		converter = e2_option_str_get ("command-encoder");
 		if (*converter != '\0')
 		{
+			gchar *command;
+#ifdef E2_VFSTMP
+	//can't sniff non-local files
+#endif
 			//sniff the file to check if conversion is really needed
 			//this is not too good for files with mixed encoding !
-			gchar *sniffer = g_strconcat ("cat ", file, NULL);
-			if (e2_fs_sniff_command_output (sniffer, &cont, 1024))
+			command = g_strconcat ("cat ", localpath, NULL);
+			if (e2_fs_sniff_command_output (command, &contents, 1024))
 			{
-				usable = e2_utf8_detect_charset (cont, &rt->charset);
-				g_free (cont);
+				usable = e2_utf8_detect_charset ((guchar *)contents, &rt->charset);
+				g_free (contents);
 			}
 			else
 				usable = FALSE;
-			g_free (sniffer);
+			g_free (command);
 
 			if (!usable)
 			{	//we need to do a conversion
-				//substitute file for any %f or %p in command string
-				gchar *command = e2_utils_expand_macros (converter, file);
+				//substitute file for any %f or %p in command string, get language etc
+				gdk_threads_enter ();	//in case there's a dialog-macro in the command
+				command = e2_utils_expand_macros (converter, localpath);
+				gdk_threads_leave ();
 				if (command == NULL || command == GINT_TO_POINTER(1))
 				{
-					g_free (file);
-					F_FREE (local);
+					g_free (localpath);
 					return FALSE;
 				}
 				else
 				{
 					rt->charset = "UNKNOWN";	//no translation
-					if (!e2_fs_get_command_output (command, &cont))
+					if (!e2_fs_get_command_output (command, &contents))
 					{
 						gchar *msg = g_strdup_printf
 						(_("Encoding conversion command '%s' failed"), converter);
+						gdk_threads_enter ();
 						e2_output_print_error (msg, TRUE);
+						gdk_threads_leave ();
 						converter = NULL;	//set flag for internal conversion fallback
 					}
 					g_free (command);
@@ -461,44 +482,56 @@ gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
 	}
 
 	if (converter == NULL	//not externally converted
-		&& !e2_fs_get_file_contents (file, &cont E2_ERR_PTR()))
+#ifdef E2_VFS
+		&& !e2_fs_get_file_contents (&ddata, &contents, NULL, TRUE E2_ERR_PTR()))
+#else
+		&& !e2_fs_get_file_contents (localpath, &contents, NULL, TRUE E2_ERR_PTR()))
+#endif
 	{
-		e2_fs_error_local (_("Error reading file %s"), local E2_ERR_MSGL());
+		e2_fs_error_local (_("Error reading file %s"),
+#ifdef E2_VFS
+		 &ddata E2_ERR_MSGL());
+#else
+		 localpath E2_ERR_MSGL());
+#endif
 		E2_ERR_CLEAR
-		g_free (file);
-		F_FREE (local);
+		g_free (localpath);
 		return FALSE;
 	}
 	//fix CR's in the text if necessary
-	rt->linebreak = e2_utils_LF_line_ends (cont);
+	rt->linebreak = e2_utils_LF_line_ends ((gchar *)contents);
 	//fix character encoding if necessary and possible
 	if (converter == NULL)	//not externally converted
 	{
-		if (*cont != '\0')
+		if (*((gchar *)contents) != '\0')
 		{
 			gchar *format = _("Conversion from %s encoding failed: \"%s\"");
 			gchar *msg, *utf;
 			GError *utf_error;
-			usable = e2_utf8_detect_charset (cont, &rt->charset);
+			usable = e2_utf8_detect_charset ((guchar *)contents, &rt->charset);
 			if (!usable)
 			{	//convert using the identified encoding if we can
 				//but no point in 'non-conversion' ...
 				if (strstr (rt->charset, "UTF-8") == NULL)
 				{
 					utf_error = NULL;
-					utf = g_convert (cont, -1, "UTF-8", rt->charset, NULL, NULL, &utf_error);
+					utf = g_convert ((gchar *)contents, -1, "UTF-8", rt->charset, NULL, NULL, &utf_error);
 					if (utf != NULL)
 					{
+						printd (DEBUG, "converted content from %s to UTF8", rt->charset);
 						usable = TRUE;
-						g_free (cont);
-						cont = utf;
+						g_free (contents);	//need free() if file buffer allocated by malloc()
+						contents = utf;
 					}
 					else
 					{
+						printd (DEBUG, "FAILED conversion of content from %s to UTF8", rt->charset);
 						if (!g_str_equal (rt->charset, "CP1252"))	//that's generally a default when we can't find the real value
 						{
 							msg = g_strdup_printf (format, rt->charset, utf_error->message);
+							gdk_threads_enter ();
 							e2_output_print_error (msg, TRUE);
+							gdk_threads_leave ();
 						}
 						g_error_free (utf_error);
 					}
@@ -515,24 +548,26 @@ gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
 					defset = "ISO-8859-1";
 				printd (DEBUG, "Conversion from %s encoding to be attempted", defset);
 				utf_error = NULL;
-				utf = g_convert (cont, -1, "UTF-8", defset, NULL, NULL, &utf_error);
+				utf = g_convert ((gchar *)contents, -1, "UTF-8", defset, NULL, NULL, &utf_error);
 				if (utf == NULL)
 				{
 					msg = g_strdup_printf (format, defset, utf_error->message);
+					gdk_threads_enter ();
 					e2_output_print_error (msg, TRUE);
+					gdk_threads_leave ();
 					g_error_free (utf_error);
-					g_free (cont);
+					g_free (contents);	//need free() if file buffer allocated by malloc()
 					return FALSE;
 				}
 				else if (!g_utf8_validate (utf, -1, NULL))
 				{
 					g_free (utf);
-					g_free (cont);
+					g_free (contents);	//need free() if file buffer allocated by malloc()
 					return FALSE;
 				}
 				rt->charset = "UNKNOWN";	//prevent attempts at saving to the unusable charset
-				g_free (cont);
-				cont = utf;
+				g_free (contents);	//need free() if file buffer allocated by malloc()
+				contents = utf;
 			}
 		}
 		else //assume default encoding for empty files
@@ -541,14 +576,19 @@ gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
 //			printd (DEBUG, "empty file, assume default charset %s", rt->charset);
 		}
 	}
-	rt->filepath = file;	//we're ok, so remember the real path
+	if (rt->localpath != NULL)
+		g_free (rt->localpath);
+	rt->localpath = localpath;	//we're ok, so remember the real path
+#ifdef E2_VFS
+	rt->spacedata = localfile->spacedata;
+#endif
 
 	//for some reason, the buffer sometimes takes a while to be created
 	while (!GTK_IS_TEXT_BUFFER (rt->textbuffer))
 		rt->textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (rt->textview));
 
-	gtk_text_buffer_set_text (rt->textbuffer, cont, -1);
-	g_free (cont);
+	gtk_text_buffer_set_text (rt->textbuffer, (gchar *)contents, -1);
+	g_free (contents);
 
 	//put cursor at start of buffer, for searching etc
 	GtkTextIter start;
@@ -557,7 +597,6 @@ gboolean e2_view_dialog_read_text (E2_ViewDialogRuntime *rt)
 
 	printd (DEBUG, "Read text file charset is %s", rt->charset);
 
-	F_FREE (local);
 	return TRUE;
 }
 /**
@@ -617,7 +656,7 @@ void e2_view_dialog_set_menu_position (GtkWidget *menu,
 	GtkAllocation alloc = rt->textview->allocation;
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y + alloc.height/2;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 #ifdef USE_GTK2_10
 /**
@@ -673,9 +712,11 @@ static void _e2_view_dialog_begin_print_cb (GtkPrintOperation *op,
 			(GDestroyNotify) g_object_unref);	//cleanup when context is destroyed
 		pango_layout_set_font_description (layout_name, font_desc);
 		pango_layout_set_width (layout_name, (gint) page_width * PANGO_SCALE);
-		gchar *page_title = g_path_get_basename (rt->filepath);
+		gchar *base = g_path_get_basename (rt->localpath);
+		gchar *page_title = F_FILENAME_FROM_LOCALE (base);
 		pango_layout_set_text (layout_name, page_title, -1);
-		g_free (page_title);
+		g_free (base);
+		F_FREE (page_title);
 		//and for page counter, if > 1 page
 		if (page_count > 1)
 		{
@@ -886,7 +927,7 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 
 /*	GtkWidget *item = e2_menu_add (menu, _("_Find"), GTK_STOCK_FIND,
 		NULL,  _e2_view_dialog_menu_find_cb, rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	e2_widget_set_toggletip (
 #else
 	gtk_tooltips_set_tip (app.tooltips,
@@ -955,7 +996,7 @@ GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
 	//as a whole and to any other element of it!!!
 	g_signal_connect_after (G_OBJECT (entry), "key-release-event",
 		G_CALLBACK (e2_view_dialog_combokey_cb), rt);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1000,7 +1041,7 @@ GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
 
 @return hbox for the searchbar
 */
-void e2_view_dialog_show_atlast (gchar *filepath, E2_ViewDialogRuntime *rt)
+void e2_view_dialog_show_atlast (VPATH *localpath, E2_ViewDialogRuntime *rt)
 {
 	GtkTextIter top;
 	GList *iterator;
@@ -1008,7 +1049,10 @@ void e2_view_dialog_show_atlast (gchar *filepath, E2_ViewDialogRuntime *rt)
 	for (iterator = view_history; iterator != NULL; iterator = iterator->next)
 	{
 		viewed = (E2_ViewHistory *) iterator->data;
-		if (g_str_equal (viewed->filepath, filepath))
+#ifdef E2_VFSTMP
+		compare spacedata ?
+#endif
+		if (g_str_equal (viewed->localpath, VPSTR (localpath)))
 			break;
 	}
 	if (iterator != NULL)
@@ -1062,7 +1106,7 @@ void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
 	for (iterator = view_history; iterator != NULL; iterator = iterator->next)
 	{
 		viewed = (E2_ViewHistory *) iterator->data;
-		if (g_str_equal (viewed->filepath, rt->filepath))
+		if (g_str_equal (viewed->localpath, rt->localpath))
 			break;
 	}
 	if (iterator == NULL)
@@ -1071,7 +1115,10 @@ void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
 		CHECKALLOCATEDWARN (viewed, );
 		if (viewed != NULL)
 		{
-			viewed->filepath = g_strdup (rt->filepath);
+#ifdef E2_VFSTMP
+			spacedata needed ?
+#endif
+			viewed->localpath = g_strdup (rt->localpath);
 			view_history = g_list_append (view_history, viewed);
 		}
 	}
@@ -1108,11 +1155,11 @@ void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
 	g_free (rt->key_binding);
 #endif
 	gtk_widget_destroy (rt->dialog);
-	g_free (rt->filepath);
-	if (rt->idle_id != 0)
-		g_source_remove (rt->idle_id);
-	DEALLOCATE (E2_ViewDialogRuntime, rt);
 	gtk_widget_grab_focus (curr_view->treeview);	//CHECKME consistency ok ?
+	g_free (rt->localpath);
+//	if (rt->idle_id != 0)
+//		g_source_remove (rt->idle_id);
+	DEALLOCATE (E2_ViewDialogRuntime, rt);
 }
 
   /*********************/
@@ -1411,9 +1458,9 @@ static void _e2_view_dialog_keybindings (E2_OptionSet *set)
 	//the category string(s) here need to match the binding name
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 	g_strconcat(_C(11),"||||",NULL),  //_("dialogs"
-	g_strconcat("\t",_A(92),"||||",NULL),  //_("view"
-	g_strconcat("\t\t|<Control>j","||",_A(108),".",_A(109),"|<Control>a",NULL),
-	g_strconcat("\t\t|<Control>k","||",_A(108),".",_A(109),"|<Control>c",NULL),
+	g_strconcat("\t",_A(101),"||||",NULL),  //_("view"
+	g_strconcat("\t\t|<Control>j","||",_A(119),".",_A(120),"|<Control>a",NULL),
+	g_strconcat("\t\t|<Control>k","||",_A(119),".",_A(120),"|<Control>c",NULL),
 	g_strdup(">"),
 	NULL);
 }
@@ -1428,12 +1475,12 @@ static void _e2_view_dialog_keybindings (E2_OptionSet *set)
 
 Expects BGL closed when we arrive here
 
-@param filename utf8 string which has path & name of item to be processed
+@param localpath localised string which has path & name of item to be processed
 @param srt pointer to store for returning dialog data, or NULL
 
 @return the dialog widget, or NULL if there's a problem
 */
-static GtkWidget *_e2_view_dialog_create (gchar *filename,
+static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	E2_ViewDialogRuntime **srt)
 {
 	//init view runtime object (0's to ensure edit-dialog things are NULL)
@@ -1445,11 +1492,10 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 			*srt = NULL;
 		return NULL;
 	}
-	rt->filepath = filename;	//interim store, for the file-read func
 	rt->textview = gtk_text_view_new ();
 	// add the item's content
 	gdk_threads_leave ();	//any downstream error message does its own mutex management
-	if (!e2_view_dialog_read_text (rt))
+	if (!e2_view_dialog_read_text (localpath, rt))
 	{
 		gdk_threads_enter ();
 		gtk_widget_destroy (rt->textview);
@@ -1468,8 +1514,10 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 	e2_view_dialog_init_hilites (rt);
 #endif
 
-	rt->dialog = e2_dialog_create (NULL, rt->filepath, _("displaying file"),
+	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
+	rt->dialog = e2_dialog_create (NULL, utf, _("displaying file"),
 		_e2_view_dialog_response_cb, rt);
+	F_FREE (utf);
 	//override some default label properties
 	GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog),
 		"e2-dialog-label");
@@ -1495,7 +1543,7 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 	e2_view_dialog_set_font (&char_width, &char_height, rt);
 	rt->window_width = e2_option_int_get ("dialog-view-width");
 	rt->window_height = e2_option_int_get ("dialog-view-height");;
-	rt->idle_id = 0;
+//	rt->idle_id = 0;
 	//init search runtime data
 	rt->history = e2_list_copy_with_data (find_history);
 
@@ -1531,7 +1579,7 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 	labeltext = _("_Hide");
 	rt->hidebtn = e2_dialog_add_undefined_button
 		(rt->dialog, GTK_STOCK_ZOOM_FIT, labeltext, E2_RESPONSE_USER3);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	gtk_widget_set_tooltip_text (
 #else
 	e2_widget_set_tooltip (NULL,
@@ -1548,7 +1596,7 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 	labeltext = _("_Find");
 	rt->findbtn = e2_dialog_add_undefined_button
 		(rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 	e2_widget_set_toggletip (
 #else
 	gtk_tooltips_set_tip (app.tooltips,
@@ -1572,8 +1620,8 @@ static GtkWidget *_e2_view_dialog_create (gchar *filename,
 #ifdef E2_TRANSIENTKEYS
 	//add dialog-specific key bindings, before the key-press callback
 	//group name (must be freeable)
-//	rt->key_binding = g_strconcat (_C(17), ".", _A(92), NULL);	//_(general.view
-	rt->key_binding = g_strconcat (_C(11), ".", _A(92), NULL);	//_(dialogs.view
+//	rt->key_binding = g_strconcat (_C(17), ".", _A(101), NULL);	//_(general.view
+	rt->key_binding = g_strconcat (_C(11), ".", _A(101), NULL);	//_(dialogs.view
 	e2_keybinding_register_transient (rt->key_binding, GTK_WIDGET (rt->textview),
 		_e2_view_dialog_keybindings);
 #endif
@@ -1615,32 +1663,45 @@ static gboolean _e2_view_dialog_reviewQ (E2_ActionTaskData *qed)
 	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
 	//".." entries filtered when names compiled
 	//for name comparisons, we need the full path
-	gchar *local = e2_utils_strcat (qed->currdir, (*iterator)->filename);
-	gchar *view_this = D_FILENAME_FROM_LOCALE (local);
-	g_free (local);
+	gchar *localpath = e2_utils_strcat (qed->currdir, (*iterator)->filename);
+#ifdef E2_VFS
+	VPATH ddata = { localpath, qed->currspace };
+#endif
 	E2_ViewDialogRuntime *vrt;
+
 	gdk_threads_enter ();
-	GtkWidget *dialog = _e2_view_dialog_create (view_this, &vrt);
+	GtkWidget *dialog = _e2_view_dialog_create
+#ifdef E2_VFS
+		(&ddata, &vrt);
+#else
+		(localpath, &vrt);
+#endif
 	gdk_threads_leave ();
 	if (dialog != NULL)
 	{
+		WAIT_FOR_EVENTS_UNLOCKED_SLOWLY
 		gdk_threads_enter ();
-		e2_view_dialog_show_atlast (view_this, vrt);
+#ifdef E2_VFS
+		e2_view_dialog_show_atlast (&ddata, vrt);
+#else
+		e2_view_dialog_show_atlast (localpath, vrt);
+#endif
 		gdk_threads_leave ();
-		g_free (view_this);
+		g_free (localpath);
 		return TRUE;
 	}
-	g_free (view_this);
+	g_free (localpath);
 	return FALSE;
 }
 /**
 @brief open file for viewing, at a location where there is a specified string
 @a art includes a string with filepath and an argument that is a text string
-to find in the file
+to find in the file. Separator between path and find-target is the first unquoted
+unescaped ' ' char (not a \t)
 This returns immediately, so if run in a Q-thread, that will end
 @param from the widget that was activated to initiate the action
 @param art runtime data for the action
-@return TRUE
+@return TRUE if dialog was created
 */
 static gboolean _e2_view_dialog_openat (gpointer from, E2_ActionRuntime *art)
 {
@@ -1649,23 +1710,32 @@ static gboolean _e2_view_dialog_openat (gpointer from, E2_ActionRuntime *art)
 }
 static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 {
+	//for a help doc, has localised doc path, and a "[title]" arg in undefined encoding
 	gchar *view_this = (gchar *) qed->rt_data;
-	gchar *target = e2_utils_find_whitespace (view_this);	//always ascii ' ', don't need g_utf8_strchr()
-//#warning ignore compiler warning about unitialized usage of c
-	gchar c;
+//	gchar c;
+	//strip off any arg FIXME doc path may have whitespace
+	gchar *target = e2_utils_bare_strchr (view_this, ' ');	//always ascii ' ', don't need g_utf8_strchr()
 	if (target != NULL)
 	{
-		c = *target;
+//		c = *target;
 		*target = '\0';
 	}
-
+#ifdef E2_VFS
+	VPATH ddata = { view_this, qed->currspace };
+#endif
 	E2_ViewDialogRuntime *vrt;
-	gdk_threads_enter ();
-	GtkWidget *dialog = _e2_view_dialog_create (view_this, &vrt);
+	gdk_threads_enter ();	//dialog setup wants BGL closed
+	GtkWidget *dialog = _e2_view_dialog_create
+#ifdef E2_VFS
+ 		(&ddata, &vrt);
+#else
+		(view_this, &vrt);
+#endif
 	gdk_threads_leave ();
 
 	if (target != NULL)
-		*target = c;	//revert the argument, ready for next time
+//		*target = c;	//revert the argument, ready for next time
+		*target = ' ';	//revert the argument, ready for next time
 
 	if (dialog != NULL)
 	{
@@ -1675,31 +1745,29 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 		{	//find the target string and open there
 			GtkTextIter iter, start, end;
 			gtk_text_buffer_get_start_iter (vrt->textbuffer, &iter);
-			gdk_threads_enter ();
 			if (gtk_text_iter_forward_search (&iter, target, 0, &start, &end, NULL))
 			{	//need to add a mark to get the scroll to work properly
+				//FIXME sometimes it scrolls a bit too far
 				GtkTextMark *mark = gtk_text_buffer_create_mark
 					(vrt->textbuffer, NULL, &start, FALSE);
-				//FIXME with ASYNC, it does not always scroll to the correct place
-				gdk_threads_leave ();
-				usleep (20000);	//pause a while for gtk to do its setups ?
+				WAIT_FOR_EVENTS_UNLOCKED
 				gdk_threads_enter ();
-
-				gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (vrt->textview), mark,
-					0.0, TRUE, 0.0, 0.1);
 				//to support incremental searching, move the cursor
 				//to the start of the located string
 				gtk_text_buffer_place_cursor (vrt->textbuffer, &start);
+				gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (vrt->textview), mark,
+					0.0, TRUE, 0.0, 0.2);
+				gdk_threads_leave ();
 			}
-			gdk_threads_leave ();
 		}
 		gdk_threads_enter ();
 		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL,
 			NULL);
 		gtk_window_present (GTK_WINDOW (dialog));
 		gdk_threads_leave ();
+		return TRUE;
 	}
-	return TRUE;
+	return FALSE;
 }
 
   /******************/
@@ -1708,19 +1776,21 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 
 /**
 @brief create and show view dialog
-@a filename may not have any path, or an absolute path. In those
-cases, curr_view->dir will be prepended
+@a localpath may not have any path, or an absolute path. In those cases,
+curr_view->dir will be prepended
 This returns immediately, so if run in a Q-thread, that will end
-@param filename utf8 string which has at least the name of item to be processed
+@param localpath localised string which has at least the name of item to be processed
+
 @return TRUE if the dialog was created
 */
-gboolean e2_view_dialog_create (gchar *filename)
+gboolean e2_view_dialog_create (VPATH *localpath)
 {
 	//there is no async thread protection here or downstream,, as this func can
 	//be called from different contexts. Such protection, if needed, must be done by the caller
-	GtkWidget *dialog = _e2_view_dialog_create (filename, NULL);
+	GtkWidget *dialog = _e2_view_dialog_create (localpath, NULL);
 	if (dialog != NULL)
 	{
+		WAIT_FOR_EVENTS_SLOWLY
 		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
 		gtk_window_present (GTK_WINDOW (dialog));
 		return TRUE;
@@ -1729,14 +1799,18 @@ gboolean e2_view_dialog_create (gchar *filename)
 }
 /**
 @brief create and show non-queued, non-logged view dialog, for user help
-This must be called only from inside BGL
-@param view_this string with help doc name and heading name
+Expects BGL on/closed
+@param view_this string with localised path of help doc, AND a heading name
+ (in undefined encoding) as its argument
 @return TRUE if the dialog was created
 */
-gboolean e2_view_dialog_create_immediate (gchar *view_this)
+gboolean e2_view_dialog_create_immediate (VPATH *view_this)
 {
 	E2_ActionTaskData qed;
-	qed.rt_data = view_this;
+	qed.rt_data = VPSTR (view_this);
+#ifdef E2_VFS
+	qed.currspace = view_this->spacedata;
+#endif
 	gdk_threads_leave ();	//turn off the BGL
 	gboolean retval = _e2_view_dialog_viewatQ (&qed);
 	gdk_threads_enter ();
@@ -1755,10 +1829,10 @@ void e2_view_dialog_actions_register (void)
 	if (e2_option_bool_get ("dialog-search-history"))
 		e2_cache_list_register ("search-history", &find_history);
 
-	gchar *action_name = g_strconcat(_A(5),".",_A(93),NULL);
+	gchar *action_name = g_strconcat(_A(5),".",_A(102),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_view_dialog_reopen, NULL, FALSE);
-	action_name = g_strconcat(_A(5),".",_A(94),NULL);
+	action_name = g_strconcat(_A(5),".",_A(103),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_view_dialog_openat, NULL, TRUE);
 }
