@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_bookmark.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -204,9 +204,9 @@ static void _e2_bookmark_add_cb (GtkWidget *widget, gchar *arg)
 			pane = E2PANE1;
 		else if (strstr (arg, _A(12)))  //_(pane2
 			pane = E2PANE2;
-		if (strstr (arg, _A(105)))  //_(top
+		if (strstr (arg, _A(114)))  //_(top
 			before = TRUE;
-		if (strstr (arg, _A(95))) //_(child
+		if (strstr (arg, _A(104))) //_(child
 		{
 			parent = TRUE;
 			before = TRUE;
@@ -352,7 +352,7 @@ gboolean e2_bookmark_click_cb (GtkWidget *widget,
 #ifdef E2_VFSTMP
 			//FIXME bookmark parsing and setting for v-dirs
 						if (0)
-							e2_pane_change_space (other_pane, "some string");
+							e2_pane_change_space_byuri (other_pane, "some string");
 #endif
 						e2_pane_change_dir (other_pane, mark);	//CHECKME mark utf8
 						if (e2_option_bool_get ("bookmarks-focus-after-open"))
@@ -381,7 +381,7 @@ gboolean e2_bookmark_click_cb (GtkWidget *widget,
 
 			menuitem = e2_menu_add (menu, _("Add as _child"), GTK_STOCK_INDENT,
 				_("Bookmark the current directory a a child of the selected bookmark"),
-				_e2_bookmark_add_cb, _A(95));	//_(child
+				_e2_bookmark_add_cb, _A(104));	//_(child
 			g_object_set_data (G_OBJECT (menuitem), "parent-mark", path);
 			//pass through the pane no. (or NULL for default) to use when adding an item
 			g_object_set_data (G_OBJECT (menuitem), "bookmark-pane", pane);
@@ -466,7 +466,7 @@ gboolean e2_bookmark_open (gpointer from, E2_ActionRuntime *art)
 #ifdef E2_VFSTMP
 		//FIXME handle case where pane is non-mounted
 			if (0)
-				e2_pane_change_space (&app.pane1, "someplace");
+				e2_pane_change_space_byuri (&app.pane1, "someplace");
 #endif
 			e2_pane_change_dir (&app.pane1, path);
 			if (e2_option_bool_get ("bookmarks-focus-after-open")
@@ -477,7 +477,7 @@ gboolean e2_bookmark_open (gpointer from, E2_ActionRuntime *art)
 #ifdef E2_VFSTMP
 		//FIXME handle case where pane is non-mounted
 			if (0)
-				e2_pane_change_space (&app.pane2, "someplace");
+				e2_pane_change_space_byuri (&app.pane2, "someplace");
 #endif
 			e2_pane_change_dir (&app.pane2, path);
 			if (e2_option_bool_get ("bookmarks-focus-after-open")
@@ -488,7 +488,7 @@ gboolean e2_bookmark_open (gpointer from, E2_ActionRuntime *art)
 #ifdef E2_VFSTMP
 		//FIXME handle case where pane is non-mounted
 			if (0)
-				e2_pane_change_space (curr_pane, "someplace");
+				e2_pane_change_space_byuri (curr_pane, "someplace");
 #endif
 			e2_pane_change_dir (NULL, path);
 			break;
