@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_bookmark.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_BOOKMARK_H
-#define E2_BOOKMARK_H
+#ifndef __E2_BOOKMARK_H__
+#define __E2_BOOKMARK_H__
 
 #include "emelfm2.h"
 
@@ -32,4 +32,4 @@ gboolean e2_bookmark_click_cb (GtkWidget *widget,
 void e2_bookmark_actions_register ();
 void e2_bookmark_options_register ();
 
-#endif //ndef E2_BOOKMARK_H
+#endif //ndef __E2_BOOKMARK_H__
