@@ -1,6 +1,6 @@
-/* $Id: e2_context_menu.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_context_menu.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -212,19 +212,30 @@ static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
 	{
 		//this is a tougher exec test than used in context menu
 //		exec = e2_fs_is_executable (info, curr_view); //runs 'file' which prompts a refresh
-		gchar *dir = e2_fs_get_localdir (curr_view);
-		if (dir == NULL)
-			return;
-		gchar *localpath = g_build_filename (dir, info->filename, NULL);
+		gchar *localpath = e2_utils_dircat (curr_view, info->filename, TRUE);
+#ifdef E2_VFS
+#ifdef E2_VFSTMP
+		//CHECKME allow exec options for non-local items ?
+#endif
+		VPATH ddata = { localpath, NULL };
+		if (curr_view->spacedata != NULL)
+			exec = FALSE;//CHECKME commands for non-local items ?
+		else
+			exec = !e2_fs_access (&ddata, X_OK E2_ERR_NONE());
+#else
 		exec = !e2_fs_access (localpath, X_OK E2_ERR_NONE());
-		g_free (dir);
+#endif
 		g_free (localpath);
 		freeext = FALSE;
 		//assumes extension is that part of the name after the leftmost '.'
 		ext = strchr (info->filename, '.');	//localised text, but assumes . is ascii
 		if (ext == info->filename //it's a dot file
 			|| (ext == NULL //no extension
+#ifdef E2_VFS
+				&& e2_fs_is_text (&ddata E2_ERR_NONE()))) //runs 'file' again!!
+#else
 				&& e2_fs_is_text (info->filename E2_ERR_NONE()))) //runs 'file' again!!
+#endif
 			//fake text extension
 			ext = ".txt";	//too bad if this is not a recognised text extension in filetypes
 		else if (ext != NULL)
@@ -274,7 +285,12 @@ static void _e2_context_menu_add_filetype_item (GtkWidget *menu)
 		{
 			path = g_build_filename (*sysdir, "applications", localname, NULL);
 			localpath = F_FILENAME_TO_LOCALE (path); //probably redundant
+#ifdef E2_VFS
+			VPATH data = { localpath, NULL };	//always local
+			if (!e2_fs_access (&data, R_OK E2_ERR_NONE()))
+#else
 			if (!e2_fs_access (localpath, R_OK E2_ERR_NONE()))
+#endif
 			{
 				e2_context_menu_create_desktop_actions_menu (menu, localpath);
 				g_free (path);
@@ -512,9 +528,9 @@ gboolean e2_context_menu_show_menu_action (gpointer from, E2_ActionRuntime *art)
 	if (arg != NULL)
 	{
 		g_strstrip (arg);
-		if (g_str_equal (arg, _A(103)))	//_(shift
+		if (g_str_equal (arg, _A(112)))	//_(shift
 			type = 1;
-		else if (g_str_equal (arg, _A(96)))	//_(ctrl
+		else if (g_str_equal (arg, _A(105)))	//_(ctrl
 			type = 2;
 		//CHECKME alt option here ?
 	}
@@ -534,26 +550,26 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 	g_strdup("context-menu=<"),  //internal name
 	g_strconcat("||false|false|",_A(5),".",_A(20),"|",NULL),
 	g_strconcat(_("Open _with.."),"|open_with_"E2IP".png|false|false|",
-		_A(5),".",_A(61),"|",NULL),
+		_A(5),".",_A(62),"|",NULL),
 	g_strconcat(_("_View"),"|view_"E2IP".png|false|false|",
-		_A(5),".",_A(92),"|",NULL),
-	g_strconcat(_("_Info"),"|gtk-dialog-info|false|false|",_A(5),".",_A(51),"|",NULL),
+		_A(5),".",_A(101),"|",NULL),
+	g_strconcat(_("_Info"),"|gtk-dialog-info|false|false|",_A(5),".",_A(52),"|",NULL),
 	g_strconcat("||false|false|",_A(18),"|",NULL),
 	g_strconcat(_("_Actions"),"|gtk-execute|false|false|",_A(19),"|",NULL),
 	g_strconcat("\t",_("_Copy"),"|gtk-copy|true|false|",_A(5),".",_A(33),"|",NULL),
-	g_strconcat("\t",_("_Move"),"|move_"E2IP".png|true|false|",_A(5),".",_A(57),"|",NULL),
-	g_strconcat("\t",_("_Link"),"|symlink_"E2IP".png|true|false|",_A(5),".",_A(82),"|",NULL),
-	g_strconcat("\t",_("_Trash"),"|trash_"E2IP".png|true|false|",_A(5),".",_A(88),"|",NULL),
+	g_strconcat("\t",_("_Move"),"|move_"E2IP".png|true|false|",_A(5),".",_A(58),"|",NULL),
+	g_strconcat("\t",_("_Link"),"|symlink_"E2IP".png|true|false|",_A(5),".",_A(91),"|",NULL),
+	g_strconcat("\t",_("_Trash"),"|trash_"E2IP".png|true|false|",_A(5),".",_A(97),"|",NULL),
 	g_strconcat("\t","<span foreground='red'>",_("_Delete"),"</span>|delete_"E2IP".png|true|false|",
 		_A(5),".",_A(38),"|",NULL),
-	g_strconcat("\t",_("_Rename.."),"|gtk-convert|true|false|",_A(5),".",_A(72),"|",NULL),
+	g_strconcat("\t",_("_Rename.."),"|gtk-convert|true|false|",_A(5),".",_A(73),"|",NULL),
 	g_strconcat("\t",_("Change _owners.."),"|owners_"E2IP".png|true|false|",
-		_A(5),".",_A(62),"|",NULL),
+		_A(5),".",_A(63),"|",NULL),
 	g_strconcat("\t",_("Change _permissions.."),"|permissions_"E2IP".png|true|false|",
-		_A(5),".",_A(66),"|",NULL),
+		_A(5),".",_A(67),"|",NULL),
 	g_strconcat("\t",_("Copy as.."),"|copy_as_"E2IP".png|true|false|",_A(5),".",_A(34),"|",NULL),
-	g_strconcat("\t",_("Move as.."),"|move_as_"E2IP".png|true|false|",_A(5),".",_A(58),"|",NULL),
-	g_strconcat("\t",_("Link as.."),"|symlink_as_"E2IP".png|true|false|",_A(5),".",_A(83),"|",NULL),
+	g_strconcat("\t",_("Move as.."),"|move_as_"E2IP".png|true|false|",_A(5),".",_A(59),"|",NULL),
+	g_strconcat("\t",_("Link as.."),"|symlink_as_"E2IP".png|true|false|",_A(5),".",_A(92),"|",NULL),
 	g_strconcat(_("_Plugins"),"|gtk-index|false|false|",_A(19),"|",NULL),
 	g_strconcat("\t||false|false|",_A(14),".",_A(24),"|",NULL),
 	g_strconcat("\t||false|false|",_A(18),"|",NULL),
@@ -569,10 +585,10 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t",_("_Free space"),"||false|true|>stat|-f %d \\| awk '/Blocks/ {printf \"%2.1f ", _("percent free"),"\",$5/$3*100}'",NULL),  //_A(17)
 	g_strconcat("\t||false|true|",_A(18),"|",NULL),
 	g_strconcat("\t",_("_Edit user commands.."),"|gtk-preferences|false|true|",_A(2),".",_A(32),"|",_C(8),NULL),
-	g_strconcat(_("Ma_ke dir.."),"|gtk-open|false|false|",_A(1),".",_A(55),"|",NULL),  //same _ as main toolbar
+	g_strconcat(_("Ma_ke dir.."),"|gtk-open|false|false|",_A(1),".",_A(56),"|",NULL),  //same _ as main toolbar
 	g_strconcat("||false|false|",_A(18),"|",NULL),
 	g_strconcat(_("_Bookmarks"),"|bookmark_"E2IP".png|false|false|",_A(19),"|",NULL),
-	g_strconcat("\t",_("Add _top"),"|add_mark_top_"E2IP".png|false|false|",_A(0),".",_A(26),"|",_A(105),NULL),
+	g_strconcat("\t",_("Add _top"),"|add_mark_top_"E2IP".png|false|false|",_A(0),".",_A(26),"|",_A(114),NULL),
 	g_strconcat("\t||false|false|",_A(0),".",_A(24),"|",NULL),
 	g_strconcat("\t",_("Add _bottom"),"|add_mark_bottom_"E2IP".png|false|false|",_A(0),".",_A(26),"|",NULL),
 	g_strconcat("\t||false|false|",_A(18),"|",NULL),
