@@ -1,6 +1,6 @@
-/* $Id: e2_context_menu.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_context_menu.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
 
 This file is part of emelFM2.
@@ -19,8 +19,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_CONTEXT_MENU_H
-#define E2_CONTEXT_MENU_H
+#ifndef __E2_CONTEXT_MENU_H__
+#define __E2_CONTEXT_MENU_H__
 
 #include "emelfm2.h"
 
@@ -32,4 +32,4 @@ void e2_context_menu_show (guint button, guint32 time, gint type);
 gboolean e2_context_menu_show_menu_action (gpointer from, E2_ActionRuntime *art);
 void e2_context_menu_options_register (void);
 
-#endif //ndef E2_CONTEXT_MENU_H
+#endif //ndef __E2_CONTEXT_MENU_H__
