@@ -1,4 +1,4 @@
-/* $Id: e2_dnd.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_dnd.c 739 2007-11-21 03:27:49Z tpgww $
 
 Copyright (C) 2006-2007 tooar <tooar@gmx.net>.
 
@@ -37,7 +37,7 @@ GtkTargetEntry target_table[] = {
 	{ "text/plain",    0, TARGET_STRING },
 //	{ "XdndDirectSave0", 0, TARGET_XDS } //this last target is supported for drops only
 };
-guint n_targets = sizeof(target_table) / sizeof(target_table[0]);
+guint n_targets = sizeof(target_table) / sizeof(GtkTargetEntry);
 
 //mouse pointer height, used for getting the path at the _top_ of the pointer
 static gint pointer_height;
@@ -140,8 +140,8 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	//must check mouse position here, as e2_dnd_drag_motion_cb() may not be called
 	//often enough when auto-scrolling a large treeview
 	//my and my are for bottom-left of pointer FIXME make it work for top-left
-	gint mx, my;
-	gdk_window_get_pointer (view->treeview->window, &mx, &my, NULL);
+	gint my;
+	gdk_window_get_pointer (view->treeview->window, NULL, &my, NULL);
 
 	if (my >= AUTO_SCROLL_MARGIN && my <= (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
 	{
@@ -179,8 +179,9 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	}
 	if (scrollnow >= 0)
 	{
+		gdk_threads_enter ();
 		gtk_adjustment_set_value (vadj, scrollnow);
-//		gtk_adjustment_value_changed (vadj);	NOT NEEDED
+		gdk_threads_leave ();
 		prior_scroll = scrollnow;
 	}
 	return TRUE;
@@ -304,8 +305,7 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 		}
 	}
 	//arrange auto-scrolling when mouse pointer is near top or bottom of treeview
-//	gint yt = y - pointer_height;	//top of pointer WON'T WORK - WHY ?
-	if (y < AUTO_SCROLL_MARGIN || y > (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
+	if (y < (pointer_height + AUTO_SCROLL_MARGIN) || y > (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
 	{
 		if (app.timers[ASCROLL_T] == 0)
 		{
@@ -687,7 +687,7 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-				dest_dir = e2_utils_dircat (view, filename);
+				dest_dir = e2_utils_dircat (view, filename, FALSE);
 #endif
 			}
 			g_free (filename);
