@@ -1,6 +1,6 @@
-/* $Id: e2_filelist.c 550 2007-07-22 10:30:40Z tpgww $
+/* $Id: e2_filelist.c 856 2008-05-05 02:00:15Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>.
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -50,8 +50,11 @@ extern sigset_t dnotify_signal_set;
 //disable-refresh counter for both filelists
 gint refresh_refcount;
 //disable-refresh counter for single filelists
-gint refresh_active_refcount;
-gint refresh_inactive_refcount;
+static gint refresh_active_refcount;
+static gint refresh_inactive_refcount;
+//refresh deferral counters
+static gint pane1repeats = 0;
+static gint pane2repeats = 0;
 
 //static gboolean case_sensitive; //local copy of option value, for faster access
 
@@ -93,13 +96,21 @@ void e2_filelist_disable_one_refresh (E2_ListChoice pane)
 	{
 		refresh_active_refcount++;
 		if (refresh_active_refcount == 1)
-			curr_view->blocked = TRUE;
+		{
+			LISTS_LOCK
+			curr_view->listcontrols.norefresh = TRUE;
+			LISTS_UNLOCK
+		}
 	}
 	else
 	{
 		refresh_inactive_refcount++;
 		if (refresh_inactive_refcount == 1)
-			other_view->blocked = TRUE;
+		{
+			LISTS_LOCK
+			other_view->listcontrols.norefresh = TRUE;
+			LISTS_UNLOCK
+		}
 	}
 }
 /**
@@ -115,7 +126,11 @@ void e2_filelist_enable_one_refresh (E2_ListChoice pane)
 	{
 		refresh_active_refcount--;
 		if (refresh_active_refcount == 0 && refresh_refcount == 0)
-			curr_view->blocked = FALSE;
+		{
+			LISTS_LOCK
+			curr_view->listcontrols.norefresh = FALSE;
+			LISTS_UNLOCK
+		}
 		if (refresh_active_refcount < 0)
 			refresh_active_refcount = 0;
 	}
@@ -123,7 +138,11 @@ void e2_filelist_enable_one_refresh (E2_ListChoice pane)
 	{
 		refresh_inactive_refcount--;
 		if (refresh_inactive_refcount == 0 && refresh_refcount == 0)
-			other_view->blocked = FALSE;
+		{
+			LISTS_LOCK
+			other_view->listcontrols.norefresh = FALSE;
+			LISTS_UNLOCK
+		}
 		if (refresh_inactive_refcount < 0)
 			refresh_inactive_refcount = 0;
 	}
@@ -141,12 +160,38 @@ gboolean e2_filelist_disable_refresh_action (gpointer from, E2_ActionRuntime *ar
 	e2_filelist_disable_refresh ();
 	return TRUE;
 }
+/**
+@brief enable -refresh action
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE always
+*/
+gboolean e2_filelist_enable_refresh_action (gpointer from, E2_ActionRuntime *art)
+{
+	e2_filelist_enable_refresh ();
+	return TRUE;
+}
+/**
+@brief reset polling of config data and pane filelists
+
+@return
+*/
+void e2_filelist_reset_refresh (void)
+{
+	if (refresh_refcount != 0)
+	{
+		refresh_refcount = 1;
+		e2_filelist_enable_refresh ();
+	}
+}
 /* These functions must encapsulate any instructions that access the FileInfo
 structs produced when a selection is interrogated, because a refresh in the
 middle of such an operation would free FileInfo structs as a side-effect.
 */
 /**
-@brief  disable refreshing of config data and pane filelists
+@brief  disable refreshing of pane filelists and config data
 
 Increase ref count, if == 1, stop the timer process which calls the refresh fns
 Why is config data check bundled here ?
@@ -171,8 +216,10 @@ void e2_filelist_disable_refresh (void)
 		{
 			//don't cancel check-dirty timer, to reduce risk that FAM
 			//dirty-reports queue may over-fill
-			curr_view->blocked = TRUE;
-			other_view->blocked = TRUE;
+			LISTS_LOCK
+			curr_view->listcontrols.norefresh = TRUE;
+			other_view->listcontrols.norefresh = TRUE;
+			LISTS_UNLOCK
 		}
 		if (e2_option_bool_get ("auto-refresh-config"))
 		{
@@ -185,79 +232,21 @@ void e2_filelist_disable_refresh (void)
 			}
 		}
 
+#ifndef E2_STATUS_DEMAND
 		e2_window_disable_status_update ();
+#endif
 	}
 }
 /**
-@brief  cause polling of config data and pane filelists at specified intervals
+@brief re-enable polling of pane filelists and config data
 
-Polling (re)starts after the interval
-reference count is back to 0
-
-@param interval  milliseconds between polls
-
-@return
-*/
-/*void e2_filelist_set_refresh (guint interval)
-{
-	e2_filelist_disable_refresh ();
-	e2_filelist_enable_refresh_custom (interval);
-}
-
-void e2_filelist_set_refresh_default (void)
-{
-	e2_filelist_disable_refresh ();
-	e2_filelist_enable_refresh_custom (E2_FILESCHECK_INTERVAL);
-} */
-/**
-@brief enable -refresh action
-
-@param from the button, menu item etc which was activated
-@param art action runtime data
-
-@return TRUE always
-*/
-gboolean e2_filelist_enable_refresh_action (gpointer from, E2_ActionRuntime *art)
-{
-	e2_filelist_enable_refresh_custom (E2_FILESCHECK_INTERVAL);
-	return TRUE;
-}
-/**
-@brief reset polling of config data and pane filelists
-
-@return
-*/
-void e2_filelist_reset_refresh (void)
-{
-	if (refresh_refcount > 0)
-	{
-		refresh_refcount = 1;
-		e2_filelist_enable_refresh_custom (E2_FILESCHECK_INTERVAL);
-	}
-}
-/**
-@brief re-enable polling of config data and pane filelists
-
-Polling (re)starts after the default 1-second interval, if the
-reference count is back to 0
+Decrease ref count, if == 0, re-enable/restart the processes which detect
+changes of filelist content and/or config data
+Config timer is bundled here because the same func refreshes the panes
 
 @return
 */
 void e2_filelist_enable_refresh (void)
-{
-	e2_filelist_enable_refresh_custom (E2_FILESCHECK_INTERVAL);
-}
-/**
-@brief  re-enable polling of config data and pane filelists, with a specified interval
-
-Decrease ref count, if == 0, restart the timers which call the refresh fns
- Config timer is bundled here because the fn refreshes the panes
-
-@param interval milliseconds between polls
-
-@return
-*/
-void e2_filelist_enable_refresh_custom (guint interval)
 {
 	refresh_refcount--;
 #ifdef E2_REFRESH_DEBUG
@@ -277,16 +266,24 @@ void e2_filelist_enable_refresh_custom (guint interval)
 	{
 		if (e2_option_bool_get ("auto-refresh"))
 		{
-			curr_view->blocked = FALSE;
-			other_view->blocked = FALSE;
+			LISTS_LOCK
+			curr_view->listcontrols.norefresh = FALSE;
+			other_view->listcontrols.norefresh = FALSE;
+			LISTS_UNLOCK
 		}
-//		printd (DEBUG, "refresh enabled, starting auto-refresh timer");
 		//this timer is used, even with fam backends
 		if (e2_option_bool_get ("auto-refresh-config"))
-			app.timers[CONFIG_T] = g_timeout_add (E2_CONFIGCHECK_INTERVAL,
+			app.timers[CONFIG_T] =
+#ifdef USE_GLIB2_14
+			g_timeout_add_seconds (E2_CONFIGCHECK_INTERVAL_S,
+#else
+			g_timeout_add (E2_CONFIGCHECK_INTERVAL,
+#endif
 				(GSourceFunc) e2_option_check_config_files, NULL);
 
+#ifndef E2_STATUS_DEMAND
 		e2_window_enable_status_update (-1);
+#endif
 	}
 }
 /**
@@ -299,52 +296,106 @@ static void _e2_filelist_timer_shutdown (gpointer data)
 	app.timers[REFRESHBEGIN_T] = 0;
 }
 /**
-@brief timer callback function which refreshes filelists as appropriate, as soon as any block is gone
+@brief timer callback function which refreshes filelist(s) as appropriate, as soon as any block is gone
 @param data pointerised value of current timer delay
-@return FALSE when there's nothing left to refresh
+@return FALSE when there's nothing left to refresh or more waiting is needed
 */
-static gboolean _e2_filelist_check_requests (gpointer data)
+static gboolean _e2_filelist_refresh_manage (gpointer data)
 {
-	//parallel filelist updates not supported
-	if (!curr_view->completed || !other_view->completed)
+	pthread_t thisID;
+	gpointer result;
+	gboolean cvdone, ovdone;
+	LISTS_LOCK
+	ViewInfo *cv = curr_view;	//log these, in case active pane is changed during this process
+	ViewInfo *ov = other_view;
+retest:
+	if (cv->listcontrols.refresh_requested)
 	{
-		printd (DEBUG, "list(s) refresh deferrred");
-		//if appropriate, lengthen the delay between attempts
-		if (data < GINT_TO_POINTER (200))
+		cvdone = !(
+		cv->listcontrols.norefresh || //norefresh flags set when refresh is disabled
+		cv->listcontrols.refresh_working ||
+		cv->listcontrols.cd_working
+#ifdef E2_STATUS_BLOCK
+		|| app.status_working
+#endif
+		);
+		if (cvdone)
 		{
-			g_source_remove (app.timers[REFRESHBEGIN_T]);
-			app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 200,
-				(GSourceFunc) _e2_filelist_check_requests, GINT_TO_POINTER (200),
-					(GDestroyNotify) _e2_filelist_timer_shutdown);
+			LISTS_UNLOCK
+#ifdef E2_VFSTMP
+			//FIXME dir when not mounted local
+#else
+			printd (DEBUG, "accepting request to refresh %s", cv->dir);
+#endif
+			//refresh function expects BGL open
+			if (pthread_create (&thisID, NULL,
+				(gpointer) e2_fileview_refresh_list, cv) == 0)
+					pthread_join (thisID, &result);
+			else
+				printd (WARN, "Failed to create list refresh thread");
+			LISTS_LOCK
+			//clear flag, too bad if a request happened late in the process
+			if (result != NULL)
+				cv->listcontrols.refresh_requested = FALSE;
 		}
-		return TRUE;
 	}
-	//blocked flags set when refresh is disabled
-	if (curr_view->refresh_requested && !curr_view->blocked)
+	else
+		cvdone = TRUE;
+retest2:
+	if (ov->listcontrols.refresh_requested)
 	{
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
-		printd (DEBUG, "accepting request to refresh %s", curr_view->dir);
+		ovdone = !(
+		ov->listcontrols.norefresh ||
+		ov->listcontrols.refresh_working ||
+		ov->listcontrols.cd_working
+#ifdef E2_STATUS_BLOCK
+		|| app.status_working
 #endif
-		//refresh function expects BGL closed
-		gdk_threads_enter ();
-		e2_fileview_refresh_list (curr_view);
-		gdk_threads_leave ();
-		curr_view->refresh_requested = FALSE;
+		);
+		if (ovdone)
+		{
+			LISTS_UNLOCK
+#ifdef E2_VFSTMP
+		//FIXME dir when not mounted local
+#else
+			printd (DEBUG, "accepting request to refresh %s", ov->dir);
+#endif
+			if (pthread_create (&thisID, NULL,
+				(gpointer) e2_fileview_refresh_list, ov) == 0)
+					pthread_join (thisID, &result);
+			else
+				printd (WARN, "Failed to create list refresh thread");
+			LISTS_LOCK
+			if (result != NULL)
+				ov->listcontrols.refresh_requested = FALSE;
+		}
 	}
-	if (other_view->refresh_requested && !other_view->blocked)
+	else
+		ovdone = TRUE;
+
+	if (cv->listcontrols.refresh_requested || ov->listcontrols.refresh_requested)
 	{
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
-		printd (DEBUG, "accepting request to refresh %s", other_view->dir);
-#endif
-		gdk_threads_enter ();
-		e2_fileview_refresh_list (other_view);
-		gdk_threads_leave ();
-		other_view->refresh_requested = FALSE;
+		//again/now/still want to do one or both panes
+		//but only go back now not blocked before
+		if (cv->listcontrols.refresh_requested && cvdone)
+		{
+			LISTS_UNLOCK
+			goto retest;
+		}
+		if (ov->listcontrols.refresh_requested && ovdone)
+		{
+			LISTS_UNLOCK
+			goto retest2;
+		}
+
+		printd (DEBUG, "more refresh request(s), deferred because busy");
+		app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 200,
+			(GSourceFunc) _e2_filelist_refresh_manage, GINT_TO_POINTER (200),
+				(GDestroyNotify) _e2_filelist_timer_shutdown);
 	}
+
+	LISTS_UNLOCK
+
 	return FALSE;
 }
 /**
@@ -365,7 +416,9 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 	if (g_str_equal (curr_view->dir, dir))
 	{
 		matched = TRUE;
-		curr_view->refresh_requested = TRUE;
+		LISTS_LOCK
+		curr_view->listcontrols.refresh_requested = TRUE;
+		LISTS_UNLOCK
 	}
 #ifdef E2_VFSTMP
 	//CHECKME v-dir ok
@@ -373,7 +426,9 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 	if (g_str_equal (other_view->dir, dir))
 	{
 		matched = TRUE;
-		other_view->refresh_requested = TRUE;
+		LISTS_LOCK
+		other_view->listcontrols.refresh_requested = TRUE;
+		LISTS_UNLOCK
 	}
 
 	if (matched && immediate)
@@ -383,37 +438,63 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 	return matched;
 }
 /**
-@brief timer callback function which activates @a focus_wid when filelist refreshing is done
-@param focus_wid widget to focus when it's ok
-@return TRUE while still waiting for completion
-*/
-/*static gboolean _e2_filelist_wait_to_focus (GtkWidget *focus_wid)
-{
-	if (curr_view->refresh_requested || other_view->refresh_requested
-		|| !curr_view->completed || !other_view->completed)
-		return TRUE;
-	gdk_threads_enter ();
-	gtk_widget_grab_focus (focus_wid);
-	gdk_threads_leave ();
-	return FALSE;
-} */
-/**
-@brief focus @a focus_wid when filelist refreshing is done
-@param focus_wid widget to focus
-@return
-*/
-/*void e2_filelist_request_focus (GtkWidget *focus_wid)
-{
-	app.timers[?] = g_timeout_add (330, (GSourceFunc) _e2_filelist_wait_to_focus,
-		focus_wid);
-}*/
-/**
-@brief refresh filepane contents (and with FAM, config too) if the corresponding changed is detected
+@brief decide whether to lengthen the interval between filelist refresh requests
+This is called when a 'dirty' flag has been detected
 
-Among other uses, this is the timeout function for filelists auto refresh
+@param pane enumerator for pane 1 or pane 2, which was reported dirty
+
+@return TRUE when
+*/
+static gboolean _e2_filelist_refresh_ok (E2_ListChoice pane)
+{
+	gboolean retval;
+	/* Ideally this would check CPU resource-usage for this app and all children,
+	   and only proceed if there's enough(?) idle capacity
+	   Instead we assume small dirs can be processed without much adverse effect */
+	gint times, itemcount = (pane == PANE1) ?
+		//these are counts of total items, not filtered items
+		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane1_view.store), NULL):
+		gtk_tree_model_iter_n_children (GTK_TREE_MODEL (app.pane2_view.store), NULL);
+	if (itemcount < 51)
+		times = 1;
+	else if (itemcount < 251)
+		times = 2;
+	else
+		times = 3;
+
+	if (pane == PANE1)
+	{
+		if (++pane1repeats >= times)
+		{
+			printd (DEBUG, "Pane 1 refresh interval count = %d", times);
+			pane1repeats = 0;
+			retval = TRUE;
+		}
+		else
+			retval = FALSE;
+	}
+	else
+	{
+		if (++pane2repeats >= times)
+		{
+			printd (DEBUG, "Pane 2 refresh interval count = %d", times);
+			pane2repeats = 0;
+			retval = TRUE;
+		}
+		else
+			retval = FALSE;
+	}
+	return retval;
+}
+/**
+@brief request refresh of filepane(s) contents (and with FAM, config too) if the corresponding changed is detected
+
+Among other uses, this is the timer callback function for filelists auto refresh
 When FAM/gamin in use, it also checks for and flags (not processes)
 config file updates, as that data comes from the same data stream.
-This expects gtk's BGL to be OFF
+In certain circumstances any dirty flag might be ignored, or deferred to lengthen
+the effective interval between refreshes
+This expects gtk's BGL to be off/open
 
 @param userdata data specified when the timer was initialised (NULL), or when called directly, non-NULL
 
@@ -421,6 +502,10 @@ This expects gtk's BGL to be OFF
 */
 gboolean e2_filelist_check_dirty (gpointer userdata)
 {
+	static gboolean busy = FALSE;
+	if (busy)
+		return TRUE;	//no reentrant usage
+	busy = TRUE;
 #ifdef E2_FAM
 	static gboolean cfgdeferred = FALSE;
 #endif
@@ -429,13 +514,19 @@ gboolean e2_filelist_check_dirty (gpointer userdata)
 #ifdef E2_FAM
 	gboolean localcfgdirty;	//use a local copy so that the extern value doesn't get cleared before it's noticed
 	extern gboolean cfgdirty;
-#endif
 
-#ifdef E2_FAM
 	if (userdata != NULL
 		 && app.monitor_type != E2_MONITOR_DEFAULT)
 			g_usleep (E2_FAMWAIT);	//wait for things to be noticed
 #endif
+
+#ifndef E2_FAM
+	if (userdata != NULL)
+	{
+		p1dirty = p2dirty = TRUE;	//force a refresh
+	}
+	else
+#endif
 	//this reports 'gone' as 'dirty', and then the list-refesh
 	//function handles choosing a replacement dir
 	e2_fs_FAM_check_dirty (&p1dirty, &p2dirty
@@ -444,27 +535,34 @@ gboolean e2_filelist_check_dirty (gpointer userdata)
 #endif
 	);
 
-	if (p1dirty)
-		app.pane1_view.refresh_requested = TRUE;
-	if (p2dirty)
-		app.pane2_view.refresh_requested = TRUE;
+	p1dirty = p1dirty || pane1repeats;
+	p2dirty = p2dirty || pane2repeats;
 
-	if (app.pane1_view.refresh_requested || app.pane2_view.refresh_requested)
+	if (p1dirty || p2dirty)
 	{
-		if (app.timers[REFRESHBEGIN_T] > 0)
+		LISTS_LOCK
+		if (p1dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE1)))
+			app.pane1_view.listcontrols.refresh_requested = TRUE;
+		else
+			p1dirty = FALSE;
+		if (p2dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE2)))
+			app.pane2_view.listcontrols.refresh_requested = TRUE;
+		else
+			p2dirty = FALSE;
+		LISTS_UNLOCK
+		if (p1dirty || p2dirty)
 		{
-			//clear any incomplete refresh request
-			g_source_remove (app.timers[REFRESHBEGIN_T]);
-		}
-//		if (_e2_filelist_check_requests (NULL)) can only do this if BGL is always open here
-//		{	//immediate refresh was blocked
-//			printd (DEBUG, "FILELIST(S) DIRTY, initiating list refresh timer");
-			//the delay between attempts may be lengthened later, if appropriate
-			app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 2,
-				(GSourceFunc) _e2_filelist_check_requests, GINT_TO_POINTER (2),
+			if (app.timers[REFRESHBEGIN_T] > 0)
+			{
+				//clear any incomplete refresh request
+				g_source_remove (app.timers[REFRESHBEGIN_T]);
+			}
+		//the delay between attempts will be lengthened later, if appropriate
+//CHECKME we may be in a timer callback here - are nested timers still acceptable to glib/gtk ?
+			app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 5,
+				(GSourceFunc) _e2_filelist_refresh_manage, GINT_TO_POINTER (5),
 				(GDestroyNotify) _e2_filelist_timer_shutdown);
-//			printd (DEBUG, "timer ID = %d", app.timers[REFRESHBEGIN_T]);
-//		}
+		}
 	}
 
 #ifdef E2_FAM
@@ -487,9 +585,32 @@ gboolean e2_filelist_check_dirty (gpointer userdata)
 	}
 #endif	//def E2_FAM
 
+	busy = FALSE;
 	return TRUE;
 }
 /**
+@brief initialise timer which polls for various 'dirty' indicator(s)
+
+The timer is never suspended or paused, to avoid the risk of overflow from
+kernel-fam processes. It may be stopped permanently if change-monitoring is
+abandoned via a config dialog
+
+@return
+*/
+void e2_filelist_start_refresh_polling (void)
+{
+	if (e2_option_bool_get ("auto-refresh"))
+	{
+		app.timers[DIRTYCHECK_T] =
+#ifdef USE_GLIB2_14
+			g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
+#else
+			g_timeout_add (E2_FILESCHECK_INTERVAL,
+#endif
+				(GSourceFunc) e2_filelist_check_dirty, NULL);
+	}
+}
+/**
 @brief idle function to clear old liststores
 
 @param user_data UNUSED data specified when the idle was setup
@@ -556,7 +677,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 	gchar modified_buf[25];
 	gchar accessed_buf[25];
 	gchar changed_buf[25];
-	gchar perm_buf[11];
+//	gchar perm_buf[11];
 	gchar uid_buf[20];
 	gchar gid_buf[20];
 	gchar *buf[NAMEKEY+1];	//pointers to strings to be inserted into each row
@@ -610,6 +731,10 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 	GtkStyle *style = gtk_rc_get_style (curr_view->treeview);
 	GdkColor *default_color = &style->text[GTK_STATE_NORMAL];
 
+#ifdef E2_ASSISTED
+	GdkColor *back_color = (e2_option_bool_get ("color-background-set")) ?
+		e2_option_color_get ("color-background") : NULL;
+#endif
 	GdkColor *link_color = e2_option_color_get ("color-ft-link");
 	GdkColor *dir_color = e2_option_color_get ("color-ft-dir");
 	GdkColor *dev_color = e2_option_color_get ("color-ft-dev");
@@ -639,7 +764,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 		{	//use condensed size format
 			if (infoptr->statbuf.st_size < 1024) //less than 1k
 			{
-			  g_snprintf(size_buf, sizeof(size_buf), "%llu",
+			  g_snprintf(size_buf, sizeof(size_buf), "%"PRIu64,
 					infoptr->statbuf.st_size);
 			}
 			else if (infoptr->statbuf.st_size < 1048576) //less than a meg
@@ -660,7 +785,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 		}
 		else
 		{	//use actual size, with commas
-			g_snprintf(size_buf, sizeof(size_buf), "%llu",
+			g_snprintf(size_buf, sizeof(size_buf), "%"PRIu64,
 				infoptr->statbuf.st_size);
 
 			guint len = strlen (size_buf);
@@ -678,9 +803,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 		}
 		buf[SIZE] = g_strdup (size_buf);	//content is ascii, no need to convert to utf
 
-		e2_fs_get_perm_string (perm_buf, sizeof(perm_buf), infoptr->statbuf.st_mode);
-//		buf[PERM] = e2_utf8_from_locale_fast (perm_buf);
-		buf[PERM] = g_strdup (perm_buf); //content is ascii, no need to convert to utf
+		buf[PERM] = e2_fs_get_perm_string (infoptr->statbuf.st_mode); //string is already utf
 
 		if ((pwd_buf = getpwuid (infoptr->statbuf.st_uid)) == NULL)
 		{
@@ -793,6 +916,9 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 			NAMEKEY, buf[NAMEKEY],
 			FINFO, infoptr,
 			FORECOLOR, foreground,
+#ifdef E2_ASSISTED
+			BACKCOLOR, back_color,
+#endif
 //			VISIBLE, TRUE,
 			-1);
 
@@ -801,6 +927,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 		for (i = 0; i <= NAMEKEY; i++)
 			g_free (buf[i]);
 
+	}
 /*
 //		gettimeofday (&strt, NULL); //FOR DEBUGGING, check time this load takes
 //		gettimeofday (&nd, NULL);
@@ -827,7 +954,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 
 //	printd (DEBUG, "populating rows took %f seconds", result.tv_sec + result.tv_usec / 1000000.0 );
 */
-	}
+//	printd (DEBUG, "populating rows finished");
 	return store;
 }
 /**
@@ -858,3 +985,49 @@ GtkListStore *e2_filelist_make_store (void)
 		);
 	return store;
 }
+
+#ifdef STORECOPY
+/**
+@brief helper function to copy each FileInfo stored in a filelist store
+
+@param model the store to be updated
+@param tpath UNUSED tree path of iter being processed ?
+@param iter pointer to iter being processed
+@param user_data UNUSED data specified when foreach was called
+
+@return FALSE if the process can continue
+*/
+static gboolean _e2_filelist_copy_storeinfos (GtkTreeModel *model,
+	GtkTreePath *tpath, GtkTreeIter *iter, gpointer user_data)
+{
+	FileInfo *info;
+	gtk_tree_model_get (model, iter, FINFO, &info, -1);
+	//FIXME duplicate the data
+#ifdef USE_GLIB2_14
+	info = (FileInfo *) g_slice_copy (sizeof (FileInfo), info);
+	gtk_list_store_set (GTK_LIST_STORE (model), iter, FINFO, info, -1);
+#else
+	FileInfo *info2 = ALLOCATE (FileInfo);
+	CHECKALLOCATEDWARN (info2, return TRUE;)
+	memcpy (info2, info, sizeof (FileInfo));
+	gtk_list_store_set (GTK_LIST_STORE (model), iter, FINFO, info2, -1);
+#endif
+//	gtk_tree_path_free (tpath); CRASHER
+	return FALSE;
+}
+/**
+@brief copy a filelist liststore with all its contents
+
+@param original the store to be copied
+
+@return the new liststore
+*/
+GtkListStore *e2_filelist_copy_store (GtkListStore *original)
+{
+	gpointer copied;
+	e2_tree_store_copy (GTK_TREE_MODEL (original), FALSE, &copied);
+	gtk_tree_model_foreach (GTK_TREE_MODEL ((GtkListStore *)copied),
+		(GtkTreeModelForeachFunc) _e2_filelist_copy_storeinfos, NULL);
+	return ((GtkListStore *)copied);
+}
+#endif
