@@ -1,6 +1,6 @@
-/* $Id: e2_filetype.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_filetype.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -89,6 +89,18 @@ Not much use - in the config data, the categories are user-editable !!
 	return NULL;
 } */
 /**
+@brief helper function to get actions list for specified file extension without regard to string case
+@param key an extension logged in the filetypes hash table
+@param value UNUSED
+@param ext the filetype extension being souht
+
+@return TRUE when a match is found
+*/
+gboolean _e2_filetype_find_any (gpointer key, gpointer value, gchar *ext)
+{
+	return (e2_utf8_caseless_match ((gchar *)key, ext, -1, -1));
+}
+/**
 @brief get actions 'list' for specified file extension
 
 Returns the actions array (in the file types hash)
@@ -102,11 +114,21 @@ Used only by the context-menu creator.
 */
 const gchar **e2_filetype_get_actions (gchar *ext)
 {
-	const gchar **actions = g_hash_table_lookup (app.filetypes, ext);
-	if (actions != NULL && *actions == '\0')
-		return NULL;
+	const gchar **actions;
+
+	if ( g_str_equal (ext, _("<directory>"))
+	  || g_str_equal (ext, _("<executable>"))
+	  || g_str_equal (ext, _("<none>"))
+	  || (!e2_option_bool_get ("anycase-filetypes") && e2_fs_mount_is_cased (curr_view)))
+		actions = (const gchar **) g_hash_table_lookup (app.filetypes, ext);
 	else
+		actions = (const gchar **) g_hash_table_find (app.filetypes,
+						(GHRFunc)_e2_filetype_find_any, ext);
+
+	if (!(actions == NULL || *actions == '\0'))
 		return actions;
+	else
+		return NULL;
 }
 /**
 @brief get default action for specified file extension
@@ -122,11 +144,11 @@ Used only by callback open_cb.
 */
 gchar *e2_filetype_get_default_action (gchar *ext)
 {
-	gchar **actions = g_hash_table_lookup (app.filetypes, ext);
-	if (actions == NULL || *actions == '\0')
-		return NULL;
-	else
+	const gchar **actions = e2_filetype_get_actions (ext);
+	if (!(actions == NULL || *actions == '\0'))
 		return g_strdup (*actions);
+	else
+		return NULL;
 }
 /**
 @brief load all filetype info into hash
@@ -471,7 +493,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("<directory>"),"|",NULL), //_I(
 		g_strconcat ("\t|",_C(6),"|",NULL),  // _("commands")
 			g_strconcat ("\t\t|",_("_Open"),"|cd %%f",NULL),	//must be no quotes in path!
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(60),NULL), //<
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(61),NULL), //<
 			g_strconcat ("\t\t|",_("_Mount"),"|mount %p",NULL),	//CHECKME quotes in path ok?
 			g_strconcat ("\t\t|",_("_Unmount"),"|umount %p",NULL),
 	g_strconcat (_("executables"),"||",NULL),
@@ -517,7 +539,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|txt|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 //not loaded	g_strconcat ("\t\t|",PLUGIN,":",_("View"),"|",_A(5),".",_("view"),NULL),  //conformed name
-			g_strconcat ("\t\t|",_("View"),"|",_A(5),".",_A(92),NULL), //_("file.view") //<
+			g_strconcat ("\t\t|",_("View"),"|",_A(5),".",_A(101),NULL), //_("file.view") //<
 			g_strconcat ("\t\t|",_("_Edit"),"|",_A(5),".",_A(39),NULL), //_("file.edit") //<
 //this removed in favour of the default context menu item open-with
 //			g_strconcat ("\t\t|",_("Edit _with.."),"|%{(editors)@",_("Editor command:"),"} %p",NULL),
@@ -575,12 +597,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tar|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("Unpack"),"|tar -xf %f",NULL),
-			g_strconcat ("\t\t|",_("Unpack in other pane"),"|",_A(13),".",_A(71),";x tar -C %D -xf %f;",_A(13),".",_A(70),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in other pane"),"|",_A(13),".",_A(72),";x tar -C %D -xf %f;",_A(13),".",_A(71),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),  //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),  //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -tf %f \\| less",NULL),
 	g_strconcat (_("gzip tarballs"),"||",NULL),
@@ -589,12 +611,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tgz|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unpack"),"|tar -zxf  %f",NULL),
-			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(71),";tar -C %D -zxf %f;",_A(13),".",_A(70),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(72),";tar -C %D -zxf %f;",_A(13),".",_A(71),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),   //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),   //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -ztf %f \\| less",NULL),
 	g_strconcat (_("bzip2 tarballs"),"||",NULL),
@@ -604,12 +626,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|tbz|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unpack"),"|>bzip2 -d -c %f \\| tar -xf -",NULL),
-			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(71),";>bzip2 -d -c %f \\| tar -C %D -xf -;",_A(13),".",_A(70),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(13),".",_A(72),";>bzip2 -d -c %f \\| tar -C %D -xf -;",_A(13),".",_A(71),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>bzip2 -d -c %f \\| tar -tf - \\| less",NULL),
 	g_strconcat (_("zip archives"),"||",NULL),
@@ -618,12 +640,12 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unzip"),"|unzip %f",NULL),
 			//space after %F in next line is needed ?
-			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(10),".",_A(81),";unzip %F;",_A(10),".",_A(81),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(10),".",_A(90),";unzip %F;",_A(10),".",_A(90),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>unzip -l %f \\| less",NULL),
 /* plugin functionality not tested yet, for these
@@ -632,20 +654,20 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|arj|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),
 #endif
 	g_strconcat (_("rar archives"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|rar|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(91),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(5),".",_A(100),NULL),
 #endif
 */
 	g_strconcat (_("RPM packages"),"||",NULL),
@@ -655,8 +677,8 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("In_formation"),"|rpm -qlip %f",NULL),
 			g_strconcat ("\t\t|",_("_Install"),"|su rpm -Uvh %f",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(91),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(107),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(5),".",_A(100),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(5),".",_A(118),NULL),
 #endif
 	g_strconcat (_("source code files"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
@@ -712,4 +734,9 @@ void e2_filetype_options_register (void)
 	e2_option_tree_create_store (set);
 
 	e2_option_tree_prepare_defaults (set, _e2_filetype_tree_defaults);
+
+	group_name = g_strconcat(_C(15) ,".",_C(26),":",_C(25),NULL);  //_("filetypes.options:miscellaneous"   //FREEME group_name not here !
+	e2_option_bool_register ("anycase-filetypes", group_name, _("case-insensitive filetypes"),
+		_("This causes text-case to always be ignored when matching a file-extension"), NULL, FALSE,
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP ); //no rebuild
 }
