@@ -1,6 +1,6 @@
-/* $Id: e2_filetype.h 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_filetype.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_FILETYPE_H
-#define E2_FILETYPE_H
+#ifndef __E2_FILETYPE_H__
+#define __E2_FILETYPE_H__
 
 #include "emelfm2.h"
 
@@ -35,4 +35,4 @@ void e2_filetype_exec_action (gchar *action);
 void	e2_filetype_actions_register (void);
 void e2_filetype_options_register (void);
 
-#endif //ndef E2_FILETYPE_H
+#endif //ndef __E2_FILETYPE_H__
