@@ -1,6 +1,6 @@
-/* $Id: e2_keybinding.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_keybinding.c 870 2008-05-06 05:52:35Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -812,6 +812,7 @@ void e2_keybinding_clean (void)
 }
 /**
 @brief display current key bindings, all or a nominated section
+Expects BGL on/closed
 @param parameters argument(s) to help command, starting with _(keys)
 
 @return
@@ -837,7 +838,7 @@ void e2_keybinding_output_help (gchar *parameters)
 	else if (!e2_tree_find_iter_from_str (mdl, 0, section, &iter, FALSE))
 	{
 		msg = g_strdup_printf (_("Cannot find a key binding named %s"), section);
-		e2_output_print_error(msg, TRUE);
+		e2_output_print_error (msg, TRUE);
 		return;
 		//revert to showing all
 //		gtk_tree_model_get_iter_first (mdl, &iter);
@@ -1002,7 +1003,7 @@ static gboolean _e2_keybinding_substitute (gpointer from, E2_ActionRuntime *art)
 */
 void e2_keybinding_actions_register (void)
 {
-	gchar *action_name = g_strconcat(_A(108),".",_A(109),NULL);	//_("key.alias"
+	gchar *action_name = g_strconcat(_A(119),".",_A(120),NULL);	//_("key.alias"
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_keybinding_substitute, NULL, TRUE);
 }
@@ -1022,13 +1023,13 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	e2_option_tree_setup_defaults (set,
 	g_strdup("keybindings=<"),  //internal name
 	g_strconcat(_C(17),"||||",NULL),  //_("general"
-	g_strconcat("\t|F1||",_A(13),".",_A(69),"|",NULL), //"refresh"
-	g_strconcat("\t|<Shift>F1||",_A(7),".",_A(50),"|",NULL), //"list.history
-	g_strconcat("\t|<Control>F1||",_A(7),".",_A(65),"|",NULL), //"list.pending
+	g_strconcat("\t|F1||",_A(13),".",_A(70),"|",NULL), //"refresh"
+	g_strconcat("\t|<Shift>F1||",_A(7),".",_A(51),"|",NULL), //"list.history
+	g_strconcat("\t|<Control>F1||",_A(7),".",_A(66),"|",NULL), //"list.pending
 	g_strconcat("\t|<Alt>F1||",_A(7),".",_A(28),"|",NULL), //"list.children
-	g_strconcat("\t|F2||",_A(5),".",_A(72),"|",NULL), //"rename"
-	g_strconcat("\t|F3||",_A(5),".",_A(92),"|",NULL), //"view"
-	g_strconcat("\t|<Shift>F3||",_A(5),".",_A(93),"|",NULL), //"view_again"
+	g_strconcat("\t|F2||",_A(5),".",_A(73),"|",NULL), //"rename"
+	g_strconcat("\t|F3||",_A(5),".",_A(101),"|",NULL), //"view"
+	g_strconcat("\t|<Shift>F3||",_A(5),".",_A(102),"|",NULL), //"view_again"
 	g_strconcat("\t|<Control>F3||",_A(5),".",_A(6),"|",NULL), //"find" (was F4)
 	g_strconcat("\t|F4||",_A(5),".",_A(39),"|",NULL), //"edit"
 	g_strconcat("\t|<Shift>F4||",_A(5),".",_A(40),"|",NULL), //"edit_again"
@@ -1036,28 +1037,29 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t|<Shift>F5||",_A(5),".",_A(34),"|",NULL),  //"copy_as"
 	g_strconcat("\t|<Control>F5||",_A(5),".",_A(36),"|",NULL),  //"copy_with_time"
 	g_strconcat("\t|<Alt>F5||",_A(5),".",_A(35),"|",NULL),  //"copy_merge"
-	g_strconcat("\t|F6||",_A(5),".",_A(57),"|",NULL), //"move"
-	g_strconcat("\t|<Shift>F6||",_A(5),".",_A(58),"|",NULL), //"move_as"
-	g_strconcat("\t|F7||",_A(1),".",_A(55),"|",NULL), //"mkdir"
-	g_strconcat("\t|F8||",_A(5),".",_A(88),"|",NULL), //"trash"
-	g_strconcat("\t|<Control>F8||",_A(6),".",_A(88),"|",NULL), //"goto trash"  modifiers not translated, gtk needs english to parse
+	g_strconcat("\t|F6||",_A(5),".",_A(58),"|",NULL), //"move"
+	g_strconcat("\t|<Shift>F6||",_A(5),".",_A(59),"|",NULL), //"move_as"
+	g_strconcat("\t|F7||",_A(1),".",_A(56),"|",NULL), //"mkdir"
+	g_strconcat("\t|F8||",_A(5),".",_A(97),"|",NULL), //"trash"
+	g_strconcat("\t|<Control>F8||",_A(6),".",_A(97),"|",NULL), //"goto trash"  modifiers not translated, gtk needs english to parse
 	g_strconcat("\t|<Shift><Control>F8||",_A(5),".",_A(38),"|",NULL), //"delete"
-	g_strconcat("\t|F9||",_A(5),".",_A(51),"|",NULL), //"file.info"
+	g_strconcat("\t|F9||",_A(5),".",_A(52),"|",NULL), //"file.info"
 #ifdef E2_TREEDIALOG
-	g_strconcat("\t|<Shift>F9||",_A(10),".",_A(90),"|",NULL), //"pane_active.tree" popup directories tree dialog
+	g_strconcat("\t|<Shift>F9||",_A(10),".",_A(99),"|",NULL), //"pane_active.tree" popup directories tree dialog
 #endif
-	g_strconcat("\t|F10||",_A(1),".",_A(68),"|",NULL), //"quit"
-	g_strconcat("\t|F11||",_A(2),".",_A(32),"|",NULL),  //"configure"
+	g_strconcat("\t|F10||",_A(1),".",_A(69),"|",NULL), //"quit"
+	g_strconcat("\t|F11||",_A(15),".",_A(44),"|",NULL), //"toggle.fullscreen"
+	g_strconcat("\t|<Shift>F11||",_A(13),".",_A(94),"|",NULL), //"panes.toggle direction"
+	g_strconcat("\t|<Control>F11||",_A(9),".",_A(26),"|",NULL), //"output.add
+	g_strconcat("\t|<Shift><Control>F11||",_A(9),".",_A(38),"|",NULL), //"output.delete
+	g_strconcat("\t|F12||",_A(2),".",_A(32),"|",NULL),  //"configure"
 #ifdef E2_VFS
-	g_strconcat("\t|<Shift>F11||",_A(10),".",_A(106),"|",NULL), //"pane_active.<virtual>"
-	g_strconcat("\t|<Control>F11||",_A(50),".",_A(106),"|",NULL), //"history.<virtual>"
+	g_strconcat("\t|<Shift>F12||",_A(10),".",_A(115),"|",NULL), //"pane_active.<spacemenu>"
+	g_strconcat("\t|<Control>F12||",_A(3),".",_A(117),"|",NULL), //"dialog.namespace"
 #endif
-	g_strconcat("\t|F12||",_A(13),".",_A(85),"|",NULL), //"toggle direction"
-	g_strconcat("\t|<Control>F12||",_A(9),".",_A(26),"|",NULL), //"output.add
-	g_strconcat("\t|<Shift><Control>F12||",_A(9),".",_A(38),"|",NULL), //"output.delete
 //	_I( single letters in the following not worth translating ?
 	//conform this help string to the following 'jump-keys'
-	g_strconcat("\t|<Control>","g","||",_A(9),".",_A(67),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
+	g_strconcat("\t|<Control>","g","||",_A(9),".",_A(68),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
 	//these 3 children are the so-called "directory jump keys"
 	g_strconcat("\t\t|","h","||cd|$HOME",NULL), //_A(17)
 	g_strconcat("\t\t|","m","||cd|/mnt",NULL),  //_A(17)
@@ -1065,59 +1067,62 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t|<Control>w","||",_A(9),".",_A(27),"|*,1",NULL),  //"adjust ratio" no arg translation
 	g_strconcat("\t|<Control>e","||",_A(9),".",_A(27),"|*,0",NULL),    //"adjust ratio" no arg translation
 	g_strconcat("\t|<Control>z","||",_A(1),".",_A(43),"|",NULL), //"focus toggle"
-	g_strconcat("\t|<Control>r","||",_A(13),".",_A(69),"|",NULL), //"refresh"
+	g_strconcat("\t|<Control>r","||",_A(13),".",_A(70),"|",NULL), //"refresh"
 	g_strconcat("\t|<Control>1","||",_A(4),".",_A(43),"|1",NULL), //"focus toggle" no arg translation
 	g_strconcat("\t|<Control>2","||",_A(4),".",_A(43),"|2",NULL), //"focus toggle"no arg translation
 	g_strconcat("\t",_C(32),"||||",NULL),  //_("panes"
 	//note "Space" not recognised, must be "space"
-//	g_strconcat("\t\t|space||",_A(10),".",_A(81),"|",NULL), //"switch"  something else makes this 'open' the selected item
-	g_strconcat("\t\t|space||",_A(10),".",_A(87),"|",NULL), //"toggle selected"
-	g_strconcat("\t\t|Tab||",_A(10),".",_A(81),"|",NULL), //"switch"
-	g_strconcat("\t\t|Return||",_A(5),".",_A(59),"|",NULL), //"open"
-	g_strconcat("\t\t|<Shift>Return||",_A(5),".",_A(60),"|",NULL), //"open in other"
-	g_strconcat("\t\t|Left||",_A(10),".",_A(44),"|",NULL), //"go back"
-	g_strconcat("\t\t|Right||",_A(10),".",_A(45),"|",NULL), //"go forward"
+//	g_strconcat("\t\t|space||",_A(10),".",_A(90),"|",NULL), //"switch"  something else makes this 'open' the selected item
+	g_strconcat("\t\t|space||",_A(10),".",_A(96),"|",NULL), //"toggle selected"
+	g_strconcat("\t\t|Tab||",_A(10),".",_A(90),"|",NULL), //"switch"
+	g_strconcat("\t\t|Return||",_A(5),".",_A(60),"|",NULL), //"open"
+	g_strconcat("\t\t|<Shift>Return||",_A(5),".",_A(61),"|",NULL), //"open in other"
+	g_strconcat("\t\t|Left||",_A(10),".",_A(45),"|",NULL), //"go back"
+	g_strconcat("\t\t|Right||",_A(10),".",_A(46),"|",NULL), //"go forward"
 	g_strconcat("\t\t|<Alt>Up||cd|..",NULL),  //_A(17)
 	g_strconcat("\t\t|BackSpace||cd|..",NULL),  //_A(17)
 	g_strconcat("\t\t|Delete||",_A(5),".",_A(38),"|",NULL), //"delete"
-	g_strconcat("\t\t|<Shift>Delete||",_A(1),".",_A(89),"|",NULL), //"trashempty"
+	g_strconcat("\t\t|<Shift>Delete||",_A(1),".",_A(98),"|",NULL), //"trashempty"
 //	g_strconcat("\t\t|Home||cd|$HOME",NULL),   //_A(17)
-	g_strconcat("\t\t|<Control>a","||",_A(10),".",_A(86),"|",NULL), //"toggle select all"
-	g_strconcat("\t\t|<Control>i","||",_A(10),".",_A(53),"|",NULL), //"invert selection"
+	g_strconcat("\t\t|<Control>a","||",_A(10),".",_A(95),"|",NULL), //"toggle select all"
+	g_strconcat("\t\t|<Control>i","||",_A(10),".",_A(54),"|",NULL), //"invert selection"
 	g_strconcat("\t\t|<Control>f","||",_A(5),".",_A(6),"|",NULL), //"find"
-	g_strconcat("\t\t|<Control>x","||",_A(13),".",_A(84),"|",NULL), //"sync"
-	g_strconcat("\t\t|<Control>h","||",_A(10),".",_A(79),"|",NULL),  //"toggle hidden"
+	g_strconcat("\t\t|<Control>x","||",_A(13),".",_A(93),"|",NULL), //"sync"
+	g_strconcat("\t\t|<Control>h","||",_A(10),".",_A(80),"|",NULL),  //"toggle hidden"
 	g_strconcat("\t\t|<Control>m","||",_A(10),".",_A(21),"|",NULL), //"pane_active<bookmarks>"
 	g_strconcat("\t\t|<Control>t","||",_A(10),".",_A(22),"|",NULL), //"pane_active<filters>"
 //	g_strconcat("\t\t|/||",_A(5),".",_A(6),"|",NULL),  //"find"
 	g_strconcat("\t\t|<Control>Left||",_A(13),".",_A(27),"|","*,1",NULL), //"adjust ratio" no arg translation
 	g_strconcat("\t\t|<Control>Right||",_A(13),".",_A(27),"|","*,0",NULL), //"adjust ratio" no arg translation
-	g_strconcat("\t\t|<Control>p","||",_A(10),".",_A(80),"|",NULL),  //"show menu"
-	g_strconcat("\t\t|<Shift><Control>p","||",_A(10),".",_A(80),"|",_A(104),NULL), //"show menu, shift"
-	g_strconcat("\t\t|<Control><Alt>p","||",_A(10),".",_A(80),"|",_A(96),NULL),  //"show menu, ctrl"
+	g_strconcat("\t\t|<Control>p","||",_A(10),".",_A(81),"|",NULL),  //"show menu"
+	g_strconcat("\t\t|<Shift><Control>p","||",_A(10),".",_A(81),"|",_A(113),NULL), //"show menu, shift"
+	g_strconcat("\t\t|<Control><Alt>p","||",_A(10),".",_A(81),"|",_A(105),NULL),  //"show menu, ctrl"
+	g_strconcat("\t\t|<Control><Alt>n","||",_A(10),".",_A(86),"|",NULL),  //"pane_active.sortname"
+	g_strconcat("\t\t|<Control><Alt>m","||",_A(10),".",_A(85),"|",NULL),  //"pane_active.sortmodified"
+	g_strconcat("\t\t|<Control><Alt>s","||",_A(10),".",_A(88),"|",NULL),  //"pane_active.sortsize"
 	g_strconcat("\t",_C(12),"||||",NULL),  //_"directory line"
-	g_strconcat("\t\t|Tab||",_A(1),".",_A(31),"|",_A(97),NULL), //"complete dirs"
-	g_strconcat("\t\t|<Control>Tab||",_A(13),".",_A(59),"|",NULL), //"open"
+	g_strconcat("\t\t|Tab||",_A(1),".",_A(31),"|",_A(106),NULL), //"complete dirs"
+	g_strconcat("\t\t|<Control>Tab||",_A(13),".",_A(60),"|",NULL), //"open"
 	g_strconcat("\t\t|<Control>Delete||",_A(1),".",_A(29),"|",NULL),  //"clear"
 	g_strconcat("\t\t|<Alt>Delete||",_A(1),".",_A(30),"|",NULL),  //"clear_history"
-	g_strconcat("\t\t|<Control>Return||",_A(1),".",_A(52),"|",_A(98),NULL), //"insert selection, escape"
-	g_strconcat("\t\t|<Alt>Return||",_A(1),".",_A(52),"|",NULL), //"insert selection"
+	g_strconcat("\t\t|<Control>Return||",_A(1),".",_A(53),"|",_A(107),NULL), //"insert selection, escape"
+	g_strconcat("\t\t|<Alt>Return||",_A(1),".",_A(53),"|",NULL), //"insert selection"
 	g_strconcat("\t",_C(5),"||||",NULL),  //_("command line"
-	g_strconcat("\t\t|<Shift>Return||",_A(1),".",_A(76),"|",NULL), //"command.send"
-	g_strconcat("\t\t|<Control>Return||",_A(1),".",_A(52),"|",_A(98),NULL), //"insert selection, escape"
-	g_strconcat("\t\t|<Alt>Return||",_A(1),".",_A(52),"|",_A(103),NULL), //"insert selection, quote"
+	g_strconcat("\t\t|<Shift>Return||",_A(1),".",_A(77),"|",NULL), //"command.send"
+	g_strconcat("\t\t|<Control>Return||",_A(1),".",_A(53),"|",_A(107),NULL), //"insert selection, escape"
+	g_strconcat("\t\t|<Alt>Return||",_A(1),".",_A(53),"|",_A(112),NULL), //"insert selection, quote"
 	g_strconcat("\t\t|<Control>Delete||",_A(1),".",_A(29),"|",NULL), //"clear"
 	g_strconcat("\t\t|<Alt>Delete||",_A(1),".",_A(30),"|",NULL),  //"clear_history"
 	g_strconcat("\t\t|Tab||",_A(1),".",_A(31),"|",NULL), //"complete"
 	g_strconcat("\t\t|<Alt>Insert||",_A(28),".",_A(24),"|",NULL), //"children<menu>"
 //	g_strconcat("\t",_C(27),"||||",NULL),  //_("output"
 //CHECKME these stay as command-line bindings ??
-	g_strconcat("\t\t|Page_Up||",_A(9),".",_A(64),"|",NULL), //"page up"
-	g_strconcat("\t\t|Page_Down||",_A(9),".",_A(63),"|",NULL), //"page down"
-	g_strconcat("\t\t|<Shift>Page_Up||",_A(9),".",_A(48),"|",NULL), //"goto top"
-	g_strconcat("\t\t|<Shift>Page_Down||",_A(9),".",_A(47),"|",NULL), //"goto bottom"
-	g_strconcat("\t\t|<Shift>Up||",_A(9),".",_A(74),"|",NULL),  //"scroll up"
-	g_strconcat("\t\t|<Shift>Down||",_A(9),".",_A(73),"|",NULL),  //"scroll down"
+	g_strconcat("\t\t|Page_Up||",_A(9),".",_A(65),"|",NULL), //"page up"
+	g_strconcat("\t\t|Page_Down||",_A(9),".",_A(64),"|",NULL), //"page down"
+	g_strconcat("\t\t|<Shift>Page_Up||",_A(9),".",_A(49),"|",NULL), //"goto top"
+	g_strconcat("\t\t|<Shift>Page_Down||",_A(9),".",_A(48),"|",NULL), //"goto bottom"
+	g_strconcat("\t\t|<Shift>Up||",_A(9),".",_A(75),"|",NULL),  //"scroll up"
+	g_strconcat("\t\t|<Shift>Down||",_A(9),".",_A(74),"|",NULL),  //"scroll down"
 	g_strdup(">"),
 	NULL);
 }
