@@ -1,6 +1,6 @@
-/* $Id: e2_main.c 556 2007-07-23 23:35:01Z tpgww $
+/* $Id: e2_main.c 893 2008-05-20 09:42:51Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -120,9 +120,6 @@ The following items are covered:
 #include "e2_plugins.h"
 #include "e2_complete.h"
 #include "e2_task.h"
-#ifdef E2_VFS
-#include "e2_vfs.h"
-#endif
 
 #define UPGRADE_PNAME "e2p_upgrade.so"
 
@@ -131,7 +128,6 @@ extern GList *open_history;
 #ifdef USE_GLIB2_10
 extern GHashTable *actions_hash;
 #endif
-extern GStaticRecMutex cd_mutex;
 
 static void _e2_main_system_shutdown (gint num); //__attribute__ ((noreturn));
 
@@ -142,22 +138,30 @@ These allow a conservative approach to locking without risk of deadlock
 */
 void e2_main_close_gdklock (void)
 {
+#ifdef NATIVE_BGL
+	gdk_threads_enter ();
+#else
 	//a single thread can't write to screen more than once at a time,
 	//so deadlocks just reflect "unnecessary" re-locks, and can be ignored
-#ifdef DEBUG_MESSAGES
+# ifdef DEBUG_MESSAGES
 	if (pthread_mutex_lock (&gdklock) == EDEADLK)
 		printd (DEBUG, "attempted BGL re-lock");
-#else
+# else
 	pthread_mutex_lock (&gdklock);
+# endif
 #endif
 }
 void e2_main_open_gdklock (void)
 {
-#ifdef DEBUG_MESSAGES
+#ifdef NATIVE_BGL
+	gdk_threads_leave ();
+#else
+# ifdef DEBUG_MESSAGES
 	if (pthread_mutex_unlock (&gdklock) == EPERM)
 		printd (DEBUG, "bad attempt to open BGL");
-#else
+# else
 	pthread_mutex_unlock (&gdklock);
+# endif
 #endif
 }
 
@@ -171,7 +175,9 @@ This tests that both view->dir strings are not ""
 static gboolean _e2_main_refresh_initialise (gpointer data)
 {
 	if (app.pane1_view.dir == NULL || *app.pane1_view.dir == '\0'
-	 || app.pane2_view.dir == NULL || *app.pane2_view.dir == '\0')
+	 || app.pane2_view.dir == NULL || *app.pane2_view.dir == '\0'
+	 || app.pane1_view.listcontrols.cd_working
+	 || app.pane2_view.listcontrols.cd_working)
 	{
 		if (data != NULL)	//not first first callback
 			return TRUE;
@@ -181,6 +187,9 @@ static gboolean _e2_main_refresh_initialise (gpointer data)
 			GINT_TO_POINTER (100));
 			return FALSE;
 	}
+	//this func may be called at some racy time when these need to be reset
+	app.pane1_view.listcontrols.norefresh = FALSE;
+	app.pane2_view.listcontrols.norefresh = FALSE;
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "enable refresh, session start after filelists filled");
 #endif
@@ -307,15 +316,26 @@ static void _e2_main_message_handler (const gchar *log_domain,
 gint main (gint argc, gchar *argv[])
 {
 	//threads needed
-//PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP isn't always available to init mutex
 	pthread_mutexattr_t attr;
+	//setup mutex to protect threaded access to cd functionality
+	pthread_mutexattr_init (&attr);
+	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+	pthread_mutex_init (&list_mutex, &attr);
+	pthread_mutex_init (&history_mutex, &attr);
+#ifndef NATIVE_BGL
+//PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP isn't always available to init mutex
+//	pthread_mutexattr_t attr;
 	pthread_mutexattr_init (&attr);
 	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_ERRORCHECK);
 	pthread_mutex_init (&gdklock, &attr);
+//	pthread_mutexattr_destroy (&attr);
+#endif
 	pthread_mutexattr_destroy (&attr);
 
 	g_thread_init (NULL);
+#ifndef NATIVE_BGL
 	gdk_threads_set_lock_functions (e2_main_close_gdklock, e2_main_open_gdklock);
+#endif
 	gdk_threads_init ();	//setup gdk mutex
 	//gtk initialization
 	printd (DEBUG, "gtk init");
@@ -331,22 +351,20 @@ gint main (gint argc, gchar *argv[])
 	GLogLevelFlags levels = G_LOG_LEVEL_MASK | G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL;
 	g_log_set_handler ("Glib", levels, _e2_main_message_handler, NULL);
 	g_log_set_handler ("Gtk", levels, _e2_main_message_handler, NULL);
+
 	//some local init stuff
-	gboolean confile_ok;
-	gboolean confdir_ok = e2_option_set_config_dir ();  //set name for config dir, make it if not there already
-	if (confdir_ok)
-		e2_cache_init ();  //read and process cache file (if any) (before options init)
 	e2_action_setup_labels ();  //need this before option_init, to avoid crashes when there is no config file
 	e2_option_setup_labels ();
 	//after labels & before options setup, setup data for referencing toolbars
 	e2_toolbar_data_create ();
 
-	e2_option_init ();  //always setup default options (before config read)
+	e2_option_init ();  //setup default options (before cache or config read)
 
-	if (confdir_ok) //read & process saved options (if any)
-		confile_ok = e2_option_file_read ();
-	else
-		confile_ok = FALSE;
+	gboolean confdir_ok = e2_option_set_config_dir ();  //set name for config dir, make it if not there already
+	e2_cache_init (confdir_ok);  //read and process cache file (if any) (before options init)
+	gboolean confile_ok = (confdir_ok) ?
+		e2_option_file_read (): //read & process saved options (if any)
+		FALSE;
 
 	if (confile_ok)
 	{
@@ -358,7 +376,9 @@ gint main (gint argc, gchar *argv[])
 			{
 				if (!p->plugin_init (p))	//do any upgrades
 					printd (ERROR, "Can't initialize upgrade plugin: "UPGRADE_PNAME);
+				gdk_threads_enter ();
 				e2_plugins_unload1 (p, FALSE);
+				gdk_threads_leave ();
 			}
 			else
 				printd (ERROR, "Can't find upgrade plugin "UPGRADE_PNAME", so can't upgrade the config file");
@@ -375,7 +395,7 @@ gint main (gint argc, gchar *argv[])
 	e2_actions_init (); //setup action data stores & register most actions
 	e2_complete_init (); //"like" actions
 
-#ifndef USE_GTK2_12
+#ifndef USE_GTK2_12TIPS
 	app.tooltips = gtk_tooltips_new ();
 #endif
 
@@ -387,12 +407,6 @@ gint main (gint argc, gchar *argv[])
 	//do this before bars/menus created
 	e2_plugins_load_all (); // load plugins specified in the config data
 
-	//window creation includes dis/enable_refresh, so the refresh count
-	//must start > 1, to prevent erroneous timer behaviour during
-	//window creation. Set to real initial value (0) later in the startup
-//	refresh_refcount = 10;
-//	app.refresh_timer_id = 0;
-
 	//build GUI
 	printd (DEBUG, "create main window");
 	e2_window_create (&app.window);
@@ -409,19 +423,16 @@ gint main (gint argc, gchar *argv[])
 	e2_fs_FAM_connect ();
 	//CHECKME ensure FAM reports don't pile up due to lack of polling
 #endif
-
-	if (e2_option_bool_get ("auto-refresh"))
-	{
-		app.timers[DIRTYCHECK_T] = g_timeout_add (E2_FILESCHECK_INTERVAL,
-			(GSourceFunc) e2_filelist_check_dirty, NULL);
-	}
-	//config checks are disabled during list creation
+	//initate dirty-checks
+	e2_filelist_start_refresh_polling ();
+	//config-change checks are disabled during list creation
 	//so the mechanism needs to be initialised beforehand
 //	app.config_timer_id = 0;
 	e2_option_enable_config_checks ();
 
+#ifndef E2_STATUS_DEMAND
 	e2_window_enable_status_update (-1);
-
+#endif
 	//set refresh counter to its real initial value
 	refresh_refcount = 0;
 #ifdef E2_REFRESH_DEBUG
@@ -432,9 +443,6 @@ gint main (gint argc, gchar *argv[])
 
 	e2_button_setup_labels ();	//maybe needed by dialog when opening dir
 
-	//setup mutex to protect threaded access to cd functionality
-	g_static_rec_mutex_init (&cd_mutex);
-
 	//session-starts with BGL closed
 //	gdk_threads_leave ();
 	//threaded cd's may start at any time, including after main loop is running
@@ -504,7 +512,7 @@ gint main (gint argc, gchar *argv[])
 	//session startup always uses local fs first, to prevent potential long
 	//delay when app window is being created
 	//now we can arrange to reinstate vfs state if appropriate
-	g_idle_add ((GSourceFunc) e2_vfs_initialise, NULL);
+//now done in plugin init	g_idle_add ((GSourceFunc) e2_vfs_initialise, NULL);
 #endif
 
 	//start change polling after both initial filelists are completed
@@ -513,6 +521,10 @@ gint main (gint argc, gchar *argv[])
 	//get all actions, for help document
 //	e2_action_list_all ();
 
+#ifdef E2_HAL
+	e2_hal_init ();
+#endif
+
 	printd (DEBUG, "enter main loop");
 	gdk_threads_enter ();
 	gtk_main ();
@@ -526,14 +538,14 @@ gint main (gint argc, gchar *argv[])
 
 /**
 @brief session-end cleanups
-
+Expects BGL on/closed, for any dialog or error message here or downstream
 @param compulsory TRUE to prevent the user from cancelling
 @param saveconfig TRUE to write config and cache files
 @param doexit TRUE to do a normal program exit
 
 @return FALSE if the user cancels or @a doexit is FALSE, else no return
 */
-static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit)
+gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit)
 {
 	if (!compulsory)
 	{
@@ -563,9 +575,20 @@ static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gb
 #ifdef E2_FAM
 	e2_fs_FAM_disconnect ();
 #endif
+#ifdef E2_HAL
+	e2_hal_disconnect ();
+#endif
 #ifdef E2_IMAGECACHE
 	e2_cache_image_clearall ();
 #endif
+	e2_plugins_unload_all (TRUE);	//backup caches, cleanup etc
+/*#ifdef E2_VFS
+	if (vfs.loaded)
+	{	//unloading does cacheing and proper cleanups
+		Plugin *p = e2_plugins_check_installed ("vfs"VERSION);
+		e2_plugins_unload1 (p, TRUE);
+	}
+#endif */
 	if (saveconfig)
 	{
 		e2_option_file_write (NULL);
@@ -576,7 +599,9 @@ static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gb
 #ifdef USE_GLIB2_10
 //	GList *member;
 	//need to deallocate the glib slices, at least
+	//NOTE this must be after all cleanups involving actions
 	g_hash_table_destroy (actions_hash);
+	//NOTE this must be after all cleanups involving options
 	g_hash_table_destroy (options_hash);
 /*	e2_cache_clean ();
 	e2_complete_clear ();
@@ -601,20 +626,20 @@ static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gb
 	//E2_ToggleBox's ?
 	//E2_ToggleData's ?
 	//anything from e2_fileview_get_selected()
-	//PlaceInfo's
-	//FileInfo's tied to treeview lines are cleaned already ??
+	//PlaceInfo's handled by unloading the vfs plugin
+	//FileInfo's tied to treeview lines are cleaned already ?? HOW?
 	//all E2_OptionTreeColumns
-	//directory history list items = view.dir_history and if cached strings, panert.opendirs
+//unref all optiontree stores ?
 	//view & edit dialog history items (if sliced!)
-	g_list_foreach (app.pane1_view.dir_history, (GFunc)g_slice_free1, NULL);
-	g_list_free (app.pane1_view.dir_history);
-	g_list_foreach (app.pane2_view.dir_history, (GFunc)g_slice_free1, NULL);
-	g_list_free (app.pane2_view.dir_history);
 	//edit dialog undo items
 	//ETC
-//	e2_plugins_unload_all ();	FIXME plugin cleaning crashes
 */
 #endif
+	//these might be in some backup place, for VFS
+	//dirline histories ?
+	//directory history list items
+	e2_fileview_clean_history (&app.pane1_view.dir_history);
+	e2_fileview_clean_history (&app.pane2_view.dir_history);
 	g_list_free (app.pane1.opendirs);	//this for non-cached list without heaped data
 	g_list_free (app.pane2.opendirs);	//ditto
 /*	printd (DEBUG, "clean options");
@@ -636,6 +661,7 @@ static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gb
 
 	if (doexit)
 	{
+		gdk_threads_leave ();
 //		printd (DEBUG, "exit (0)");
 		exit (0);
 	}
@@ -651,23 +677,10 @@ static gboolean _e2_main_closedown (gboolean compulsory, gboolean saveconfig, gb
 */
 gboolean e2_main_user_shutdown (gpointer from, E2_ActionRuntime *art)
 {
-	_e2_main_closedown (FALSE, TRUE, TRUE);
+	e2_main_closedown (FALSE, TRUE, TRUE);
 	return FALSE;
 }
 /**
-@brief callback for main window "delete-event" signal
-This can also be called directly.
-@param widget UNUSED the window button which was activated
-@param event UNUSED pointer to event data
-@param forced pointerised gboolean, NULL if the user is permitted to choose to keep the window open
-
-@return TRUE if the user cancelled the close, or else it does not return at all
-*/
-gboolean e2_main_shutdown (GtkWidget *widget, GdkEvent *event, gpointer forced)
-{
-	return (!_e2_main_closedown (GPOINTER_TO_INT (forced), TRUE, TRUE));
-}
-/**
 @brief perform system-initiated shutdown in repsonse to a shutdown signal
 This is performed the "proper" way, so as to not block signalling to related
 processes
@@ -679,7 +692,7 @@ static void _e2_main_system_shutdown (gint num)
 {
 	printd (DEBUG, "system shutdown initiated by signal %d", num);
 	gdk_threads_enter ();	//downstream errors expect BGL closed
-	_e2_main_closedown (TRUE, (num != SIGINT && num != SIGKILL), FALSE);
+	e2_main_closedown (TRUE, (num != SIGINT && num != SIGKILL), FALSE);
 	gdk_threads_leave ();
 	//in this handler, current signal is blocked, unblock it for this use
 	struct sigaction sigdata;
