@@ -1,6 +1,6 @@
-/* $Id: e2_output.c 536 2007-07-17 00:15:26Z tpgww $
+/* $Id: e2_output.c 854 2008-04-19 11:45:33Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 2004 Florian Zähringer <flo.zaehringer@web.de>
 Portions copyright (C) 1999 Michael Clark.
 
@@ -56,21 +56,24 @@ ToDo
 //#define VOL
 
 static GtkWidget *_e2_output_create_view (E2_OutputTabRuntime *rt);
-
+static void _e2_output_tabchange_cb (GtkNotebook *notebook,
+	GtkNotebookPage *page, guint page_num, gpointer data);
+#ifdef E2_TABS_DETACH
+static void _e2_output_tabgone_cb (GtkNotebook *notebook, GtkWidget *child,
+	guint page_num, GtkWidget *window);
+#endif
 //static gboolean output_activated = FALSE;
 static GStaticRecMutex print_mutex;
 extern pthread_mutex_t task_mutex;
 
-/*UNUSED until tab re-dragging is working
 #ifdef E2_TABS_DETACH
 #define TARGET_NOTEBOOK_TAB 3
 static GtkTargetEntry target_table2[] =
 {
-	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_NOTEBOOK_TAB }, //flags => 0 ??
+	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_NOTEBOOK_TAB },
 };
-static guint n_targets2 = sizeof(target_table2) / sizeof(target_table2[0]);
+static guint n_targets2 = sizeof(target_table2) / sizeof(GtkTargetEntry);
 #endif
-*/
 
   /*****************/
  /***** utils *****/
@@ -84,7 +87,7 @@ static guint n_targets2 = sizeof(target_table2) / sizeof(target_table2[0]);
 @return TRUE if @a iter relates to a point in @a text_view that is NOT presently visible
 */
 static inline gboolean _e2_output_iter_offscreen
-	(GtkTextView *text_view, volatile GtkTextIter *iter)
+	(GtkTextView *text_view, VOLATILE GtkTextIter *iter)
 {
 	GdkRectangle visible_rect, iter_rect;
 	gtk_text_view_get_visible_rect (text_view, &visible_rect);
@@ -198,8 +201,8 @@ set @a push_in to TRUE for menu completely inside the screen,
 FALSE for menu clamped to screen size
 
 @param menu the GtkMenu to be positioned
-@param x	place to store gint representing the menu left
-@param y  place to store gint representing the menu top
+@param x place to store gint representing the menu left
+@param y place to store gint representing the menu top
 @param push_in place to store pushin flag
 @param textview output pane widget in focus when the menu key was pressed
 
@@ -213,7 +216,7 @@ void e2_output_set_menu_position (GtkWidget *menu,
 	GtkAllocation alloc = textview->allocation;
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y + alloc.height/2;
-	*push_in = TRUE;
+	*push_in = FALSE;
 }
 
 #ifdef E2_TABS_DETACH
@@ -242,85 +245,168 @@ static void _e2_output_clear_buffer (E2_OutputTabRuntime *rt)
 	g_object_unref (G_OBJECT (buffer));
 }
 /**
-@brief (re)attach a notebook tab to the output pane
+@brief set or clear detached-related data for tab whose widget is @a child
 
-@param widget
-@param sw the child GtkWidget to be dropped
-@param context
-@param x X coordinate where the drop happens
-@param y Y coordinate where the drop happens
-@param selection_data
-@param info
-@param time
-@param user_data data specified when the arrangement was set up
+@param child the notebook tab widget
+@param dest_notebook the notebook into which a moved tab will go, NULL = app.outbook
 
 @return
 */
-/* FIXME doesn't work
-static void _e2_output_tabdrag_data_received_cb (
-	GtkWidget        *widget,
-	GdkDragContext   *context,
-	gint              x,
-	gint              y,
-	GtkSelectionData *selection_data,
-	guint             info,
-	guint             time,
-	gpointer          user_data)
+static void _e2_output_tab_set_detached_state (GtkWidget *child, GtkWidget *dest_notebook)
 {
-	printd (DEBUG, "_e2_output_tab_drag_data_received_cb ()");
-	GtkWidget *notebook = gtk_drag_get_source_widget (context);
-	GtkWidget **child = (void *) selection_data->data;
-	const gchar *text = gtk_notebook_get_tab_label_text
-		(GTK_NOTEBOOK (notebook), *child);
-	printd (DEBUG, "DnD widget 1 = %x", widget);
-	printd (DEBUG, "DnD widget 2 = %x", notebook);
-	GtkWidget *label = gtk_label_new (text);
-	gtk_notebook_append_page (GTK_NOTEBOOK (widget), *child, label);
-	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (widget), *child, TRUE);
-	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (widget), *child, TRUE);
-//	if (notebook == app.outbook)
-//	{ CHECKME any consequentials ?
-//	}
-	gtk_drag_finish (context, TRUE, TRUE, time);
-}
-*/
-/**
-@brief move tab(s) from @a newbook in a drop-window, back to @a oldbook
+	gboolean attaching;
+	GtkWidget *window;
 
-@param newbook notebook from which the tab(s) will be removed
-@param oldbook notebook to which the tab(s) will be appended
+	attaching = (dest_notebook == NULL || dest_notebook == app.outbook);
+	if (attaching)	//moving to main notebook
+		window = NULL;
+	else //moving from main notebook or some other drop window
+	{
+		window = gtk_widget_get_toplevel (dest_notebook);
+		if (!GTK_WIDGET_TOPLEVEL (window))
+		{
+			//FIXME
+			printd (DEBUG, "can't find top window for moved tab");
+			return;
+		}
+	}
 
-@return
-*/
-static void _e2_output_tab_reattach (GtkNotebook *newbook, GtkNotebook *oldbook)
-{
-//	gint i, j = gtk_notebook_get_n_pages (newbook);
-//	for (i = 0; i < j; i++)
-//	{
-	GtkWidget *child = gtk_notebook_get_nth_page (newbook, 0);	//i);
-	GtkWidget *tab_label = gtk_notebook_get_tab_label (newbook, child);
-	g_object_ref (G_OBJECT (child));
-	g_object_ref (G_OBJECT (tab_label));
-	gtk_container_remove (GTK_CONTAINER (newbook), child);
-	gtk_notebook_append_page (oldbook, child, tab_label);
-	g_object_unref (G_OBJECT (child));
-	g_object_unref (G_OBJECT (tab_label));
-	gtk_notebook_set_tab_reorderable (oldbook, child, TRUE);
-	gtk_notebook_set_tab_detachable (oldbook, child, TRUE);
-
-	//clear flag so tab context-menu shows items relevant to attached tab
+	if (child == app.tab.scroll)
+	{
+		app.tab.detached = !attaching;
+		app.tab.dropwindow = window;
+	}
 	GList *member;
 	for (member = app.tabslist; member != NULL; member = member->next)
 	{
 		if (((E2_OutputTabRuntime *)member->data)->scroll == child)
 		{
-			((E2_OutputTabRuntime *)member->data)->detached = FALSE;
-			printd (DEBUG, "clear detached flag of tab %d",
-				((E2_OutputTabRuntime *)member->data)->labelnum);
+			((E2_OutputTabRuntime *)member->data)->detached = !attaching;
+			printd (DEBUG, "detached-flag of tab %d set %s",
+				((E2_OutputTabRuntime *)member->data)->labelnum, (attaching) ? "FALSE":"TRUE");
+			((E2_OutputTabRuntime *)member->data)->dropwindow = window;
 			break;
 		}
 	}
-//	}
+}
+/**
+@brief move tab whose widget is @a child from @a newbook to @a oldbook
+
+@param child tab widget
+@param frombook notebook from which the tab will be removed
+@param tobook notebook to which the tab will be appended
+
+@return
+*/
+static void _e2_output_tab_move (GtkWidget *child,
+	GtkNotebook *frombook, GtkNotebook *tobook)
+{
+	GtkWidget *tab_label, *menu_label;
+	gboolean tab_expand, tab_fill;	//, reorderable, detachable;
+	guint tab_pack;
+	//this is essentially the same process that gtk uses
+	g_object_ref (G_OBJECT (child));
+	tab_label = gtk_notebook_get_tab_label (frombook, child);
+	if (tab_label)
+		g_object_ref (G_OBJECT (tab_label));
+	menu_label = gtk_notebook_get_menu_label (frombook, child);
+	if (menu_label)
+		g_object_ref (G_OBJECT (menu_label));
+
+	gtk_container_child_get (GTK_CONTAINER (frombook), child,
+		"tab-expand", &tab_expand,
+		"tab-fill", &tab_fill,
+		"tab-pack", &tab_pack,
+//		"reorderable", &reorderable,
+//		"detachable", &detachable,
+		NULL);
+
+	gtk_container_remove (GTK_CONTAINER (frombook), child);
+	gtk_notebook_append_page_menu (tobook, child, tab_label, menu_label);
+
+	gtk_container_child_set (GTK_CONTAINER (tobook), child,
+		"tab-pack", tab_pack,
+		"tab-expand", tab_expand,
+		"tab-fill", tab_fill,
+		"reorderable", TRUE,	//reorderable,
+		"detachable", TRUE,	//detachable,
+		NULL);
+
+	g_object_unref (G_OBJECT (child));
+	if (tab_label)
+		g_object_unref (G_OBJECT (tab_label));
+	if (menu_label)
+		g_object_unref (G_OBJECT (menu_label));
+
+	if (child == app.tab.scroll)
+	{	//this is the default tab
+		gtk_notebook_set_current_page (tobook, -1);
+	}
+	//update detached-data for the tab
+	_e2_output_tab_set_detached_state (child, GTK_WIDGET (tobook));
+}
+
+/**
+@brief process a tab being dragged
+This is called for second and later drops onto a tabdrop window and for all
+drops back onto the output-pane main notebook
+Page-widget properties are not changed when the page is dragged
+@param dest_notebook the notebook to which a tab is being dragged
+@param context drag context data
+@param x X coordinate where the drop happens
+@param y Y coordinate where the drop happens
+@param sel_data the received data
+@param info the info registered for the target in target_table2
+@param time timestamp at which the data was received
+@param user_data UNUSED data specified when the callback was connected
+
+@return
+*/
+static void _e2_output_tabdrag_data_received_cb (
+	GtkWidget        *dest_notebook,
+	GdkDragContext   *context,
+	gint              x,
+	gint              y,
+	GtkSelectionData *sel_data,
+	guint             info,
+	guint             time,
+	gpointer          user_data)
+{
+	printd (DEBUG, "_e2_output_tab_drag_data_received_cb");
+	gboolean success;
+	if (sel_data->length > 0)	//&& sel_data->format == 8)
+	{
+		GtkWidget *source_notebook = gtk_drag_get_source_widget (context);
+		GtkWidget *child = *(GtkWidget **)sel_data->data;
+		if (source_notebook == dest_notebook)
+		{
+//			printd (DEBUG, "trying to drag to same place");
+			success = FALSE;
+		}
+		else if (source_notebook == app.outbook
+			&& gtk_notebook_get_n_pages (GTK_NOTEBOOK (source_notebook)) == 1)
+		{	//prevent dragging the only output-pane tab
+//			printd (DEBUG, "trying to drag last output tab");
+			success = FALSE;
+		}
+		else
+		{
+			success = TRUE;
+			_e2_output_tab_set_detached_state (child, dest_notebook);
+			//prevent changes of default tab, by block here
+			//(WAS unblock in tab-removed cb, but that spits error - seems
+			//that gtk unblocks this all by itself !
+			g_signal_handlers_block_by_func (G_OBJECT (source_notebook),
+				_e2_output_tabchange_cb, NULL);
+		}
+	}
+	else
+		success = FALSE;
+
+	if (!success)
+		sel_data->target = GDK_NONE;
+
+	gtk_drag_finish (context, success, FALSE, time);
 }
 /**
 @brief cleanup when closing a tab-drop window
@@ -334,9 +420,35 @@ This moves the tab(s) in the closing window back to the output-pane notebook
 static gboolean _e2_output_tabrestore_cb (GtkWidget *window, GdkEvent *event,
 	gpointer data)
 {
+	printd (DEBUG, "_e2_output_tabrestore_cb ()");
+	gint j;
 	GtkNotebook *newbook = GTK_NOTEBOOK (gtk_bin_get_child (GTK_BIN (window)));
-	if (newbook != NULL && gtk_notebook_get_n_pages (newbook) > 0)
-		_e2_output_tab_reattach (newbook, GTK_NOTEBOOK (app.outbook));
+	if (newbook != NULL && (j = gtk_notebook_get_n_pages (newbook)) > 0)
+	{
+		//prevent sequential tab-changes during the cleanout process
+		g_signal_handlers_disconnect_by_func (G_OBJECT (newbook),
+			_e2_output_tabchange_cb, NULL);
+		//no need to go back and check for empty book
+		g_signal_handlers_disconnect_by_func (G_OBJECT (newbook),
+			_e2_output_tabgone_cb, window);
+		GtkWidget *defchild = app.tab.scroll;	//remember which page is default now
+		gint i;
+		for (i = 0; i < j; i++)
+		{
+			GtkWidget *child = gtk_notebook_get_nth_page (newbook, 0);
+			_e2_output_tab_move (child, newbook, GTK_NOTEBOOK (app.outbook));
+		}
+		//reset the default page in the destination book
+		gint indx = gtk_notebook_page_num (GTK_NOTEBOOK (app.outbook), defchild);
+		if (indx != -1)
+		{
+			g_signal_handlers_block_by_func (G_OBJECT (app.outbook),
+				_e2_output_tabchange_cb, NULL);
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (app.outbook), indx);
+			g_signal_handlers_unblock_by_func (G_OBJECT (app.outbook),
+				_e2_output_tabchange_cb, NULL);
+		}
+	}
 	return FALSE;
 }
 /**
@@ -349,16 +461,72 @@ This is a callback for the "page-removed" signal on @a notebook
 
 @return
 */
-static void _e2_output_gone_cb (GtkNotebook *notebook, GtkWidget *child,
+static void _e2_output_tabgone_cb (GtkNotebook *notebook, GtkWidget *child,
 	guint page_num, GtkWidget *window)
 {
-	if (gtk_notebook_get_n_pages (notebook) == 0)
-		gtk_widget_destroy (window);
+	printd (DEBUG, "_e2_output_tabgone_cb");
+	if (notebook != GTK_NOTEBOOK (app.outbook) &&
+		gtk_notebook_get_n_pages (notebook) == 0)
+			gtk_widget_destroy (window);
+/* this generates warning - seems as if gtk already unblocked the thing
+	else
+		//revert the block added when the drag was in progress
+		g_signal_handlers_unblock_by_func (G_OBJECT (notebook),
+			_e2_output_tabchange_cb, NULL);
+*/
 }
 /**
+@brief when a mouse-button is pressed on tab "label", update current-page data if needed
+This is a callback for the "grab-focus" signal on @a notebook
+It's called after a tab-change, if any
+@param notebook the affected notebook
+@param user_data UNUSED data specified when the callback was connected
+
+@return
+*/
+static void _e2_output_grab_focus_cb (GtkWidget *notebook, gpointer user_data)
+{
+	printd (DEBUG, "_e2_output_grab_focus_cb");
+	gint curindx = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+	GtkWidget *child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
+		curindx);
+	GList *member;
+	for (member = app.tabslist; member != NULL; member = member->next)
+	{
+		E2_OutputTabRuntime *tab;
+		tab = (E2_OutputTabRuntime *)member->data;
+		if (tab->scroll == child)
+		{
+			if (tab != curr_tab)
+			{
+				//swap with mimimum race-risk ...
+				*curr_tab = app.tab;	//backup current tab's data from stack to heap
+				app.tab = *tab;	//get the replacement stuff into stackspace
+				curr_tab = tab;
+				//adjust all relevant child foreground-tab pointers to/from the stacked tab data
+				e2_command_retab_children (&app.tab, curr_tab);
+				printd (DEBUG, "output tab change, new current-tab ID is %d", tab->labelnum);
+			}
+			break;
+		}
+	}
+}
+//for "focus-in-event" NO USE
+/*static gboolean _e2_output_focus_in_cb (GtkWidget *widget, GdkEventFocus *event,
+	gpointer user_data)
+{
+	printd (DEBUG, "_e2_output_focus_in_cb");
+	return FALSE;
+}
+void _e2_output_page_add_cb (GtkNotebook *notebook, GtkWidget *child,
+		guint page_num, gpointer user_data)
+{
+	printd (DEBUG, "_e2_output_page_add_cb");
+} */
+/**
 @brief when a detached notebook tab is dropped in an empty area, create a
 window containing a notebook to receive the tab
-
+Page-widget properties are not changed when dragged
 @param source the source GtkNotebook of the drag operation
 @param sw the child GtkWidget to be dropped
 @param x the X coordinate where the drop happens
@@ -377,11 +545,11 @@ static GtkNotebook *_e2_output_tab_drop_new (GtkNotebook *source, GtkWidget *sw,
 		return NULL;	//prevent dragging of the only tab
 
 	GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	E2_DIALOG_TITLE(window, _("output tabs"));
+	e2_window_set_title (window, _("output tabs"));
 	gtk_window_set_role (GTK_WINDOW (window), "tabdrop");
 //	gtk_window_set_wmclass (GTK_WINDOW (window), "main", BINNAME);
 #ifdef E2_COMPOSIT
-	e2_window_set_translucency (window, -1);
+	e2_window_set_opacity (window, -1);
 #endif
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
 	//restore dropped tab(s) when new window is closed
@@ -389,26 +557,44 @@ static GtkNotebook *_e2_output_tab_drop_new (GtkNotebook *source, GtkWidget *sw,
 		G_CALLBACK (_e2_output_tabrestore_cb), NULL);
 
 	GtkWidget *notebook = gtk_notebook_new ();
+	printd (DEBUG, "new notebook widget = %x", notebook);
 	gtk_container_add (GTK_CONTAINER (window), notebook);
-	//FIXME next is a crasher
-//	gtk_drag_source_set (notebook, GDK_BUTTON1_MASK, target_table2, n_targets2,
+//CRASHER gtk_drag_source_set (notebook, GDK_BUTTON1_MASK, target_table2, n_targets2,
 //		GDK_ACTION_PRIVATE);
-/* FIXME doesn't work
+	//setup to process further tabs being dragged here
 	gtk_drag_dest_set (notebook, GTK_DEST_DEFAULT_DROP, target_table2, n_targets2,
-		GDK_ACTION_PRIVATE);
+		GDK_ACTION_MOVE);
 	g_signal_connect (G_OBJECT (notebook), "drag-data-received",
 		G_CALLBACK (_e2_output_tabdrag_data_received_cb), NULL);	//CHECKME user_data
-*/
-	//close window when all tabs dragged out
+	//close window when all tabs dragged out or re-attached by menu-selection
 	g_signal_connect (G_OBJECT (notebook), "page-removed",
-		G_CALLBACK (_e2_output_gone_cb), window);
+		G_CALLBACK (_e2_output_tabgone_cb), window);
+	//capture clicks on tab labels
+	g_signal_connect (G_OBJECT (notebook), "grab-focus",
+		G_CALLBACK (_e2_output_grab_focus_cb), NULL);
+//	g_signal_connect (G_OBJECT (notebook), "focus-in-event",
+//		G_CALLBACK (_e2_output_focus_in_cb), NULL);
+//	g_signal_connect (G_OBJECT (notebook), "page-added",
+//		G_CALLBACK (_e2_output_page_add_cb), NULL);
+	g_signal_connect (G_OBJECT (notebook), "switch-page",
+		G_CALLBACK (_e2_output_tabchange_cb), NULL);	//no data
+#ifdef USE_GTK2_12DND
+	//allow further dragging from this window
+	//(this seems to be un-necessary here, though that may be a gtk deficiency)
+	g_signal_connect (G_OBJECT (notebook), "create-window",
+		G_CALLBACK (_e2_output_tab_drop_new), NULL);	//CHECKME user_data
+#endif
 
 	GtkNotebook *book = GTK_NOTEBOOK (notebook);
+	gtk_notebook_popup_enable (book);
+	gtk_notebook_set_scrollable (book, TRUE);
+	gtk_notebook_set_show_border (book, FALSE);
 	gtk_notebook_set_show_tabs (book, TRUE);
 	gtk_notebook_set_tab_pos (book, GTK_POS_LEFT);
-	gtk_notebook_set_tab_reorderable (book, sw, TRUE);
-	gtk_notebook_set_tab_detachable (book, sw, TRUE);
-#ifdef USE_GTK2_12
+	// these 2 needed ?
+//	gtk_notebook_set_tab_reorderable (book, sw, TRUE);
+//	gtk_notebook_set_tab_detachable (book, sw, TRUE);
+#ifdef USE_GTK2_12DND
 	gpointer group = gtk_notebook_get_group (source);
 	gtk_notebook_set_group (book, group);
 #else
@@ -424,8 +610,9 @@ static GtkNotebook *_e2_output_tab_drop_new (GtkNotebook *source, GtkWidget *sw,
 	app.tab.dropwindow = window;
 	//set flag for tab context-menu so it shows only items relevant to separate window
 	app.tab.detached = TRUE;
-	printd (DEBUG, "set detached flag of tab %d", app.tab.labelnum);
-	//gtk will change the active tab in the source notebook
+	printd (DEBUG, "detached-flag of tab %d set TRUE", app.tab.labelnum);
+	//gtk will change the active tab in the source notebook and then the active
+	//page in the dest notebook - any way to prevent the former of these ?
 
 	return book;
 }
@@ -500,7 +687,15 @@ static void _e2_output_add_filetype_menu (gchar *text, GtkWidget *menu)
 	struct stat statbuf;
 	E2_ERR_DECLARE
 	gchar *local = F_FILENAME_TO_LOCALE (text);
+#ifdef E2_VFS
+# ifdef E2_VFSTMP
+	get relevant spacedata, or allow only local (NULL)
+# endif
+	VPATH sdata = {local , NULL};
+	if (e2_fs_stat (&sdata, &statbuf E2_ERR_PTR()))
+#else
 	if (e2_fs_stat (local, &statbuf E2_ERR_PTR()))
+#endif
 	{
 #ifdef E2_VFSTMP
 		//FIXME handle error
@@ -513,18 +708,30 @@ static void _e2_output_add_filetype_menu (gchar *text, GtkWidget *menu)
 	const gchar **actions = NULL;
 	gchar *ext, *ext2 = NULL;
 	gchar *base = g_path_get_basename (text);
+#ifdef E2_VFS
+	if (e2_fs_is_dir3 (&sdata E2_ERR_NONE()))
+#else
 	if (e2_fs_is_dir3 (local E2_ERR_NONE()))
+#endif
 	{
 		exec = FALSE;	//no special treatment of dirs
 		ext = ext2 = g_strconcat (".", _("<directory>"), NULL);
 	}
 	else
 	{
+#ifdef E2_VFS
+		exec = !e2_fs_access (&sdata, X_OK E2_ERR_NONE());
+#else
 		exec = !e2_fs_access (local, X_OK E2_ERR_NONE());
+#endif
 		//assumes extension is that part of the name after the leftmost '.'
 		ext = strchr (base, '.');	//assumes '.' is ascii
 		if (ext == base //it's a dot file
+#ifdef E2_VFS
+			|| (ext == NULL && e2_fs_is_text (&sdata E2_ERR_NONE()))) //no extension
+#else
 			|| (ext == NULL && e2_fs_is_text (local E2_ERR_NONE()))) //no extension
+#endif
 			//fake text extension
 			//too bad if this is not a recognised text extension in filetypes
 			ext = ".txt";
@@ -563,7 +770,12 @@ static void _e2_output_add_filetype_menu (gchar *text, GtkWidget *menu)
 		{
 			path = g_build_filename (*sysdir, "applications", localname, NULL);
 			localpath = F_FILENAME_TO_LOCALE (path); //probably redundant
+#ifdef E2_VFS
+			sdata.localpath = localpath;
+			if (!e2_fs_access (&sdata, R_OK E2_ERR_NONE()))
+#else
 			if (!e2_fs_access (localpath, R_OK E2_ERR_NONE()))
+#endif
 			{
 				e2_context_menu_create_desktop_actions_menu (menu, localpath);
 				g_free (path);
@@ -584,7 +796,7 @@ static void _e2_output_add_filetype_menu (gchar *text, GtkWidget *menu)
 }
 /**
 @brief check for or open a taskable item
-
+Expects BGL on/closed
 @param text utf8 string describing the item to open
 @param run TRUE to execute a matched command, FALSE to just return the match status
 
@@ -596,7 +808,16 @@ static gboolean _e2_output_open_text (gchar *text, gboolean run)
 	struct stat statbuf;
 	gchar *local = F_FILENAME_TO_LOCALE (text);
 	//make sure the text means something
+#ifdef E2_VFS
+# ifdef E2_VFSTMP
+	assume text from local command run on local items
+# endif
+	VPATH sdata = { local, NULL };
+
+	if (e2_fs_stat (&sdata, &statbuf E2_ERR_NONE()))
+#else
 	if (e2_fs_stat (local, &statbuf E2_ERR_NONE()))
+#endif
 	{
 		retval = FALSE;
 		if (run)
@@ -607,12 +828,21 @@ static gboolean _e2_output_open_text (gchar *text, gboolean run)
 	}
 	else if (run)
 	{
+#ifdef E2_VFS
+		retval = e2_task_backend_open (&sdata, FALSE);
+#else
 		retval = e2_task_backend_open (local, FALSE);
+#endif
 	}
 	else
 	{	//just check for a known filetype
+#ifdef E2_VFS
+		if (e2_fs_is_dir3 (&sdata E2_ERR_NONE())	//we can always handle dirs
+		|| !e2_fs_access (&sdata, X_OK E2_ERR_NONE()))	//and executable items
+#else
 		if (e2_fs_is_dir3 (local E2_ERR_NONE())	//we can always handle dirs
 		|| !e2_fs_access (local, X_OK E2_ERR_NONE()))	//and executable items
+#endif
 			retval = TRUE;
 		else
 		{
@@ -621,7 +851,11 @@ static gboolean _e2_output_open_text (gchar *text, gboolean run)
 			if (ext == NULL //no extension
 				|| ext == base) //hidden file
 			{
+#ifdef E2_VFS
+				retval = e2_fs_is_text (&sdata E2_ERR_NONE());
+#else
 				retval = e2_fs_is_text (local E2_ERR_NONE());
+#endif
 			}
 			else
 			{
@@ -1051,7 +1285,11 @@ static void _e2_output_set_op_wrap (gpointer pvalue, E2_OutputTabRuntime *rt)
 */
 /*static void _e2_output_savesel_cb (GtkWidget *menuitem,	E2_OutputTabRuntime *rt)
 {
-	e2_edit_dialog_save_selected (rt->buffer, app.main_window);
+	e2_edit_dialog_save_selected (rt->buffer,
+#ifdef E2_VFS
+	NULL,	//local namespace assumed
+#endif
+	app.main_window);
 } */
 /**
 @brief edit output-pane-tab text
@@ -1076,8 +1314,7 @@ static void _e2_output_edit_cb (GtkWidget *menuitem, E2_OutputTabRuntime *rt)
 static void _e2_output_tabattach_cb (GtkWidget *menuitem, E2_OutputTabRuntime *rt)
 {
 	GtkNotebook *newbook = GTK_NOTEBOOK (gtk_bin_get_child (GTK_BIN (rt->dropwindow)));
-	_e2_output_tab_reattach (newbook, GTK_NOTEBOOK (app.outbook));
-	gtk_widget_destroy (rt->dropwindow);
+	_e2_output_tab_move (rt->scroll, newbook, GTK_NOTEBOOK (app.outbook));
 }
 
 static void _e2_output_clear_cb (GtkWidget *menuitem, E2_OutputTabRuntime *rt)
@@ -1171,7 +1408,7 @@ static void _e2_output_show_context_menu (GtkWidget *textview,
 		_e2_output_add_filetype_menu (item_name, submenu);
 		g_free (item_name);	//??
 	}
-/*	item_name = g_strconcat (_A(9),".",_A(49),NULL);
+/*	item_name = g_strconcat (_A(9),".",_A(50),NULL);
 	e2_menu_add_action (menu, _("C_ommand help"), GTK_STOCK_HELP,
 		_("Show information about using the command line"),
 		item_name, "", NULL);
@@ -1275,6 +1512,23 @@ static gboolean _e2_output_button_press_cb (GtkWidget *textview,
 		return FALSE;
 */
 	printd (DEBUG, "output button press cb");
+#ifdef E2_TABS_DETACH
+	//just changing notebook page is not sufficient to set current tab when
+	//there's > 1 notebook
+	if (event->type == GDK_BUTTON_PRESS)
+	{
+		if (rt != curr_tab)
+		{
+			//swap with mimimum race-risk ...
+			*curr_tab = app.tab;	//backup current tab's data from stack to heap
+			app.tab = *rt;	//get the replacement stuff into stackspace
+			curr_tab = rt;
+			//adjust all relevant child foreground-tab pointers to/from the stacked tab data
+			e2_command_retab_children (&app.tab, curr_tab);
+			printd (DEBUG, "output tab change, new current-tab ID is %d", curr_tab->labelnum);
+		}
+	}
+#endif
 	E2_OutputTabRuntime *rrt = (rt == curr_tab) ? &app.tab : rt;
 	/*for double-clicks, the callback sequence is:
 	1, then another < click interval, then 1 more with 0 click interval
@@ -1348,7 +1602,11 @@ static gboolean _e2_output_button_press_cb (GtkWidget *textview,
 		//FIXME stop the selected text from being de-selected in the
 		//button-release callback - but we can't just block that !!
 	}
-	else if (event->button == 2)
+	else if (event->button == 2
+#ifdef E2_TABS_DETACH
+		&& !rrt->detached
+#endif
+		)
 	{
 		e2_window_output_hide (NULL, NULL, NULL);
 		return TRUE;
@@ -1469,7 +1727,7 @@ static void _e2_output_tabchange_cb (GtkNotebook *notebook,
 	GtkNotebookPage *page, guint page_num, gpointer data)
 {
 	E2_OutputTabRuntime *newtab;
-#ifdef USE_GTK2_10
+#ifdef E2_TABS_DETACH
 	GtkWidget *child = gtk_notebook_get_nth_page (notebook, page_num);
 	GList *member;
 	for (member = app.tabslist; member != NULL; member = member->next)
@@ -1549,6 +1807,7 @@ void e2_output_print_strerrno (void)
 @a msg is converted to utf-8 if it's not that form already
 Provides special handling of any '\\b', '\\r' char(s) in @a msg
 for an error message, style parameters are ignored
+Expects BGL to be on/closed
 
 @param tab pointer to data structure for the tab to get the message
 @param msg actual message
@@ -1573,9 +1832,7 @@ void e2_output_print (E2_OutputTabRuntime *tab,
 	if (//!debug &&
 		!app.output.visible && e2_option_bool_get_direct (app.output.opt_show_on_new))
 	{
-//		gdk_threads_enter ();
 		e2_window_output_show (NULL, NULL);
-//		gdk_threads_leave ();
 	}
 
 	//parse the msg string to handle special characters
@@ -2013,13 +2270,6 @@ GtkWidget *e2_output_initialise (void)
 	gtk_notebook_set_tab_pos (book, GTK_POS_LEFT);
 	gtk_notebook_set_scrollable (book, TRUE);
 	gtk_notebook_set_show_border (book, FALSE);
-#ifdef E2_TABS_DETACH
-	//enable tab dragging to/from new window
-	gtk_notebook_set_window_creation_hook
-		((GtkNotebookWindowCreationFunc) _e2_output_tab_drop_new,
-		NULL, //FIXME gpointer data
-		NULL);	//(GDestroyNotify) destroy
-#endif
 	const gchar *fntname = e2_utils_get_output_font ();
 	PangoFontDescription *font_desc = pango_font_description_from_string
 			(fntname);
@@ -2051,11 +2301,11 @@ GtkWidget *e2_output_initialise (void)
 		gtk_notebook_prepend_page (book, sw, label);
 		g_free (labltxt);
 #ifdef USE_GTK2_10
+		tab->labelnum = i;	//save tab id for matching
 		gtk_notebook_set_tab_reorderable (book, sw, TRUE);
 # ifdef E2_TABS_DETACH
 		gtk_notebook_set_tab_detachable (book, sw, TRUE);
-		tab->labelnum = i;	//save tab id for matching
-#  ifdef USE_GTK2_12
+#  ifdef USE_GTK2_12DND
 		gtk_notebook_set_group (book, app.outbook);	//instance-specific pointer
 #  else
 		gtk_notebook_set_group_id (book, getpid());
@@ -2068,9 +2318,35 @@ GtkWidget *e2_output_initialise (void)
 	app.tab = *tab;
 	curr_tab = tab;
 
+#ifdef E2_TABS_DETACH
+	//enable tab dragging to new windows
+# ifdef USE_GTK2_12DND
+	g_signal_connect (G_OBJECT (app.outbook), "create-window",
+		G_CALLBACK (_e2_output_tab_drop_new), NULL);	//CHECKME user_data
+# else
+	//this is the gtk 2.10 approach
+	gtk_notebook_set_window_creation_hook
+		((GtkNotebookWindowCreationFunc) _e2_output_tab_drop_new,
+		NULL, //CHECKME gpointer data
+		NULL);	//(GDestroyNotify) destroy
+# endif
+	//enable processing of tabs being dragged back from a new window
+	gtk_drag_dest_set (app.outbook, GTK_DEST_DEFAULT_DROP, target_table2, n_targets2,
+		GDK_ACTION_MOVE);
+	g_signal_connect (G_OBJECT (app.outbook), "drag-data-received",
+		G_CALLBACK (_e2_output_tabdrag_data_received_cb), NULL);	//CHECKME user_data
+	//capture clicks on tab labels
+	g_signal_connect (G_OBJECT (app.outbook), "grab-focus",
+		G_CALLBACK (_e2_output_grab_focus_cb), NULL);
+//	g_signal_connect (G_OBJECT (app.outbook), "focus-in-event",
+//		G_CALLBACK (_e2_output_focus_in_cb), NULL);
+	//for unblocking
+	g_signal_connect (G_OBJECT (app.outbook), "page-removed",
+		G_CALLBACK (_e2_output_tabgone_cb), app.main_window);
+#endif
+
 	g_signal_connect (G_OBJECT (app.outbook), "switch-page",
 		G_CALLBACK (_e2_output_tabchange_cb), NULL);	//no data
-
 	//setup data common to all tabs
 	//repetitive, but option data structs may move after options hash recreation,
 	//and possibly without re-running the options init function
@@ -2096,28 +2372,28 @@ void e2_output_actions_register (void)
 	gchar *action_name = g_strconcat(_A(9),".",_A(29),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_clear_action, NULL, FALSE);
-	action_name = g_strconcat(_A(9),".",_A(67),NULL);
+	action_name = g_strconcat(_A(9),".",_A(68),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_print_action, NULL, TRUE);
-	action_name = g_strconcat(_A(9),".",_A(49),NULL);  //output.help
+	action_name = g_strconcat(_A(9),".",_A(50),NULL);  //output.help
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_help_action, NULL, TRUE);
-	action_name = g_strconcat(_A(9),".",_A(73),NULL);  //output.scrolldown
+	action_name = g_strconcat(_A(9),".",_A(74),NULL);  //output.scrolldown
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_step, GINT_TO_POINTER (TRUE), TRUE);
-	action_name = g_strconcat(_A(9),".",_A(74),NULL);  //output.scrollup
+	action_name = g_strconcat(_A(9),".",_A(75),NULL);  //output.scrollup
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_step, GINT_TO_POINTER (FALSE), TRUE);
-	action_name = g_strconcat(_A(9),".",_A(63),NULL); //output.pagedown
+	action_name = g_strconcat(_A(9),".",_A(64),NULL); //output.pagedown
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_page, GINT_TO_POINTER (TRUE), TRUE);
-	action_name = g_strconcat(_A(9),".",_A(64),NULL); //output.pageup
+	action_name = g_strconcat(_A(9),".",_A(65),NULL); //output.pageup
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_page, GINT_TO_POINTER (FALSE), TRUE);
-	action_name = g_strconcat(_A(9),".",_A(47),NULL); //output.goto bottom
+	action_name = g_strconcat(_A(9),".",_A(48),NULL); //output.goto bottom
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_all, GINT_TO_POINTER (TRUE), TRUE);
-	action_name = g_strconcat(_A(9),".",_A(48),NULL); //output.goto top
+	action_name = g_strconcat(_A(9),".",_A(49),NULL); //output.goto top
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_output_scroll_all, GINT_TO_POINTER (FALSE), TRUE);
 	action_name = g_strconcat (_A(9),".",_A(26),NULL); //output.add
