@@ -1,6 +1,6 @@
-/* $Id: e2_pane.c 557 2007-07-24 11:59:11Z tpgww $
+/* $Id: e2_pane.c 894 2008-05-20 21:26:22Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -33,9 +33,7 @@ ToDo
 
 #include "emelfm2.h"
 //#include <unistd.h>
-#ifdef E2_VFS
 #include <string.h>
-#endif
 #include <pthread.h>
 #include "e2_pane.h"
 #include "e2_action.h"
@@ -45,12 +43,8 @@ ToDo
 #endif
 #include "e2_context_menu.h"
 #include "e2_filelist.h"
-#ifdef E2_VFS
-#include "e2_vfs.h"
-#endif
 
 static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
-//	E2_CDType *completed_flag,
 	gboolean history, gboolean hook);
 extern guint last_selected_rows;
 
@@ -61,79 +55,66 @@ extern guint last_selected_rows;
 #ifdef E2_VFS
 /**
 @brief setup the pane to which @a rt applies to show data for a specified namespace
-This does not do anything about the dir shown in the pane
 Any error messages downstream expect BGL closed here
-@param rt pointer to pane data struct
-@param space_uri pointer to namespace identifier
+@a spacedescriptor can have various forms, from full-URI to "-1" - see plugin for details
+@param rt pointer to pane data struct, NULL for active pane
+@param spacedescriptor utf8 string to be interpreted into existing or new namespace
 
-@return
+@return TRUE on successful completion or no change needed
 */
-void e2_pane_change_space (E2_PaneRuntime *rt, gchar *space_uri)
+gboolean e2_pane_change_space_byuri (E2_PaneRuntime *rt, gchar *spacedescriptor)
 {
-#ifdef E2_VFSTMP
-	//parse the URI
-	gchar *s, *pathhost = NULL;
-	//g_filename_from_uri() no good here (works only for file:// and strips protocol)
-	if ((s = strstr (space_uri, "://")) == NULL)
+	if (e2_fs_vfsfunc_ready ((gpointer *)&vfs.set_namedspace))
 	{
-		//split string into 2 parts, if we can
-		s += 3;	//skip located ascii bytes
-		s = strchr (s, G_DIR_SEPARATOR);
-		if (s == NULL)
-			return;
-//		pathhost = g_strndup (path, s - path);
-//		path = g_strdup (s);
+		ViewInfo *view;
+		if (rt == NULL)
+			view = curr_view;
+		else
+			view = (rt == &app.pane1) ? &app.pane1_view : &app.pane2_view;
+		return (vfs.set_namedspace (view, spacedescriptor));
+		//CHECKME last-used dir is opened when we change space ?
 	}
 	else
 	{
+		printd (WARN, "Cannot find vfs plugin");
+		return FALSE;
 	}
-#endif
-	PlaceInfo *spacedata;
-#ifdef E2_VFSTMP
-	//find an existing v-ptr that matches the parsed uri
-	spacedata = NULL;
-#else
-	spacedata = NULL;
-#endif
-	if (spacedata == NULL)
-	{
-		spacedata = ALLOCATE0 (PlaceInfo);
-		CHECKALLOCATEDFATAL (spacedata);
-#ifdef E2_VFSTMP
-		//populate it
-#endif
-	}
-	e2_pane_change_space_pointer (rt, spacedata);
 }
 /**
 @brief setup the pane to which @a rt applies to show data for a specified namespace
-This does not do anything about the dir shown in the pane
+If @a utfpath has a NULL localpath, then the relevant history or vtab path will be used
 @param rt pointer to pane data struct
-@param spacedata pointer to namespace data struct
+@param localpath pointer to path and namespace data struct
 
-@return
+@return TRUE on successful change or no change needed
 */
-void e2_pane_change_space_pointer (E2_PaneRuntime *rt, PlaceInfo *spacedata)
+gboolean e2_pane_change_space (E2_PaneRuntime *rt, VPATH *utfpath)
 {
 	PlaceInfo *current = rt->view->spacedata;
-	if (current == NULL && spacedata == NULL)
-		return;	//neither old nor new space are non-standard
-	rt->view->spacedata = spacedata;
-	if (current != NULL)	//any spacedata
-	{	//setup for reverting to mounted dirspace
-#ifdef E2_VFSTMP
-		//swap 2 history lists
-#endif
-		rt->view->dirtype = FS_LOCAL;	//FIXME displayed dir may be fuse mountpoint
+	if (current == utfpath->spacedata)
+		return TRUE;	//nothing to change
+//E2_VFSTMP may want to change dir anyway
+	if (e2_fs_vfsfunc_ready ((gpointer *)&vfs.set_space))
+	{
+		ViewInfo *view;
+		if (rt == NULL)
+			view = curr_view;
+		else
+			view = (rt == &app.pane1) ? &app.pane1_view : &app.pane2_view;
+		if (!vfs.set_space (view, utfpath->spacedata, TRUE))
+			return FALSE;
+//E2_VFSTMP CHECKME last-used dir is opened when we change space ?
+		if (utfpath->localpath == NULL)
+			_e2_pane_change_dir (rt, "FIXME-HISTORY-ITEM", FALSE, TRUE);
+		else
+			_e2_pane_change_dir (rt, (gchar *)utfpath->localpath, TRUE, TRUE);
+		return TRUE;
 	}
-	else //current == NULL, spacedata != NULL
-	{	//setup for showing virtual space
-#ifdef E2_VFSTMP
-		//swap 2 history lists
-#endif
-		rt->view->dirtype = spacedata->dirtype;
+	else
+	{
+		printd (WARN, "Cannot find vfs plugin");
+		return FALSE;
 	}
-//	_e2_pane_change_dir (rt, ??path, NULL, TRUE, TRUE);
 }
 #endif //fdef E2_VFS
 
@@ -191,7 +172,7 @@ void e2_pane_flag_active (void)
 }
 /**
 @brief change active pane from the current one to the other one
-
+Assumes BGL is closed on arrival here
 @return
 */
 void e2_pane_activate_other (void)
@@ -202,21 +183,17 @@ void e2_pane_activate_other (void)
 	curr_pane = tmp;
 	//force a status-line update
 	last_selected_rows = -1;
-	//DEBUG try this
-//	if (curr_view->completed && other_view->completed)
-//	{
-//		printd (DEBUG, "NOT waiting for fileview completion");
-		e2_fileview_switch_views ();
-//		gtk_widget_grab_focus (tmp->focus_widget);
-		if (GTK_WIDGET_HAS_FOCUS (other_view->treeview))
-			gtk_widget_grab_focus (curr_view->treeview);
-		e2_hook_list_run (&app.hook_pane_focus_changed, tmp);
-/*	}
-	else
-	{  //wait until all is done
-		printd (DEBUG, "start waiting for fileview completion");
-		app.timers[?] = g_timeout_add (200, _e2_pane_views_complete, NULL);
-	} */
+#ifdef E2_STATUS_DEMAND
+BGL
+	e2_window_update_status_bar (NULL);
+#endif
+	e2_fileview_switch_views ();
+//	gtk_widget_grab_focus (tmp->focus_widget);
+	if (GTK_WIDGET_HAS_FOCUS (other_view->treeview))
+		gtk_widget_grab_focus (curr_view->treeview);
+	gdk_threads_leave ();
+	e2_hook_list_run (&app.hook_pane_focus_changed, tmp);
+	gdk_threads_enter ();
 }
 /* *
 @brief change-directory completion-watch timer-function
@@ -260,7 +237,7 @@ void e2_pane_change_dir_watch (E2_PaneRuntime *rt, gchar *path,
 
 This is a wrapper for _e2_pane_change_dir (), with update of
 pane history list and hooklist
-It must be called with gtk's thread-lock closed
+gtk's thread-lock (BGL) may be open or closed
 
 @param rt pointer to pane data struct, or NULL for current pane
 @param path utf8 string containing actual or virtual path to directory to be opened
@@ -273,11 +250,14 @@ void e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path)
 }
 /**
 @brief change directory displayed in a specified pane
-Downstream functions assume BGL is active, and this func does too. Any gdk
-mutex management must be handled by caller
+Makes no assumption about whether BGL is active, but uses timer to force-open
+BGL for downstream processing where needed
 ~ at start of path is interpreted to $HOME
-redundant separators are removed, and one is added to the end,
+Surrounding single- or double-quotes are removed
+Redundant separators are removed, and one is added to the end,
 if not already there
+Any ".." at start or later in the path is corrected if possible (or if not,
+ignored)
 Does nothing if the path string is empty, shows error msg if
 path is unreachable
 Session-start check for valid path done in main(), not here
@@ -285,33 +265,22 @@ The supplied path string is unchanged.
 New path is stored in heap @ rt->path
 New path is also stored in array @ rt->view->dir
 
-when @a fsclass reflects a NEW virtual, @a prefix contains a sufficient
-identifier to allow the place to be opened
-when @a fsclass reflects an OLD virtual, @a prefix contains initial part of
-real local path used for relevant purposes e.g. unpacking
-
 @param rt pointer to pane data struct, or NULL for current pane
-@param path utf8 string containing actual or virtual path to directory to be opened
-@param completed_flag store for completed flag, or NULL when not watching for completion
+@param path utf8 string containing space-relative path of directory to be opened
 @param history TRUE to update pane history list
-@param hook TRUE to run change-dir hook functions (if any)
+@param hook TRUE to run change-dir hook functions (if any, normally has at least update dirline)
 
 @return
 */
 static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
-	//E2_CDType *completed_flag,
 	gboolean history, gboolean hook)
 {
-	printd (DEBUG, "_e2_pane_change_dir (rt:,path:%s,update:%d,hook:%d)",
+	gchar *freeme, *currpath;
+	printd (DEBUG, "_e2_pane_change_dir (rt:,path:\"%s\",update:%d,hook:%d)",
 		path, history, hook);
 
-	gchar *freeme;
-
 	if (path == NULL)
 		return;
-	//work on currently active file pane if argument is NULL
-	if (rt == NULL)
-		rt = curr_pane;
 
 	path = g_strdup (path);
 	g_strchug (path);	//trailing whitespace may be deliberate
@@ -322,8 +291,29 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 		g_free (path);
 		return;
 	}
-	else if (g_str_equal (path, "..")
-			&& g_str_equal (rt->path, G_DIR_SEPARATOR_S)
+	else if (path[0] == '"' || path[0] == '\'')
+	{
+		gint len = strlen (path);
+		gchar *s = path + len - sizeof (gchar);
+		if (*s == path[0])
+		{
+			path[0] = ' ';
+			*s = '\0';
+			g_strchug (path);	//trailing whitespace may be deliberate
+		}
+	}
+
+	//work on currently active file pane if argument is NULL
+	if (rt == NULL)
+		rt = curr_pane;
+
+	//work with a copy of path string which might be in transition by a prior cd
+	LISTS_LOCK
+	currpath = g_strdup (rt->path);
+	LISTS_UNLOCK
+
+	if (g_str_equal (path, "..") &&
+		g_str_equal (currpath, G_DIR_SEPARATOR_S)
 #ifdef E2_VFSTMP
 	//CHECKME how does updir from archive-root work here ?
 #endif
@@ -331,6 +321,7 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 	{
 		//ignore impossible change
 		g_free (path);
+		g_free (currpath);
 		return;
 	}
 	//CHECKME do a full interpretation here ?
@@ -380,7 +371,7 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 #ifdef E2_VFSTMP
 	//FIXME for changes up from a virtual root ...
 	if ( !g_str_equal (path, "..")
-	  || !g_str_equal (rt->path, G_DIR_SEPARATOR_S))
+	  || !g_str_equal (currpath, G_DIR_SEPARATOR_S))
 	{
 #endif
 	//cleanup the new path string
@@ -393,10 +384,8 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 			freeme = path;
 #ifdef E2_VFSTMP
 		//CHECKME confirm this is ok for non-local work-dir
-			path = e2_utils_translate_relative_path (rt->path, path);
-#else
-			path = e2_utils_translate_relative_path (rt->path, path);
 #endif
+			path = e2_utils_translate_relative_path (currpath, path);
 			g_free (freeme);
 		}
 		else
@@ -404,7 +393,7 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 			if (!g_path_is_absolute (path))
 			{
 				freeme = path;
-				path = g_strconcat (rt->path, path, NULL);
+				path = g_strconcat (currpath, path, NULL);
 				g_free (freeme);
 			}
 			//make sure the user didn't enter a 'dirty' path string
@@ -419,32 +408,36 @@ static void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path,
 	}
 #endif
 
-extern E2_CDdata p1cd_data;
-extern E2_CDdata p2cd_data;
-//extern pthread_mutex_t cd_mutex;
-extern GStaticRecMutex cd_mutex;
-//extern pthread_t cthreadID;
-//extern gboolean cd_blocked;
+	g_free (currpath);
 
-	E2_CDdata *data = (rt == &app.pane1) ? &p1cd_data : &p2cd_data;
-//	pthread_mutex_lock (&cd_mutex);
-	g_static_rec_mutex_lock (&cd_mutex);
+	E2_Listman *data = (rt == &app.pane1) ?
+		&app.pane1_view.listcontrols : &app.pane2_view.listcontrols;
+	LISTS_LOCK
 	data->view = rt->view;
 	data->history = history;
 	data->hook = hook;
 	//now signal we're ready to do a new cd
 //	any prior data->newpath is copied and cleared downstream, when ready
 	data->newpath = path;	//this is a copy
-	//use a timer, not a thread, coz a thread can't ensure that BGL is off
-	gint interval = (data->cd_blocked) ? 90 :	//check about every 90 mS for completion of prior cd
+	LISTS_UNLOCK
+	/*we use a timer, not a thread directly, to control the cd process cuz:
+		. we're happy to do the cd at a reasonably non-busy time
+		. if gtk's "native" BGL mutex is in play, _must_ ensure that BGL is off,
+		  for the downstream code that performs the cd
+	  (BUT this does make it tough to wait for the end of any specific cd)
+	  we check in the timer callback, not here, for any duplicated timer for this
+	  same pane (or the other one), so as to eliminate any race and missed cd request */
+	gint interval = (data->cd_working) ? 90 :	//check about every 90 mS for completion of prior cd
 		4; //short initial delay in case there's a following request for other pane
-	data->timer = g_timeout_add_full (G_PRIORITY_HIGH, interval,
+//	data->timer = with possible repeated timers for the same pane, the id is useless
+//#ifdef DEBUG_MESSAGES
+//	guint tid =
+//#endif
+	g_timeout_add_full (G_PRIORITY_HIGH, interval,
 		(GSourceFunc) e2_fileview_cd_manage, data, NULL);
-	printd (DEBUG, "initiated cd timer = %d", data->timer);
-//	pthread_mutex_unlock (&cd_mutex);
-	g_static_rec_mutex_unlock (&cd_mutex);
-/*
-	//setup for completion-watching if requested
+//	printd (DEBUG, "initiated cd timer = %d", tid);
+//	LISTS_UNLOCK
+/*	//setup for completion-watching if requested
 	if (completed_flag != NULL)
 	{
 		E2_CDwatch *data2 = ALLOCATE (E2_CDwatch);
@@ -512,11 +505,16 @@ gboolean e2_pane_goto_choice (E2_PaneRuntime *rt, GtkWidget *entry)
 	if (g_str_has_prefix (choice, G_DIR_SEPARATOR_S))
 		dir = g_strdup (choice);
 	else
+	{
 #ifdef E2_VFSTMP
 	//FIXME path for non-mounted dirs
 #else
+		//path string might be involved in a current cd
+		LISTS_LOCK
 		dir = g_strconcat (prt->path, choice, NULL);
+		LISTS_UNLOCK
 #endif
+	}
 #ifdef E2_VFSTMP
 	//FIXME handle vfs error reasonably
 #endif
@@ -545,14 +543,17 @@ gboolean e2_pane_goto_choice (E2_PaneRuntime *rt, GtkWidget *entry)
 */
 /*gboolean _e2_pane_views_complete (gpointer userdata)
 {
-	if (curr_view->completed && other_view->completed)
+	if (!(curr_view->listing || other_view->listing))
 	{
+		printd (DEBUG, "finished waiting for fileview completion");
+		gdk_threads_enter ();
 		e2_fileview_switch_views ();
 		gtk_widget_grab_focus (curr_view->treeview);
+		gdk_threads_leave ();
+		//run this with BGL open/off
 		e2_hook_list_run (&app.hook_pane_focus_changed, curr_pane);
 		return FALSE;
 	}
-	printd (DEBUG, "finished waiting for fileview completion");
 	return TRUE;  //continue waiting
 }
 */
@@ -682,8 +683,12 @@ static gboolean _e2_pane_go_forward (gpointer from, E2_ActionRuntime *art)
 	if (rt == NULL)
 		rt = curr_pane;
 	//history entries are prepended to list, forward in history = back in list
+	HISTORY_LOCK
 	if (rt->opendirs == NULL || rt->opendir_cur == 0)
+	{
+		HISTORY_UNLOCK
 		return FALSE;	//nowhere to go to
+	}
 
 	if (g_str_equal (IS_A ((GtkWidget *)from), "GtkButton"))
 	{
@@ -692,10 +697,10 @@ static gboolean _e2_pane_go_forward (gpointer from, E2_ActionRuntime *art)
 		{
 			GtkWidget *menu = gtk_menu_new ();
 			GtkWidget *item;
-			GList *member;
 			guint curr = (rt->opendir_cur > 1) ? rt->opendir_cur - 1 : 0;
-			for (member = g_list_nth (rt->opendirs, curr);
-					member != NULL; member = member->prev, curr--)
+			GList *member = g_list_nth (rt->opendirs, curr);
+			HISTORY_UNLOCK
+			while (member != NULL)
 			{
 				item = gtk_menu_item_new_with_label ((gchar *)member->data);
 				gtk_widget_show_all (item);
@@ -704,6 +709,10 @@ static gboolean _e2_pane_go_forward (gpointer from, E2_ActionRuntime *art)
 					GUINT_TO_POINTER (curr));
 				g_signal_connect (G_OBJECT (item), "activate",
 					G_CALLBACK (_e2_pane_history_selected_cb), rt);
+				HISTORY_LOCK
+				member = member->prev;
+				HISTORY_UNLOCK
+				curr--;
 			}
 			g_signal_connect (G_OBJECT (menu), "selection-done",
 				G_CALLBACK (e2_menu_destroy_cb), NULL);
@@ -717,6 +726,7 @@ static gboolean _e2_pane_go_forward (gpointer from, E2_ActionRuntime *art)
 	rt->opendir_cur--;
 //	gchar *dir = F_FILENAME_TO_LOCALE (
 	gchar *dir = g_list_nth_data (rt->opendirs, rt->opendir_cur);
+	HISTORY_UNLOCK
 	_e2_pane_change_dir (rt, dir, FALSE, TRUE);
 //	F_FREE (dir);
 
@@ -743,7 +753,7 @@ NOTE - IF USED, THIS MAY NEED A BASE-INDEX CORRECTION (1 TO 0) AND LIST-DIRECTIO
 	if (old != rt->opendir_cur)
 	{
 		gchar *dir = g_list_nth_data (rt->history, rt->opendir_cur - 1);
-		_e2_pane_change_dir (rt, dir, NULL, FALSE, TRUE);
+		_e2_pane_change_dir (rt, dir, FALSE, TRUE);
 	}
 }*/
 /**
@@ -761,8 +771,12 @@ static gboolean _e2_pane_go_back (gpointer from, E2_ActionRuntime *art)
 	if (rt == NULL)
 		rt = curr_pane;
 	//history entries are prepended to list, so back in history = forward in list
+	HISTORY_LOCK
 	if (rt->opendirs == NULL || rt->opendir_cur == g_list_length (rt->opendirs) - 1)
+	{
+		HISTORY_UNLOCK
 		return FALSE;
+	}
 
 	if (g_str_equal (IS_A ((GtkWidget *)from), "GtkButton"))
 	{
@@ -771,10 +785,10 @@ static gboolean _e2_pane_go_back (gpointer from, E2_ActionRuntime *art)
 		{
 			GtkWidget *menu = gtk_menu_new ();
 			GtkWidget *item;
-			GList *member;
 			guint curr = rt->opendir_cur + 1;
-			for (member = g_list_nth (rt->opendirs, curr); member != NULL;
-				member = member->next, curr++)
+			GList *member = g_list_nth (rt->opendirs, curr);
+			HISTORY_UNLOCK
+			while (member != NULL)
 			{
 				item = gtk_menu_item_new_with_label ((gchar *)member->data);
 				gtk_widget_show_all (item);
@@ -783,6 +797,10 @@ static gboolean _e2_pane_go_back (gpointer from, E2_ActionRuntime *art)
 					GUINT_TO_POINTER (curr));
 				g_signal_connect (G_OBJECT (item), "activate",
 					G_CALLBACK (_e2_pane_history_selected_cb), rt);
+				HISTORY_LOCK
+				member = member->next;
+				HISTORY_UNLOCK
+				curr++;
 			}
 			g_signal_connect (G_OBJECT (menu), "selection-done",
 				G_CALLBACK (e2_menu_destroy_cb), NULL);
@@ -795,6 +813,7 @@ static gboolean _e2_pane_go_back (gpointer from, E2_ActionRuntime *art)
 
 	rt->opendir_cur++;
 	gchar *dir = g_list_nth_data (rt->opendirs, rt->opendir_cur);
+	HISTORY_UNLOCK
 	_e2_pane_change_dir (rt, dir, FALSE, TRUE);
 
 	return TRUE;
@@ -827,7 +846,11 @@ static gboolean _e2_pane_mirror (gpointer from, E2_ActionRuntime *art)
 {
 	E2_PaneRuntime *rt = (E2_PaneRuntime *)art->action->data;
 	E2_PaneRuntime *ort = (rt == &app.pane1) ? &app.pane2 : &app.pane1;
-	_e2_pane_change_dir (rt, ort->path, TRUE, TRUE);
+	LISTS_LOCK
+	gchar *pathcopy = g_strdup (ort->path);
+	LISTS_UNLOCK
+	_e2_pane_change_dir (rt, pathcopy, TRUE, TRUE);
+	g_free (pathcopy);
 	return TRUE;
 }
 /**
@@ -871,7 +894,7 @@ static gboolean _e2_pane_goto_trash (gpointer from, E2_ActionRuntime *art)
 				break;
 			}
 		}
-		_e2_pane_change_dir (other_pane, (gchar *) tp, TRUE, FALSE);
+		_e2_pane_change_dir (other_pane, (gchar *) tp, TRUE, FALSE);	//NOTE no hook so no change of dirline !
 
 		return TRUE;
 	}
@@ -919,6 +942,34 @@ static gboolean _e2_pane_filters_show (gpointer from, E2_ActionRuntime *art)
 }
 #ifdef E2_VFS
 /**
+@brief select-namespace action, where a string argument is supplied
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE if the change was successful
+*/
+static gboolean _e2_pane_select_space (gpointer from, E2_ActionRuntime *art)
+{
+	return (e2_pane_change_space_byuri ((E2_PaneRuntime *)art->action->data, (gchar *)art->data));
+}
+/**
+@brief vfs menu-item callback
+This must be here, not in vfs plugin, as it is one mechanism to load that plugin
+Other callbacks are in the plugin
+@param menu_item the selected menu item widget
+@param view pointer to data struct for view to be changed
+
+@return
+*/
+static void _e2_pane_vfs_dialog_cb (GtkWidget *menu_item, ViewInfo *view)
+{
+	if (e2_fs_vfsfunc_ready ((gpointer *)&vfs.show_historydialog))
+		vfs.show_historydialog (view);
+	else
+		printd (WARN, "Cannot find vfs plugin");
+}
+/**
 @brief respond to pane vfs-button click
 
 @param from the button, menu item etc which was activated
@@ -926,7 +977,7 @@ static gboolean _e2_pane_filters_show (gpointer from, E2_ActionRuntime *art)
 
 @return TRUE
 */
-gboolean e2_pane_vfs_menu_show (gpointer from, E2_ActionRuntime *art)
+static gboolean _e2_pane_vfs_menu_show (gpointer from, E2_ActionRuntime *art)
 {
 	E2_PaneRuntime *rt = (E2_PaneRuntime *)art->action->data;	//will be NULL for active-pane action
 	ViewInfo *view;
@@ -934,7 +985,27 @@ gboolean e2_pane_vfs_menu_show (gpointer from, E2_ActionRuntime *art)
 		view = curr_view;
 	else
 		view = (rt == &app.pane1) ? &app.pane1_view : &app.pane2_view;
-	GtkWidget *menu = e2_menu_create_vfs_menu (view);
+
+	GtkWidget *item, *menu = gtk_menu_new ();
+
+	if (e2_fs_vfsfunc_ready ((gpointer *)&vfs.create_placesmenu))	//note no cursor change in event of slow loading
+	{
+		gboolean history = FALSE;
+		if (g_str_equal (IS_A ((GtkWidget *)from), "GtkButton"))
+		{
+			GdkModifierType mask = e2_utils_get_modifiers ();
+			if (mask & GDK_CONTROL_MASK)
+				history = TRUE;
+		}
+		vfs.create_placesmenu (menu, view, history);	//add plugin-specific items to menu
+	}
+
+	item = e2_menu_add (menu, _("_Edit places"), NULL, NULL,
+		_e2_pane_vfs_dialog_cb, view);
+
+	g_signal_connect (G_OBJECT (menu), "selection-done",
+		G_CALLBACK (e2_menu_destroy_cb), NULL);
+
 	guint32 time = gtk_get_current_event_time ();
 	if (rt == NULL)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -960,6 +1031,10 @@ static gboolean _e2_pane_select_all (gpointer from, E2_ActionRuntime *art)
 	if (rt != curr_pane)
 		e2_pane_activate_other ();
 	e2_fileview_select_all (NULL, curr_view);
+#ifdef E2_STATUS_DEMAND
+use a selection-change cb ? BGL management
+	e2_window_update_status_bar (NULL);
+#endif
 	return TRUE;
 }
 /**
@@ -973,6 +1048,10 @@ static gboolean _e2_pane_select_all (gpointer from, E2_ActionRuntime *art)
 static gboolean _e2_pane_current_select_all (gpointer from, E2_ActionRuntime *art)
 {
 	e2_fileview_select_all (NULL, curr_view);
+#ifdef E2_STATUS_DEMAND
+use a selection-change cb ? BGL management
+	e2_window_update_status_bar (NULL);
+#endif
 	return TRUE;
 }
 /**
@@ -997,6 +1076,10 @@ static gboolean _e2_pane_current_invert_all (gpointer from, E2_ActionRuntime *ar
 			else
 				gtk_tree_selection_select_iter (sel, &iter);
 		} while (gtk_tree_model_iter_next (mdl, &iter));
+#ifdef E2_STATUS_DEMAND
+use a selection-change cb ? BGL management
+		e2_window_update_status_bar (NULL);
+#endif
 		return TRUE;
 	}
 	return FALSE;
@@ -1019,6 +1102,10 @@ static gboolean _e2_pane_current_invert_current (gpointer from, E2_ActionRuntime
 			gtk_tree_selection_unselect_iter (sel, &iter);
 		else
 			gtk_tree_selection_select_iter (sel, &iter);
+#ifdef E2_STATUS_DEMAND
+use a selection-change cb ? BGL management
+		e2_window_update_status_bar (NULL);
+#endif
 		return TRUE;
 	}
 	return FALSE;
@@ -1028,6 +1115,13 @@ static gboolean _e2_pane_toggle_hidden_ (ViewInfo *view)
 {
 	view->show_hidden = !view->show_hidden;
 	e2_fileview_refilter_list (view);
+#ifdef E2_STATUS_DEMAND
+	if (view == curr_view)
+	{
+use a selection-change cb ? BGL management
+		e2_window_update_status_bar (NULL);
+	}
+#endif
 	//toggle the button
 	E2_ToggleType num = (view == &app.pane1_view) ?
 		E2_TOGGLE_PANE1HIDDEN : E2_TOGGLE_PANE2HIDDEN;
@@ -1149,6 +1243,18 @@ static gboolean _e2_pane_current_focus_bottom (gpointer from, E2_ActionRuntime *
 	}
 	return FALSE;
 }
+/**
+@brief action to sort a visible column in active pane
+This is primarily intended for a custom key-binding
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE always
+*/
+static gboolean _e2_pane_sort_active (gpointer from, E2_ActionRuntime *art)
+{
+	return (e2_fileview_sort_column (GPOINTER_TO_INT (art->action->data), curr_view));
+}
 
   /******************/
  /***** public *****/
@@ -1249,26 +1355,26 @@ void e2_pane_create (E2_PaneRuntime *rt)
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_focus_action, rt, FALSE);
 
-	action_name = g_strconcat (pstr,".",_A(45), NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_pane_go_forward, rt, FALSE);
-	action_name = g_strconcat (pstr,".",_A(44), NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_pane_go_back, rt, FALSE);
 	action_name = g_strconcat (pstr,".",_A(46), NULL);
+	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_go_forward, rt, FALSE);
+	action_name = g_strconcat (pstr,".",_A(45), NULL);
+	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_go_back, rt, FALSE);
+	action_name = g_strconcat (pstr,".",_A(47), NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_go_up, rt, FALSE);
-	action_name = g_strconcat (pstr,".",_A(59), NULL);
+	action_name = g_strconcat (pstr,".",_A(60), NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_goto_choice, rt, FALSE);
-	action_name = g_strconcat (pstr,".",_A(54),NULL);
+	action_name = g_strconcat (pstr,".",_A(55),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_mirror, rt, FALSE);
 	action_name = g_strdup ((num == 1) ?
 		toggles_array [E2_TOGGLE_PANE1HIDDEN] : toggles_array [E2_TOGGLE_PANE2HIDDEN]);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_toggle_hidden, rt->view, FALSE);
-	action_name = g_strconcat (pstr,".",_A(86),NULL);
+	action_name = g_strconcat (pstr,".",_A(95),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_select_all, rt, FALSE);
 	action_name = g_strconcat (pstr,".",_A(21), NULL);
@@ -1280,10 +1386,13 @@ void e2_pane_create (E2_PaneRuntime *rt)
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_filter_menu_create, rt, FALSE);
 #ifdef E2_VFS
-	action_name = g_strdup ((num == 1) ?
-		toggles_array [E2_TOGGLE_PANE1VIRTUAL] : toggles_array [E2_TOGGLE_PANE2VIRTUAL]);
+	action_name = g_strconcat (pstr,".",_A(117),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		e2_pane_vfs_menu_show, rt, FALSE);
+		_e2_pane_select_space, rt, TRUE);
+	action_name = g_strdup ((num == 1) ?
+		toggles_array [E2_TOGGLE_PANE1SPACE] : toggles_array [E2_TOGGLE_PANE2SPACE]);
+ 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_vfs_menu_show, rt, FALSE);
 #endif
 	//after the actions are in place, fill in the pane contents
 	e2_pane_create_part (rt);
@@ -1381,39 +1490,39 @@ Actions with pane-specific function are registered in e2_pane_create()
 */
 void e2_pane_actions_register (void)
 {  //these strings must be freeable
-	gchar *action_name = g_strconcat(_A(10),".",_A(45),NULL);
+	gchar *action_name = g_strconcat(_A(10),".",_A(46),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_go_forward, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(44),NULL);
+	action_name = g_strconcat(_A(10),".",_A(45),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_go_back, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(46),NULL);
+	action_name = g_strconcat(_A(10),".",_A(47),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_go_up, NULL, FALSE);
-	action_name = g_strconcat(_A(13),".",_A(59),NULL);
+	action_name = g_strconcat(_A(13),".",_A(60),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_goto_choice, NULL, FALSE);
-	action_name = g_strconcat(_A(6),".",_A(88),NULL);
+	action_name = g_strconcat(_A(6),".",_A(97),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_goto_trash, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(79),NULL);
+	action_name = g_strconcat(_A(10),".",_A(80),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_toggle_hidden, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(53),NULL);
+	action_name = g_strconcat(_A(10),".",_A(54),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_invert_all, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(87),NULL);
+	action_name = g_strconcat(_A(10),".",_A(96),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_invert_current, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(86),NULL);
+	action_name = g_strconcat(_A(10),".",_A(95),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_select_all, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(80),NULL);
+	action_name = g_strconcat(_A(10),".",_A(81),NULL);
 	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
 		e2_context_menu_show_menu_action, NULL, TRUE,
 		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_TOOLBAR, NULL);
 #ifdef E2_TREEDIALOG
-	action_name = g_strconcat(_A(10),".",_A(90),NULL);
+	action_name = g_strconcat(_A(10),".",_A(99),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		e2_tree_dialog_show_action, NULL, FALSE);
 #endif
@@ -1425,15 +1534,53 @@ void e2_pane_actions_register (void)
 	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_focus, NULL, FALSE,
 		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_TOOLBAR, NULL);
-	action_name = g_strconcat(_A(10),".",_A(48),NULL);
+	action_name = g_strconcat(_A(10),".",_A(49),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_focus_top, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(47),NULL);
+	action_name = g_strconcat(_A(10),".",_A(48),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_current_focus_bottom, NULL, FALSE);
 	action_name = g_strconcat(_A(10),".",_A(22),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_filters_show, NULL, FALSE);
+	//CHECKME which exclusions for these sort-actions ?
+	action_name = g_strconcat(_A(10),".",_A(86),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (FILENAME), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(88),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (SIZE), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(87),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (PERM), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(89),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (OWNER), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(84),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (GROUP), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(85),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (MODIFIED), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(82),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (ACCESSED), FALSE,
+		0, NULL);
+	action_name = g_strconcat(_A(10),".",_A(83),NULL);
+	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_sort_active, GINT_TO_POINTER (CHANGED), FALSE,
+		0, NULL);
+#ifdef E2_VFS
+	action_name = g_strconcat (_A(10),".",_A(117),NULL);
+ 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_select_space, NULL, TRUE);
+#endif
 }
 /**
 @brief register pane-related config options
@@ -1444,8 +1591,6 @@ Panebar default contents are set here, too
 */
 void e2_pane_options_register (gint num)
 {
-	e2_toolbar_panebar_register (num);	//setup the pane bar
-
 	gint i; gchar *option_name, *desc, *tip, *grey_me;
 	E2_OptionFlags flags;
 	desc = (num == 1) ? _A(11) : _A(12);
@@ -1469,6 +1614,9 @@ void e2_pane_options_register (gint num)
 		e2_option_bool_register (option_name, group_name, desc, tip, grey_me,
 		TRUE, flags);
 	}
+	//this needs to be after at least one non-tree options, so that the
+	//first-registered option is non-tree and dialog scrolling is setup properly
+	e2_toolbar_panebar_register (num);	//setup the pane bar
 
 	if (num == 1)
 		e2_toolbar_options_register (E2_BAR_PANE1);  //set general toolbar options
