@@ -1,4 +1,4 @@
-/* $Id: e2_plugins.c 515 2007-07-15 02:49:26Z tpgww $
+/* $Id: e2_plugins.c 650 2007-09-05 03:31:53Z tpgww $
 
 Copyright (C) 2003-2007 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
@@ -253,6 +253,7 @@ Any child plugins are cleared, also from app.plugins
 Adjusts app.plugins, so is not very suited to use during a walk of that list
 Any related config treestore data are not affected - any update of those
 must be done before coming here
+Expects BGL on/closed
 
 @param p plugin data struct
 @param force TRUE to force removal even if not cleaned up by the plugin
@@ -266,18 +267,32 @@ gboolean e2_plugins_unload1 (Plugin *p, gboolean force)
 	if (p->module != NULL) //this is not a child plugin
 	{
 		//FIXME handle plugins that are not allowed to be unloaded
-		if ((g_module_symbol (p->module, "clean_plugin", (gpointer) &clean) && clean (p))
-			|| force)
+		if ((g_module_symbol (p->module, "clean_plugin", (gpointer) &clean)
+			&& clean (p)) || force)
 		{
 			g_module_close (p->module);
 			GList *member;
+			Plugin *pc;
 			for (member = p->child_list; member != NULL; member = member->next)
 			{
-				app.plugins = g_list_remove (app.plugins, member->data);
-				DEALLOCATE (Plugin, member->data);
+				pc = member->data;
+				if (pc->cleanflags & E2P_CLEANICON)
+					g_free (pc->icon);
+				if (pc->cleanflags & E2P_CLEANLABEL)
+					g_free (pc->menu_name);
+				if (pc->cleanflags & E2P_CLEANTIP)
+					g_free (pc->description);
+				DEALLOCATE (Plugin, pc);
+				app.plugins = g_list_remove (app.plugins, pc);
 			}
 			if (p->child_list != NULL)
 				g_list_free (p->child_list);
+			if (p->cleanflags & E2P_CLEANICON)
+				g_free (p->icon);
+			if (p->cleanflags & E2P_CLEANLABEL)
+				g_free (p->menu_name);
+			if (p->cleanflags & E2P_CLEANTIP)
+				g_free (p->description);
 			DEALLOCATE (Plugin, p);
 			return TRUE;
 		}
@@ -292,9 +307,11 @@ gboolean e2_plugins_unload1 (Plugin *p, gboolean force)
 To the extent possible, each plugin listed in app.plugins is removed, and the
 list itself is cleared.
 Related config treestore data for plugins are not affected.
+@param force TRUE to force removal even if not cleaned up by the plugin
+
 @return
 */
-void e2_plugins_unload_all (void)
+void e2_plugins_unload_all (gboolean force)
 {
 	gboolean (*clean)(Plugin *);
 	Plugin *p;
@@ -307,20 +324,37 @@ void e2_plugins_unload_all (void)
 			&& p->module != NULL) //not a child plugin to be cleared
 		{
 			GList *childp, *mainmember;
-			if (g_module_symbol (p->module, "clean_plugin", (void *) &clean))
-				clean (p);	//don't care if this fails
-			g_module_close (p->module);
-			for (childp = p->child_list; childp != NULL; childp = childp->next)
+			Plugin *pc;
+			if ((g_module_symbol (p->module, "clean_plugin", (gpointer) &clean)
+				&& clean (p)) || force)
 			{
-				mainmember = g_list_find (app.plugins, childp->data);
-				if (mainmember != NULL)
-					mainmember->data = NULL;
-				DEALLOCATE (Plugin, childp->data);
+				g_module_close (p->module);
+				for (childp = p->child_list; childp != NULL; childp = childp->next)
+				{
+					pc = childp->data;
+					if (pc->cleanflags & E2P_CLEANICON)
+						g_free (pc->icon);
+					if (pc->cleanflags & E2P_CLEANLABEL)
+						g_free (pc->menu_name);
+					if (pc->cleanflags & E2P_CLEANTIP)
+						g_free (pc->description);
+					mainmember = g_list_find (app.plugins, pc);
+					if (mainmember != NULL)
+						mainmember->data = NULL;	//avoid double-cleans
+					DEALLOCATE (Plugin, pc);	//FIXME leaks strings
+				}
+				if (p->child_list != NULL)
+					g_list_free (p->child_list);
+				if (p->cleanflags & E2P_CLEANICON)
+					g_free (p->icon);
+				if (p->cleanflags & E2P_CLEANLABEL)
+					g_free (p->menu_name);
+				if (p->cleanflags & E2P_CLEANTIP)
+					g_free (p->description);
+
+				DEALLOCATE (Plugin, p);
+				member->data = NULL;
 			}
-			if (p->child_list != NULL)
-				g_list_free (p->child_list);
-			DEALLOCATE (Plugin, p);
-			member->data = NULL;
 		}
 	}
 	app.plugins = g_list_remove_all (app.plugins, NULL);
@@ -477,17 +511,20 @@ static void _e2_plugins_record_children (GtkTreeModel *model, GtkTreeIter *iter,
 						//this repetition not really needed, but just in case the user edited the sig ...
 						6, pc->signature,	//formatted as n-ANAME, n=0,1,....
 						-1);
-/*FIXME plugin setup probably but not necessarily makes these strings constant
-  and here we're replacing with allocated == leak when cleaning up
-  probably each plugin's internal cleanup function will need to handle cleanup */
-
 					pc->show_in_menu = childon;
+					if (pc->cleanflags & E2P_CLEANICON)
+						g_free (pc->icon);
+					if (pc->cleanflags & E2P_CLEANLABEL)
+						g_free (pc->menu_name);
+					if (pc->cleanflags & E2P_CLEANTIP)
+						g_free (pc->description);
 					gtk_tree_model_get (model, &iter2,
 //						1, &pc->show_in_menu,
 						2, &pc->menu_name,
 						3, &pc->icon,
 						4, &pc->description,
 						-1);
+					pc->cleanflags = E2P_CLEANALL;
 					//progressively build replacement list in same order as config data
 					reordered = g_list_append (reordered, pc);
 				}
@@ -592,7 +629,12 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 		//no path specified, so use the default plugins path
 		localdir = PLUGINS_DIR;	//localised
 
+#ifdef E2_VFS
+	VPATH data = { localdir, NULL };	//only local dirs for plugins
+	if (e2_fs_is_dir3 (&data E2_ERR_NONE()))	//only local places for plugins
+#else
 	if (e2_fs_is_dir3 (localdir E2_ERR_NONE()))	//only local places for plugins
+#endif
 	{
 		gchar *file_name = F_FILENAME_TO_LOCALE (utf_name);
 		localpath = g_build_filename (localdir, file_name, NULL);
@@ -612,18 +654,36 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 					{
 						//update the UI data if need be
 						p->show_in_menu = in_menu;
-						//NOTE replacements below can't be freed, they may be constant strings
-						//but if not constant, the replacement will leak
 						if (*label == '\0' && p->menu_name != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 								2, p->menu_name, -1);
 						else if (p->menu_name == NULL || !g_str_equal (p->menu_name, label))
+						{
+							if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
+								g_free (p->menu_name);
 							p->menu_name = g_strdup (label);
+							p->cleanflags |= E2P_CLEANLABEL;
+						}
 						if (*icon == '\0' && p->icon != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 								3, p->icon, -1);
 						else if (p->icon == NULL || !g_str_equal (p->icon, icon))
+						{
+							if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
+								g_free (p->icon);
 							p->icon = g_strdup (icon);
+							p->cleanflags |= E2P_CLEANICON;
+						}
+						if (*tip == '\0' && p->description != NULL)
+							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+								4, p->description, -1);
+						else if (p->description == NULL || !g_str_equal (p->description, tip))
+						{
+							if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
+								g_free (p->description);
+							p->description = g_strdup (tip);
+							p->cleanflags |= E2P_CLEANTIP;
+						}
 
 						app.plugins = g_list_append (app.plugins, p); //parent listed before any children
 
@@ -754,9 +814,8 @@ reloaded )
 */
 void e2_plugins_update (void)
 {
-	gchar *icon;
-	gchar *name;
-	gboolean load, menu;
+	gchar *icon, *name, *tip;
+	gboolean load, menu, freestrings;
 	GList *oldlist, *member;
 	Plugin *p;
 	E2_OptionSet *set = e2_option_get_simple ("plugins");
@@ -767,8 +826,9 @@ void e2_plugins_update (void)
 		app.plugins = NULL;
 		do
 		{
+			freestrings = TRUE;
 			gtk_tree_model_get (set->ex.tree.model, &iter,
-				0, &load, 1, &menu, 2, &name, 3, &icon, -1);
+				0, &load, 1, &menu, 2, &name, 3, &icon, 4, &tip, -1);
 			if (menu && !load) //fix any mistake by user
 			{
 				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter,
@@ -789,6 +849,7 @@ void e2_plugins_update (void)
 						break;
 				}
 			}
+
 			if (load && member != NULL)
 			{	//this one is to keep
 				oldlist = g_list_remove_link (oldlist, member);
@@ -796,8 +857,20 @@ void e2_plugins_update (void)
 				p = (Plugin *)member->data;
 				//update on-menu status in case was edited
 				p->show_in_menu = menu;
-				//update icon in case it was edited
+				//update strings in case they were edited
+				if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
+					g_free (p->icon);
 				p->icon = icon;
+				p->cleanflags |= E2P_CLEANICON;
+				if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
+					g_free (p->menu_name);
+				p->menu_name = name;
+				p->cleanflags |= E2P_CLEANLABEL;
+				if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
+					g_free (p->description);
+				p->description = tip;
+				p->cleanflags |= E2P_CLEANTIP;
+				freestrings = FALSE;
 				//process any child plugins
 				if (p->child_list != NULL)
 				{
@@ -845,7 +918,12 @@ void e2_plugins_update (void)
 			{
 				_e2_plugins_load1 (set->ex.tree.model, &iter);
 			}
-			g_free (name);
+			if (freestrings)
+			{
+				g_free (name);
+				g_free (icon);
+				g_free (tip);
+			}
 		} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter));
 
 		if (oldlist != NULL)
@@ -858,7 +936,7 @@ void e2_plugins_update (void)
 		}
 	}
 	else	//nothing in the store
-		e2_plugins_unload_all ();	//cleanup any existing runtime data
+		e2_plugins_unload_all (FALSE);	//cleanup any existing runtime data
 }
 /**
 @brief get the list of installed plugins
@@ -893,10 +971,96 @@ Plugin *e2_plugins_check_installed (const gchar *signature)
 	}
 	return NULL;
 }
+/**
+@brief load plugin with specified @a signature, if it's not loaded already
+
+@param signature unique ID string for the desired plugin, may be localised
+
+@return pointer to data struct for plugin which has the specified ID, else NULL
+*/
+Plugin *e2_plugins_load_plugin (const gchar *signature)
+{
+	Plugin *p = e2_plugins_check_installed (signature);
+	if (p == NULL)
+	{
+		gchar *ppath, *pname, *local, *s;
+
+		ppath = NULL;
+		s = strstr (signature, VERSION);
+		s = g_strndup (signature, s-signature);
+		pname = g_strconcat ("e2p_", s, ".so", NULL);
+		g_free (s);
+		GtkTreeIter iter;
+		E2_OptionSet *set = e2_option_get ("plugins");
+		GtkTreeModel *mdl = set->ex.tree.model;
+		gboolean inconfig = FALSE;
+		if (gtk_tree_model_get_iter_first (mdl, &iter))
+		{
+			if (e2_tree_find_iter_from_str_same (mdl, 5, pname, &iter))
+			{
+				inconfig = TRUE;
+				gtk_tree_model_get (mdl, &iter, 6, &ppath, -1);
+				if (ppath != NULL && *ppath != '\0')
+				{	//plugin is recorded in config data
+					s = ppath;
+					local = F_FILENAME_TO_LOCALE (ppath);
+					ppath = g_build_filename (local, pname, NULL); //create path from config data
+					g_free (s);
+					F_FREE (local);
+				}
+				else if (ppath != NULL)
+				{
+					g_free (ppath);
+					ppath = NULL;
+				}
+			}
+		}
+		if (ppath == NULL)
+		{	//try to get from default place
+			ppath = e2_utils_strcat (PLUGINS_DIR G_DIR_SEPARATOR_S, pname);	//localised path
+		}
+		p = e2_plugins_open1 (ppath);
+		g_free (ppath);
+		if (p != NULL)
+		{
+			if (!p->plugin_init (p))
+			{
+				printd (ERROR, "Can't initialize plugin %s", pname);
+				e2_plugins_unload1 (p, FALSE);
+				p = NULL;
+			}
+			if (inconfig)
+				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter, 0, TRUE, -1);
+			else
+			{
+				//add to config data
+				const gchar *label = (p->menu_name != NULL) ? p->menu_name : "";
+				const gchar *icon = (p->icon != NULL) ? p->icon : "";
+				const gchar *tip = (p->description != NULL) ? p->description : "";
+				gchar *name = F_FILENAME_FROM_LOCALE (pname);
+#ifdef USE_GTK2_10
+				gtk_tree_store_insert_with_values (GTK_TREE_STORE (set->ex.tree.model),
+					&iter, NULL, -1,
+#else
+				gtk_tree_store_insert (GTK_TREE_STORE (set->ex.tree.model), &iter, NULL, -1);
+				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter,
+#endif
+					//if path is not already in config, then it must be default
+					0, TRUE, 1, FALSE, 2, label, 3, icon, 4, tip, 5, name, 6, "", -1);
+
+				F_FREE (name);
+			}
+		}
+		else
+			printd (ERROR, "Can't find plugin %s", pname);
+	}
+	return p;
+}
+
 /**
 @brief find address of a function in another plugin
 
-This is intended to be used by plugins, if they want to interact.
+This is intended to be used by main code or other plugins which want to interact.
 If the plugin with the desired function is not presently loaded, any unloaded
 plugins recorded in config data are polled for a matching signature, and if so,
 the plugin will be loaded
@@ -910,52 +1074,13 @@ the plugin will be loaded
 gboolean e2_plugins_find_function (const gchar *signature, const gchar *func_name,
 	gpointer *address)
 {
-	Plugin *p = e2_plugins_check_installed (signature);
+	Plugin *p = e2_plugins_load_plugin (signature);
 	if (p == NULL)
-	{
-		//find and load the plugin
-		E2_OptionSet *set = e2_option_get_simple ("plugins");
-		GtkTreeIter iter;
-		if ((set != NULL) &&
-			gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
-		{
-			gboolean loaded;
-			do
-			{
-				gtk_tree_model_get (set->ex.tree.model, &iter, 0, &loaded, -1);
-				if (!loaded)
-				{
-					gtk_tree_store_set (set->ex.tree.model, &iter, 0, TRUE, -1);
-					_e2_plugins_load1 (set->ex.tree.model, &iter);
-					GList *member = g_list_last (app.plugins);
-					p = member->data;
-					if (g_str_equal (p->signature, signature))
-						break;
-					else
-					{
-						//no change to any child-plugin config data before dumping
-						if (e2_plugins_unload1 (p, FALSE))
-							gtk_tree_store_set (set->ex.tree.model, &iter, 0, FALSE, -1);
-						p = NULL;
-					}
-				}
-			} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter));
-			if (p == NULL)
-			{
-				printd (WARN, "module with %s not found", signature);
-				return FALSE;
-			}
-		}
-		else
-		{
-			printd (WARN, "plugins config doesn't exist");
-			return FALSE;
-		}
-	}
+		return FALSE;
 	if (!g_module_symbol (p->module, func_name, address))
 	{
-		printd (WARN, "couldn't find %s in module %s: %s",
-			func_name, signature, g_module_error());
+		printd (WARN, "couldn't find %s in module %s: %s", func_name, signature,
+			g_module_error());
 		return FALSE;
 	}
 	return TRUE;
@@ -1124,8 +1249,14 @@ static void _e2_plugins_tree_defaults (E2_OptionSet *set)
 	g_strdup ("false|false||||e2p_config.so|"),
 	g_strdup ("false|false||||e2p_clone.so|"),
 	g_strdup ("false|false||||e2p_view.so|"),
+#ifdef E2_VFS
+	g_strdup ("false|false||||e2p_vfs.so|"),
+	g_strdup ("false|false||||e2p_gvfs.so|"),
+#endif
 	g_strdup(">"),
 	NULL);
+	//FIXME setup some mapping between filename and signature name to enable
+	//lookup by signature
 }
 /**
 @brief register plugins actions
