@@ -1,6 +1,6 @@
-/* $Id: e2_window.c 557 2007-07-24 11:59:11Z tpgww $
+/* $Id: e2_window.c 850 2008-04-07 10:33:34Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -47,6 +47,9 @@ ToDo - description of how this works
 extern GList *cols_data;
 extern gint col_width_store[2][MAX_COLUMNS];
 extern gint stored_col_order[2][MAX_COLUMNS];
+//main-window size before fullscreen enacted
+gint real_width;
+gint real_height;
 
 static gdouble _e2_window_get_pos (GtkWidget *paned);
 
@@ -154,7 +157,7 @@ static gint _e2_window_get_pos_from_str (gchar *str, GtkPaned *paned)
 Value(s) provided in @a arg is (are both) a proportion of the relevant window
 dimension, or (the first one) may be * which causes the stored prior value
 of the fraction to be used
-
+Expects BGL to be on/closed on arrival here
 @param paned ptr to paned widget to be adjusted
 @param pos pointer to store for paned position value
 @param ratio_last pointer to store for the prior value of the 'paned ratio' (position / max. position)
@@ -162,8 +165,6 @@ of the fraction to be used
 
 @return
 */
-//FIXME setting the ratio to 0 when the taskbar is in pane 1 generates a
-//gtk warning about -ve widget size
 static void _e2_window_adjust_panes_ratio (GtkPaned *paned, gint *pos,
 	gdouble *ratio_last, gchar *arg)
 {
@@ -220,20 +221,29 @@ static void _e2_window_adjust_panes_ratio (GtkPaned *paned, gint *pos,
 	}
 	gtk_paned_set_position (paned, cur);
 
+	gdouble ratio_now = cur / paned->max_position;
+	if (ratio_now > 0.001 && ratio_now < 0.999)
+		*ratio_last = ratio_now;
+
 	if (paned == GTK_PANED (app.window.panes_paned))
 	{	//do this only for filelist divider changes (may hang for output pane divider)
-		WAIT_FOR_EVENTS
-		if (cur > 0.99 && GTK_WIDGET_HAS_FOCUS (app.pane2_view.treeview))
+		if (ratio_now > 0.99 && GTK_WIDGET_HAS_FOCUS (app.pane2_view.treeview))
+		{
+			WAIT_FOR_EVENTS	//note - expects BGL closed
 			gtk_widget_grab_focus (app.pane1_view.treeview);
-		else if (cur < 0.01 && GTK_WIDGET_HAS_FOCUS (app.pane1_view.treeview))
+		}
+		else if (ratio_now < 0.01 && GTK_WIDGET_HAS_FOCUS (app.pane1_view.treeview))
+		{
+			WAIT_FOR_EVENTS
 			gtk_widget_grab_focus (app.pane2_view.treeview);
+		}
 	}
 }
 /**
 @brief change position of file-panes separator
 
 This is a wrapper for _e2_window_adjust_panes_ratio ()
-
+Downstream expects BGL closed
 @param arg string with new ratio to be set
 
 @return
@@ -294,8 +304,14 @@ static void _e2_window_set_icon (void)
 			list = g_list_append (list, pixbuf);
 		}
 	}
+	if (list != NULL)
+	{
+		gtk_window_set_default_icon_list (list);
+		//gtk copies list and ref's its items, so cleanup here
+		g_list_foreach (list, (GFunc) g_object_unref, NULL);
+		g_list_free (list);
+	}
 	g_free (icons_dir);
-	gtk_window_set_default_icon_list (list);
 	e2_fs_dir_close (d);
 }
 
@@ -303,11 +319,10 @@ static void _e2_window_set_icon (void)
  /***** callbacks *****/
 /*********************/
 /**
-@brief callback for signal emitted when file panes divider is moved
-
-
+@brief callback for "size-allocate" signal on app.pane1.outer_box
+This happens when file panes divider moves (among other instances)
 @param widget UNUSED the resized widget, app.pane1.outer_box
-@param alloc
+@param alloc the allocation for @a widget
 @param rt pointer to window data struct
 
 @return
@@ -315,15 +330,17 @@ static void _e2_window_set_icon (void)
 static void _e2_window_filepanes_adjusted_cb (GtkWidget *widget,
 	GtkAllocation *alloc, E2_WindowRuntime *rt)
 {
-//	printd (DEBUG, "_e2_window_filepanes_adjusted_cb (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
-	rt->panes_paned_ratio = _e2_window_get_pos (rt->panes_paned);
-//	printd (DEBUG, "1ratio: %f", rt->panes_paned_ratio);
-/* DON'T BOTHER AUTOSWAP (AND IT IS NOW CIRCULAR)
-	if (((rt->panes_paned_ratio == 0.0) && (curr_pane == &app.pane1)) ||
-		((rt->panes_paned_ratio == 1.0) && (curr_pane == &app.pane2)))
-		e2_pane_activate_other (); */
-	if ((rt->panes_paned_ratio != 0.0) && (rt->panes_paned_ratio != 1.0))
-		rt->panes_paned_ratio_last = rt->panes_paned_ratio;
+	static gint prev_size = -1;
+	if ((rt->panes_horizontal && alloc->height != prev_size)
+	 || (!rt->panes_horizontal && alloc->width != prev_size))
+	{	//this is not just some gtk housekeeping
+		printd (DEBUG, "_e2_window_filepanes_adjusted_cb (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
+		prev_size = (rt->panes_horizontal) ? alloc->height : alloc->width;
+		rt->panes_paned_ratio = _e2_window_get_pos (rt->panes_paned);
+//		printd (DEBUG, "1ratio: %f", rt->panes_paned_ratio);
+		if (rt->panes_paned_ratio > 0.001 && rt->panes_paned_ratio < 0.999)
+			rt->panes_paned_ratio_last = rt->panes_paned_ratio;
+	}
 }
 /**
 @brief after a file pane is mapped or unmapped, ensure the correct toggle button is visible in the other pane's toolbar
@@ -344,11 +361,11 @@ static void _e2_window_map_pane_cb (GtkWidget *widget, gpointer state)
 		e2_toolbar_button_toggle (toggles_array [num]);
 }
 /**
-@brief callback for signal emitted when output pane size is adjusted
-
-
+@brief callback for signal emitted when output pane size is allocated
+This happens when output content change varies the size of @a widget,
+and also when the output-paned divider is moved
 @param widget UNUSED the resized widget, app.outbook
-@param alloc
+@param alloc the allocation for @a widget
 @param rt pointer to window data struct
 
 @return TRUE, always
@@ -356,19 +373,24 @@ static void _e2_window_map_pane_cb (GtkWidget *widget, gpointer state)
 static void _e2_window_outputpane_adjusted_cb (GtkWidget *widget,
 	GtkAllocation *alloc, E2_WindowRuntime *rt)
 {
-//	printd (DEBUG, "_e2_window_outputpane_adjusted_cb (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
-	rt->output_paned_ratio = _e2_window_get_pos (app.window.output_paned);
-//	printd (DEBUG, "2ratio: %f", rt->output_paned_ratio);
-	//remember visibility state
-	app.output.visible = (rt->output_paned_ratio != 1.0);
-	if ((rt->output_paned_ratio != 0.0) && (rt->output_paned_ratio != 1.0))
-		rt->output_paned_ratio_last = rt->output_paned_ratio;
-	//make sure the correct toggle btn is shown
-	gboolean truenow =
-	e2_toolbar_toggle_button_get_state (toggles_array [E2_TOGGLE_OUTPUTFULL]);
-	if ((!truenow && rt->output_paned_ratio < 0.001)
-	  ||(truenow && rt->output_paned_ratio >= 0.001))
-		e2_toolbar_button_toggle (toggles_array [E2_TOGGLE_OUTPUTFULL]);
+	static gint prev_height = -1;
+	if (alloc->height != prev_height)
+	{	//this is not just more output being displayed
+		printd (DEBUG, "_e2_window_outputpane_adjusted_cb (widget:,alloc:%d, %d, %d-%d,rt:_)", alloc->x, alloc->y, alloc->width, alloc->height);
+		prev_height = alloc->height;
+		rt->output_paned_ratio = _e2_window_get_pos (app.window.output_paned);
+//		printd (DEBUG, "2ratio: %f", rt->output_paned_ratio);
+		//remember visibility state
+		app.output.visible = (rt->output_paned_ratio != 1.0);
+		if (rt->output_paned_ratio > 0.001 && rt->output_paned_ratio < 0.999)
+			rt->output_paned_ratio_last = rt->output_paned_ratio;
+		//make sure the correct toggle btn is shown
+		gboolean truenow =
+		e2_toolbar_toggle_button_get_state (toggles_array [E2_TOGGLE_OUTPUTFULL]);
+		if ((!truenow && rt->output_paned_ratio < 0.001)
+		  ||(truenow && rt->output_paned_ratio >= 0.001))
+			e2_toolbar_button_toggle (toggles_array [E2_TOGGLE_OUTPUTFULL]);
+	}
 }
 /**
 @brief after the output pane is mapped or unmapped, ensure the correct toggle button is visible in the commandbar
@@ -395,6 +417,7 @@ static void _e2_window_map_output_cb (GtkWidget *widget, gpointer state)
 This is a callback for ouptput pane "focus-in-event",
 and also used generally
 Runs output adjust ratio action
+Downstream expects BGL to be on/closed
 
 @param widget UNUSED the newly-focused entry widget or NULL
 @param data UNUSED
@@ -417,6 +440,7 @@ gboolean e2_window_output_show (GtkWidget *widget, gpointer data)
 This is a callback for command-line "focus-out-event",
 and also used generally
 Runs output adjust ratio action
+Downstream expects BGL to be on/closed
 
 @param widget UNUSED newly-departed widget or NULL
 @param event UNUSED pointer to event data struct
@@ -437,6 +461,29 @@ gboolean e2_window_output_hide (GtkWidget *widget, GdkEventFocus *event,
 	return FALSE;
 }
 /**
+@brief callback for "window-state-event" signal emitted when the main window changes
+@param widget the main-window object
+@param event the GdkEventWindowState which triggered this signal
+@param user_data UNUSED data specified when the signal handler was connected
+
+@return FALSE always, so event propogates
+*/
+static gboolean _e2_window_change_cb (GtkWidget *widget, GdkEventWindowState *event,
+	gpointer user_data)
+{
+	//GdkWindowState event->changed_mask;
+	//GdkWindowState event->new_window_state
+	if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		//toggling fullscreen on or off will grab the "unfull" size each time
+		real_width = app.main_window->allocation.width;
+		real_height = app.main_window->allocation.height;
+		printd (DEBUG, "window-change cb main window size: W %d H %d", real_width, real_height);
+	}
+	app.mainwindow_state = event->new_window_state;
+	return FALSE;
+}
+/**
 @brief callback for signal emitted when the main window is first shown
 
 paned dividers' positions are 'crudely' set when those widgets are created,
@@ -454,15 +501,20 @@ again, to avoid unnecessary callbacks
 static void _e2_window_show_cb (GtkWidget *window, E2_WindowRuntime *rt)
 {
 	printd (DEBUG, "show main window cb");
+	//initialize window flags
+	app.mainwindow_state = gdk_window_get_state (app.main_window->window);
+	//notice future window-state changes
+	g_signal_connect (G_OBJECT(app.main_window), "window-state-event",
+		G_CALLBACK(_e2_window_change_cb), NULL);
 	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
 		GTK_PANED (rt->panes_paned)->max_position * rt->panes_paned_ratio);
 	gtk_paned_set_position (GTK_PANED (app.window.output_paned),
 		GTK_PANED (app.window.output_paned)->max_position * rt->output_paned_ratio);
 	rt->panes_paned_pos = gtk_paned_get_position (GTK_PANED (rt->panes_paned));
 	rt->output_paned_pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
-
+	//ensure inter-list divider-position is logged when divider is moved
 	g_signal_connect (G_OBJECT (app.pane1.outer_box), "size-allocate",
-		G_CALLBACK (_e2_window_filepanes_adjusted_cb), rt);
+			G_CALLBACK (_e2_window_filepanes_adjusted_cb), rt);
 	//ensure correct toggle button is displayed when a pane divider is dragged
 	g_signal_connect (G_OBJECT (app.pane1.pane_sw), "map",
 			G_CALLBACK (_e2_window_map_pane_cb), GINT_TO_POINTER (TRUE));
@@ -472,8 +524,9 @@ static void _e2_window_show_cb (GtkWidget *window, E2_WindowRuntime *rt)
 			G_CALLBACK (_e2_window_map_pane_cb), GINT_TO_POINTER (TRUE));
 	g_signal_connect (G_OBJECT (app.pane2.pane_sw), "unmap",
 			G_CALLBACK (_e2_window_map_pane_cb), GINT_TO_POINTER (FALSE));
+	//ensure inter-pane divider-position is logged when divider is moved
 	g_signal_connect (G_OBJECT (app.outbook), "size-allocate",
-		G_CALLBACK (_e2_window_outputpane_adjusted_cb), rt);
+			G_CALLBACK (_e2_window_outputpane_adjusted_cb), rt);
 	g_signal_connect (G_OBJECT (app.outbook), "map",
 			G_CALLBACK (_e2_window_map_output_cb), GINT_TO_POINTER (TRUE));
 	g_signal_connect (G_OBJECT (app.outbook), "unmap",
@@ -486,8 +539,34 @@ static void _e2_window_show_cb (GtkWidget *window, E2_WindowRuntime *rt)
 	//now that window is established, we are ready to initialise all toolbars' overflow handling
 	e2_toolbar_initialise_space_handler (NULL);
 }
-#ifdef E2_COMPOSIT
 /**
+@brief callback for main window "destroy-event", "delete-event" signals
+
+@param widget UNUSED the window button which was activated
+@param event UNUSED pointer to event data
+@param forced pointerised gboolean, NULL if the user is permitted to choose to keep the window open
+
+@return TRUE if the user cancelled the close, or else it does not return at all
+*/
+static gboolean _e2_window_gone_cb (GtkWidget *widget, GdkEvent *event, gpointer forced)
+{
+	e2_main_closedown (GPOINTER_TO_INT (forced), TRUE, TRUE);
+	return TRUE;
+}
+/* *
+@brief callback for main window "destroy" signal
+@param object UNUSED the window object
+@param user_data UNUSED
+@return
+*/
+/*static void _e2_window_destroy_cb (GtkObject *object, gpointer user_data)
+{
+	printd (DEBUG, "main window \"destroy\" signal callback");
+	e2_main_closedown (TRUE, TRUE, TRUE);
+}
+*/
+#ifdef E2_COMPOSIT
+/* *
 @brief callback for "expose-event" signal on @a widget
 This is called when we need to draw the window's contents. The X server sends
 an expose event when the window becomes visible on screen, meaning we need to
@@ -500,7 +579,7 @@ be sent whenever the window is uncovered by another.)
 
 @return FALSE always, to propogate the event to other handlers
 */
-static gboolean _e2_window_expose_cb (GtkWidget *widget, GdkEventExpose *event,
+/*static gboolean _e2_window_expose_cb (GtkWidget *widget, GdkEventExpose *event,
 	gpointer data)
 {
 	cairo_t *cr = gdk_cairo_create (widget->window);
@@ -516,7 +595,7 @@ static gboolean _e2_window_expose_cb (GtkWidget *widget, GdkEventExpose *event,
 		e2_option_int_get ("window-bleed"):
 		GPOINTER_TO_INT (data);
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-/*	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0); //fully transparent
+/ *	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0); //fully transparent
 	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0); //opaque white
  or partially transparent
 	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, (gfloat) level / 100.0);	//black transparent as wanted
@@ -532,13 +611,14 @@ static gboolean _e2_window_expose_cb (GtkWidget *widget, GdkEventExpose *event,
 
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 	cairo_paint (cr);
-*/
+* /
 	cairo_paint_with_alpha (cr, (gfloat) level / 100.0);
 
 	cairo_destroy (cr);
 	return FALSE;
 }
-/**
+*/
+/* *
 @brief callback for "screen-changed" signal on @a widget
 GTK supports migration of running applications between X servers, which might not
 support the same features (and in partucular, an alpha channel).
@@ -551,7 +631,7 @@ accordingly.
 
 @return
 */
-static void _e2_window_screen_changed_cb (GtkWidget *widget,
+/*static void _e2_window_screen_changed_cb (GtkWidget *widget,
 	GdkScreen *old_screen, gpointer data)
 {
 	GdkScreen *new_screen;
@@ -559,7 +639,7 @@ static void _e2_window_screen_changed_cb (GtkWidget *widget,
 //#ifdef USE_GTK2_12
 //	GdkDisplay *display = ; what for old screen ??
 //	if gdk_display_supports_composite (display))
-//#el
+//#else
 #if defined (USE_GTK2_10)
 	if (gdk_screen_is_composited (old_screen))
 #else //def USE_GTK2_8
@@ -580,13 +660,13 @@ static void _e2_window_screen_changed_cb (GtkWidget *widget,
 #endif
 		{	//but this one doesn't, make some changes
 #ifdef USE_GTK2_12
-/*CECKME
+/ *CHECKME
 			GtkWindowType wtype;
 			g_object_get (G_OBJECT (widget), "type", &wtype, NULL);
 			if (wtype == GTK_WINDOW_TOPLEVEL)
 				gdk_window_set_opacity (widget->window, gdouble opacity);
 			else //<child window
-*/
+* /
 				gdk_window_set_composited (widget->window, FALSE);
 #endif
 			g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
@@ -660,6 +740,7 @@ setup:
 	}
 #endif
 }
+*/
 #endif //def E2_COMPOSIT
 
   /*******************/
@@ -684,6 +765,7 @@ static gboolean _e2_window_toggle_panes_direction
 @brief change the position of the filepanes separator
 
 This is a wrapper for _e2_window_adjust_panes_ratio ()
+Downstream expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
@@ -703,6 +785,7 @@ static gboolean _e2_window_adjust_pane_ratio_action
 
 The visible toggle button is swapped, and the panes-ajust fn
 is called with either "1" or "*"
+Expects BGL closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
@@ -723,6 +806,7 @@ static gboolean _e2_window_toggle_full_pane1 (gpointer from, E2_ActionRuntime *a
 
 The visible toggle button is swapped, and the panes-ajust fn
 is called with either "0" or "*"
+Expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
@@ -743,6 +827,7 @@ static gboolean _e2_window_toggle_full_pane2 (gpointer from, E2_ActionRuntime *a
 
 The visible toggle button is swapped or changed, and the panes-adjust fn
 is called with either "0" or "*"
+Expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
@@ -788,6 +873,7 @@ static gboolean _e2_window_toggle_full_output (gpointer from, E2_ActionRuntime *
 
 The visible toggle button is swapped, and the panes-ajust fn
 is called with either "1" or "*"
+Expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
@@ -809,16 +895,12 @@ static gboolean _e2_window_toggle_visible_output (gpointer from, E2_ActionRuntim
 This is a wrapper for _e2_window_toggle etc
 The visible toggle button is swapped, and the panes-ajust fn
 is called with "*" ",1" or ",0"
+Expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
 @param art action runtime data
 
 @return TRUE if the argument string is valid
-
-* @param rt ptr to window data struct
-@param arg string with new ratio to be set
-
-@return
 */
 static gboolean _e2_window_adjust_output_ratio_action
 	(gpointer from, E2_ActionRuntime *art)
@@ -854,6 +936,22 @@ static gboolean _e2_window_adjust_output_ratio_action
 	}
 	return FALSE;
 }
+/**
+@brief window-fullscreen toggle action
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE
+*/
+static gboolean _e2_window_toggle_full_screen (gpointer from, E2_ActionRuntime *art)
+{
+	if (app.mainwindow_state & GDK_WINDOW_STATE_FULLSCREEN)
+		gtk_window_unfullscreen (GTK_WINDOW(app.main_window));
+	else
+		gtk_window_fullscreen (GTK_WINDOW(app.main_window));
+	return TRUE;
+}
 
   /******************/
  /***** public *****/
@@ -865,18 +963,19 @@ static gboolean _e2_window_adjust_output_ratio_action
 Levels < 10 are treated as 0 and > 95 is ignored
 Expects BGL on/active
 @param window the widget for the window to be processed
-@param level translucence %, 0 (opaque) to 50 (faint), or -1 (config default level)
+@param level opacity %, 50 (faint) to 100 (opaque), or -1 (config default level)
 
 @return
 */
-void e2_window_set_translucency (GtkWidget *window, gint level)
+void e2_window_set_opacity (GtkWidget *window, gint level)
 {
-	GdkScreen *screen;
+/*	GdkScreen *screen;
 #ifndef USE_GTK2_10
 	GdkColormap *colormap;
 #endif
-	gint deflevel = e2_option_int_get ("window-bleed");	//0=opaque ... 50=faint
-	if (deflevel > 0)	//compositing is active
+*/
+	gint deflevel = e2_option_int_get ("window-opacity");	//50=faint...100=opaque
+	if (deflevel < 100)	//compositing is active
 	{
 		if (level < 0)
 			level = deflevel;
@@ -884,14 +983,17 @@ void e2_window_set_translucency (GtkWidget *window, gint level)
 		GtkWindowType wtype;
 		g_object_get (G_OBJECT (window), "type", &wtype, NULL);
 		if (wtype != GTK_WINDOW_TOPLEVEL)
-			level = MIN (deflevel - 5, level);
-		if (level > 50)	//igonre unusably-invisible levels
-			level = 50;
-		else if (level < 3)	//ignore nearly-opaque levels
-			level = 0;
+			level = MAX (deflevel + 5, level);
+		if (level < 30)	//ignore unusably-invisible levels
+			level = 30;
+		else if (level > 96)	//ignore nearly-opaque or > 100 levels
+			level = 100;
 
-		level = 100 - level;	//convert from translucence to opacity
+#ifdef USE_GTK2_12
+		gtk_window_set_opacity (GTK_WINDOW (window), (gdouble) level / 100.0);
+#endif
 
+/* FIXME
 		//turn off translucence if it was on, or turn on or adjust translucence
 #ifdef USE_GTK2_12
 		if (gtk_widget_is_composited (window))
@@ -937,10 +1039,29 @@ void e2_window_set_translucency (GtkWidget *window, gint level)
 #endif
 			_e2_window_screen_changed_cb (window, screen, GINT_TO_POINTER (100));
 		}
+*/
 	}
 }
 #endif
 /**
+@brief setup custom window title
+This probably needs BGL to be closed upon arrival
+@param widget dialog or window widget
+@param title custom string to append to window titlebar, or NULL
+@return
+*/
+void e2_window_set_title (GtkWidget *wid, const gchar *title)
+{
+	if (title != NULL)
+	{
+		gchar *tt = e2_utils_strcat (PROGNAME ": ", title);
+		gtk_window_set_title (GTK_WINDOW (wid), tt);	//needs BGL ?
+		g_free (tt);
+	}
+	else
+		gtk_window_set_title (GTK_WINDOW (wid), PROGNAME);
+}
+/**
 @brief set app window cursor to @a type
 Expects BGL on/active
 @param type enumerator of desired cursor type
@@ -954,10 +1075,11 @@ void e2_window_set_cursor (GdkCursorType type)
 	gdk_cursor_unref (cursor);
 	gdk_flush ();
 }
-/**
+/* *
 @brief set output pane size  UNUSED
 
 for internal use, not a toolbar toggle
+Downstream expects BGL to be on/closed
 
 @param rt window rt data structure
 @param arg string indicating the desired state of the pane e.g "*" or "0"
@@ -968,7 +1090,7 @@ for internal use, not a toolbar toggle
 {
 	_e2_window_adjust_panes_ratio (GTK_PANED (rt->output_paned),
 		&rt->output_paned_pos, &rt->output_paned_ratio_last, arg);
-}  */
+}*/
 
 guint last_selected_rows = -1;	//-1 ensures it always reports at session start
 static guint last_total_rows;
@@ -987,7 +1109,19 @@ Expects BGL off/open
 */
 gboolean e2_window_update_status_bar (gpointer userdata)
 {
-// 	gchar status_text[128];
+	//CHECKME make this happen after specific events (refresh, refilter, cd, swap panes)
+	//not on a timer
+#ifdef E2_STATUS_BLOCK
+	if (app.status_working)
+	{
+		printd (DEBUG, "update status bar BLOCKED");
+		return TRUE;
+	}
+#endif
+//	printd (DEBUG, "e2_window_update_status_bar");
+#ifdef E2_STATUS_BLOCK
+	app.status_working = TRUE;
+#endif
 	gint selected_rows, total_rows;
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (curr_view->treeview));
 	GtkTreeSelection *sel = curr_view->selection;
@@ -1038,8 +1172,17 @@ gboolean e2_window_update_status_bar (gpointer userdata)
 			g_free (status_text1);
 		g_free (status_text2);
 	}
+#ifdef E2_STATUS_BLOCK
+	app.status_working = FALSE;
+#endif
+//	printd (DEBUG, "e2_window_update_status_bar ends");
 	return TRUE;  //never turn this off
 }
+#ifdef E2_STATUS_REF
+gint statref_count = 0;
+#endif
+gint lastinterval = E2_STATUSREPORT_INTERVAL;
+#ifndef E2_STATUS_DEMAND
 /**
 @brief turn on status bar 'selected files' message refreshing
 
@@ -1050,8 +1193,23 @@ gboolean e2_window_update_status_bar (gpointer userdata)
 void e2_window_enable_status_update (gint interval)
 {
 	if (interval < 0)
-		interval = E2_STATUSREPORT_INTERVAL;
-	app.timers[STATUS_T] = g_timeout_add (interval, e2_window_update_status_bar, NULL);
+		interval = lastinterval;
+	else
+		lastinterval = interval;
+#ifdef E2_STATUS_REF
+	statref_count--;
+	if (statref_count == 0)
+	{
+#endif
+#ifdef E2_STATUS_BLOCK
+		app.status_working = FALSE;
+#endif
+		app.timers[STATUS_T] = g_timeout_add (interval, e2_window_update_status_bar, NULL);
+#ifdef E2_STATUS_REF
+	}
+	else if (statref_count < 0)
+		statref_count = 0;
+#endif
 }
 /**
 @brief turn off status bar 'selected files' message refreshing
@@ -1060,12 +1218,22 @@ void e2_window_enable_status_update (gint interval)
 */
 void e2_window_disable_status_update (void)
 {
-	if (app.timers[STATUS_T] > 0)
+#ifdef E2_STATUS_REF
+	statref_count++;
+	if (statref_count == 1)
 	{
-		g_source_remove (app.timers[STATUS_T]);
-		app.timers[STATUS_T] = 0;
+//		printd (BEBUG, "block update status bar cb");
+#endif
+		if (app.timers[STATUS_T] > 0)
+		{
+			g_source_remove (app.timers[STATUS_T]);
+			app.timers[STATUS_T] = 0;
+#ifdef E2_STATUS_REF
+		}
+#endif
 	}
 }
+#endif	//ndef E2_STATUS_DEMAND
 /* *
 @brief display status bar custom message
 
@@ -1125,7 +1293,9 @@ void e2_window_show_status_message (gchar *message)
 	GList *children = gtk_container_get_children (GTK_CONTAINER (app.status_bar_box3));
 	if (children != NULL)
 	{
+		gdk_threads_enter ();	//maybe the message is already displayed
 		gtk_widget_destroy (children->data);	//FIXME make this a friendly Q
+		gdk_threads_leave ();
 		g_list_free (children);
 	}
 //	GtkWidget *label = gtk_label_new (message);
@@ -1133,10 +1303,10 @@ void e2_window_show_status_message (gchar *message)
 	gchar *public = g_markup_escape_text (message, -1);
 	gtk_label_set_markup (GTK_LABEL (label), public);
 	g_free (public);
+	gdk_threads_enter ();
 //	gtk_box_pack_start (GTK_BOX (app.status_bar_box3), label, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (app.status_bar_box3), label);
 
-	gdk_threads_enter ();
 	gtk_widget_show_all (app.status_bar_box3);
 	gdk_threads_leave ();
 }
@@ -1189,29 +1359,30 @@ void e2_window_create (E2_WindowRuntime *rt)
 	gtk_window_set_wmclass (GTK_WINDOW (app.main_window), "main", BINNAME);
 	gtk_widget_set_size_request (app.main_window, 0, 0);	//window as small as possible
 	//default size 640 x 480
+	//FIXME these are wrong when window is maximised
 	e2_cache_int_register ("window-width", &app.main_window->allocation.width, 640);
 	e2_cache_int_register ("window-height", &app.main_window->allocation.height, 480);
 	gtk_window_set_default_size (GTK_WINDOW (app.main_window),
 		app.main_window->allocation.width, app.main_window->allocation.height);
 	gtk_window_set_resizable (GTK_WINDOW (app.main_window), TRUE);
 #ifdef E2_COMPOSIT
-	e2_window_set_translucency (app.main_window, -1);
+	e2_window_set_opacity (app.main_window, -1);
 #endif
 	g_signal_connect (G_OBJECT (app.main_window), "show",
 		G_CALLBACK (_e2_window_show_cb), rt);
 	//arrange to clean up gracefully
-	g_signal_connect (G_OBJECT (app.main_window), "destroy",
-		G_CALLBACK (e2_main_shutdown), NULL);	//NULL data = optional cancellation of window-close
+	g_signal_connect (G_OBJECT (app.main_window), "delete-event",
+		G_CALLBACK (_e2_window_gone_cb), NULL); //NULL data = optional cancellation of window-close
 /*FIXME if the system needs shutdown-feedback. support
 	different processes for user- and system-initiated shutdowns?
 	g_signal_connect (G_OBJECT (app.main_window),  ??,
 		G_CALLBACK (system_shutdown), NULL); */
 	//trap signals that might (but probably don't) get issued when a
 	//session-manager initiates a shutdown
-	g_signal_connect (G_OBJECT (app.main_window), "delete-event",
-		G_CALLBACK (e2_main_shutdown), (gpointer)TRUE);
+//	g_signal_connect (GTK_OBJECT (app.main_window), "destroy",
+//		G_CALLBACK (_e2_window_destroy_cb), NULL);
 	g_signal_connect (G_OBJECT (app.main_window), "destroy-event",
-		G_CALLBACK (e2_main_shutdown), (gpointer)TRUE);
+		G_CALLBACK (_e2_window_gone_cb), (gpointer)TRUE);
 
 	app.hbox_main = gtk_hbox_new (FALSE, 0);
 	app.vbox_main = gtk_vbox_new (FALSE, 0);
@@ -1484,7 +1655,11 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 	other_thumbx = gtk_adjustment_get_value (adj);
 	other_upperx = adj->upper;  //no fn for this
 
-	if (curr_view->completed)	//no racing with in-progress refresh
+	LISTS_LOCK
+	gboolean busy = curr_view->listcontrols.cd_working
+				 || curr_view->listcontrols.refresh_working;
+	LISTS_UNLOCK
+	if (!busy) //no racing with in-progress re-list
 	{
 		//save current selections
 		if (curr_view->selected_names != NULL)
@@ -1493,7 +1668,11 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 				(GSourceFunc) e2_fileview_treehash_free, curr_view->selected_names, NULL);
 		curr_view->selected_names = e2_fileview_log_selected_names (curr_view);
 	}
-	if (other_view->completed)
+	LISTS_LOCK
+	busy = other_view->listcontrols.cd_working
+		|| other_view->listcontrols.refresh_working;
+	LISTS_UNLOCK
+	if (!busy)
 	{
 		if (other_view->selected_names != NULL)
 			g_idle_add_full (G_PRIORITY_LOW,
@@ -1519,8 +1698,18 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 	e2_filelist_disable_refresh ();
 	//wait until any in-progress refresh is completed
 	//CHECKME do this at start of this func ?
-	while (!curr_view->completed || !other_view->completed)
+	while (TRUE)
+	{
+		LISTS_LOCK
+		busy = curr_view->listcontrols.cd_working
+			|| curr_view->listcontrols.refresh_working
+			|| other_view->listcontrols.cd_working
+			|| other_view->listcontrols.refresh_working;
+		LISTS_UNLOCK
+		if (!busy)
+			break;
 		usleep (50000);
+	}
 
 	//destroy the widgets
 	//toolbar(s) may be outside rt->panes_outer_box, so destroy them independently
@@ -1530,6 +1719,10 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 	gtk_widget_destroy (app.commandbar.toolbar_container);
 	gtk_widget_destroy (rt->panes_outer_box);
 
+#ifdef E2_COMPOSIT
+	e2_window_set_opacity (app.main_window, -1);
+#endif
+
 	e2_keybinding_register (_C(17), app.main_window);  //_("general"
 
 	_e2_window_create_pane_boxes (rt);
@@ -1555,7 +1748,10 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 	//reinstate former liststore, so its data can be recovered
 	gtk_tree_view_set_model (GTK_TREE_VIEW (curr_view->treeview), NULL);
 	curr_view->store = curr_store;
-	curr_view->refreshtype = E2_RECREATE;
+	//FIXME block new filelist if currently busy
+//	LISTS_LOCK
+	curr_view->listcontrols.refreshtype = E2_RECREATE;
+//	LISTS_UNLOCK
 	e2_fileview_prepare_list (curr_view);	//apply a rebuilt liststore
 
 	//revert the vert scroll
@@ -1578,7 +1774,10 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 #endif
 	gtk_tree_view_set_model (GTK_TREE_VIEW (other_view->treeview), NULL);
 	other_view->store = other_store;
-	other_view->refreshtype = E2_RECREATE;
+	//FIXME block new filelist if currently busy
+//	LISTS_LOCK
+	other_view->listcontrols.refreshtype = E2_RECREATE;
+//	LISTS_UNLOCK
 	e2_fileview_prepare_list (other_view);	//apply a rebuilt liststore
 
 	e2_fileview_scroll_to_position (other_view, 0, other_yscroll);
@@ -1614,22 +1813,25 @@ void e2_window_actions_register (void)
 	action_name = g_strconcat(_A(13),".",_A(27),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_window_adjust_pane_ratio_action, &app.window, TRUE);
-	action_name = g_strconcat(_A(13),".",_A(69),NULL);
+	action_name = g_strconcat(_A(15),".",_A(44),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		e2_task_refresh, NULL, FALSE);
-	action_name = g_strconcat(_A(13),".",_A(71),NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		e2_filelist_disable_refresh_action, NULL, FALSE);
+		_e2_window_toggle_full_screen, NULL, FALSE);
 	action_name = g_strconcat(_A(13),".",_A(70),NULL);
+	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		e2_task_refresh, NULL, FALSE);
+	action_name = g_strconcat(_A(13),".",_A(72),NULL);
+	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		e2_filelist_disable_refresh_action, NULL, FALSE);
+	action_name = g_strconcat(_A(13),".",_A(71),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		e2_filelist_enable_refresh_action, NULL, FALSE);
-	action_name = g_strconcat(_A(13),".",_A(84),NULL);
+	action_name = g_strconcat(_A(13),".",_A(93),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		e2_task_sync_dirs, NULL, FALSE);
-	action_name = g_strconcat(_A(13),".",_A(85),NULL);
+	action_name = g_strconcat(_A(13),".",_A(94),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_window_toggle_panes_direction, NULL, FALSE);
-	action_name = g_strconcat(_A(10),".",_A(81),NULL);
+	action_name = g_strconcat(_A(10),".",_A(90),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		e2_pane_activate_other_action, NULL, FALSE);
 	//these are "self-managed" toggle actions, with names already logged
