@@ -1,6 +1,6 @@
-/* $Id: emelfm2.h 565 2007-07-25 05:33:39Z tpgww $
+/* $Id: emelfm2.h 893 2008-05-20 09:42:51Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -20,11 +20,12 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 #ifndef __EMELFM2_H__
 #define __EMELFM2_H__
-//see also: Makefile has build-time conditional #defines
+//see also: Makefile has build-time conditional #defines and compiler parameters
 #define _GNU_SOURCE
-//default 64-bit functions (handle files > 2GB)
-// also in makefile
+//default 64-bit functions (handle files > 2GB) (also in Makefile)
 #define _FILE_OFFSET_BITS 64
+//some of the Makefile define's are transferred in this header
+#include "build.h"
 
 #include <sys/types.h>
 //needed for various statbuf declarations
@@ -39,10 +40,24 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <gdk/gdkkeysyms.h>
 #include <gmodule.h>
 
-#include "build.h"
 #define GETTEXT_PACKAGE BINNAME
 #include <glib/gi18n-lib.h>
 
+//#define E2_ASSISTED see build.h
+#ifdef E2_ASSISTED
+# include <atk/atk.h>
+#endif
+
+#ifndef PRIu64
+//for systems that don't define PRI??? macros
+# if defined(__WORDSIZE) && __WORDSIZE == 64
+#  define PRIu64 "lu"
+# else
+//guess
+#  define PRIu64 "llu"
+# endif
+#endif
+
 //include code valid only for gtk versions > 2.6, according to the
 //version in use when compiling
 //#define E2_CURRENTGTK_VERSION see makefile and build.h
@@ -74,6 +89,23 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #endif
 #endif
 
+#ifdef USE_GTK2_12
+//this token governs use of gtk 2.12 tooltips
+#define USE_GTK2_12TIPS
+//this token governs use of gtk 2.12 notebook-tab DnD
+#define USE_GTK2_12DND
+#endif
+
+//tag for development of on-demand status-bar updates
+//NOTE need to check all BGL effects
+//#define E2_STATUS_DEMAND
+#ifndef E2_STATUS_DEMAND
+//tag for refcounting enable/disable status-bar updates
+//#define E2_STATUS_REF
+//allow checking for status-in progess when updating filelists
+//#define E2_STATUS_BLOCK
+#endif
+
 //include error-message code regardless of debug state
 //#define DEBUG_MESSAGES_ALWAYS
 
@@ -121,13 +153,15 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //#define NAME_MAX 255
 //length of various buffers
 #define E2_MAX_LEN 1024
-//msec interval between open-directory polls
-//(long enough to avoid too many refreshes during
-//e.g. archive creation)
-#define E2_FILESCHECK_INTERVAL 3000
+//msec interval between pane-directory polls
+//NOTE that the actual refresh interval may be a multiple (up to 3) times this
+//to avoid too many refreshes during e.g. archive creation
+#define E2_FILESCHECK_INTERVAL 1000
+#define E2_FILESCHECK_INTERVAL_S 1
 //msec interval between config-directory polls
 //this is only used if E2_FAM not defined
 #define E2_CONFIGCHECK_INTERVAL 5000
+#define E2_CONFIGCHECK_INTERVAL_S 5
 //msec interval between status line updates
 #define E2_STATUSREPORT_INTERVAL 1500
 
@@ -170,14 +204,18 @@ typedef struct _E2_ColorData
 } E2_ColorData;
 #endif
 
+//maybe useful for multi-processor optimisation ?
+//#define VOLATILE volatile
+#define VOLATILE
+
 //support key.alias action
 //#define E2_KEYALIAS
 //support addition of items to tree options (eg keybindings)
 //#define E2_TRANSIENTKEYS see Makefile for this
 #ifdef E2_TRANSIENTKEYS
-#ifndef E2_KEYALIAS
-#define E2_KEYALIAS
-#endif
+# ifndef E2_KEYALIAS
+#  define E2_KEYALIAS
+# endif
 #endif
 //support addition of treestore iters from strings like 1.2.3
 //#define E2_TREEINCREMENT
@@ -185,59 +223,7 @@ typedef struct _E2_ColorData
 //support mountpoint-related capabilities
 #define E2_FS_MOUNTABLE
 
-//support vfs for file/dir processing
-//NOT FINISHED
-//needs cacheing of view data, backend data structs, history, marks
-//lib interrogation, task backends etc etc
-//#define E2_VFS
-//transition management
-#ifdef E2_VFS
-//#define E2_VFSTMP
-#ifdef E2_VFSTMP
-//#define E2_VFSTMPOK
-#endif
-#endif
-
-//support for custom error handling with vfs
-#ifdef E2_VFS
-# define E2_ERR_NAME __sys_err
-# define E2_ERR_INIT __sys_err = NULL;
-# define E2_ERR_DECLARE GError *__sys_err = NULL;
-//in these, there should never br any "a", we just need to get the leading comma
-# define E2_ERR_NONE(a) a, NULL
-# define E2_ERR_PTR(a) a, &__sys_err
-# define E2_ERR_ARG(a) a, GError **__sys_err
-# define E2_ERR_SAMEARG(a) a, __sys_err
-# define E2_ERR_MSGC(a) a, (*__sys_err)->message
-//these are for GError *, not for GError **
-# define E2_ERR_MSGL(a) a, __sys_err->message
-# define E2_ERR_IS(num) (__sys_err != NULL && __sys_err->code == num)
-# define E2_ERR_ISNOT(num) (__sys_err == NULL || __sys_err->code != num)
-//these are for use in a func with GError ** argument
-# define E2_ERR_PIS(num) (__sys_err != NULL && (*__sys_err)->code == num)
-# define E2_ERR_PISNOT(num) (__sys_err == NULL || (*__sys_err)->code != num)
-# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
-#else
-# define E2_ERR_NAME
-# define E2_ERR_INIT
-# define E2_ERR_DECLARE
-# define E2_ERR_NONE(a)
-# define E2_ERR_PTR(a)
-# define E2_ERR_ARG(a)
-# define E2_ERR_SAMEARG(a)
-# define E2_ERR_MSGC(a)
-# define E2_ERR_MSGL(a)
-# define E2_ERR_IS(num) (errno==num)
-# define E2_ERR_ISNOT(num) (errno!=num)
-# define E2_ERR_PIS(num) (errno==num)
-# define E2_ERR_PISNOT(num) (errno!=num)
-# define E2_ERR_CLEAR
-#endif
-
 //support detachable tabs for gtk >= 2.10
-/*DOES NOT WORK ENTIRELY WELL, needs
-tab dragging to already-open windows, including back to output pane
-*/
 #ifdef USE_GTK2_10
 #define E2_TABS_DETACH
 #endif
@@ -251,16 +237,148 @@ tab dragging to already-open windows, including back to output pane
 //enable filesystem-tree-navigation dialogs
 #define E2_TREEDIALOG
 
-//support app-specific window translucence NOT WORKING
+//support app-specific window translucence NOT WORKING FOR gtk < 2.12
 //anyhow, good enough for the window-manager to do this equally for all apps
-#ifdef USE_GTK2_8
-//# define E2_COMPOSIT
-# ifdef E2_COMPOSIT
-//default translucence % for dialogs
-#  define DIALOG_TRANS_LEVEL 10
+//#define E2_COMPOSIT see Makefile
+#ifdef E2_COMPOSIT
+# ifdef USE_GTK2_12
+//minimum opacity % for dialogs
+#  define DIALOG_OPACITY_LEVEL 95
+# else
+#  undef E2_COMPOSIT
 # endif
 #endif
 
+//support vfs for file/dir processing
+//NOT FINISHED
+//needs cacheing of view data, backend data structs, history, marks
+//lib interrogation, task backends etc etc
+//#define E2_VFS see Makefile
+//transition management
+#ifdef E2_VFS
+//# define E2_VFSTMP
+# ifdef E2_VFSTMP
+#  define E2_VFSTMPOK
+# endif
+#endif
+
+//enable filelist liststore copying instead of getting fresh data from source
+#ifdef E2_VFS
+#define STORECOPY
+#endif
+
+//support for custom error handling with vfs
+#ifdef E2_VFS
+# define E2_ERR_NAME __sys_err
+# define E2_ERR_INIT __sys_err = NULL;
+# define E2_ERR_DECLARE GError *__sys_err = NULL;
+# define E2_ERR_CLEARBACKUP(a) if (a != NULL) g_error_free (a);
+//in these, there should never be any "a", we just need to get the leading comma
+# define E2_ERR_NONE(a) a, NULL
+# define E2_ERR_ARG(a) a, GError **__sys_err
+# define E2_ERR_SAMEARG(a) a, __sys_err
+# define E2_ERR_PTR(a) a, &__sys_err
+# define E2_ERR_MSGC(a) a, (*__sys_err)->message
+//these are for GError *, not for GError **
+# define E2_ERR_MSGL(a) a, __sys_err->message
+# define E2_ERR_IS(num) (__sys_err != NULL && __sys_err->code == num)
+# define E2_ERR_ISNOT(num) (__sys_err == NULL || __sys_err->code != num)
+//these are for use in a func with GError ** argument
+# define E2_ERR_PIS(num) (__sys_err != NULL && (*__sys_err)->code == num)
+# define E2_ERR_PISNOT(num) (__sys_err == NULL || (*__sys_err)->code != num)
+# define E2_ERR_BACKUP(a) GError *a = NULL;if (__sys_err == NULL) __sys_err = &a;
+# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
+#else
+# define E2_ERR_NAME
+# define E2_ERR_INIT
+# define E2_ERR_DECLARE
+# define E2_ERR_CLEARBACKUP(a)
+# define E2_ERR_NONE(a)
+# define E2_ERR_PTR(a)
+# define E2_ERR_ARG(a)
+# define E2_ERR_SAMEARG(a)
+# define E2_ERR_MSGC(a)
+# define E2_ERR_MSGL(a)
+# define E2_ERR_IS(num) (errno==num)
+# define E2_ERR_ISNOT(num) (errno!=num)
+# define E2_ERR_PIS(num) (errno==num)
+# define E2_ERR_PISNOT(num) (errno!=num)
+# define E2_ERR_CLEAR
+# define E2_ERR_BACKUP(a)
+#endif
+
+//support for tailored argument(s) for fs functions
+#ifdef E2_VFS
+# define VPATH vpath
+# define VPCSTR(p) *((const gchar**)p)
+# define VPSTR(p) *((gchar**)p)
+#else
+# define VPATH const gchar
+# define VPCSTR(p) p
+# define VPSTR(p) (gchar*)p
+#endif
+
+//type of "dir" in a pane
+typedef enum
+{
+	FS_LOCAL   = 0,		//default, mounted-local dir
+	FS_FUSE    = 1,		//virtual local dir that needs special handling
+	FS_REMOTE  = 1 << 1,//virtual non-mounted remote dir
+	FS_ARCHIVE = 1 << 2,//archive (can be local or remote)
+	FS_KERNEL  = 1 << 3,//a namespace in a multi-space local filesystem
+	FS_SYNTH   = 1 << 4,//a synthesised virtual dir e.g. from a search process or metadata derived
+} E2_FSType;
+//types of virtual dir that are non-native, and warrant timeout if misbehaved
+#define FS_SLOW (FS_FUSE | FS_REMOTE | FS_ARCHIVE)
+
+#ifdef E2_VFS
+
+typedef enum { E2PLACE_UNMOUNTED, E2PLACE_MOUNTING, E2PLACE_MOUNTED } PlaceReady;
+typedef struct _E2_PlaceInfo
+{
+	gchar *priv_name; /* for mounted dir NULL CHECKME synth dir
+	   for remote site escaped utf8, no trailer, login-ready URI before the filepath proper with p/w etc
+	   for archive localised absolute path, with trailer, of temp dir used for unpacking whole or part
+	   constructed from vtab data (not stored there, it may have P/W text in URI)
+	*/
+	gchar *tip; /* for mounted dir: NULL CHECKME synth dir
+	   for remote site: utf8, no trailer, public components of 1st part of URI, before the filepath proper
+	   for archive: utf8, no trailer, pseudo-path of archive, with all ancestor(s) including remote URI and other archive
+	*/
+	gchar *plain_pw;	//edit-ready password, utf8, or NULL if no P/W
+	gchar *alias;	//user-friendly short-form utf8 for menu label etc, NULL or "" if not in use
+	gchar *workplace;	//path of local fs dir, absolute localised no trailer
+	E2_FSType dirtype;	//flags for type of namespace this is
+//UNUSED	E2_FSHandler handler;	//enumerator of lib/plugin which handles fs-operations for this space
+									//CHECK could be op-specific
+//UNUSED	E2_VFSOpFlags cando;	//flags for the sorts of operations that can be performed
+	PlaceReady mountstate;	//this place has been initialised by its backend handler
+						//(maybe redundant if only mounted places can have a PlaceInfo)
+	gboolean monitored;	//alteration monitoring in effect for this place (even if just polling)
+	//directory histories for this namespace
+	GList *pane1_entered; //pane 1 dirline history, specific to current namespace
+	GList *pane2_entered; //pane 2 dirline history, specific to current namespace
+	GList *pane1_history; //pane1 dir_history, but specific to this namespace
+	GList *pane2_history; //pane2 dir_history, but specific to this namespace
+	GList *pane1_visited; //pane1 visited, opendirs, but specific to this namespace
+	GList *pane2_visited; //pane2 visited, opendirs, but specific to this namespace
+
+	struct _E2_PlaceInfo* parentplace;	//for cd .. past root
+	//etc
+} PlaceInfo;
+
+//data for vfs operations uning path-argument(s)
+typedef struct _E2_Vpath
+{
+	const gchar *localpath; /*first, so that vpath* == gchar **
+						typically an absolute path string
+						encoding may actually be utf-8 if a handler lib
+						requires that */
+	PlaceInfo *spacedata;
+//	GError *err; //replacement for error args in fs functions
+} vpath;
+#endif
+
 #include "debug.h"
 #include "e2_pane.h"
 #include "e2_fileview.h"
@@ -286,11 +404,23 @@ tab dragging to already-open windows, including back to output pane
 #include "e2_window.h"
 
 // some nice helper/utility macros
-#define WAIT_FOR_EVENTS while (gtk_events_pending ()) { gtk_main_iteration (); }
+//these 2 must both be called with BGL closed
+//the gtk-versions block until at least one event is processed
+//#define WAIT_FOR_EVENTS while (gtk_events_pending ()) { gtk_main_iteration (); }
+//#define WAIT_FOR_EVENTS_SLOWLY while (gtk_events_pending ()) { gtk_main_iteration ();usleep(5000); }
+#define WAIT_FOR_EVENTS gdk_threads_leave (); \
+	while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); } \
+	gdk_threads_enter ();
+#define WAIT_FOR_EVENTS_SLOWLY gdk_threads_leave (); \
+	while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); usleep(5000); } \
+	gdk_threads_enter ();
+//and these 2 are the corresponding ones with BGL open
+#define WAIT_FOR_EVENTS_UNLOCKED while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); }
+#define WAIT_FOR_EVENTS_UNLOCKED_SLOWLY while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); usleep(5000); }
+
 #define ITEM_ISHIDDEN(expr) (expr[0]=='.')
 //#define ITEM_ISHIDDEN(expr) (*expr=='.')
 
-
 // development-phase string tagger
 // search for _I( for general strings that need attention
 #define _I(d) d
@@ -414,7 +544,7 @@ typedef struct _E2_MainData
 #endif
 	guint timers[MAX_TIMERS];	//glib timer id's
 	time_t config_mtime;	//last-logged (seconds) timestamp of config file
-#ifndef USE_GTK2_12
+#ifndef USE_GTK2_12TIPS
 	GtkTooltips *tooltips;
 #endif
 	GList *plugins;	//list of data structs for all loaded plugins, "non-children"
@@ -431,18 +561,18 @@ typedef struct _E2_MainData
 #endif
 	GSList *used_stores;	//for deferred liststore clearing
 	GList *taskhistory;
-#ifdef E2_VFS
-	GtkListStore *vfs_data;	//store for history/bookmarks runtime data
-	gboolean vfs_available;	//TRUE when at least one vfs plugin is available
-#endif
 #ifndef E2_FILES_UTF8ONLY
 	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
 #endif
+	GdkWindowState mainwindow_state;	//flags indicating changes to main_window
 	GtkWidget *context_menu;	//main context-menu when it is popped up
 	gint cfgdlg_width;  //config dialog width (cached)
 	gint cfgdlg_height; //config dialog height (cached)
 	gchar cfgfile_version[20];  //store for config version string eg "0.1.9", used only for upgrades
 	gboolean rebuild_enabled;	//set FALSE to block window recreation
+#ifdef E2_STATUS_BLOCK
+	gboolean status_working;	//set TRUE to block reentrant status-checking etc
+#endif
 	GHookList hook_pane_focus_changed;
 } E2_MainData;
 
@@ -453,6 +583,17 @@ ViewInfo *curr_view;
 ViewInfo *other_view;
 E2_OutputTabRuntime *curr_tab;	//currently-focused member of tabslist
 
+pthread_mutex_t list_mutex;
+#define LISTS_LOCK pthread_mutex_lock (&list_mutex);
+#define LISTS_UNLOCK pthread_mutex_unlock (&list_mutex);
+//CHECKME worth having a separate mutex for this ?
+pthread_mutex_t history_mutex;
+#define HISTORY_LOCK pthread_mutex_lock (&history_mutex);
+#define HISTORY_UNLOCK pthread_mutex_unlock (&history_mutex);
+
+//for testing - define to use gtk's own mutex for BGL
+//#define NATIVE_BGL
+#ifndef NATIVE_BGL
 //local management of gdk mutex, to enable tolerant re-locking by the same thread
 #include <pthread.h>
 //direct mutex-manipulation
@@ -462,8 +603,32 @@ pthread_mutex_t gdklock;	//BGL replacement
 void e2_main_close_gdklock (void);
 void e2_main_open_gdklock (void);
 
+	//not locked now and not already locked - this should never happen
+	//prevent unlock
+#define CLOSEBGL_IF_OPEN \
+	gint _lockres; \
+	_lockres = pthread_mutex_trylock (&gdklock); \
+	printd (DEBUG, "dialog title trylock result %d", _lockres); \
+	if (!(_lockres == 0 || _lockres == EBUSY)) \
+	{ \
+		printd (WARN, "dialog title close BGL failed"); \
+		WAIT_FOR_EVENTS_UNLOCKED \
+		_lockres = 1; \
+	}
+
+#define	OPENBGL_IF_CLOSED \
+	if (_lockres == 0) \
+		pthread_mutex_unlock (&gdklock);
+
+#else //def NATIVE_BGL
+	//WAIT_FOR_EVENTS expects BGL closed
+#define CLOSEBGL_IF_OPEN \
+	WAIT_FOR_EVENTS_UNLOCKED
+#define	OPENBGL_IF_CLOSED
+#endif
+
+gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit);
 gboolean e2_main_user_shutdown (gpointer from, E2_ActionRuntime *art);
-gboolean e2_main_shutdown (GtkWidget *widget, GdkEvent *event, gpointer forced);
 //	__attribute__ ((noreturn));
 
 #endif //ndef __EMELFM2_H__
