@@ -1,4 +1,4 @@
-/* $Id: e2_fs_FAM_dnotify.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_fs_FAM_dnotify.c 574 2007-08-06 04:41:08Z tpgww $
 
 Copyright (C) 2005-2007 tooar <tooar@gmx.net>
 
@@ -214,7 +214,7 @@ static gint _e2_fs_FAM_dnotify_monitor_dir (gchar *path)
 
 	//otherwise register it with dnotify
 	gchar *local = F_FILENAME_TO_LOCALE (path);
-	gint fd = open (local, O_RDONLY | O_NONBLOCK);
+	gint fd = e2_fs_safeopen (local, O_RDONLY | O_NONBLOCK, 0);
 	F_FREE (local);
 
 	if (fd >= 0)
@@ -244,7 +244,7 @@ static gint _e2_fs_FAM_dnotify_monitor_dir (gchar *path)
 				return retval;
 		}
 		else	//connection failed for some reason
-			close (fd);
+			e2_fs_safeclose (fd);
 	}
 	printd (WARN, "failed to add dnotify monitoring of %s\n%s", path, strerror (errno));
 	return -1;
