@@ -1,4 +1,4 @@
-/* $Id: e2_fs_walk.c 540 2007-07-18 15:00:05Z tpgww $
+/* $Id: e2_fs_walk.c 744 2007-11-22 21:36:34Z tpgww $
 
 Copyright (C) 2005-2007 tooar <tooar@gmx.net>
 
@@ -111,14 +111,15 @@ original_path newly-allocated localised path string, = CWD when the walk is star
 */
 typedef struct _E2_TwData
 {
-	gint (*twstat) (const gchar *, struct stat *
 #ifdef E2_VFS
-		, GError **
+	gint (*twstat) (VPATH *, struct stat *, GError **);
+#else
+	gint (*twstat) (const gchar *, struct stat *);
 #endif
-	);
-	E2_TwResult (*cbfunc) (const gchar *, const struct stat *, E2_TwStatus, gpointer);
+	E2_TwResult (*cbfunc) (VPATH *, const struct stat *, E2_TwStatus, gpointer);
 	gpointer user_data;
 	gint max_depth;	//-1 for unlimited
+	E2_FsReadWatch watchdir;
 	E2_TwFlags exec_flags;
 	dev_t device;
 } E2_TwData;
@@ -135,11 +136,11 @@ NOTE for GRP and OTH this does not work, for dirs at least
 
 @return mode of @a localpath, not masked with ALLPERMS, maybe same as current mode, 0 after error
 */
-mode_t e2_fs_tw_adjust_dirmode (const gchar *localpath, const struct stat *statptr,
+mode_t e2_fs_tw_adjust_dirmode (VPATH *localpath, const struct stat *statptr,
 	gint howflags)
 {
 	E2_ERR_DECLARE
-	if (!e2_fs_access3 ((gchar *)localpath, howflags E2_ERR_PTR()))
+	if (!e2_fs_access3 (localpath, howflags E2_ERR_PTR()))
 		return statptr->st_mode;
 	E2_ERR_CLEAR
 
@@ -173,7 +174,7 @@ mode_t e2_fs_tw_adjust_dirmode (const gchar *localpath, const struct stat *statp
 		//X permission on dir when trying to process its contents
 		gchar *fmt = ((howflags & X_OK) && S_ISDIR (statptr->st_mode)) ?
 			_("Cannot change anything in %s") : _("Cannot change permissions of %s");
-		e2_fs_error_local (fmt, (gchar *)localpath E2_ERR_MSGL());
+		e2_fs_error_local (fmt, localpath E2_ERR_MSGL());
 		E2_ERR_CLEAR
 		newmode = 0;
 	}
@@ -190,7 +191,7 @@ So this uses a kludgy but robust alternative approach
 
 @return callbck status code in accordance with whether the link is valid
 */
-static E2_TwResult _e2_fs_tw_link (const gchar *localpath)
+static E2_TwResult _e2_fs_tw_link (VPATH *localpath)
 {
 	E2_TwStatus code;
 	gchar buf[PATH_MAX];
@@ -202,7 +203,7 @@ static E2_TwResult _e2_fs_tw_link (const gchar *localpath)
 	else
 	{
 		buf[read] = '\0';
-		gchar *converted1 = F_FILENAME_FROM_LOCALE (localpath);
+		gchar *converted1 = F_FILENAME_FROM_LOCALE (VPSTR(localpath));
 		gchar *converted2 = F_FILENAME_FROM_LOCALE (buf);
 		gchar *targetpath = e2_utils_translate_relative_path (converted1, converted2);
 		F_FREE (converted1);
@@ -212,7 +213,12 @@ static E2_TwResult _e2_fs_tw_link (const gchar *localpath)
 
 		converted1 = F_FILENAME_TO_LOCALE (targetpath);
 		//access() looks through links
+#ifdef E2_VFS
+		VPATH data = { converted1, localpath->spacedata };
+		code = (e2_fs_access (&data, F_OK E2_ERR_NONE())) ? E2TW_SLN : E2TW_SL;
+#else
 		code = (e2_fs_access (converted1, F_OK E2_ERR_NONE())) ? E2TW_SLN : E2TW_SL;
+#endif
 		g_free (targetpath);
 		F_FREE (converted1);
 	}
@@ -232,7 +238,7 @@ into @a statptr, before this function is called
 
 @return result code in accordance with callback-function response
 */
-static E2_TwResult _e2_fs_tw_nondir (const gchar *localpath, const struct stat *statptr,
+static E2_TwResult _e2_fs_tw_nondir (VPATH *localpath, const struct stat *statptr,
 	E2_TwData *twdata)
 {
 	E2_TwStatus code;
@@ -261,8 +267,8 @@ static E2_TwResult _e2_fs_tw_nondir (const gchar *localpath, const struct stat *
 @brief process directory @a localpath, as part of a treewalk
 
 if possible, and in accord with the walk-mode determined by flags in @a twdata,
-@a localpath is opened and its contents are each 'reported' to the callback
-function recorded in @a twdata.
+@a localpath is opened and its contents are listed and each listed item is
+'reported' to the callback function recorded in @a twdata.
 Any subdir found among the contents will recurse into this function.
 Non-dirs are handled by another function, before coming here or when found here.
 
@@ -273,7 +279,7 @@ Non-dirs are handled by another function, before coming here or when found here.
 
 @return result code in accordance with callback-function returns, E2TW_STOP or E2TW_CONTINUE if all was well
 */
- static E2_TwResult _e2_fs_tw (const gchar *localpath, gint depth, E2_TwData *twdata)
+static E2_TwResult _e2_fs_tw_dir (VPATH *localpath, gint depth, E2_TwData *twdata)
 {
 	//FIXME support an exit-code for cleanup-this-level then stop
 	E2_TwResult result;
@@ -290,13 +296,9 @@ Non-dirs are handled by another function, before coming here or when found here.
 	gchar *itempath, *itemname;
 	GList *entries, *parked_dirs, *member;
 #ifdef E2_VFS
+	VPATH ddata;
 	GError **callerr;
 	E2_ERR_DECLARE
-#endif
-#ifdef E2_VFSTMP
-	E2_FSType dirtype = ?; //FIXME
-#else
-	E2_FSType dirtype = curr_view->dirtype; //FIXME
 #endif
 	struct stat statbuf;	//assume it's ok to recursively stack this much data
 
@@ -307,7 +309,7 @@ Non-dirs are handled by another function, before coming here or when found here.
 		//convenience we report to the user unless instructed otherwise
 		if (!(twdata->exec_flags & E2TW_XQT))
 			e2_fs_error_local (_("Cannot get information about %s"),
-				(gchar *)localpath E2_ERR_MSGL());
+				localpath E2_ERR_MSGL());
 
 		result = (*twdata->cbfunc) (localpath, &statbuf, E2TW_NS, twdata->user_data);
 
@@ -328,7 +330,7 @@ Non-dirs are handled by another function, before coming here or when found here.
 	if ((twdata->exec_flags & E2TW_MOUNT) && twdata->device != statbuf.st_dev)
 	{
 		if (!(twdata->exec_flags & (E2TW_QT | E2TW_XQT)))
-			e2_fs_error_simple (_("Directory %s not opened"), (gchar *)localpath);
+			e2_fs_error_simple (_("Directory %s not opened"), localpath);
 		result = (*twdata->cbfunc) (localpath, &statbuf, E2TW_DM, twdata->user_data);
 		return ((result & E2TW_STOP) ? E2TW_STOP : E2TW_CONTINUE); //unwanted filesystem, would expect normally to continue
 	}
@@ -336,14 +338,14 @@ Non-dirs are handled by another function, before coming here or when found here.
 	if (twdata->max_depth != -1 && newdepth > twdata->max_depth)
 	{
 		if (!(twdata->exec_flags & (E2TW_QT | E2TW_XQT)))
-			e2_fs_error_simple (_("Directory %s not opened"), (gchar *)localpath);
+			e2_fs_error_simple (_("Directory %s not opened"), localpath);
 		result = (*twdata->cbfunc) (localpath, &statbuf, E2TW_DL, twdata->user_data);
 		return ((result & E2TW_STOP) ? E2TW_STOP : E2TW_CONTINUE); //max depth reached, would expect normally to continue
 	}
 
 	//create a re-usable path-string buffer
 	//since this function may recurse indefinitely, buffer is on heap, not stack
-	localpathlen = strlen (localpath);
+	localpathlen = strlen (VPSTR(localpath));
 	//space to append a FileInfo->filename (NAME_MAX+1), plus a few chars (/ or 0)
 	pathbuflen = (localpathlen + NAME_MAX + 1)/8*8 + 8;
 #ifdef USE_GLIB2_10
@@ -362,10 +364,8 @@ Non-dirs are handled by another function, before coming here or when found here.
 	}
 
 	//get list of item-names in the dir
-	gdk_threads_enter ();
-	entries = (GList *)e2_fs_dir_foreach ((gchar *)localpath, dirtype,
-		 NULL, NULL, NULL E2_ERR_PTR());
-	gdk_threads_leave ();
+	entries = (GList *)e2_fs_dir_foreach (localpath, twdata->watchdir,
+		NULL, NULL, NULL E2_ERR_PTR());
 	//check whether it was possible to open the dir
 	if (E2DREAD_FAILED (entries))
 	{
@@ -375,10 +375,8 @@ Non-dirs are handled by another function, before coming here or when found here.
 			newmode = e2_fs_tw_adjust_dirmode (localpath, &statbuf, (R_OK | X_OK));
 			if (newmode != firstmode && newmode != 0)
 			{	//try again
-				gdk_threads_enter ();
-				entries = (GList *)e2_fs_dir_foreach ((gchar *)localpath, dirtype,
-					 NULL, NULL, NULL E2_ERR_PTR());
-				gdk_threads_leave ();
+				entries = (GList *)e2_fs_dir_foreach (localpath, twdata->watchdir,
+					NULL, NULL, NULL E2_ERR_PTR());
 			}
 		}
 		//was 1st or possible 2nd attempted read successful ?
@@ -386,7 +384,7 @@ Non-dirs are handled by another function, before coming here or when found here.
 		{
 			if (!(twdata->exec_flags & E2TW_XQT))
 				e2_fs_error_local (_("Cannot open directory %s"),
-					(gchar *)localpath E2_ERR_MSGL());
+					localpath E2_ERR_MSGL());
 
 			result = (*twdata->cbfunc) (localpath, &statbuf, E2TW_DNR, twdata->user_data);
 
@@ -435,9 +433,9 @@ Non-dirs are handled by another function, before coming here or when found here.
 //==== DIR HAS BEEN READ & REPORTED AS SUCH
 
 #ifdef __USE_GNU
-	itemname = mempcpy (itempath, localpath, localpathlen);
+	itemname = mempcpy (itempath, VPSTR (localpath), localpathlen);
 #else
-	memcpy (itempath, localpath, localpathlen);
+	memcpy (itempath, VPSTR (localpath), localpathlen);
 	itemname = itempath + localpathlen;
 #endif
 	bufspace = pathbuflen - localpathlen;
@@ -450,7 +448,10 @@ Non-dirs are handled by another function, before coming here or when found here.
 	//for logging dirs in breadth-first mode, to be processed when we're ready
 	//to open them, after all non-dirs at this level have been processed
 	parked_dirs = NULL;
-
+#ifdef E2_VFS
+	ddata.localpath = itempath;
+	ddata.spacedata = localpath->spacedata;
+#endif
 	//walk the list of entries in the dir (tree-depth now = newdepth)
 	for (member = entries; member != NULL ; member = member->next)
 	{
@@ -459,14 +460,22 @@ Non-dirs are handled by another function, before coming here or when found here.
 
 		g_strlcpy (itemname, (gchar *) member->data, bufspace);
 
+#ifdef E2_VFS
+		if ((*twdata->twstat) (&ddata, &statbuf E2_ERR_PTR()))
+#else
 		if ((*twdata->twstat) (itempath, &statbuf E2_ERR_PTR()))
+#endif
 		{	//process stat failure
 			//NOTE statting a hanging link to a dir does NOT fail here, glib BUG ?
 			if (!(twdata->exec_flags & E2TW_XQT))
 				e2_fs_error_local (_("Cannot get information about %s"),
-					itempath E2_ERR_MSGL());
-			result = (*twdata->cbfunc) (itempath, &statbuf, E2TW_NS, twdata->user_data);
 #ifdef E2_VFS
+					&ddata E2_ERR_MSGL());
+#else
+					itempath E2_ERR_MSGL());
+#endif
+#ifdef E2_VFS
+			result = (*twdata->cbfunc) (&ddata, &statbuf, E2TW_NS, twdata->user_data);
 			if (twdata->exec_flags & E2TW_XERR)
 			{
 				callerr = (GError **)twdata->user_data;
@@ -475,6 +484,8 @@ Non-dirs are handled by another function, before coming here or when found here.
 			}
 			else
 				E2_ERR_CLEAR
+#else
+			result = (*twdata->cbfunc) (itempath, &statbuf, E2TW_NS, twdata->user_data);
 #endif
 			if ((result & E2TW_CLEAN) && reportdirs)
 			{
@@ -525,15 +536,18 @@ Non-dirs are handled by another function, before coming here or when found here.
 */
 				//breadth-first walk
 				//log the subdir for processing after all non-subdirs
-				parked_dirs = g_list_append (parked_dirs,
+				parked_dirs = g_list_prepend (parked_dirs,
 					GINT_TO_POINTER (g_list_position (entries, member)));
 //			}
 		}
 		else	//not a dir (it could be a hanging link to a dir)
 			if (reportnondirs)
 		{
+#ifdef E2_VFS
+			result = _e2_fs_tw_nondir (&ddata, &statbuf, twdata);
+#else
 			result = _e2_fs_tw_nondir (itempath, &statbuf, twdata);
-
+#endif
 			if ((result & E2TW_CLEAN) && reportdirs)
 			{
 				result |=
@@ -566,7 +580,11 @@ Non-dirs are handled by another function, before coming here or when found here.
 		gint indx = GPOINTER_TO_INT (member->data);
 		gchar *parkname = g_list_nth_data (entries, indx);
 		g_strlcpy (itemname, parkname, bufspace);
-		result |= (_e2_fs_tw (itempath, newdepth, twdata));
+#ifdef E2_VFS
+		result |= (_e2_fs_tw_dir (&ddata, newdepth, twdata));
+#else
+		result |= (_e2_fs_tw_dir (itempath, newdepth, twdata));
+#endif
 	}
 	if (parked_dirs != NULL)
 		g_list_free (parked_dirs);
@@ -632,7 +650,7 @@ Non-dirs are handled by another function, before coming here or when found here.
 			{
 				if (!(twdata->exec_flags & E2TW_XQT))
 					e2_fs_error_local (_("Cannot change permissions of %s"),
-						(gchar *)localpath E2_ERR_MSGL());
+						localpath E2_ERR_MSGL());
 #ifdef E2_VFS
 				if (twdata->exec_flags & E2TW_XERR)
 				{
@@ -663,16 +681,17 @@ must assume that too.
 
 @return TRUE if @a start_item (and any descendants) was processed in way(s) that @a callback considered ok
 */
-gboolean e2_fs_tw (gchar *start_item, E2_TwResult (*callback) (),
+gboolean e2_fs_tw (VPATH *start_item, E2_TwResult (*callback) (),
 	gpointer user_data, gint max_depth, E2_TwFlags exec_flags E2_ERR_ARG())
 {
 	E2_TwResult result;
 	struct stat statbuf;
-	gint (*twstat) (const gchar *, struct stat *
+	gint (*twstat) (VPATH *, struct stat *
 #ifdef E2_VFS
 		, GError **
 #endif
 	);
+	E2_ERR_BACKUP (localerr);
 
 	twstat = (exec_flags & E2TW_PHYS) ? e2_fs_lstat : e2_fs_stat;
 	if ((twstat) (start_item, &statbuf E2_ERR_SAMEARG()))
@@ -684,12 +703,15 @@ gboolean e2_fs_tw (gchar *start_item, E2_TwResult (*callback) (),
 	}
 	else
 	{
+		E2_FsReadWatch watchdir =
+			(S_ISDIR (statbuf.st_mode) && e2_fs_mount_is_fusepoint (start_item)) ?
+			E2_DIRWATCH_YES : E2_DIRWATCH_NO;
 		//setup walk-data for downstream functions
 		E2_TwData data =
-		{ twstat, callback, user_data, max_depth, exec_flags, statbuf.st_dev };
+		{ twstat, callback, user_data, max_depth, watchdir, exec_flags, statbuf.st_dev };
 
 		result = (S_ISDIR (statbuf.st_mode)) ?
-			_e2_fs_tw (start_item, 0, &data):
+			_e2_fs_tw_dir (start_item, 0, &data):
 			_e2_fs_tw_nondir (start_item, &statbuf, &data); //should never be needed, but ...
 	}
 
@@ -701,6 +723,7 @@ gboolean e2_fs_tw (gchar *start_item, E2_TwResult (*callback) (),
 			*E2_ERR_NAME = *callerr;
 	}
 #endif
+	E2_ERR_CLEARBACKUP (localerr);
 
 	return (result == E2TW_CONTINUE);
 }
