@@ -1,6 +1,6 @@
-/* $Id: e2_list.h 514 2007-07-15 02:46:39Z tpgww $
+/* $Id: e2_list.h 874 2008-05-07 14:47:27Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -18,8 +18,8 @@ along with emelFM2; see the file GPL. If not, contact the Free Software
 Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef E2_LIST_H
-#define E2_LIST_H
+#ifndef __E2_LIST_H__
+#define __E2_LIST_H__
 
 #include "emelfm2.h"
 
@@ -31,5 +31,8 @@ GList *e2_list_copy_with_data (GList *list);
 void e2_list_free_with_data (GList **list);
 //void e2_list_free_data_only (GList **list);
 //GList *e2_list_find_data_string (GList *list, gchar *search_text);
+void e2_list_nth_break (GList **list, guint breaker, GList **part1, GList **part2);
+void e2_list_break (GList **list, GList *breaker, gboolean nullatend, GList **part1,
+	GList **part2);
 
-#endif // ifndef E2_LIST_H
+#endif //ndef __E2_LIST_H__
