@@ -1,6 +1,6 @@
-/* $Id: e2_menu.c 559 2007-07-24 12:16:46Z tpgww $
+/* $Id: e2_menu.c 853 2008-04-07 10:38:17Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,9 +32,6 @@ This file contains utility and helper functions for menus.
 #include "e2_dialog.h"
 #include "e2_plugins.h"
 #include "e2_task.h"
-#ifdef E2_VFS
-# include "e2_vfs.h"
-#endif
 #ifdef E2_FS_MOUNTABLE
 # include "e2_filelist.h"
 #endif
@@ -72,6 +69,78 @@ void e2_menu_control_cb (GtkWidget *menu_item, gpointer widget)
 	e2_option_connect (widget, gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item)));
 }
+/**
+@brief update set value when tied check menu item is activated
+@param menu_item the item that was activated
+@param set pointer to data for optionset tied to @a menu_item
+@return
+*/
+static void _e2_menu_check_value_changed_cb (GtkWidget *menu_item, E2_OptionSet *set)
+{
+	GtkWidget *controller = g_object_get_data (G_OBJECT (menu_item),
+		"e2-controller-widget");
+	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
+		"e2-controller-blocked")))
+		e2_option_bool_set_direct (set, gtk_check_menu_item_get_active
+			(GTK_CHECK_MENU_ITEM (menu_item)));
+}
+/**
+@brief set @a menu_item state if it's not blocked and not already at the desired state
+This is a hook-function callack upon change of set data associated with @a menu_item
+@param state pointerised TRUE/FALSE, the value to apply to @a menu_item
+@param menu_item the widget to change
+
+@return TRUE always
+*/
+static gboolean _e2_menu_check_change_value (gpointer state, GtkWidget *menu_item)
+{
+	GtkWidget *controller = g_object_get_data (G_OBJECT (menu_item),
+		"e2-controller-widget");
+	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
+		"e2-controller-blocked")))
+	{
+		gboolean value = GPOINTER_TO_INT (state);
+		gboolean current = gtk_check_menu_item_get_active
+			(GTK_CHECK_MENU_ITEM (menu_item));
+		if (value != current)
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), value);
+	}
+	return TRUE;
+}
+/**
+@brief add tied check item to @a menu
+
+@param menu menu widget
+@param set pointer to data for set whose value is tied to the check button
+@param controller widget which may have "e2-controller-blocked" data to prevent set value being updated when item is activated, can be NULL
+@param func void* callback function for handling menu item selection, or NULL
+@param data pointer to data to send to the callback
+
+@return the menu item widget
+*/
+static GtkWidget *_e2_menu_add_tied_check (GtkWidget *menu, E2_OptionSet *set,
+	GtkWidget *controller, gpointer func, gpointer data)
+{
+	GtkWidget *menu_item = e2_menu_add_check (menu, set->desc,
+		e2_option_bool_get_direct (set), func, data);
+	g_object_set_data (G_OBJECT (menu_item), "e2-controller-widget", controller);
+	//this will update the set value when the check menu item is activated
+	g_signal_connect (G_OBJECT (menu_item), "toggled",
+		G_CALLBACK (_e2_menu_check_value_changed_cb), set);
+	//this will update the check menu item when the config data value changes
+	e2_option_attach_value_changed_simple (set, menu_item,
+		_e2_menu_check_change_value, menu_item);
+#ifdef USE_GTK2_12TIPS
+	gtk_widget_set_tooltip_text (
+#else
+	e2_widget_set_tooltip (NULL,
+#endif
+		menu_item, set->tip);
+	//conform to dependent set if any
+	e2_widget_handle_depends (menu_item, set);
+
+	return menu_item;
+}
 
   /****************/
  /**** public ****/
@@ -117,7 +186,7 @@ GtkWidget *e2_menu_add (GtkWidget *menu, gchar *label, gchar *icon,
 	gtk_widget_show_all (menu_item);
 
 	if (tip != NULL && *tip != '\0')
-#ifdef USE_GTK2_12
+#ifdef USE_GTK2_12TIPS
 		gtk_widget_set_tooltip_text (
 #else
 		e2_widget_set_tooltip (NULL,
@@ -274,7 +343,7 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gchar *label, gchar *icon,
 	{	//process 1st of a pair
 		first = FALSE;
 		//park until we get the paired name, so we can make hash key
-		firststate = g_str_has_suffix (type, _A(102));	//_(on
+		firststate = g_str_has_suffix (type, _A(111));	//_(on
 		firstlabel = g_strdup (label);
 		firsticon = g_strdup (icon);
 		firsttip = g_strdup (tip);
@@ -391,7 +460,7 @@ GtkWidget *e2_menu_create_options_menu (GtkWidget *controller, GtkWidget *menu,
 		switch (set->type)
 		{
 			case E2_OPTION_TYPE_BOOL:
-				e2_option_bool_add_menu_widget (controller, menu, set, func, data);
+				_e2_menu_add_tied_check (menu, set, controller, func, data);
 				break;
 			case E2_OPTION_TYPE_SEL:
 				e2_option_sel_add_menu_widget (controller, menu, set, func, data);
@@ -723,7 +792,7 @@ GtkWidget *e2_menu_create_filter_menu (ViewInfo *view)
 The state of @a item when it arrives here is opposite to that
 shown in the menu, when clicked
 @param item the selected menu item
-@param UNUSED user_data data specified when the item was constructed
+@param user_data UNUSED data specified when the item was constructed
 
 @return
 */
@@ -731,8 +800,11 @@ static void _e2_menu_mount_cb (GtkWidget *item, gpointer user_data)
 {
 	gboolean newstate = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
 //	gint result;
-//	gchar *olddir;
+	gchar *elsewhere = NULL;
 	gchar *point = g_object_get_data (G_OBJECT (item), "_mountpoint_");
+#ifdef E2_HAL
+	gboolean remove = e2_fs_mount_is_ejectable (point);	//check this before unmount
+#endif
 	//CHECKME synchronous umount command to mimimize risk of CWD error
 	gchar *cmd = g_strdup_printf ("%s \"%s\"", (newstate) ? "mount" : "|umount", point);	//CHECKME _I(
 	if (!newstate //doing an unmount
@@ -741,31 +813,57 @@ static void _e2_menu_mount_cb (GtkWidget *item, gpointer user_data)
 #else
 		&& g_str_has_prefix (curr_view->dir, point))
 #endif
-	{	//if possible, move out of current dir so we don't ourself block the unmount
-		gchar *s = strrchr (point, G_DIR_SEPARATOR);
+	{	//if possible, change CWD out of mountpoint so we don't ourself block the unmount
+		gchar *s = strrchr (point, G_DIR_SEPARATOR);	//assumes no trailer on mountpoint string
 		if (s > point+1) //not doing root dir
-		{
-			*s = '\0';
-//			result =
-			e2_command_run_at (cmd, point, E2_COMMAND_RANGE_DEFAULT);
-			//CHECKME
-			//if (result == 0) //cd only if the command succeeds
-			e2_pane_change_dir (curr_pane, point);
-//			*s = G_DIR_SEPARATOR;
-			g_free (cmd);
-			return;
-		}
+			elsewhere = g_strndup (point, s - point);
+		else //can't cd
+			elsewhere = g_strdup (G_DIR_SEPARATOR_S);
+
+		e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE());
+//		if (e2_fs_chdir (elsewhere E2_ERR_NONE()))
+		//result =
+			e2_command_run_at (cmd, elsewhere, E2_COMMAND_RANGE_DEFAULT);	//has temporary change CWD
 	}
+	else
 //	result =
 #ifdef E2_COMMANDQ
-	e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT, FALSE);
+		e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT, FALSE);
 #else
-	e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT);
+		e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT);
 #endif
 	g_free (cmd);
 
 	if (newstate) //doing a mount
-		e2_filelist_check_dirty (GINT_TO_POINTER (1));	//FIXME doesn't work
+	{
+#ifdef E2_FAM
+		e2_filelist_request_refresh (curr_view->dir, FALSE);
+		e2_filelist_request_refresh (other_view->dir, TRUE);
+#else
+		e2_filelist_check_dirty (GINT_TO_POINTER (1));
+#endif
+	}
+	else	//doing an unmount
+	{
+#ifdef E2_HAL
+		if (remove)
+		{
+			cmd = g_strdup_printf ("eject \"%s\"", point);	//CHECKME _I(
+# ifdef E2_COMMANDQ
+			e2_command_run_at (cmd, elsewhere, E2_COMMAND_RANGE_DEFAULT, FALSE);
+# else
+			e2_command_run_at (cmd, elsewhere, E2_COMMAND_RANGE_DEFAULT);
+# endif
+			g_free (cmd);
+		}
+#endif
+		if (elsewhere != NULL)
+		{
+//E2_VFSTMP may change space too
+//			e2_pane_change_dir (curr_pane, point);	//not needed, with inofify at least
+			g_free (elsewhere);
+		}
+	}
 
 /*	if (!(newstate || result == 0))
 	{	//change back to where we started
@@ -858,59 +956,6 @@ gboolean e2_menu_create_mounts_menu (gpointer from, E2_ActionRuntime *art)
 	return TRUE;
 }
 #endif	//def E2_FS_MOUNTABLE
-#ifdef E2_VFS
-/**
-@brief vfs menu-item callbacks
-
-@param menu_item the selected menu item widget
-@param view data struct for view to be changed
-
-@return
-*/
-static void _e2_menu_vfs_item_local_cb (GtkWidget *menu_item, ViewInfo *view)
-{
-#ifdef E2_VFSTMP
-	FIXME
-	e2_vfs_open_local (view);
-#endif
-}
-static void _e2_menu_vfs_item_remote_cb (GtkWidget *menu_item, ViewInfo *view)
-{
-	e2_vfs_dialog_create (view, TRUE);
-}
-static void _e2_menu_vfs_item_archive_cb (GtkWidget *menu_item, ViewInfo *view)
-{
-	e2_vfs_dialog_create (view, FALSE);
-}
-/**
-@brief create vfs menu for showing in pane to which @a view belongs
-
-@param view data struct for view to be changed
-
-@return the menu widget
-*/
-GtkWidget *e2_menu_create_vfs_menu (ViewInfo *view)
-{
-	GtkWidget *menu = gtk_menu_new ();
-	GtkWidget *item =
-	e2_menu_add_check (menu, _("_Mounted directory"),
-		view->dirtype == FS_LOCAL, _e2_menu_vfs_item_local_cb, view);
-	//don't use real radio menu items, to prevent double callbacks
-	gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
-//	e2_menu_add_separator (menu);
-	item = e2_menu_add_check (menu, _("_Remote directory"),
-		view->dirtype == FS_REMOTE, _e2_menu_vfs_item_remote_cb, view);
-	gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
-	item = e2_menu_add_check (menu, _("_Archive"),
-		view->dirtype == FS_ARCHIVE, _e2_menu_vfs_item_archive_cb, view);
-	gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
-
-	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
-
-	return menu;
-}
-#endif
 /**
 @brief install default tree options for custom menus
 This function is called only if the default is missing from the config file
