@@ -1,4 +1,4 @@
-/* $Id: e2_tree.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_tree.c 639 2007-09-05 03:10:56Z tpgww $
 
 Copyright (C) 2004-2007 tooar <tooar@gmx.net>
 
@@ -801,3 +801,119 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 		g_list_free (selpaths);
 	}
 }
+#ifdef STORECOPY
+/**
+@brief helper function to recursively copy a treestore
+@param model treemodel for store being copied
+@param parent pointer to treeiter which has been confirmed to have child(ren)
+@param ncols no. of columns in @a model
+@param colnums array of column indices
+@param types array of GTypes corresponding to columns
+@param values array in which to store values being transferred
+
+@return
+*/
+static void _e2_tree_copy_descendants (GtkTreeModel *model, GtkTreeIter *parent, gint ncols,
+	gint colnums[], GType types[], GValue values[])
+{
+	gint i;
+	GtkTreeStore *newtstore = GTK_TREE_STORE (model);
+	GtkTreeIter child;
+	gtk_tree_model_iter_children (model, &child, parent);
+	do
+	{
+		if (gtk_tree_model_iter_has_child (model, &child))
+			_e2_tree_copy_descendants (model, &child, ncols, colnums, types, values);
+		//read iter into pointers array
+		for (i = 0; i < ncols; i++)
+			gtk_tree_model_get_value (model, &child, i, &values[i]);
+#ifdef USE_GTK2_10
+		gtk_tree_store_insert_with_valuesv (newtstore, &child, parent, -1, colnums, values, ncols);
+#else
+		gtk_tree_store_append (newtstore, &child, parent);
+		for (i = 0; i < ncols; i++)
+			gtk_tree_store_set_value (newtstore, &child, i, &values[i]);
+#endif
+		for (i = 0; i < ncols; i++)
+			g_value_unset (&values[i]);
+	} while (gtk_tree_model_iter_next (model, &child));
+}
+/**
+@brief copy a liststore or treestore
+This is for when reffing is not enough e.g. need independent sorting in
+attached views. Pointers to data are simply copied, so if the "source" data will
+be cleared before fhe new store, then all such data must be replicated or otherwise
+preserved by code elsewhere which understands what that data really is.
+Officially, the new store will be unsorted, though in practice it will be the
+same as the old store.
+@param model treemodel of the store being copied
+@param treetype TRUE for treestore, FALSE for liststore
+@param newstore store for pointer to new store
+
+@a return
+*/
+void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newstore)
+{
+	GtkTreeModel *newmodel;
+	GtkTreeIter src;
+	GtkTreeIter dest;
+	gint i, ncols = gtk_tree_model_get_n_columns (model);
+	gint colnums[ncols];
+	GType types[ncols];
+	GValue values[ncols];
+
+	for (i = 0; i < ncols; i++)
+	{
+		colnums[i] = i;
+		types[i] = gtk_tree_model_get_column_type (model, i);
+		memset (&values[i], 0, sizeof (GValue));
+	}
+
+	if (treetype)
+	{
+		GtkTreeStore *newtstore;
+		newtstore = gtk_tree_store_newv (ncols, types);
+		if (gtk_tree_model_get_iter_first (model, &src))
+		{
+			newmodel = GTK_TREE_MODEL (newtstore);
+			do
+			{
+				//read iter into values array
+				for (i = 0; i < ncols; i++)
+					gtk_tree_model_get_value (model, &src, i, &values[i]);
+#ifdef USE_GTK2_10
+				gtk_tree_store_insert_with_valuesv (newtstore, &dest, NULL, -1, colnums, values, ncols);
+#else
+				gtk_tree_store_append (newtstore, &dest, NULL);
+				for (i = 0; i < ncols; i++)
+					gtk_tree_store_set_value (newtstore, &dest, i, &values[i]);
+#endif
+				for (i = 0; i < ncols; i++)
+					g_value_unset (&values[i]);
+				if (gtk_tree_model_iter_has_child (model, &src))
+					_e2_tree_copy_descendants (model, NULL, ncols, colnums, types, values);
+
+			} while (gtk_tree_model_iter_next (model, &src));
+		}
+		*newstore = newtstore;
+	}
+	else
+	{
+		GtkListStore *newlstore;
+		newlstore = gtk_list_store_newv (ncols, types);
+		if (gtk_tree_model_get_iter_first (model, &src))
+		{
+			newmodel = GTK_TREE_MODEL (newlstore);
+			do
+			{
+				for (i = 0; i < ncols; i++)
+					gtk_tree_model_get_value (model, &src, i, &values[i]);
+				gtk_list_store_insert_with_valuesv (newlstore, &dest, -1, colnums, values, ncols);
+				for (i = 0; i < ncols; i++)
+					g_value_unset (&values[i]);
+			} while (gtk_tree_model_iter_next (model, &src));
+		}
+		*newstore = newlstore;
+	}
+}
+#endif
