@@ -1,6 +1,6 @@
-/* $Id: e2_utf8.c 469 2007-07-06 22:58:30Z tpgww $
+/* $Id: e2_utf8.c 881 2008-05-12 22:54:30Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 Portion _maybe_ copyright (C) 2004-2005 Tarot Osuji
 
 This file is part of emelFM2.
@@ -180,27 +180,33 @@ gboolean e2_utf8_caseless_match (const gchar *s1, const gchar *s2,
 	gchar *casefold;
 	gchar *normalized_s1;
 	gchar *normalized_s2;
-	gint len_s1;
-	gint len_s2;
-	gboolean ret = FALSE;
+//	gint len_s1;
+//	gint len_s2;
+	gboolean ret;
 
 	g_return_val_if_fail (s1 != NULL, FALSE);
 	g_return_val_if_fail (s2 != NULL, FALSE);
-	g_return_val_if_fail (n1 > 0, FALSE);
-	g_return_val_if_fail (n2 > 0, FALSE);
+	g_return_val_if_fail (n1 >= -1, FALSE);
+	g_return_val_if_fail (n2 >= -1, FALSE);
+
+	if (n1 == 0 && n2 == 0)
+		return TRUE;
+	else if (n1 == 0 || n2 == 0)
+		return FALSE;
 
 	casefold = g_utf8_casefold (s1, n1);
 	normalized_s1 = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
 	g_free (casefold);
-	len_s1 = strlen (normalized_s1);
+//	len_s1 = strlen (normalized_s1);
 
 	casefold = g_utf8_casefold (s2, n2);
 	normalized_s2 = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
 	g_free (casefold);
-	len_s2 = strlen (normalized_s2);
+//	len_s2 = strlen (normalized_s2);
 
-	if (len_s1 >= len_s2)
-		ret = (strncmp (normalized_s1, normalized_s2, len_s2) == 0);
+//	ret = (len_s1 == len_s2) ?
+//		(strcmp (normalized_s1, normalized_s2) == 0) : FALSE;
+	ret = (strcmp (normalized_s1, normalized_s2) == 0);
 
 	g_free (normalized_s1);
 	g_free (normalized_s2);
@@ -425,19 +431,21 @@ gchar *e2_utf8_to_locale_fallback (const gchar *d)
 //#endif //0
 
 /*************************************
-character detection code imported from leafpad 0.8.7
-by Tarot Osuji
-But maybe originally from enca ?
+encoding-detection code adapted from leafpad 0.8.7 by Tarot Osuji
 */
 
-#define MAX_COUNTRY_NUM 10
+/* A complete list of charsets officially supported in glibc 2.2.3 is:
+ISO-8859-{1,2,3,5,6,7,8,9,13,15}, CP1251, UTF-8, EUC-{KR,JP,TW}, KOI8-{R,U},
+GB2312, GB18030, GBK, BIG5, BIG5-HKSCS and TIS-620
+(Romanian may be switching to ISO-8859-16)
+*/
 
 enum
 {
 	IANA = 0,
 	OPENI18N,
 	CODEPAGE,
-	ENCODING_MAX_ITEM_NUM
+	ENCODING_TYPECOUNT
 };
 
 enum
@@ -445,132 +453,136 @@ enum
 	LATIN1 = 0,
 	LATIN2,
 	LATIN3,
-	LATIN4,
-	LATINC,
-	LATINC_UA,
-	LATINC_TJ,
-	LATINA,
-	LATING,
-	LATINH,
+//	LATIN4,
 	LATIN5,
+	LATIN6,
+	LATIN7,
+	ARABIC,
+	CYRILIC,
+	CYRILIC_TJ,
+	CYRILIC_UA,
+	GREEK,
+	HEBREW,
 	CHINESE_CN,
-	CHINESE_TW,
 	CHINESE_HK,
+	CHINESE_TW,
 	JAPANESE,
 	KOREAN,
 	VIETNAMESE,
 	THAI,
 	GEORGIAN,
-	END_CODE
+	ENCODE_COUNT
 };
 
-typedef struct
-{
-	const gchar *item[ENCODING_MAX_ITEM_NUM];
-} EncArray;
+//up to this many locales per encoding
+#define MAX_LOCALES 10
 
-const gchar *country_table[][MAX_COUNTRY_NUM] =
+const gchar *locale_table[ENCODE_COUNT][MAX_LOCALES] =
 {
-	/* LATIN1 */        {NULL},
-	/* LATIN2 */        {"cs", "hr", "hu", "pl", "ro", "sk", "sl", "sq", "sr", "uz"},
-	/* LATIN3 */        {"eo", "mt", NULL},
-	/* LATIN4 */        {"et", "lt", "lv", "mi", NULL},
-	/* LATINC */        {"be", "bg", "ky", "mk", "mn", "ru", "tt", NULL},
-	/* LATINC_UA */     {"uk", NULL},
-	/* LATINC_TJ */     {"tg", NULL},
-	/* LATINA */        {"ar", "fa", "ur", NULL},
-	/* LATING */        {"el", NULL},
-	/* LATINH */        {"he", "yi", NULL},
-	/* LATIN5 */        {"az", "tr", NULL},
-	/* CHINESE_CN */    {"zh_CN", NULL},
-	/* CHINESE_TW */    {"zh_TW", NULL},
-	/* CHINESE_HK */    {"zh_HK", NULL},
-	/* JAPANESE */      {"ja", NULL},
-	/* KOREAN */        {"ko", NULL},
-	/* VIETNAMESE */    {"vi", NULL},
-	/* THAI */          {"th", NULL},
-	/* GEORGIAN */      {"ka", NULL},
+/* LATIN1 */	{NULL},
+/* LATIN2 */	{"cs", "de", "hr", "hu", "ro", "pl", "sk", "sq", "sr", "uz"},	//this effectively sets MAX_LOCALES
+/* LATIN3 */	{"eo", "gl", "mt", NULL},
+// / * LATIN4 * /	{"et", "mi", NULL}, Estonian, Latvian, and Lithuanian essentially obsolete;
+/* LATIN5 */	{"az", "tr", NULL},
+/* LATIN6 */	{"et", "ga", "is", "kl", "mi", NULL},
+/* LATIN7 */	{"da", "fi", "lt", "lv", "no", "pl", "se", "sl", "sv", NULL},
+/* ARABIC */	{"ar", "fa", "ur", NULL},
+/* CYRILIC */	{"be", "bg", "ky", "mk", "mn", "ru", "tt", NULL},
+/* CYRILIC_TJ */{"tg", NULL},
+/* CYRILIC_UA */{"uk", NULL},
+/* GREEK */		{"el", NULL},	//modern
+/* HEBREW */	{"he", "iw", "yi", NULL}, //modern
+/* CHINESE_CN */{"zh_CN", NULL},
+/* CHINESE_HK */{"zh_HK", NULL},
+/* CHINESE_TW */{"zh_TW", NULL},
+/* JAPANESE */	{"ja", NULL},
+/* KOREAN */	{"ko", NULL},
+/* VIETNAMESE */{"vi", NULL},
+/* THAI */		{"th", NULL},
+/* GEORGIAN */	{"ka", NULL}
 };
 
-const gchar *encoding_table[][ENCODING_MAX_ITEM_NUM] =
+/*******************************************
+/ * LATIN8 * /	{ "ISO-8859-14",  , },	//a.k.a celtic adds missing gaelic and welsh (cy) to latin1 mainly Old Gaelic, also ok for Albanian, Breton, Catalan, Danish, Dutch, English, Finnish, French, Gaelic, German, Irish, Italian, Norwegian, Portuguese, Swedish and Welsh.
+/ * LATIN9 * /	{ "ISO-8859-15",  , },	//latin1 with euro and some missing things
+/ * LATIN10 * /	{ "ISO-8859-16",  , },	//for Albanian, Croatian, Hungarian, Polish, Romanian and Slovenian, but also French, German, Italian and Irish Gaelic (new orthography). It differs from the other
+/ * ARABIC * /	{ "ISO-8859-6-E",  , },	//explicit bi-directional vartiant
+/ * ARABIC * /	{ "ISO-8859-6-I",  , }, //implicit bi-directional variant
+/ * HEBREW * /	{ "ISO-8859-8-E",  , }, //explicit bi-directional variant
+/ * HEBREW * /	{ "ISO-8859-8-I",  , }, //implicit bi-directional variant
+******************************************/
+
+const gchar *encoding_table [ENCODE_COUNT][ENCODING_TYPECOUNT] =
 {
-	                  /*  IANA           OpenI18N       Codepage */
-	/* LATIN1 */        { "ISO-8859-1",  "ISO-8859-15", "CP1252" },
-	/* LATIN2 */        { "ISO-8859-2",  "ISO-8859-16", "CP1250" },
-	/* LATIN3 */        { "ISO-8859-3",  NULL,          NULL },
-	/* LATIN4 */        { "ISO-8859-4",  "ISO-8859-13", "CP1257" },
-	/* LATINC */        { "ISO-8859-5",  "KOI8-R",      "CP1251" },
-	/* LATINC_UA */     { "ISO-8859-5",  "KOI8-U",      "CP1251" },
-	/* LATINC_TJ */     { "ISO-8859-5",  "KOI8-T",      "CP1251" },
-	/* LATINA */        { "ISO-8859-6",  NULL,          "CP1256" },
-	/* LATING */        { "ISO-8859-7",  NULL,          "CP1253" },
-	/* LATINH */        { "ISO-8859-8",  NULL,          "CP1255" },
-	/* LATIN5 */        { "ISO-8859-9",  NULL,          "CP1254" },
-	/* CHINESE_CN */    { "GB2312",      "GB18030",     "CP936" },
-	/* CHINESE_TW */    { "BIG5",        "EUC-TW",      "CP950" },
-	/* CHINESE_HK */    { "BIG5",        "BIG5-HKSCS",  "CP950" },
-	/* JAPANESE */      { "ISO-2022-JP", "EUC-JP",      "CP932" },
-	/* KOREAN */        { "ISO-2022-KR", "EUC-KR",      "CP949" },
-	/* VIETNAMESE */    { NULL,          "VISCII",      "CP1258" },
-	/* THAI */          { NULL,          "TIS-620",     "CP874" },
-	/* GEORGIAN */      { NULL,          "GEORGIAN-PS", NULL },
+				/*  IANA          OpenI18N       Codepage */
+/* LATIN1 */	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" },
+/* LATIN2 */	{ "ISO-8859-2",  "ISO-8859-16", "CP1250" },
+/* LATIN3 */	{ "ISO-8859-3",  NULL,          NULL },
+// /* LATIN4 */	{ "ISO-8859-4",  "ISO-8859-13", "CP1257" },
+/* LATIN5 */	{ "ISO-8859-9",  NULL,          "CP1254" },
+/* LATIN6 */	{ "ISO-8859-10", NULL,          "CP1257" },
+/* LATIN7 */	{ "ISO-8859-13", NULL,          "CP1257" },
+/* ARABIC */	{ "ISO-8859-6",  NULL,          "CP1256" },
+//ISO-8859-5 for e.g. bulgarian, belarusian, russian and macedonian, not ukrainian, "never really caught on"
+/* CYRILIC */	{ "ISO-8859-5",  "KOI8-R",      "CP1251" },	//russian, macedonian
+/* CYRILIC_TJ */{ "ISO-8859-5",  "KOI8-T",      "CP1251" }, //tajic
+/* CYRILIC_UA */{ "ISO-8859-5",  "KOI8-U",      "CP1251" },	//ukrainian
+/* GREEK */		{ "ISO-8859-7",  NULL,          "CP1253" },	//modern
+/* HEBREW */	{ "ISO-8859-8",  NULL,          "CP1255" }, //modern
+/* CHINESE_CN */{ "GB2312",      "GB18030",     "CP936" },
+/* CHINESE_HK */{ "BIG5",        "BIG5-HKSCS",  "CP950" },
+/* CHINESE_TW */{ "BIG5",        "EUC-TW",      "CP950" },
+/* JAPANESE */	{ "ISO-2022-JP", "EUC-JP",      "CP932" },
+/* KOREAN */	{ "ISO-2022-KR", "EUC-KR",      "CP949" },
+/* VIETNAMESE */{ NULL,          "VISCII",      "CP1258" },
+/* THAI */		{ "ISO-8859-11", "TIS-620",     "CP874" }, //ISO-8859-11 is unofficial
+/* GEORGIAN */	{ NULL,          "GEORGIAN-PS", NULL }
 };
 
-guint get_encoding_code(void)
-{
-	static guint code = END_CODE;
-	const gchar *env;
-	guint i, j = 1;
 
-	if (code == END_CODE) {
-		env = g_getenv("LC_ALL");
-		if (!env)
-			env = g_getenv("LANG");
-		if (env && strlen(env) >= 2)
-			while (code == END_CODE && j < END_CODE) {
-				for (i = 0; i < MAX_COUNTRY_NUM; i++) {
-					if (!country_table[j][i])
+static guint get_locale_index (void)
+{
+	static guint code = ENCODE_COUNT;
+
+	if (code == ENCODE_COUNT)	//we only look this up once
+	{
+		const gchar *env = g_getenv ("LC_ALL");
+		if (env == NULL)
+			env = g_getenv ("LANG");
+		if (env != NULL && strlen (env) >= 2)
+		{
+			guint i, j = 1;
+			while (code == ENCODE_COUNT && j < ENCODE_COUNT)
+			{
+				for (i = 0; i < MAX_LOCALES; i++)
+				{
+					if (locale_table[j][i] == NULL)
 						break;
-					if (strncmp(env, country_table[j][i], strlen(country_table[j][i])) == 0) {
+					if (strncmp (env, locale_table[j][i], strlen(locale_table[j][i])) == 0)
+					{
 						code = j;
 						break;
 					}
 				}
 				j++;
 			}
-		if (code == END_CODE)
-			code = 0;
+		}
+		if (code == ENCODE_COUNT)
+			code = LATIN1;	//default if we can't find anything supported
 	}
 
 	return code;
 }
 
-EncArray *get_encoding_items (guint code)
+static const gchar *detect_charset_cyrillic (const guchar *text)
 {
-	gint i;
-	static EncArray *array = NULL;
-
-	if (array == NULL)
-	{
-		array = ALLOCATE (EncArray);	//FIXME might be too small for slice, never deallocated
-		CHECKALLOCATEDFATAL (array);
-		for (i = 0; i < ENCODING_MAX_ITEM_NUM; i++)
-			array->item[i] = encoding_table[code][i] ?
-				encoding_table[code][i] : NULL;
-	}
-
-	return array;
-}
-
-static const gchar *detect_charset_cyrillic(const gchar *text)
-{
-	guint8 c = *text;
+	guchar c;
 	gboolean noniso = FALSE;
 	guint32 xc = 0, xd = 0, xef = 0;
+	const gchar *charset;
 
-	const gchar *charset = get_encoding_items(get_encoding_code())->item[OPENI18N];
-
-	while ((c = *text++) != '\0') {
+	while ((c = *text++) != '\0')
+	{
 		if (c >= 0x80 && c <= 0x9F)
 			noniso = TRUE;
 		else if (c >= 0xC0 && c <= 0xCF)
@@ -585,82 +597,119 @@ static const gchar *detect_charset_cyrillic(const gchar *text)
 		charset = "ISO-8859-5";
 	else if ((xc + xd) < xef)
 		charset = "CP1251";
+	else
+		charset = encoding_table [get_locale_index()][OPENI18N];
 
 	return charset;
 }
 
-static const gchar *detect_charset_chinese(const gchar *text)
+static const gchar *detect_charset_chinese (const guchar *text)
 {
-	guint8 c = *text;
+	guchar c;
 
-	const gchar *charset = get_encoding_items(get_encoding_code())->item[IANA];
+	const gchar *charset = NULL;
 
-	while ((c = *text++) != '\0') {
-		if (c >= 0x81 && c <= 0x87) {
+	while ((c = *text++) != '\0')
+	{
+		if (c >= 0x81 && c <= 0x87)
+		{
 			charset = "GB18030";
 			break;
 		}
-		else if (c >= 0x88 && c <= 0xA0) {
+		else if (c >= 0x88 && c <= 0xA0)
+		{
 			c = *text++;
-			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
+			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+			{
 				charset = "GB18030";
 				break;
 			} //else GBK/Big5-HKSCS cannot determine
 		}
-		else if ((c >= 0xA1 && c <= 0xC6) || (c >= 0xC9 && c <= 0xF9)) {
+		else if ((c >= 0xA1 && c <= 0xC6) || (c >= 0xC9 && c <= 0xF9))
+		{
 			c = *text++;
 			if (c >= 0x40 && c <= 0x7E)
+			{
 				charset = "BIG5";
-			else if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
+				break;
+			}
+			else if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+			{
 				charset = "GB18030";
 				break;
 			}
 		}
-		else if (c >= 0xC7) {
+		else if (c >= 0xC7)
+		{
 			c = *text++;
-			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0)) {
+			if ((c >= 0x30 && c <= 0x39) || (c >= 0x80 && c <= 0xA0))
+			{
 				charset = "GB18030";
 				break;
 			}
 		}
 	}
 
+	if (charset == NULL)
+		charset = encoding_table [get_locale_index()][IANA];
+
 	return charset;
 }
 
-static const gchar *detect_charset_japanese(const gchar *text)
+static const gchar *detect_charset_japanese (const guchar *text)
 {
-	guint8 c = *text;
+	guchar c;
 	gchar *charset = NULL;
 
-	while (charset == NULL && (c = *text++) != '\0') {
-		if (c >= 0x81 && c <= 0x9F) {
-			if (c == 0x8E) /* SS2 */ {
+	while ((c = *text++) != '\0')
+	{
+		if (c >= 0x81 && c <= 0x9F)
+		{
+			if (c == 0x8E) /* SS2 */
+			{
 				c = *text++;
 				if ((c >= 0x40 && c <= 0xA0) || (c >= 0xE0 && c <= 0xFC))
+				{
 					charset = "CP932";
+					break;
+				}
 			}
-			else if (c == 0x8F) /* SS3 */ {
+			else if (c == 0x8F) /* SS3 */
+			{
 				c = *text++;
 				if (c >= 0x40 && c <= 0xA0)
+				{
 					charset = "CP932";
+					break;
+				}
 				else if (c >= 0xFD)
 					break;
 			}
 			else
+			{
 				charset = "CP932";
+				break;
+			}
 		}
-		else if (c >= 0xA1 && c <= 0xDF) {
+		else if (c >= 0xA1 && c <= 0xDF)
+		{
 			c = *text++;
 			if (c <= 0x9F)
+			{
 				charset = "CP932";
+				break;
+			}
 			else if (c >= 0xFD)
 				break;
 		}
-		else if (c >= 0xE0 && c <= 0xEF) {
+		else if (c >= 0xE0 && c <= 0xEF)
+		{
 			c = *text++;
 			if (c >= 0x40 && c <= 0xA0)
+			{
 				charset = "CP932";
+				break;
+			}
 			else if (c >= 0xFD)
 				break;
 		}
@@ -674,59 +723,90 @@ static const gchar *detect_charset_japanese(const gchar *text)
 	return charset;
 }
 
-static const gchar *detect_charset_korean(const gchar *text)
+static const gchar *detect_charset_korean (const guchar *text)
 {
-	guint8 c = *text;
+	guchar c;
 	gboolean noneuc = FALSE;
 	gboolean nonjohab = FALSE;
 	gchar *charset = NULL;
 
-	while (charset == NULL && (c = *text++) != '\0') {
-		if (c >= 0x81 && c < 0x84) {
+	while ((c = *text++) != '\0')
+	{
+		if (c >= 0x81 && c < 0x84)
+		{
 			charset = "CP949";
+			break;
 		}
-		else if (c >= 0x84 && c < 0xA1) {
+		else if (c >= 0x84 && c < 0xA1)
+		{
 			noneuc = TRUE;
 			c = *text++;
 			if ((c > 0x5A && c < 0x61) || (c > 0x7A && c < 0x81))
+			{
 				charset = "CP1361";
+				break;
+			}
 			else if (c == 0x52 || c == 0x72 || c == 0x92 || (c > 0x9D && c < 0xA1)
 				|| c == 0xB2 || (c > 0xBD && c < 0xC1) || c == 0xD2
 				|| (c > 0xDD && c < 0xE1) || c == 0xF2 || c == 0xFE)
+			{
 				charset = "CP949";
+				break;
+			}
 		}
-		else if (c >= 0xA1 && c <= 0xC6) {
+		else if (c >= 0xA1 && c <= 0xC6)
+		{
 			c = *text++;
-			if (c < 0xA1) {
+			if (c < 0xA1)
+			{
 				noneuc = TRUE;
 				if ((c > 0x5A && c < 0x61) || (c > 0x7A && c < 0x81))
+				{
 					charset = "CP1361";
+					break;
+				}
 				else if (c == 0x52 || c == 0x72 || c == 0x92 || (c > 0x9D && c < 0xA1))
+				{
 					charset = "CP949";
+					break;
+				}
 				else if (c == 0xB2 || (c > 0xBD && c < 0xC1) || c == 0xD2
 					|| (c > 0xDD && c < 0xE1) || c == 0xF2 || c == 0xFE)
 					nonjohab = TRUE;
 			}
 		}
-		else if (c > 0xC6 && c <= 0xD3) {
+		else if (c > 0xC6 && c <= 0xD3)
+		{
 			c = *text++;
 			if (c < 0xA1)
+			{
 				charset = "CP1361";
+				break;
+			}
 		}
-		else if (c > 0xD3 && c < 0xD8) {
+		else if (c > 0xD3 && c < 0xD8)
+		{
 			nonjohab = TRUE;
 			c = *text++;
 		}
-		else if (c >= 0xD8) {
+		else if (c >= 0xD8)
+		{
 			c = *text++;
 			if (c < 0xA1)
+			{
 				charset = "CP1361";
+				break;
+			}
 		}
 		if (noneuc && nonjohab)
+		{
 			charset = "CP949";
+			break;
+		}
 	}
 
-	if (charset == NULL) {
+	if (charset == NULL)
+	{
 		if (noneuc)
 			charset = "CP949";
 		else
@@ -736,88 +816,224 @@ static const gchar *detect_charset_korean(const gchar *text)
 	return charset;
 }
 
-static gboolean detect_noniso(const gchar *text)
+static gboolean detect_noniso (const guchar *text)
 {
-	guint8 c = *text;
+	guchar c;
 
-	while ((c = *text++) != '\0') {
+	while ((c = *text++) != '\0')
+	{
 		if (c >= 0x80 && c <= 0x9F)
 			return TRUE;
 	}
 	return FALSE;
 }
-/**
-@brief detect character encoding of string @a text
-@a text may be validated UTF-8 in spite of a nominally different coding
-@a charset is set to "UNKNOWN" if nothing specific is recognised
-@param text the string to be processed
-@param charset store for name of detected charset
-@return TRUE if @a text is found to be valid UTF-8 or plain ascii
-*/
-gboolean e2_utf8_detect_charset (const gchar *text, const gchar **charset)
-{
-	*charset = NULL;
 
-	gboolean isutf8 = g_utf8_validate (text, -1, NULL);
-	if (isutf8)
+/**
+@brief detect any recognised byte-order-mark at start of @a text
+@param text the string to be processed
+Note
+There's no distinction between flavours of UTF-16 and the corresponding
+UCS-2's, or between UTF-32's and corresponding UCS-4's
+No checking for file length, particularly in the case of UTF-32
+@return encoding name if any is recognised, or NULL
+*/
+static const gchar *detect_charset_BOM (const guchar *text)
+{
+	guchar c;
+	gint count;
+	guchar array[3];
+
+	count = 0;
+	switch (*text++)
 	{
-		guint8 c = *text;
-		while ((c = *text++) != '\0')
+		case '+':
+			//2B 2F 76 and one of the following bytes: [ 3C | 3D | 3E | 3F ]
+			array[0] = '/';
+			array[1] = 'v';
+			while (count < 2 && (c = *text++) == array[count]) {count++;}
+			if (count == 2 && ((c = *text) >= '\\' && c <= '_'))
+				return "UTF-7";
+			break;
+		case 0xEF:
+			//EF BB BF
+			array[0] = 0xBB;
+			array[1] = 0xBF;
+			while (count < 2 && (c = *text++) == array[count]) {count++;}
+			if (count == 2)
+				return "UTF-8";
+			break;
+		/*UCS-2, UCS-2, UCS-2BE, UCS-2LE (often conflated with UTF-16 but they're not the same)
+		  have same BOM as corresponding UTF-16. Ditto for UCS-4's and UTF-32's	*/
+		case 0xFE:
+			//Big Endian FE FF
+			if (*text == 0xFF)
+				return "UTF-16BE"; //could actually be UCS-2BE, not quite the same
+			break;
+		case 0xFF:
+			//Little Endian	FF FE 00 00
+			//Little Endian FF FE
+			array[0] = 0xFE;
+			array[1] = 0;
+			array[2] = 0;
+			while (count < 3 && (c = *text++) == array[count]) {count++;}
+			if (count == 3)
+				return "UTF-32LE";
+			if (count == 1)
+				return "UTF-16LE"; //could actually be UCS-2LE, not quite the same
+			break;
+		case 0: //CHECKME if file is empty, next checks may be a buffer overflow ?
+			//Big Endian 00 00 FE FF
+			array[0] = 0;
+			array[1] = 0xFE;
+			array[2] = 0xFF;
+			while (count < 3 && (c = *text++) == array[count]) {count++;}
+			if (count == 3)
+				return "UTF-32BE";
+			break;
+		case 0x0E:
+			//0E FE FF
+			array[0] = 0xFE;
+			array[1] = 0xFF;
+			while (count < 2 && (c = *text++) == array[count]) {count++;}
+			if (count == 2)
+				return "SCSU";
+			break;
+		case 0xDD:
+			//DD 73 66 73
+			array[0] = 's';
+			array[1] = 'f';
+			array[2] = 's';
+			while (count < 3 && (c = *text++) == array[count]) {count++;}
+			if (count == 3)
+				return "UTF-EBCDIC";
+			break;
+		case 0xFB:
+			//FB EE 28
+			array[0] = 0xEE;
+			array[1] = 0x28;
+			while (count < 2 && (c = *text++) == array[count]) {count++;}
+			if (count == 2)
+				return "BOCU-1";
+		default:
+			break;
+	}
+	return NULL;
+}
+
+/**
+@brief detect character encoding of 0-terminated string @a text
+@a text may be validated as UTF-8 in spite of a nominally different coding
+@a charset is set to "UNKNOWN" if nothing specific is recognised
+@param text 0-terminated string to be processed
+@param charset store for name of detected charset
+@return TRUE if @a text is found to be valid UTF-8 (which includes plain ascii)
+*/
+gboolean e2_utf8_detect_charset (const guchar *text, const gchar **charset)
+{
+	const guchar *start;
+	guchar c;
+	gboolean isutf8;
+
+	*charset = detect_charset_BOM (text);
+	if (*charset != NULL)	//FIXME test for short file length if UTF-32*
+		return (g_str_equal (*charset, "UTF-8"));
+
+	start = text;
+	isutf8 = FALSE;
+	while ((c = *text++) != '\0')
+	{
+		if (c > 0x7F)
 		{
-			if (c > 0x7F)
+			if ((c & 0xC0) == 0xC0)
+				isutf8 = g_utf8_validate ((gchar *)--text, -1, NULL);
+			break;
+		}
+		else if (c == '~')
+		{
+			c = *text++;
+			if (c == '{')
 			{
-				*charset = "UTF-8";
-				break;
-			}
-			if (c == 0x1B) /* ESC */
-			{
-				c = *text++;
-				if (c == '$')
+				c = *text++;	//maybe 1st GB2312 (chinese) char
+				if (c != '\0')
 				{
-					c = *text++;
-					switch (c)
+					c = *text++;	//maybe } or 2nd GB2312 (chinese) char
+					if (c == '}' && *text == '~')
 					{
-						case 'B': // JIS X 0208-1983
-						case '@': // JIS X 0208-1978
-							*charset = "ISO-2022-JP";
-							continue;
-						case 'A': // GB2312-1980
-							*charset = "ISO-2022-JP-2";
+						*charset = "HZ";
+						break;
+					}
+					else
+					{
+						c = *text++;	//maybe } after 2nd GB2312 (chinese) char
+						if (c == '}' && *text == '~' && *(text-2) > 0x7F)
+						{
+							*charset = "HZ";
 							break;
-						case '(':
-							c = *text++;
-							switch (c)
-							{
-								case 'C': // KSC5601-1987
-								case 'D': // JIS X 0212-1990
-									*charset = "ISO-2022-JP-2";
-							}
-							break;
-						case ')':
-							c = *text++;
-							if (c == 'C')
-								*charset = "ISO-2022-KR"; // KSC5601-1987
+						}
 					}
-					break;
 				}
 			}
 		}
-/*		if (*charset == NULL)
+		else if (c == 0x1B) /* ESC */
 		{
-			//it's probably just ascii, assume it's default coding
-			e2_utils_get_charset (charset);
-//			printd (DEBUG, "Specific UTF8 detection failed, reverted to DEFAULT CHARSET %s", *charset);
+			c = *text++;
+			if (c == '$')
+			{
+				c = *text++;
+				switch (c)
+				{
+					case 'B': // JIS X 0208-1983
+					case '@': // JIS X 0208-1978
+						*charset = "ISO-2022-JP";
+						break;
+					case 'A': // GB2312-1980
+						*charset = "ISO-2022-JP-2";
+						break;
+					case '(':
+						c = *text++;
+						switch (c)
+						{
+							case 'C': // KSC5601-1987
+							case 'D': // JIS X 0212-1990
+								*charset = "ISO-2022-JP-2";
+						}
+						break;
+					case ')':
+						c = *text++;
+						if (c == 'C')
+							*charset = "ISO-2022-KR"; // KSC5601-1987
+					default:
+						break;
+				}
+				if (*charset != NULL)
+					break;
+			}
 		}
-*/
 	}
 
-	if (*charset == NULL)
+	if (isutf8)
+	{	//validation succeeded
+		if (!g_get_charset (charset))	//use default encoding if it's utf8-compatible
+			*charset = "UTF-8";
+	}
+	else	//no validation requested, or validation failed
+		if (c == '\0')	//reached end of text without detection
+	{	//it's almost certainly ASCII, assume the default encoding applies
+		//FIXME except if that's incompatible with ascii
+//		printd (DEBUG, "Specific UTF8 detection failed, reverted to DEFAULT CHARSET %s", *charset);
+		e2_utils_get_charset (charset);
+		isutf8 = TRUE;	//no conversion needed
+	}
+	else // !isutf8 && not scanned to end of buffer
+		if (*charset == NULL)	//not detected ISO2022 or HZ
 	{
-		switch (get_encoding_code())
+		//next, assume that the file originates in the user's locale
+		guint code = get_locale_index();
+		switch (code)
 		{
-			case LATINC:
-			case LATINC_UA:
-			case LATINC_TJ:
+			case CYRILIC:
+			case CYRILIC_UA:
+			case CYRILIC_TJ:
 				*charset = detect_charset_cyrillic(text); // fuzzy...
 				break;
 			case CHINESE_CN:
@@ -834,18 +1050,20 @@ gboolean e2_utf8_detect_charset (const gchar *text, const gchar **charset)
 			case VIETNAMESE:
 			case THAI:
 			case GEORGIAN:
-				*charset = get_encoding_items(get_encoding_code())->item[OPENI18N];
+				*charset = encoding_table [code][OPENI18N];
 				break;
 			default:
-				if (!g_get_charset (charset))
+				e2_utils_get_charset (charset);
+				if (strcmp (*charset, "UTF-8") == 0)	//CHECKME this is no use as the text isn't compliant
 				{
-					if (detect_noniso(text))
-						*charset = get_encoding_items(get_encoding_code())->item[CODEPAGE];
+					if (detect_noniso (text))
+						*charset = encoding_table [code][CODEPAGE];
 					else
-						*charset = get_encoding_items(get_encoding_code())->item[OPENI18N];
+						*charset = encoding_table [code][OPENI18N];
 				}
 				if (*charset == NULL)
-					*charset = get_encoding_items(get_encoding_code())->item[IANA];
+					*charset = encoding_table [code][IANA];
+				break;
 		}
 	}
 
