@@ -1,6 +1,6 @@
-/* $Id: e2_utils.c 566 2007-07-25 05:34:09Z tpgww $
+/* $Id: e2_utils.c 896 2008-05-21 20:46:53Z tpgww $
 
-Copyright (C) 2003-2007 tooar <tooar@gmx.net>
+Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -34,7 +34,6 @@ typedef struct _E2_WildData
 	guint last_depth;		//count of the last path element
 	guint first_wild_depth;	//first element that has wildcard char(s)
 	guint last_wild_depth;	//last element to be matched (maybe 1 more than last depth)
-	E2_FSType dirtype;
 	struct stat *statptr;	//access to statbuf for shared use
 } E2_WildData;
 
@@ -102,6 +101,7 @@ void e2_utils_nontet_destroy (E2_Nontet *n)
 }
 /**
 @brief display error message about insufficient memory
+Expects BGL to be closed
 @return
 */
 void e2_utils_show_memory_message (void)
@@ -110,17 +110,19 @@ void e2_utils_show_memory_message (void)
 }
 /**
 @brief handle fatal lack of memory
+Expects BGL to be open
 @return never does
 */
 void e2_utils_memory_error (void)
 {
 	g_critical ("Not enough memory");
 	gdk_threads_enter (); 	//called func expects BGL closed
-	e2_main_shutdown (NULL, NULL, GINT_TO_POINTER (TRUE));	//prevents user cancellation
+	e2_main_closedown (TRUE, TRUE, TRUE); //no user-cancellation
 }
 /**
 @brief show user help at the heading @a title
-@param title heading string (NO []) to search for in the main help doc
+Expects BGL to be on/closed
+@param title heading string (NO []) to search for in the main help doc, undefined encoding
 @return
 */
 void e2_utils_show_help (gchar *title)
@@ -129,8 +131,26 @@ void e2_utils_show_help (gchar *title)
 	if (helpfile != NULL)
 	{
 		gchar *local = F_FILENAME_TO_LOCALE (helpfile);
-		if (e2_fs_access (local, R_OK E2_ERR_NONE()))
-	  		helpfile = NULL;
+#ifdef E2_VFS
+		VPATH ddata = { local, NULL };	//local helpfiles only
+		if (!e2_fs_access (&ddata, R_OK E2_ERR_NONE()))
+#else
+		if (!e2_fs_access (local, R_OK E2_ERR_NONE()))
+#endif
+		{
+			//setup help doc name and heading name, as parameters for command
+			gchar *filepath = g_strdup_printf ("%s [%s]", local, title);
+#ifdef E2_VFS
+			ddata.localpath = filepath; //possibly mixed encoding
+			e2_view_dialog_create_immediate (&ddata);
+#else
+			e2_view_dialog_create_immediate (filepath);
+#endif
+			g_free (filepath);
+		}
+		else
+			helpfile = NULL;
+
 		F_FREE (local);
 	}
 
@@ -139,13 +159,6 @@ void e2_utils_show_help (gchar *title)
 		gchar *msg = g_strdup_printf (_("Cannot read USAGE help document"));
 		e2_output_print_error (msg, TRUE);
 	}
-	else
-	{
-		//setup help doc name and heading name, as parameters for command
-		gchar *filepath = g_strdup_printf ("%s [%s]", helpfile, title);
-		e2_view_dialog_create_immediate (filepath);
-		g_free (filepath);
-	}
 }
 /**
 @brief convert color data to string form
@@ -165,7 +178,7 @@ This uses strsplit and strjoinv. No utf8 consideration.
 @param new the replacement string for @a old
 @return newly allocated string with the repacements made
 */
-gchar *e2_utils_str_replace (gchar *str, gchar *old, gchar *new)
+gchar *e2_utils_str_replace (const gchar *str, const gchar *old, const gchar *new)
 {
 	gchar **split = g_strsplit (str, old, -1);
 	gchar *join = g_strjoinv (new, split);
@@ -278,6 +291,38 @@ gchar *e2_utils_str_shorten (gchar *string, gint limit, E2_DotMode position)
 	}
 	return g_strdup (string);
 }
+
+/**
+@brief convert from a character-offset to a byte-offset within @a utf8_string
+@param utf8_string the string in which we want the offset
+@param charoffset 0-based character-position in @a utf8_string, may be < 0 to indicate position relative to end
+@return the byte-offset, or -1 upon error
+*/
+gint e2_utils_get_byte_position (const gchar *utf8_string, gint charoffset)
+{
+	if (charoffset == 0)
+		return 0;
+	//glib doesn't do any bounds check, so we do that here
+	gint len = g_utf8_strlen (utf8_string, -1);
+	if (charoffset > 0)
+	{
+		if (charoffset >= len)
+			return -1;
+	}
+	else
+#ifdef USE_GLIB2_10
+	{
+		if (-charoffset >= len)
+			return -1;
+	}
+#else
+		return -1;
+#endif
+
+	const gchar *s = g_utf8_offset_to_pointer (utf8_string, charoffset);
+	return (s - utf8_string);
+}
+
 /**
 @brief if necessary, replace "non-UNIX" line-separtors in @a text with LF
 In-place conversion is performed.
@@ -397,38 +442,24 @@ gchar *e2_utils_str_to_lower (gchar *string)
 {
 	return g_utf8_strdown (string, -1);
 }
-#ifdef E2_VFS
 /**
-@brief append @a string to the full "fake" current dir for @a view
-
-This is for joining a path with a prepended "v-pointer"
+@brief append @a string to the full path of the dir shown in @a view
 
 @param view pointer to view data struct
-@param string the string which will be appended, utf-8
+@param string the string which will be appended, localised or utf-8, in accord with @a localised
+@param localised TRUE to localise dir path before joining
 
-@return newly allocated joined utf8 string, or NULL
+@return newly allocated joined localised or utf8 string, or NULL on error
 */
-gchar *e2_utils_dircat (ViewInfo *view, const gchar *string)
+gchar *e2_utils_dircat (ViewInfo *view, const gchar *string, gboolean localised)
 {
-	gint len1 = strlen (view->dir);
-	gint len2 = strlen (string);
-	//include the leading pointer and one trailing 0 in the space allocation
-	gchar *result = g_try_malloc (len1 + len2 + sizeof (PlaceInfo *) + sizeof (gchar));
-	if (result != NULL)
-	{
-		PlaceInfo **vptr = (PlaceInfo **)result;
-		*vptr = view->spacedata;
-# ifdef __USE_GNU
-		gchar *next = mempcpy (result + sizeof (PlaceInfo *), view->dir, len1);
-		next = mempcpy (next, string, len2 + sizeof (gchar));
-# else
-		memcpy (result + sizeof (PlaceInfo *), view->dir, len1);
-		memcpy (result + len1 + sizeof (PlaceInfo *), string, len2 + sizeof (gchar));
-# endif
-	}
-	return result;
+	gchar *dir, *joined;
+	dir = (localised) ? F_FILENAME_TO_LOCALE (view->dir) : view->dir;
+	joined = e2_utils_strcat (dir, string);
+	if (localised)
+		F_FREE (dir);
+	return joined;
 }
-#endif
 /**
 @brief join strings @a string1 and @a string2
 
@@ -605,6 +636,50 @@ gchar *e2_utils_pass_whitespace (gchar *string)
 	return NULL;
 }
 /**
+@brief get a copy of @a utf8string without quotes if any
+@param utf8string the string which is to be processed
+Quotes may be ' or "
+
+@return newly-allocated string, or NULL if string is empty
+*/
+gchar *e2_utils_unquote_string (const gchar *utf8string)
+{
+	gboolean quoted;
+	gunichar uc1;
+	gchar *p;
+
+	p = e2_utils_pass_whitespace ((gchar *)utf8string);
+	if (p == NULL)
+		return NULL;
+	uc1 = g_utf8_get_char (p);
+	quoted = ((gchar)uc1 == '"' || (gchar)uc1 == '\'');
+	if (quoted)
+	{
+		gint len;
+		gunichar uc2;
+		gchar *q;
+		len = g_utf8_strlen (p, -1);
+		q = g_utf8_offset_to_pointer (p, len - 1);
+getlast:
+		uc2 = g_utf8_get_char (q);
+		if (uc1 == uc2)
+		{
+			p = g_utf8_next_char (p);
+			p = g_strndup (p, q - p);
+		}
+		else if ((gchar)uc2 == ' ' || (gchar)uc2 == '\t')
+		{
+			q = g_utf8_prev_char (q);
+			goto getlast;
+		}
+		else
+			p = g_strdup (p);
+	}
+	else
+		p = g_strdup (p);
+	return p;
+}
+/**
 @brief find the first (if any) argument-substring in @a string
 If @a quoted is TRUE, but @a string has no "", then quotes are added
 only if @a string contains whitespace. AND then, the whole of @a string is
@@ -704,6 +779,7 @@ Supported macro codes are %c, [%]%d, [%]%D, [%]%f, [%]%F, [%]%p, [%]%P(= F),
 These 'letters' are hard-coded CHECKME should the letters be translatable, non-ascii ?
 (%c macro is also processed in rename plugin, so that counter(s) can be
 recorded for incrementation purposes)
+Expects BGL on/closed
 
 @param text utf-8 (or ascii) string, possibly with macros to be expanded
 @param for_each utf8 string with name(s) or path(s) of item(s) to substitute for any "%f" or "%p", or NULL to use selected items
@@ -743,12 +819,8 @@ gchar *e2_utils_expand_macros (gchar *text, gchar *for_each)
 					//the supplied name may or may not have a path
 					name = g_path_get_basename (for_each);
 					if (g_str_equal (name, for_each)) //no path in for_each
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
 						p = (*s == 'f') ?
-							for_each : e2_utils_dircat (curr_view, for_each);
-#endif
+							for_each : e2_utils_dircat (curr_view, for_each, FALSE);
 					else //path in for_each
 						p = (*s == 'f') ? name : for_each;
 					if (with_quotes)
@@ -789,11 +861,7 @@ gchar *e2_utils_expand_macros (gchar *text, gchar *for_each)
 									(prefix == NULL) ? "" : prefix, utf);
 							else
 								g_string_append_printf (command_string, "%s%s%s",
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
 									(prefix == NULL) ? "" : prefix, curr_view->dir, utf);
-#endif
 							if (with_quotes)
 								command_string = g_string_append_c (command_string, '"');
 							if (tmp->next != NULL)
@@ -833,21 +901,13 @@ gchar *e2_utils_expand_macros (gchar *text, gchar *for_each)
 						{
 							g_string_append_printf (command_string, "\"%s%s%s\"%s",
 								(prefix == NULL) ? "" : prefix,
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
 								other_view->dir, utf, tmp->next != NULL ? " " : "");
-#endif
 						}
 						else
 						{
 							g_string_append_printf (command_string, "%s%s%s%s",
 								(prefix == NULL) ? "" : prefix,
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
 								other_view->dir, utf, tmp->next != NULL ? " " : "");
-#endif
 						}
 						F_FREE (utf);
 					}
@@ -859,11 +919,7 @@ gchar *e2_utils_expand_macros (gchar *text, gchar *for_each)
 			case 'D':
 			{
 				gchar *s1, *s2;
-#ifdef E2_VFSTMP
-	//FIXME dir when not mounted local
-#else
 				s1 = (*s == 'd') ? curr_view->dir : other_view->dir;
-#endif
 				s1 = g_strdup (s1);
 				s2 = s1 + strlen (s1) - 1;
 				if (*s2 == G_DIR_SEPARATOR)
@@ -1066,15 +1122,16 @@ This differs from e2_utils_expand_macros(), due to no distinction between
 %f and %p, and a slightly different approach to quoting
 @param text string to be processed
 @param eachitem replacement string
+
 @return newly-allocted replacement string, or NULL if no replacement done
 */
-gchar *e2_utils_replace_name (gchar *text, gchar *eachitem)
+gchar *e2_utils_replace_name (const gchar *text, const gchar *eachitem)
 {
 	gchar *retval, *withquotes, *freeme;
-	withquotes = eachitem + strlen (eachitem) - 1;
+	withquotes = (gchar *) eachitem + strlen (eachitem) - 1;
 	gboolean quoted = ((*eachitem == '"' && *withquotes == '"')
 		|| (*eachitem == '\'' && *withquotes == '\''));
-	withquotes = (quoted) ? eachitem : g_strconcat ("\"", eachitem, "\"", NULL);
+	withquotes = (quoted) ? (gchar *)eachitem : g_strconcat ("\"", eachitem, "\"", NULL);
 	retval = g_strdup (text);
 
 	if (strstr (retval, "%%p") != NULL)
@@ -1193,12 +1250,16 @@ gchar *e2_utils_replace_name2 (gchar *text, gchar *path, GPtrArray *array,
 	return retval;
 }
 /**
-@brief construct a temporary itemname by adding a suffix to @a localpath
+@brief construct a temporary local itemname by adding a suffix to @a localpath
 @param localpath path of item to be processed, localised string
 @return newly-allocated, localised, path string comprising the temp name
 */
-gchar *e2_utils_get_tempname (gchar *localpath)
+gchar *e2_utils_get_tempname (const gchar *localpath)
 {
+#ifdef E2_VFS
+	VPATH ddata;
+	ddata.spacedata = NULL;
+#endif
 	gchar *temppath;
 	guint i = 0;
 	while (TRUE)
@@ -1210,7 +1271,12 @@ gchar *e2_utils_get_tempname (gchar *localpath)
 			*s = '\0';
 		}
 		E2_ERR_DECLARE
+#ifdef E2_VFS
+		ddata.localpath = temppath;
+		if (e2_fs_access2 (&ddata E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
+#else
 		if (e2_fs_access2 (temppath E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
+#endif
 		{
 			E2_ERR_CLEAR
 			break;
@@ -1232,7 +1298,7 @@ gboolean e2_utils_goto_accessible_path (E2_PaneRuntime *rt)
 {
 	printd (DEBUG, "check that %s is accessible", rt->path);
 #ifdef E2_VFSTMP
-	//FIXME path for non-mounted dirs
+	//FIXME path for non-native dirs
 #else
 	gchar *checkpath = g_strdup (rt->path);	//don't clobber the rt->path string
 #endif
@@ -1256,185 +1322,206 @@ gboolean e2_utils_goto_accessible_path (E2_PaneRuntime *rt)
 	return retval;
 }
 /**
-@brief ensure path string @a path has appropriate separators
+@brief truncate @a path at the right-most separator, if any
+Multiple adjacent separators are treated as one.
+@param path path string to be processed
+@param ignore_trailer TRUE to strip any trailing separator before checking
 
-Prepends curr_view_dir if @a path is not absolute,
-removes superfluous separators from inside @a path,
-appends separator if not one there already.
-Unless @a path is just "/", the returned string will be newly
-allocated, and the old one will have been freed.
+@return TRUE if parent path was created
+*/
+gboolean e2_utils_get_parent_path (gchar *path, gboolean ignore_trailer)
+{
+	gchar *s;
+	gint len = strlen (path);
+	if (ignore_trailer)
+	{
+		s = path + len - sizeof (gchar);
+		if (s > path && *s == G_DIR_SEPARATOR)
+		{
+			*s = '\0';
+			len -= sizeof (gchar);
+		}
+	}
+#ifdef E2_VFSTMP
+	//FIXME handle going past namespace root
+#endif
+	if (len == sizeof (gchar))
+		return FALSE;	//path this short may be root or not, but can't have a parent
+	s = strrchr (path, G_DIR_SEPARATOR);
+	if (s == NULL)
+		return FALSE;
+	if (s > path)
+	{
+		while (*(--s) == G_DIR_SEPARATOR && s > path) {}
+	}
+	*(s + sizeof (gchar)) = '\0';
+	return TRUE;
+}
+/**
+@brief ensure path string @a path has appropriate separators and embedded relativities
 
-@param path absolute or relative path string to be checked (probably UTF)
+Removes superfluous separators, and "./", "../" from inside @a path, but doesn't
+change any leading instances of "../".
+Appends separator if not one there already.
+Unless @a path is just "/", the returned string will be newly allocated, and the
+old one will have been freed, so the returned string may be assigned to the
+supplied path argument.
 
-@return pointer to cleaned absolute path string (utf8)
+@param path absolute or relative path string to be checked (UTF-8), must be freeable
+
+@return pointer to cleaned absolute path string (UTF-8), may be same as @a path
 */
-// (what a clunker this is, with utf parsing !!)
 gchar *e2_utils_path_clean (gchar *path)
 {
-	gchar *freeme = path;
-	path = g_strchug (path);	//allow (maybe deliberate) trailing whitespace
+	g_strchug (path);	//trailing whitespace may be deliberate
 
-	if (!g_str_equal (path, G_DIR_SEPARATOR_S))
-	{	//path is not just the file system root
-		//remove any superfluous separator from within it
-		gunichar c;
-		gint path_len = strlen (path);
-		gchar clean[path_len + 4];  //space for an extra trailer if needed
-		gchar *po = path, *pp = path, *pn = clean;
-		while ((pp = strchr (pp, G_DIR_SEPARATOR)) != NULL ) //if always ascii /, don't need g_utf8_strchr()
-		{
-			//check for a repeated separator, or 0
-			pp++;	//NCHR(pp);
-			c = g_utf8_get_char (pp);
-			if (c == '\0')
-				break;
-			if (c != G_DIR_SEPARATOR)
-			{
-				NCHR (pp);
-				continue;
-			}
-			//found at least 2 separators,
-			//copy old to clean, with the first separator
-			while (po < pp)
-				*(pn++) = *(po++);
-			//skip any more separators
-			while ((c = g_utf8_get_char (pp)) == G_DIR_SEPARATOR)
-				NCHR (pp);
-			po = pp;  //point to the next start for copying
-		}
-		//copy rest, to the terminator
-		while (*po != '\0')
-			*(pn++) = *(po++);
-		//check for trailing /
-		PCHR (pn);
-		c = g_utf8_get_char (pn);
-		NCHR (pn);
+	if (path[0] == G_DIR_SEPARATOR && e2_utils_pass_whitespace (path + sizeof (gchar)) == NULL)
+		return path;
+	//path is not just the file system root
+	//(what a clunker this is, with utf parsing !!) FIXME
+	gunichar c;
+	gchar *pn, *po, *pp, *s;
+
+	gint path_len = strlen (path);
+	gchar clean[path_len + 4];  //space for an extra trailer if needed
+	po = pp = path;
+	pn = clean;
+	//remove any superfluous separator from within it
+	while ((pp = strchr (pp, G_DIR_SEPARATOR)) != NULL ) //if always ascii /, don't need g_utf8_strchr()
+	{
+		//check for a repeated separator, or 0
+		pp++;	//NCHR(pp);
+		c = g_utf8_get_char (pp);
+		if (c == '\0')
+			break;
 		if (c != G_DIR_SEPARATOR)
-		{	//we need to add a trailer
-			//g_strlcpy (pn, G_DIR_SEPARATOR_S, path_len-pn+4);
-			//since separator is ascii, just shove it into place
-			*pn = G_DIR_SEPARATOR;
-			NCHR (pn);
+		{
+			NCHR (pp);
+			continue;
 		}
-		*(pn) = '\0';
-		path = strdup (clean); //get a copy we can keep
-		g_free (freeme);	//and cleanup the one supplied
+		//found at least 2 separators,
+		//copy old to clean, with the first separator
+		while (po < pp)
+			*(pn++) = *(po++);
+		//skip any more separators
+		while ((c = g_utf8_get_char (pp)) == G_DIR_SEPARATOR)
+			NCHR (pp);
+		po = pp;  //point to the next start for copying
 	}
-	return path;
+	//copy rest, to the terminator
+	while (*po != '\0')
+		*(pn++) = *(po++);
+	//check for trailing /
+	PCHR (pn);
+	c = g_utf8_get_char (pn);
+	NCHR (pn);
+	if (c != G_DIR_SEPARATOR)
+	{	//we need to add a trailer
+		//g_strlcpy (pn, G_DIR_SEPARATOR_S, path_len-pn+4);
+		//since separator is ascii, just shove it into place
+		*pn = G_DIR_SEPARATOR;
+		NCHR (pn);
+	}
+	*(pn) = '\0';
+
+	//remove any superfluous . from within it
+	pp = clean;
+	while ((pp = strchr (pp, '.')) != NULL) //always ascii '.', don't need g_utf8_strchr()
+	{
+		s = pp + sizeof (gchar);	//s = pp; NCHR (s);
+		if (*s == G_DIR_SEPARATOR &&
+			(pp == clean || *(pp - sizeof (gchar)) == G_DIR_SEPARATOR))
+		{	//eliminate "./" from path
+			pn = pp;
+			po = s + sizeof (gchar);
+			while (*po != '\0')
+				*(pn++) = *(po++);
+			*(pn) = '\0';
+		}
+		else if (*s == '.' &&
+				(*(s + sizeof (gchar)) == G_DIR_SEPARATOR || *(s+1) == '\0') &&
+				(pp == clean || *(pp - sizeof (gchar)) == G_DIR_SEPARATOR))
+		{	//backup or ignore "../" in path
+			*pp = '\0';
+
+			if (e2_utils_get_parent_path (clean, TRUE))
+			{
+				if (*(s + sizeof (gchar)) == G_DIR_SEPARATOR)
+				{
+					pp = pn = clean + strlen (clean);
+					if (pp == clean || *(pp - sizeof (gchar)) == G_DIR_SEPARATOR)
+						po = s + 2 * sizeof (gchar);	//don't want duplicate separator
+					else
+						po = s + sizeof (gchar);
+					while (*po != '\0')
+						*(pn++) = *(po++);
+					*(pn) = '\0';
+				}
+				else	//".." is at end of path string
+					break;	//finished now
+			}
+			else	//no parent
+			{
+				if (*(s + sizeof (gchar)) == G_DIR_SEPARATOR)
+				{
+					pn = pp;
+					if (pp == clean || *(pp - sizeof (gchar)) == G_DIR_SEPARATOR)
+						po = s + 2 * sizeof (gchar);	//don't want duplicate separator
+					else
+						po = s + sizeof (gchar);
+					while (*po != '\0')
+						*(pn++) = *(po++);
+					*(pn) = '\0';
+				}
+				else	//".." is at end of path string
+				{
+					if (pp == clean)
+						*pp = '.';	//reinstate ".." is the whole path
+					break;
+				}
+			}
+		}
+		else
+			pp++;
+	}
+
+	g_free (path);	//cleanup the one supplied
+	return (strdup (clean)); //send back a copy
 }
 /**
 @brief translate relative path
 
-Creates an absolute new-path string from a
-relative one, or if the new one is already absolute,
-just makes sure it is 'clean'.
-Works with UTF (if splitting on G_DIR_SEPARATOR_S is ok)
-and with non-UTF strings.
-Returned string is newly allocated, the old one needs
-to be freed by caller.
+Creates an absolute path string from a relative one, or if the supplied path is
+already absolute, just makes sure it is 'clean'.
+Works with UTF and with non-UTF path strings.
 
-@param current_dir current-path string (probably UTF)
-@param new_dir absolute or relative new-path string (probably UTF)
+@param current_dir current-path string, UTF-8
+@param new_dir absolute or relative new-path string, UTF-8
 
-* @return newly-allocated, cleaned, absolute path
+* @return newly-allocated, cleaned, absolute path with trailing separator
 */
 gchar *e2_utils_translate_relative_path (gchar *current_dir, gchar *new_dir)
 {
-	//work with a copy, esp so that
-	// e2_utils_path_clean doesn't free anything essential
-	gchar *retval = g_strdup (new_dir);
-	//clean new path, add trailer if need be
-	retval = e2_utils_path_clean (retval);
-	//do we need to do anything?
-	if (g_path_is_absolute (retval))
-		return retval;  //no, send back the copy
-
-	//split current path into its 'segments' (plus leading & trailing nulls)
-	gchar **split_old = g_strsplit (current_dir, G_DIR_SEPARATOR_S, -1);
-	gint split_old_count = 0;
-	while (split_old[split_old_count] != NULL) split_old_count++;
-
-	//likewise, split new dir path
-	//note: new dir path has trailing /, & may have leading / or ./
-	gchar **split = g_strsplit (retval, G_DIR_SEPARATOR_S, -1);
-	gint split_count = 0;
-	while (split[split_count] != NULL) split_count++;
-
-	//suitable size for the segment ptr  array
-	gint join_max_count = split_old_count + split_count + 2;
-	//this is used to point to final path segments
-	gchar *join[join_max_count];
-	gint i, join_count = 0;
-
-	//to get the point of reference for relative cd(s),
-	//prepend the current path segments
-	for (i = 0; i < split_old_count; i++)
-		join[join_count++] = split_old[i];
-	//point back to trailer, to suit updir at start of new path
-	//also = place to store any / at start of new path
-	//CHECKME what if new path is relative, with no leading ./
-	join_count--;
-	//append the new path segments (including trailer)
-	gchar *p;
-	for (i = 0; i < split_count; i++)
-	{
-		if (g_utf8_get_char_validated (split[i], 1) == '.')
-		{
-			p = split[i];
-			NCHR(p);
-			if (*p == '\0' && i == 0)
-			{ //ignore leading reference to current dir
-				continue;
-			}
-			else if (g_utf8_get_char_validated (p, 1) == '.')
-			{
-				NCHR(p);
-				if (*p == '\0')
-				{	//up dir, move back 1 segment if we can
-					//ottherwise ignore it
-					if ( join_count > 0)
-						join_count--;
-					continue;
-				}
-			}
-		}
-		join[join_count++] = split[i];
-	}
-
-	if (join_count == 0)  //we've updir'ed past the root !
-		join[join_count++] = "";  //setup for joined path = root
-
-	//fill the leftover (empty) element field with NULL
-//	no, 1 is enough to terminate the concat
-//		for (; join_count < join_max_count; join_count++)
-	join[join_count] = NULL;
-
-//	g_free (retval); FIXME = this bombs, regardless of the variable name !!
-	//concat the retval
-	retval = g_strjoinv (G_DIR_SEPARATOR_S, join);
-//	}
-	//clean up
-	g_strfreev (split);
-	g_strfreev (split_old);
-	return retval;
+	gchar *s = (g_path_is_absolute (new_dir)) ?
+		g_strdup (new_dir): //copy so that e2_utils_path_clean() can free it
+		g_strconcat (current_dir, G_DIR_SEPARATOR_S, new_dir, NULL);
+	return (e2_utils_path_clean (s));
 }
 /**
-@brief create relative path
-This only works for single-byte separator-characters in file/path names
-If @a src already has relative path, that is returned unchanged
-If @a src is not an absolute path, that is returned unchanged
+@brief create relative path of @a src relative to @a dest
+If @a src is not an absolute path, it is returned unchanged
+This only works for ascii single-byte separator-characters in filepaths
 @param src string containing the path to be relativised
 @param dest string containing the reference path
-@return newly-allocated path of @a src relative to @a dest
+@return newly-allocated path string
 */
-gchar *e2_utils_create_relative_path (gchar *src, gchar *dest)
+gchar *e2_utils_create_relative_path (const gchar *src, const gchar *dest)
 {
-	if (g_str_has_prefix (src, ".."G_DIR_SEPARATOR_S)
-		|| g_str_has_prefix (src, "."G_DIR_SEPARATOR_S)
-		|| src[0] != G_DIR_SEPARATOR)	//not an absolute path, we don't know how to relativise
+	if (*src != G_DIR_SEPARATOR)
+		//not an absolute path, we don't want, or don't know how, to relativise
 		return g_strdup (src);
 
-	gchar *sp = src, *dp = dest, *tsp = NULL;
+	const gchar *sp = src, *dp = dest, *tsp = NULL;
 	GString *rel = g_string_sized_new (PATH_MAX);
 
 	// strip common path
@@ -1460,11 +1547,10 @@ gchar *e2_utils_create_relative_path (gchar *src, gchar *dest)
 //	if (rel->len == 0)
 //		g_string_append (rel, "."G_DIR_SEPARATOR_S);
 	// and add short src path
-	g_string_append (rel, tsp);
+	if (tsp != NULL)
+		g_string_append (rel, tsp);
 
-	tsp = rel->str;
-	g_string_free (rel, FALSE);
-	return tsp;
+	return (g_string_free (rel, FALSE));
 }
 /**
 @brief get unique temp dir name
@@ -1803,7 +1889,7 @@ GtkIconSize e2_utils_get_best_icon_size (gint psize)
 
 @return TRUE if @a name represents a stock-icon
 */
-gboolean e2_utils_check_stock_icon (gchar *name)
+gboolean e2_utils_check_stock_icon (const gchar *name)
 {
 /* gtk_stock_lookup is badly unreliable !!
 	GtkStockItem item;
@@ -1887,6 +1973,7 @@ void e2_utils_update_gtk_settings (void)
 
 /**
 @brief helper function to find matches for last or only path segment
+Arrives here with BGL off
 Note: to eliminate BGL-racing, no UI-change from here
 @param parent absolute path of dir being processed, localised string with trailer
 @param itemname name of discovered item, localised string
@@ -1895,8 +1982,8 @@ Note: to eliminate BGL-racing, no UI-change from here
 
 @return TRUE to signal the read has not been aborted
 */
-static gboolean _e2_utils_drcb_match_wild_last (gchar *parent, gchar *itemname,
-	GList **found, E2_Duo *pair)
+static gboolean _e2_utils_drcb_match_wild_last (VPATH *parent,
+	const gchar *itemname, GList **found, E2_Duo *pair)
 {
 	if (!g_str_equal (itemname, ".."))
 	{
@@ -1913,8 +2000,13 @@ static gboolean _e2_utils_drcb_match_wild_last (gchar *parent, gchar *itemname,
 			}
 			else
 			{
-				gchar *freeme = e2_utils_strcat (parent, itemname);
+				gchar *freeme = e2_utils_strcat (VPSTR (parent), itemname);
+#ifdef E2_VFS
+				VPATH ddata = { freeme, parent->spacedata };
+				if (e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
+#else
 				if (e2_fs_is_dir3 (freeme E2_ERR_NONE()))
+#endif
 				{
 					escname = e2_utf8_escape (utfname, ' ');
 					*found = g_list_append (*found, escname);
@@ -1928,11 +2020,12 @@ static gboolean _e2_utils_drcb_match_wild_last (gchar *parent, gchar *itemname,
 }
 /**
 @brief find matches for last or only path segment of @a arg, if that contains * and/or ?
+Searching is in $PATH directories
 @a arg may include an absolute or relative path.
 Relative path is assumed relative to curr_view->dir
 If @a arg does include any path, that is expected to be all non-wild.
 "." and ".." items are excluded
-For any downstream UI changes, assumes BGL is on/closed
+Expects BGL to be on/closed on arrival here
 @param arg path or itemname which may have wildcard(s) in its only or last segment, utf-8 string
 @param all TRUE to match any type of item, FALSE to match dirs only
 
@@ -1986,20 +2079,29 @@ static GList *_e2_utils_match_wild_last (gchar *arg, gboolean all)
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-			path = e2_utils_dircat (curr_view, path);
+			path = e2_utils_dircat (curr_view, path, FALSE);
 #endif
 			g_free (freeme);
 		}
 		freepath = freename = TRUE;
 	}
 
+	gdk_threads_leave ();	//needed for dirforeach
 	localpath = F_FILENAME_TO_LOCALE (path);
 	GPatternSpec *pattern = g_pattern_spec_new (name);
 	E2_Duo pair = { GINT_TO_POINTER (all), pattern };
 
-	GList *matches = (GList *)e2_fs_dir_foreach (localpath, dirtype,
-			_e2_utils_drcb_match_wild_last, &pair, NULL E2_ERR_NONE());
-
+#ifdef E2_VFS
+#ifdef E2_VFSTMP
+	FIXME placedata
+#endif
+	VPATH ddata = { localpath, NULL };
+	GList *matches = (GList *)e2_fs_dir_foreach (&ddata,
+#else
+	GList *matches = (GList *)e2_fs_dir_foreach (localpath,
+#endif
+		E2_DIRWATCH_NO,	//$PATH is local places only
+		_e2_utils_drcb_match_wild_last, &pair, NULL E2_ERR_NONE());
 	//conform results to API
 	if (E2DREAD_FAILED (matches))
 		matches = NULL;
@@ -2012,12 +2114,13 @@ static GList *_e2_utils_match_wild_last (gchar *arg, gboolean all)
 	F_FREE (localpath);
 	if (freename)
 		g_free (name);
+	gdk_threads_enter ();
 	return matches;
 }
 
-static gboolean _e2_utils_match_wild_path (gchar *parent, E2_WildData *wdata);
 /**
 @brief helper function to match path which has wildcard(s) in its element(s)
+Arrives with BGL open/off
 Note: to eliminate BGL-racing, no UI-change from here
 Reentrant use of _e2_utils_match_wild_path() assmumes BGL is still off here
 @param parent UNUSED absolute path of dir being processed, localised string with or without trailer
@@ -2027,8 +2130,8 @@ Reentrant use of _e2_utils_match_wild_path() assmumes BGL is still off here
 
 @return TRUE to signal the read has not been aborted
 */
-static gboolean _e2_utils_drcb_match_wild_path (gchar *parent, gchar *itemname,
-	GList **found, GPatternSpec *pattern)
+static gboolean _e2_utils_drcb_match_wild_path (VPATH *parent,
+	const gchar *itemname, GList **found, GPatternSpec *pattern)
 {
 	if (!g_str_equal (itemname, ".."))	//"." entries are filtered at source
 	{
@@ -2046,7 +2149,7 @@ This scans @a parent depth-first, until either a complete match is found, or
 no match is possible, in which case it backs up a level and tries the next
 match at that level, and so on.
 Matching path segments are stored in @a wdata
-Any downstream UI changes expect BGL to be closed, here
+Expects BGL to be closed on arrival here
 @param parent absolute path, no wildcard char(s) or redundant separators or trailer, localised string
 @param wdata pointer to struct with parameters for use here
 
@@ -2060,13 +2163,27 @@ static gboolean _e2_utils_match_wild_path (gchar *parent, E2_WildData *wdata)
 	GString *checker;
 	GPatternSpec *pattern;
 	GList *matches, *member;
+#ifdef E2_VFS
+	VPATH ddata;
+#ifdef E2_VFSTMP
+	FIXME valid spacedata for parent
+#endif
+	ddata.spacedata = NULL;
+#endif
+	gdk_threads_leave ();	//needed for dirforeach
 
 	here = wdata->curr_depth;
 	checker = g_string_sized_new (256);
 	checker = g_string_append (checker, parent);
 	pattern = g_pattern_spec_new (wdata->path_elements[here]);
-	matches = (GList *)e2_fs_dir_foreach (checker->str, wdata->dirtype,
-			_e2_utils_drcb_match_wild_path, pattern, NULL E2_ERR_NONE());
+#ifdef E2_VFS
+	ddata.localpath = checker->str;
+	matches = (GList *)e2_fs_dir_foreach (&ddata,
+#else
+	matches = (GList *)e2_fs_dir_foreach (checker->str,
+#endif
+		E2_DIRWATCH_NO,	//$PATH is local places only
+		_e2_utils_drcb_match_wild_path, pattern, NULL E2_ERR_NONE());
 	retval = !(matches == NULL || E2DREAD_FAILED (matches));
 
 	if (retval)
@@ -2079,7 +2196,12 @@ static gboolean _e2_utils_match_wild_path (gchar *parent, E2_WildData *wdata)
 			for (member = matches; member != NULL; member = member->next)
 			{
 				g_string_append_printf (checker, format, (gchar *)member->data);
+#ifdef E2_VFS
+				ddata.localpath = checker->str;
+				retval = !e2_fs_stat (&ddata, wdata->statptr E2_ERR_NONE())
+#else
 				retval = !e2_fs_stat (checker->str, wdata->statptr E2_ERR_NONE())
+#endif
 						&& S_ISDIR (wdata->statptr->st_mode)
 						&& _e2_utils_match_wild_path (checker->str, wdata);	//recurse into dir at next level
 				if (retval)
@@ -2103,6 +2225,7 @@ static gboolean _e2_utils_match_wild_path (gchar *parent, E2_WildData *wdata)
 	g_pattern_spec_free (pattern);
 	g_string_free (checker, TRUE);
 
+	gdk_threads_enter ();
 	return retval;
 }
 /**
@@ -2155,7 +2278,7 @@ static gchar *_e2_utils_match_wild_segment (gchar *string)
 #else
 			path = (abs) ?
 				g_strdup (string):	//use whole string in case basename is also wild
-				e2_utils_dircat (curr_view, string);
+				e2_utils_dircat (curr_view, string, FALSE);
 #endif
 			//making GPatternSpec's requires utf-8 patterns
 			wdata.path_elements = g_strsplit (path, G_DIR_SEPARATOR_S, -1);
@@ -2249,11 +2372,6 @@ static gchar *_e2_utils_match_wild_segment (gchar *string)
 			//scan the filesystem to match the full path
 			struct stat statbuf;
 			wdata.statptr = &statbuf;
-#ifdef E2_VFSTMP
-			wdata.dirtype = ?;
-#else
-			wdata.dirtype = curr_view->dirtype;	//assume all commands apply locally
-#endif
 			if (_e2_utils_match_wild_path (s, &wdata))
 			{
 				g_free (s);
@@ -2432,9 +2550,11 @@ gchar *e2_utils_replace_wildcards (gchar *raw)
 	g_free (raw);
 	return expanded;
 }
-//no. of single-parenthesis characters found in the replacement string,
+//no. of single-quote characters found in the replacement string,
 //up to the end of the current segment
 static gint p_count;
+//and correspondingly for double-quotes
+static gint p2_count;
 static gchar *dollar = "$";
 /**
 @brief add segment to the replacement string
@@ -2460,8 +2580,15 @@ static void _e2_utils_replace_vars_add (gchar *string, gchar *join[], gint *join
 	gchar *p = string;
 	while (*p != '\0')
 	{
-		if (*p++ == '\'')
-			p_count++;
+		if (*p != '\\')
+		{
+			if (*p++ == '\'')
+				p_count++;
+			if (*p == '"')
+				p2_count++;
+		}
+		else
+			p++;
 	}
 }
 /**
@@ -2516,25 +2643,44 @@ gchar *e2_utils_replace_vars (gchar *raw)
 //	printd (DEBUG, "e2_utils_replace_vars (raw:%s)", raw);
 	//replace all relevant occurrences of ~ with ${HOME}
 	gchar *s = g_strdup (raw), *p = s, *h = "${HOME}", *freeme;
-	while ((p = strchr (p, '~')) != NULL) //if always ascii ~, don't need g_utf8_strchr()
+	gint cnt1 = 0; //counter for ' chars
+	gint cnt2 = 0; //counter for " chars
+	while (*p != '\0')
 	{
-		if (
-			   (p == s || *(p-1) == ' ' || *(p-1) == '\t' ) //preceeded by nothing or whitespace
-			&& (*(p+1) == G_DIR_SEPARATOR //and followed by '/'
-					|| *(p+1) == '\0' || *(p+1) == ' ' || *(p+1) == '\t' )	//or lone char in argument
-		)
+		if (*p == '\'')
 		{
-			*p = '\0';
-			freeme = s;
-			s = g_strconcat (s, h, p+1, NULL);
-			p = s + (p - freeme);
-			g_free (freeme);
+			if (p == s || *(p-1) != '\\')
+				cnt1++;
 		}
-		else
-			p++;
+		else if (*p == '"')
+		{
+			if (p == s || *(p-1) != '\\')
+				cnt2++;
+		}
+		else if (*p == '~')	//always ascii ~, don't need utf8 handler
+		{	//check if tilde seems to be outside parentheses
+			if (cnt1 % 2 == 0 && cnt2 % 2 == 0)
+			{ //found a candidate for replacement
+				if (
+					   (p == s || *(p-1) == ' ' || *(p-1) == '\t' ) //preceeded by nothing or whitespace
+					&& (*(p+1) == G_DIR_SEPARATOR //and followed by '/'
+							|| *(p+1) == '\0' || *(p+1) == ' ' || *(p+1) == '\t' )	//or lone char in argument
+					)
+				{
+					*p = '\0';
+					freeme = s;
+					s = g_strconcat (s, h, p+1, NULL);
+					p = s + (p - freeme);
+					g_free (freeme);
+					continue;
+				}
+			}
+		}
+		p++;
 	}
-	//replace all $... occurrences
-	if ((p = strchr (s, '$')) != NULL)	//if always ascii $, don't need g_utf8_strchr()
+
+	//replace $... occurrences, taking quoting into account
+	if (strchr (s, '$') != NULL)	//if always ascii $, don't need g_utf8_strchr()
 	{
 		//break into pieces and count them
 		gchar **split = g_strsplit (s, "$", -1);
@@ -2544,8 +2690,8 @@ gchar *e2_utils_replace_vars (gchar *raw)
 		//init the stack-store for the pieces of the replacement string
 		gint join_count = 0;
 		gchar *join[split_count * 2 + 1];
-		//init the count of ' chars
-		p_count = 0;
+		//init the counts of quote-chars
+		p_count = p2_count = 0;
 		gchar *rest;
 
 		_e2_utils_replace_vars_add (split[0], join, &join_count);	//save the segment before the 1st '$'
@@ -2602,7 +2748,7 @@ gchar *e2_utils_replace_vars (gchar *raw)
 			else if (*split[i] == '{')	//variable to replace
 			{
 				//make sure separator was outside ' '
-				if (p_count%2 == 1)
+				if (p_count % 2 == 1)
 					_e2_utils_replace_vars_ignore (split[i], join, &join_count);
 				else
 				{
@@ -2631,16 +2777,29 @@ gchar *e2_utils_replace_vars (gchar *raw)
 			else	//something else (maybe a variable)
 			{
 				//make sure separator was outside ' '
-				if (p_count%2 == 1)
+				if (p_count % 2 == 1)
 					_e2_utils_replace_vars_ignore (split[i], join, &join_count);
 				else
 				{
+					gchar c = 0;
+					gchar *st;
 					rest = e2_utils_find_whitespace (split[i]);
+					if (p2_count % 2 == 1)
+					{	//inside double-quote
+						st = strchr (split[i], '"');
+						if (st != NULL && (rest == NULL || st < rest))
+							rest = st;
+					}
 					if (rest != NULL)
+					{
+						c = *rest;
 						*rest = '\0';
-					const gchar *env = e2_command_get_variable_value (split[i]);
+					}
+					st = e2_utils_strcat ("$", split[i]);
+					const gchar *env = e2_command_get_variable_value (st);
+					g_free (st);
 					if (env == NULL)
-						env = g_getenv (split[i]);
+						env = g_getenv (split[i]); //WHAT IS THIS FOR?
 					if (rest == NULL)
 					{
 						if (env != NULL)
@@ -2650,7 +2809,7 @@ gchar *e2_utils_replace_vars (gchar *raw)
 					}
 					else //rest != NULL
 					{
-						*rest = ' ';	//too bad if it was \t
+						*rest = c;
 						if (env != NULL)
 						{
 							join[join_count++] = (gchar *) env;
@@ -2671,7 +2830,7 @@ gchar *e2_utils_replace_vars (gchar *raw)
 	return s;
 }
 /**
-@brief get coordinates of @a widget relative to the current screeen
+@brief get coordinates of @a widget relative to its current screen
 @param widget the activated widget whose position is to be calculated
 @param x pointer to gint storage for the x (left) coordinate
 @param y pointer to gint storage for the y (top) coordinate
@@ -2684,11 +2843,13 @@ void e2_utils_get_abs_pos (GtkWidget *widget, gint *x, gint *y)
 	else
 	{
 		GdkWindow *window;
+		GtkWidget *current = widget;
+		while (GTK_WIDGET_NO_WINDOW (current) && (current = gtk_widget_get_parent(current)) != NULL) {}
 
-		if (GTK_WIDGET_NO_WINDOW (widget))
-			window = widget->window;
-		else
-			window = gdk_window_get_parent (widget->window);
+		if (current != NULL)
+			window = current->window;
+		else	//should never happen
+			window = app.main_window->window;
 
 		gdk_window_get_origin (window, x, y);
 
